@@ -230,14 +230,13 @@ image_particles = Image("particles", "particles.png")
 image_game = Image("game", "game.png")
 image_emoticons = Image("emoticons", "emoticons.png")
 image_speedup_arrow = Image("speedup_arrow", "editor/speed_arrow.png")
-image_demobuttons = Image("demobuttons", "demo_buttons.png")
-image_fileicons = Image("fileicons", "file_icons.png")
 image_guibuttons = Image("guibuttons", "gui_buttons.png")
 image_guiicons = Image("guiicons", "gui_icons.png")
 image_arrow = Image("arrow", "arrow.png")
-image_demobuttons2 = Image("demobuttons2", "demo_buttons2.png")
 image_audio_source = Image("audio_source", "editor/audio_source.png")
 image_strongweak = Image("strongweak", "strong_weak.png")
+image_hud = Image("hud", "hud.png")
+image_extras = Image("extras", "extras.png")
 
 container.images.Add(image_null)
 container.images.Add(image_game)
@@ -248,17 +247,20 @@ container.images.Add(image_emoticons)
 container.images.Add(Image("console_bg", "console.png"))
 container.images.Add(Image("console_bar", "console_bar.png"))
 container.images.Add(image_speedup_arrow)
-container.images.Add(image_demobuttons)
-container.images.Add(image_fileicons)
 container.images.Add(image_guibuttons)
 container.images.Add(image_guiicons)
 container.images.Add(image_arrow)
-container.images.Add(image_demobuttons2)
 container.images.Add(image_audio_source)
 container.images.Add(image_strongweak)
+container.images.Add(image_hud)
+container.images.Add(image_extras)
 
 container.pickups.Add(Pickup("health"))
 container.pickups.Add(Pickup("armor"))
+container.pickups.Add(Pickup("armor_shotgun"))
+container.pickups.Add(Pickup("armor_grenade"))
+container.pickups.Add(Pickup("armor_laser"))
+container.pickups.Add(Pickup("armor_ninja"))
 container.pickups.Add(Pickup("weapon"))
 container.pickups.Add(Pickup("ninja", 90, 90))
 
@@ -267,26 +269,24 @@ set_game = SpriteSet("game", image_game, 32, 16)
 set_tee = SpriteSet("tee", image_null, 8, 4)
 set_emoticons = SpriteSet("emoticons", image_emoticons, 4, 4)
 set_speedup_arrow = SpriteSet("speedup_arrow", image_speedup_arrow, 1, 1)
-set_demobuttons = SpriteSet("demobuttons", image_demobuttons, 5, 1)
-set_fileicons = SpriteSet("fileicons", image_fileicons, 8, 1)
 set_guibuttons = SpriteSet("guibuttons", image_guibuttons, 12, 4)
 set_guiicons = SpriteSet("guiicons", image_guiicons, 12, 2)
-set_demobuttons2 = SpriteSet("demobuttons2", image_demobuttons2, 4, 1)
 set_audio_source = SpriteSet("audio_source", image_audio_source, 1, 1)
 set_strongweak = SpriteSet("strongweak", image_strongweak, 2, 1)
+set_hud = SpriteSet("hud", image_hud, 16, 16)
+set_extras = SpriteSet("extras", image_extras, 16, 16)
 
 container.spritesets.Add(set_particles)
 container.spritesets.Add(set_game)
 container.spritesets.Add(set_tee)
 container.spritesets.Add(set_emoticons)
 container.spritesets.Add(set_speedup_arrow)
-container.spritesets.Add(set_demobuttons)
-container.spritesets.Add(set_fileicons)
 container.spritesets.Add(set_guibuttons)
 container.spritesets.Add(set_guiicons)
-container.spritesets.Add(set_demobuttons2)
 container.spritesets.Add(set_audio_source)
 container.spritesets.Add(set_strongweak)
+container.spritesets.Add(set_hud)
+container.spritesets.Add(set_extras)
 
 container.sprites.Add(Sprite("part_slice", set_particles, 0,0,1,1))
 container.sprites.Add(Sprite("part_ball", set_particles, 1,0,1,1))
@@ -358,8 +358,16 @@ container.sprites.Add(Sprite("weapon_ninja_muzzle3", set_game, 25,8,7,4))
 
 container.sprites.Add(Sprite("pickup_health", set_game, 10,2,2,2))
 container.sprites.Add(Sprite("pickup_armor", set_game, 12,2,2,2))
-container.sprites.Add(Sprite("pickup_weapon", set_game, 3,0,6,2))
+container.sprites.Add(Sprite("pickup_hammer", set_game, 2,1,4,3))
+container.sprites.Add(Sprite("pickup_gun", set_game, 2,4,4,2))
+container.sprites.Add(Sprite("pickup_shotgun", set_game, 2,6,8,2))
+container.sprites.Add(Sprite("pickup_grenade", set_game, 2,8,7,2))
+container.sprites.Add(Sprite("pickup_laser", set_game, 2,12,7,3))
 container.sprites.Add(Sprite("pickup_ninja", set_game, 2,10,8,2))
+container.sprites.Add(Sprite("pickup_armor_shotgun", set_game, 15,2,2,2))
+container.sprites.Add(Sprite("pickup_armor_grenade", set_game, 17,2,2,2))
+container.sprites.Add(Sprite("pickup_armor_ninja", set_game, 10,10,2,2))
+container.sprites.Add(Sprite("pickup_armor_laser", set_game, 19,2,2,2))
 
 container.sprites.Add(Sprite("flag_blue", set_game, 12,8,4,8))
 container.sprites.Add(Sprite("flag_red", set_game, 16,8,4,8))
@@ -396,23 +404,6 @@ container.sprites.Add(Sprite("question", set_emoticons, 3, 3, 1, 1))
 
 container.sprites.Add(Sprite("speedup_arrow", set_speedup_arrow, 0,0,1,1))
 
-container.sprites.Add(Sprite("demobutton_play", set_demobuttons, 0,0,1,1))
-container.sprites.Add(Sprite("demobutton_pause", set_demobuttons, 1,0,1,1))
-container.sprites.Add(Sprite("demobutton_stop", set_demobuttons, 2,0,1,1))
-container.sprites.Add(Sprite("demobutton_slower", set_demobuttons, 3,0,1,1))
-container.sprites.Add(Sprite("demobutton_faster", set_demobuttons, 4,0,1,1))
-
-container.sprites.Add(Sprite("demobutton_slice_begin", set_demobuttons2, 0,0,1,1))
-container.sprites.Add(Sprite("demobutton_slice_end", set_demobuttons2, 1,0,1,1))
-container.sprites.Add(Sprite("demobutton_shortcuts_enabled", set_demobuttons2, 2,0,1,1))
-container.sprites.Add(Sprite("demobutton_shortcuts_disabled", set_demobuttons2, 3,0,1,1))
-
-container.sprites.Add(Sprite("file_demo1", set_fileicons, 0,0,1,1))
-container.sprites.Add(Sprite("file_demo2", set_fileicons, 1,0,1,1))
-container.sprites.Add(Sprite("file_folder", set_fileicons, 2,0,1,1))
-container.sprites.Add(Sprite("file_map1", set_fileicons, 5,0,1,1))
-container.sprites.Add(Sprite("file_map2", set_fileicons, 6,0,1,1))
-
 container.sprites.Add(Sprite("guibutton_off", set_guibuttons, 0,0,4,4))
 container.sprites.Add(Sprite("guibutton_on", set_guibuttons, 4,0,4,4))
 container.sprites.Add(Sprite("guibutton_hover", set_guibuttons, 8,0,4,4))
@@ -426,6 +417,39 @@ container.sprites.Add(Sprite("audio_source", set_audio_source, 0,0,1,1))
 container.sprites.Add(Sprite("hook_strong", set_strongweak, 0,0,1,1))
 container.sprites.Add(Sprite("hook_weak", set_strongweak, 1,0,1,1))
 
+container.sprites.Add(Sprite("hud_airjump", set_hud, 0,0,2,2))
+container.sprites.Add(Sprite("hud_airjump_empty", set_hud, 2,0,2,2))
+container.sprites.Add(Sprite("hud_solo", set_hud, 4,0,2,2))
+container.sprites.Add(Sprite("hud_collision_disabled", set_hud, 6,0,2,2))
+container.sprites.Add(Sprite("hud_endless_jump", set_hud, 8,0,2,2))
+container.sprites.Add(Sprite("hud_endless_hook", set_hud, 10,0,2,2))
+container.sprites.Add(Sprite("hud_jetpack", set_hud, 12,0,2,2))
+container.sprites.Add(Sprite("hud_freeze_bar_full_left", set_hud, 0,2,1,1))
+container.sprites.Add(Sprite("hud_freeze_bar_full", set_hud, 1,2,1,1))
+container.sprites.Add(Sprite("hud_freeze_bar_empty", set_hud, 2,2,1,1))
+container.sprites.Add(Sprite("hud_freeze_bar_empty_right", set_hud, 3,2,1,1))
+container.sprites.Add(Sprite("hud_ninja_bar_full_left", set_hud, 0,3,1,1))
+container.sprites.Add(Sprite("hud_ninja_bar_full", set_hud, 1,3,1,1))
+container.sprites.Add(Sprite("hud_ninja_bar_empty", set_hud, 2,3,1,1))
+container.sprites.Add(Sprite("hud_ninja_bar_empty_right", set_hud, 3,3,1,1))
+container.sprites.Add(Sprite("hud_hook_hit_disabled", set_hud, 4,2,2,2))
+container.sprites.Add(Sprite("hud_hammer_hit_disabled", set_hud, 6,2,2,2))
+container.sprites.Add(Sprite("hud_shotgun_hit_disabled", set_hud, 8,2,2,2))
+container.sprites.Add(Sprite("hud_grenade_hit_disabled", set_hud, 10,2,2,2))
+container.sprites.Add(Sprite("hud_laser_hit_disabled", set_hud, 12,2,2,2))
+container.sprites.Add(Sprite("hud_gun_hit_disabled", set_hud, 14,2,2,2))
+container.sprites.Add(Sprite("hud_deep_frozen", set_hud, 10,4,2,2))
+container.sprites.Add(Sprite("hud_live_frozen", set_hud, 12,4,2,2))
+container.sprites.Add(Sprite("hud_teleport_grenade", set_hud, 4,4,2,2))
+container.sprites.Add(Sprite("hud_teleport_gun", set_hud, 6,4,2,2))
+container.sprites.Add(Sprite("hud_teleport_laser", set_hud, 8,4,2,2))
+container.sprites.Add(Sprite("hud_practice_mode", set_hud, 4,6,2,2))
+container.sprites.Add(Sprite("hud_dummy_hammer", set_hud, 6,6,2,2))
+container.sprites.Add(Sprite("hud_dummy_copy", set_hud, 8,6,2,2))
+
+container.sprites.Add(Sprite("part_snowflake", set_extras, 0,0,2,2))
+
+
 anim = Animation("base")
 anim.body.frames.Add(AnimKeyframe(0, 0, -4, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0, 0, 10, 0))
@@ -442,6 +466,18 @@ anim.back_foot.frames.Add(AnimKeyframe(0, -3, 0, -0.1))
 anim.front_foot.frames.Add(AnimKeyframe(0, 3, 0, -0.1))
 container.animations.Add(anim)
 
+anim = Animation("sit_left")
+anim.body.frames.Add(AnimKeyframe(0, 0, 3, 0))
+anim.back_foot.frames.Add(AnimKeyframe(0, -12, 0, 0.1))
+anim.front_foot.frames.Add(AnimKeyframe(0, -8, 0, 0.1))
+container.animations.Add(anim)
+
+anim = Animation("sit_right")
+anim.body.frames.Add(AnimKeyframe(0, 0, 3, 0))
+anim.back_foot.frames.Add(AnimKeyframe(0, 12, 0, -0.1))
+anim.front_foot.frames.Add(AnimKeyframe(0, 8, 0, -0.1))
+container.animations.Add(anim)
+
 anim = Animation("walk")
 anim.body.frames.Add(AnimKeyframe(0.0, 0, 0, 0))
 anim.body.frames.Add(AnimKeyframe(0.2, 0,-1, 0))
@@ -465,6 +501,52 @@ anim.front_foot.frames.Add(AnimKeyframe(0.8, 8, 0, 0))
 anim.front_foot.frames.Add(AnimKeyframe(1.0,-10,-4, 0.2))
 container.animations.Add(anim)
 
+anim = Animation("run_left")
+anim.body.frames.Add(AnimKeyframe(0.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.2, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.4, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.6, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.8, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(1.0, 0, -2, 0))
+
+anim.back_foot.frames.Add(AnimKeyframe(0.0, 18, -11, -0.27))
+anim.back_foot.frames.Add(AnimKeyframe(0.2, 6, 0, 0))
+anim.back_foot.frames.Add(AnimKeyframe(0.4, -7, 0, 0))
+anim.back_foot.frames.Add(AnimKeyframe(0.6, -13, -4.5, 0.05))
+anim.back_foot.frames.Add(AnimKeyframe(0.8, 0, -11, -0.2))
+anim.back_foot.frames.Add(AnimKeyframe(1.0, 18, -11, -0.27))
+
+anim.front_foot.frames.Add(AnimKeyframe(0.0, -13, -4.5, 0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.2, -14, -7, 0.1))
+anim.front_foot.frames.Add(AnimKeyframe(0.4, 11, -13, -0.3))
+anim.front_foot.frames.Add(AnimKeyframe(0.6, 18, -11, -0.27))
+anim.front_foot.frames.Add(AnimKeyframe(0.8, 3, 0, -0.02))
+anim.front_foot.frames.Add(AnimKeyframe(1.0, -13, -4.5, 0.05))
+container.animations.Add(anim)
+
+anim = Animation("run_right")
+anim.body.frames.Add(AnimKeyframe(0.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.2, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.4, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.6, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.8, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(1.0, 0, -2, 0))
+
+anim.back_foot.frames.Add(AnimKeyframe(0.0, -18, -11, 0.27))
+anim.back_foot.frames.Add(AnimKeyframe(0.2, 0, -11, 0.2))
+anim.back_foot.frames.Add(AnimKeyframe(0.4, 13, -4.5, -0.05))
+anim.back_foot.frames.Add(AnimKeyframe(0.6, 7, 0, 0))
+anim.back_foot.frames.Add(AnimKeyframe(0.8, -6, 0, 0))
+anim.back_foot.frames.Add(AnimKeyframe(1.0, -18, -11, 0.27))
+
+anim.front_foot.frames.Add(AnimKeyframe(0.0, 13, -4.5, -0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.2, -3, 0, 0.02))
+anim.front_foot.frames.Add(AnimKeyframe(0.4, -18, -11, 0.27))
+anim.front_foot.frames.Add(AnimKeyframe(0.6, -11, -13, 0.3))
+anim.front_foot.frames.Add(AnimKeyframe(0.8, 14, -7, -0.1))
+anim.front_foot.frames.Add(AnimKeyframe(1.0, 13, -4.5, -0.05))
+container.animations.Add(anim)
+
 anim = Animation("hammer_swing")
 anim.attach.frames.Add(AnimKeyframe(0.0, 0, 0, -0.10))
 anim.attach.frames.Add(AnimKeyframe(0.3, 0, 0, 0.25))
