@@ -1,8 +1,11 @@
 #!/usr/bin/env bash
 
-# You need asciidoc installed
-# Debian/Ubuntu: sudo apt install asciidoc
-# http://asciidoc.org/
+command -v a2x >/dev/null 2>&1 || {
+echo >&2 "You need asciidoc installed";
+echo >&2 "Debian/Ubuntu: sudo apt install asciidoc";
+echo >&2 "http://asciidoc.org/";
+exit 1;
+}
 
 set -ex
 
@@ -16,4 +19,5 @@ done
 EOF
 
 a2x --doctype manpage --format manpage DDNet.adoc
-a2x --doctype manpage --format manpage DDNetServer.adoc
\ No newline at end of file
+a2x --doctype manpage --format manpage DDNetServer.adoc
+
