@@ -6,7 +6,9 @@ os.chdir(os.path.dirname(__file__) + "/..")
 
 PATH = "src/"
 EXCEPTIONS = [
+        "src/base/unicode/confusables.h",
 	"src/base/unicode/confusables_data.h",
+        "src/base/unicode/tolower.h",
 	"src/base/unicode/tolower_data.h",
 	"src/tools/config_common.h"
 ]
@@ -15,20 +17,20 @@ def check_file(filename):
 	if filename in EXCEPTIONS:
 		return False
 	error = False
-	with open(filename) as file:
+	with open(filename, encoding="utf-8") as file:
 		for line in file:
 			if line == "// This file can be included several times.\n":
 				break
 			if line[0] == "/" or line[0] == "*" or line[0] == "\r" or line[0] == "\n" or line[0] == "\t":
 				continue
+			header_guard = "#ifndef " + ("_".join(filename.split(PATH)[1].split("/"))[:-2]).upper() + "_H"
 			if line.startswith("#ifndef"):
-				header_guard = "#ifndef " + ("_".join(filename.split(PATH)[1].split("/"))[:-2]).upper() + "_H"
 				if line[:-1] != header_guard:
 					error = True
-					print("Wrong header guard in {}".format(filename))
+					print(f"Wrong header guard in {filename}, is: {line[:-1]}, should be: {header_guard}")
 			else:
 				error = True
-				print("Missing header guard in {}".format(filename))
+				print(f"Missing header guard in {filename}, should be: {header_guard}")
 			break
 	return error
 
