@@ -4,18 +4,18 @@
 # - http://www.unicode.org/Public/<VERSION>/ucd/UnicodeData.txt
 #
 # If executed as a script, it will generate the contents of the file
-# `src/base/unicode/tolower_data.h`.
+# python3 scripts/generate_unicode_tolower.py header > `src/base/unicode/tolower.h`,
+# python3 scripts/generate_unicode_tolower.py data > `src/base/unicode/tolower_data.h`.
 
+import sys
 import unicode
 
 def generate_cases():
 	ud = unicode.data()
 	return [(unicode.unhex(u["Value"]), unicode.unhex(u["Simple_Lowercase_Mapping"])) for u in ud if u["Simple_Lowercase_Mapping"]]
 
-def main():
-	cases = generate_cases()
-
-	print("""\
+def gen_header(cases):
+	print(f"""\
 #include <stdint.h>
 
 struct UPPER_LOWER
@@ -26,13 +26,32 @@ struct UPPER_LOWER
 
 enum
 {{
-\tNUM_TOLOWER={},
+\tNUM_TOLOWER = {len(cases)},
 }};
 
-static const struct UPPER_LOWER tolower[NUM_TOLOWER] = {{""".format(len(cases)))
+extern const struct UPPER_LOWER tolowermap[];""")
+
+def gen_data(cases):
+	print("""\
+#ifndef TOLOWER_DATA
+#error "This file must only be included in `tolower.cpp`"
+#endif
+
+const struct UPPER_LOWER tolowermap[] = {""")
 	for upper_code, lower_code in cases:
-		print("\t{{{}, {}}},".format(upper_code, lower_code))
+		print(f"\t{{{upper_code}, {lower_code}}},")
 	print("};")
 
+def main():
+	cases = generate_cases()
+
+	header = "header" in sys.argv
+	data = "data" in sys.argv
+
+	if header:
+		gen_header(cases)
+	elif data:
+		gen_data(cases)
+
 if __name__ == '__main__':
 	main()
