@@ -5,7 +5,8 @@ import sys
 import twlang
 
 def copy_fix(infile, delete_unused, append_missing, delete_empty):
-	content = open(infile).readlines()
+	with open(infile, encoding="utf-8") as f:
+		content = f.readlines()
 	trans = twlang.translations(infile)
 	if delete_unused or append_missing:
 		local = twlang.localizes()
@@ -59,7 +60,8 @@ def main(argv):
 
 	content = copy_fix(infile, delete_unused, append_missing, delete_empty)
 
-	open(outfile, "w").write("".join(content))
+	with open(outfile, "w", encoding="utf-8") as f:
+		f.write("".join(content))
 	print("Successfully created '" + outfile + "'.")
 
 if __name__ == '__main__':
