@@ -1,7 +1,7 @@
 #ifndef BASE_HASH_H
 #define BASE_HASH_H
 
-#include <stddef.h>
+#include <cstddef> // size_t
 
 enum
 {
@@ -11,15 +11,15 @@ enum
 	MD5_MAXSTRSIZE = 2 * MD5_DIGEST_LENGTH + 1,
 };
 
-typedef struct
+struct SHA256_DIGEST
 {
 	unsigned char data[SHA256_DIGEST_LENGTH];
-} SHA256_DIGEST;
+};
 
-typedef struct
+struct MD5_DIGEST
 {
 	unsigned char data[MD5_DIGEST_LENGTH];
-} MD5_DIGEST;
+};
 
 SHA256_DIGEST sha256(const void *message, size_t message_len);
 void sha256_str(SHA256_DIGEST digest, char *str, size_t max_len);
@@ -31,8 +31,7 @@ void md5_str(MD5_DIGEST digest, char *str, size_t max_len);
 int md5_from_str(MD5_DIGEST *out, const char *str);
 int md5_comp(MD5_DIGEST digest1, MD5_DIGEST digest2);
 
-static const SHA256_DIGEST SHA256_ZEROED = {{0}};
-static const MD5_DIGEST MD5_ZEROED = {{0}};
+extern const SHA256_DIGEST SHA256_ZEROED;
 
 inline bool operator==(const SHA256_DIGEST &that, const SHA256_DIGEST &other)
 {
