@@ -1,6 +1,6 @@
 #include <cstdlib>
 
-#include "tolower_data.h"
+#include "tolower.h"
 
 static int compul(const void *a, const void *b)
 {
@@ -15,10 +15,14 @@ int str_utf8_tolower(int code)
 	struct UPPER_LOWER key;
 	struct UPPER_LOWER *res;
 	key.upper = code;
-	res = (UPPER_LOWER *)bsearch(&key, tolower, NUM_TOLOWER, sizeof(struct UPPER_LOWER), compul);
+	res = (UPPER_LOWER *)bsearch(&key, tolowermap, NUM_TOLOWER, sizeof(struct UPPER_LOWER), compul);
 
 	if(res == NULL)
 		return code;
 	return res->lower;
 }
 }
+
+#define TOLOWER_DATA
+#include "tolower_data.h"
+#undef TOLOWER_DATA
