@@ -10,8 +10,8 @@
 #define ENGINE_CLIENT_BACKEND_OPENGL_OPENGL_SL_PROGRAM_H_AS_ES
 #endif
 
-#include <base/detect.h>
-
+#include <base/color.h>
+#include <base/vmath.h>
 #include <engine/client/graphics_defines.h>
 
 class CGLSL;
@@ -41,7 +41,7 @@ public:
 	void SetUniform(int Loc, int Count, const float *pValues);
 
 	//for performance reason we do not use SetUniform with using strings... save the Locations of the variables instead
-	int GetUniformLoc(const char *Name);
+	int GetUniformLoc(const char *pName);
 
 	CGLSLProgram();
 	virtual ~CGLSLProgram();
@@ -57,7 +57,7 @@ public:
 	CGLSLTWProgram() :
 		m_LocPos(-1), m_LocTextureSampler(-1), m_LastTextureSampler(-1), m_LastIsTextured(-1)
 	{
-		m_LastScreen[0] = m_LastScreen[1] = m_LastScreen[2] = m_LastScreen[3] = -1.f;
+		m_LastScreenTL = m_LastScreenBR = vec2(-1, -1);
 	}
 
 	int m_LocPos;
@@ -65,7 +65,8 @@ public:
 
 	int m_LastTextureSampler;
 	int m_LastIsTextured;
-	float m_LastScreen[4];
+	vec2 m_LastScreenTL;
+	vec2 m_LastScreenBR;
 };
 
 class CGLSLTextProgram : public CGLSLTWProgram
@@ -86,8 +87,8 @@ public:
 	int m_LocTextOutlineSampler;
 	int m_LocTextureSize;
 
-	float m_LastColor[4];
-	float m_LastOutlineColor[4];
+	ColorRGBA m_LastColor;
+	ColorRGBA m_LastOutlineColor;
 	int m_LastTextSampler;
 	int m_LastTextOutlineSampler;
 	int m_LastTextureSize;
@@ -105,8 +106,8 @@ public:
 
 	{
 		m_LastRotation = 0.f;
-		m_LastCenter[0] = m_LastCenter[1] = 0.f;
-		m_LastVertciesColor[0] = m_LastVertciesColor[1] = m_LastVertciesColor[2] = m_LastVertciesColor[3] = -1.f;
+		m_LastCenter = vec2(0, 0);
+		m_LastVerticesColor = ColorRGBA(-1, -1, -1, -1);
 	}
 
 	int m_LocRotation;
@@ -114,8 +115,8 @@ public:
 	int m_LocVertciesColor;
 
 	float m_LastRotation;
-	float m_LastCenter[2];
-	float m_LastVertciesColor[4];
+	vec2 m_LastCenter;
+	ColorRGBA m_LastVerticesColor;
 };
 
 class CGLSLSpriteMultipleProgram : public CGLSLTWProgram
@@ -124,16 +125,16 @@ public:
 	CGLSLSpriteMultipleProgram()
 
 	{
-		m_LastCenter[0] = m_LastCenter[1] = 0.f;
-		m_LastVertciesColor[0] = m_LastVertciesColor[1] = m_LastVertciesColor[2] = m_LastVertciesColor[3] = -1.f;
+		m_LastCenter = vec2(0, 0);
+		m_LastVerticesColor = ColorRGBA(-1, -1, -1, -1);
 	}
 
 	int m_LocRSP;
 	int m_LocCenter;
 	int m_LocVertciesColor;
 
-	float m_LastCenter[2];
-	float m_LastVertciesColor[4];
+	vec2 m_LastCenter;
+	ColorRGBA m_LastVerticesColor;
 };
 
 class CGLSLQuadProgram : public CGLSLTWProgram
