@@ -1,8 +1,7 @@
 #ifndef ENGINE_CLIENT_BACKEND_VULKAN_BACKEND_VULKAN_H
 #define ENGINE_CLIENT_BACKEND_VULKAN_BACKEND_VULKAN_H
 
-#include <base/detect.h>
-#include <engine/client/backend_sdl.h>
+class CCommandProcessorFragment_GLBase;
 
 static constexpr int gs_BackendVulkanMajor = 1;
 static constexpr int gs_BackendVulkanMinor = 0;
