@@ -7,6 +7,8 @@
 
 #include <algorithm>
 #include <climits>
+#include <unordered_set>
+#include <vector>
 
 #include <base/hash_ctxt.h>
 #include <base/math.h>
@@ -14,6 +16,7 @@
 
 #include <engine/shared/config.h>
 #include <engine/shared/json.h>
+#include <engine/shared/masterserver.h>
 #include <engine/shared/memheap.h>
 #include <engine/shared/network.h>
 #include <engine/shared/protocol.h>
@@ -22,12 +25,11 @@
 #include <engine/config.h>
 #include <engine/console.h>
 #include <engine/engine.h>
+#include <engine/favorites.h>
 #include <engine/friends.h>
 #include <engine/serverbrowser.h>
 #include <engine/storage.h>
 
-#include <mastersrv/mastersrv.h>
-
 #include <engine/external/json-parser/json.h>
 
 #include <game/client/components/menus.h> // PAGE_DDNET
@@ -39,8 +41,8 @@ class SortWrap
 	CServerBrowser *m_pThis;
 
 public:
-	SortWrap(CServerBrowser *t, SortFunc f) :
-		m_pfnSort(f), m_pThis(t) {}
+	SortWrap(CServerBrowser *pServer, SortFunc Func) :
+		m_pfnSort(Func), m_pThis(pServer) {}
 	bool operator()(int a, int b) { return (g_Config.m_BrSortOrder ? (m_pThis->*m_pfnSort)(b, a) : (m_pThis->*m_pfnSort)(a, b)); }
 };
 
@@ -49,10 +51,6 @@ CServerBrowser::CServerBrowser()
 	m_ppServerlist = 0;
 	m_pSortedServerlist = 0;
 
-	m_NumFavoriteServers = 0;
-
-	mem_zero(m_aServerlistIp, sizeof(m_aServerlistIp));
-
 	m_pFirstReqServer = 0; // request list
 	m_pLastReqServer = 0;
 	m_NumRequests = 0;
@@ -90,14 +88,12 @@ CServerBrowser::~CServerBrowser()
 void CServerBrowser::SetBaseInfo(class CNetClient *pClient, const char *pNetVersion)
 {
 	m_pNetClient = pClient;
-	str_copy(m_aNetVersion, pNetVersion, sizeof(m_aNetVersion));
+	str_copy(m_aNetVersion, pNetVersion);
 	m_pConsole = Kernel()->RequestInterface<IConsole>();
 	m_pEngine = Kernel()->RequestInterface<IEngine>();
+	m_pFavorites = Kernel()->RequestInterface<IFavorites>();
 	m_pFriends = Kernel()->RequestInterface<IFriends>();
 	m_pStorage = Kernel()->RequestInterface<IStorage>();
-	IConfigManager *pConfigManager = Kernel()->RequestInterface<IConfigManager>();
-	if(pConfigManager)
-		pConfigManager->RegisterCallback(ConfigSaveCallback, this);
 	m_pPingCache = CreateServerBrowserPingCache(m_pConsole, m_pStorage);
 
 	RegisterCommands();
@@ -117,7 +113,9 @@ void CServerBrowser::Con_LeakIpAddress(IConsole::IResult *pResult, void *pUserDa
 {
 	CServerBrowser *pThis = (CServerBrowser *)pUserData;
 
-	std::vector<int> aSortedServers;
+	// We only consider the first address of every server.
+
+	std::vector<int> vSortedServers;
 	// Sort servers by IP address, ignoring port.
 	class CAddrComparer
 	{
@@ -125,44 +123,44 @@ void CServerBrowser::Con_LeakIpAddress(IConsole::IResult *pResult, void *pUserDa
 		CServerBrowser *m_pThis;
 		bool operator()(int i, int j)
 		{
-			NETADDR Addr1 = m_pThis->m_ppServerlist[i]->m_Addr;
-			NETADDR Addr2 = m_pThis->m_ppServerlist[j]->m_Addr;
+			NETADDR Addr1 = m_pThis->m_ppServerlist[i]->m_Info.m_aAddresses[0];
+			NETADDR Addr2 = m_pThis->m_ppServerlist[j]->m_Info.m_aAddresses[0];
 			Addr1.port = 0;
 			Addr2.port = 0;
 			return net_addr_comp(&Addr1, &Addr2) < 0;
 		}
 	};
-	aSortedServers.reserve(pThis->m_NumServers);
+	vSortedServers.reserve(pThis->m_NumServers);
 	for(int i = 0; i < pThis->m_NumServers; i++)
 	{
-		aSortedServers.push_back(i);
+		vSortedServers.push_back(i);
 	}
-	std::sort(aSortedServers.begin(), aSortedServers.end(), CAddrComparer{pThis});
+	std::sort(vSortedServers.begin(), vSortedServers.end(), CAddrComparer{pThis});
 
 	// Group the servers into those with same IP address (but differing
 	// port).
 	NETADDR Addr;
 	int Start = -1;
-	for(int i = 0; i <= (int)aSortedServers.size(); i++)
+	for(int i = 0; i <= (int)vSortedServers.size(); i++)
 	{
 		NETADDR NextAddr;
-		if(i < (int)aSortedServers.size())
+		if(i < (int)vSortedServers.size())
 		{
-			NextAddr = pThis->m_ppServerlist[aSortedServers[i]]->m_Addr;
+			NextAddr = pThis->m_ppServerlist[vSortedServers[i]]->m_Info.m_aAddresses[0];
 			NextAddr.port = 0;
 		}
-		bool New = Start == -1 || i == (int)aSortedServers.size() || net_addr_comp(&Addr, &NextAddr) != 0;
+		bool New = Start == -1 || i == (int)vSortedServers.size() || net_addr_comp(&Addr, &NextAddr) != 0;
 		if(Start != -1 && New)
 		{
 			int Chosen = Start + secure_rand_below(i - Start);
-			CServerEntry *pChosen = pThis->m_ppServerlist[aSortedServers[Chosen]];
+			CServerEntry *pChosen = pThis->m_ppServerlist[vSortedServers[Chosen]];
 			pChosen->m_RequestIgnoreInfo = true;
 			pThis->QueueRequest(pChosen);
 			char aAddr[NETADDR_MAXSTRSIZE];
-			net_addr_str(&pChosen->m_Addr, aAddr, sizeof(aAddr), true);
+			net_addr_str(&pChosen->m_Info.m_aAddresses[0], aAddr, sizeof(aAddr), true);
 			dbg_msg("serverbrowse/dbg", "queuing ping request for %s", aAddr);
 		}
-		if(i < (int)aSortedServers.size() && New)
+		if(i < (int)vSortedServers.size() && New)
 		{
 			Start = i;
 			Addr = NextAddr;
@@ -199,59 +197,59 @@ int CServerBrowser::GetExtraToken(int Token)
 
 bool CServerBrowser::SortCompareName(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
 	//	make sure empty entries are listed last
-	return (a->m_GotInfo && b->m_GotInfo) || (!a->m_GotInfo && !b->m_GotInfo) ? str_comp(a->m_Info.m_aName, b->m_Info.m_aName) < 0 :
-										    a->m_GotInfo != 0;
+	return (pIndex1->m_GotInfo && pIndex2->m_GotInfo) || (!pIndex1->m_GotInfo && !pIndex2->m_GotInfo) ? str_comp(pIndex1->m_Info.m_aName, pIndex2->m_Info.m_aName) < 0 :
+													    pIndex1->m_GotInfo != 0;
 }
 
 bool CServerBrowser::SortCompareMap(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
-	return str_comp(a->m_Info.m_aMap, b->m_Info.m_aMap) < 0;
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
+	return str_comp(pIndex1->m_Info.m_aMap, pIndex2->m_Info.m_aMap) < 0;
 }
 
 bool CServerBrowser::SortComparePing(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
-	return a->m_Info.m_Latency < b->m_Info.m_Latency;
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
+	return pIndex1->m_Info.m_Latency < pIndex2->m_Info.m_Latency;
 }
 
 bool CServerBrowser::SortCompareGametype(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
-	return str_comp(a->m_Info.m_aGameType, b->m_Info.m_aGameType) < 0;
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
+	return str_comp(pIndex1->m_Info.m_aGameType, pIndex2->m_Info.m_aGameType) < 0;
 }
 
 bool CServerBrowser::SortCompareNumPlayers(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
-	return a->m_Info.m_NumFilteredPlayers > b->m_Info.m_NumFilteredPlayers;
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
+	return pIndex1->m_Info.m_NumFilteredPlayers > pIndex2->m_Info.m_NumFilteredPlayers;
 }
 
 bool CServerBrowser::SortCompareNumClients(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
-	return a->m_Info.m_NumClients > b->m_Info.m_NumClients;
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
+	return pIndex1->m_Info.m_NumClients > pIndex2->m_Info.m_NumClients;
 }
 
 bool CServerBrowser::SortCompareNumPlayersAndPing(int Index1, int Index2) const
 {
-	CServerEntry *a = m_ppServerlist[Index1];
-	CServerEntry *b = m_ppServerlist[Index2];
+	CServerEntry *pIndex1 = m_ppServerlist[Index1];
+	CServerEntry *pIndex2 = m_ppServerlist[Index2];
 
-	if(a->m_Info.m_NumFilteredPlayers == b->m_Info.m_NumFilteredPlayers)
-		return a->m_Info.m_Latency > b->m_Info.m_Latency;
-	else if(a->m_Info.m_NumFilteredPlayers == 0 || b->m_Info.m_NumFilteredPlayers == 0 || a->m_Info.m_Latency / 100 == b->m_Info.m_Latency / 100)
-		return a->m_Info.m_NumFilteredPlayers < b->m_Info.m_NumFilteredPlayers;
+	if(pIndex1->m_Info.m_NumFilteredPlayers == pIndex2->m_Info.m_NumFilteredPlayers)
+		return pIndex1->m_Info.m_Latency > pIndex2->m_Info.m_Latency;
+	else if(pIndex1->m_Info.m_NumFilteredPlayers == 0 || pIndex2->m_Info.m_NumFilteredPlayers == 0 || pIndex1->m_Info.m_Latency / 100 == pIndex2->m_Info.m_Latency / 100)
+		return pIndex1->m_Info.m_NumFilteredPlayers < pIndex2->m_Info.m_NumFilteredPlayers;
 	else
-		return a->m_Info.m_Latency > b->m_Info.m_Latency;
+		return pIndex1->m_Info.m_Latency > pIndex2->m_Info.m_Latency;
 }
 
 void CServerBrowser::Filter()
@@ -438,8 +436,8 @@ void CServerBrowser::Sort()
 	else if(g_Config.m_BrSort == IServerBrowser::SORT_GAMETYPE)
 		std::stable_sort(m_pSortedServerlist, m_pSortedServerlist + m_NumSortedServers, SortWrap(this, &CServerBrowser::SortCompareGametype));
 
-	str_copy(m_aFilterGametypeString, g_Config.m_BrFilterGametype, sizeof(m_aFilterGametypeString));
-	str_copy(m_aFilterString, g_Config.m_BrFilterString, sizeof(m_aFilterString));
+	str_copy(m_aFilterGametypeString, g_Config.m_BrFilterGametype);
+	str_copy(m_aFilterString, g_Config.m_BrFilterString);
 	m_Sorthash = SortHash();
 }
 
@@ -465,14 +463,12 @@ void CServerBrowser::RemoveRequest(CServerEntry *pEntry)
 
 CServerBrowser::CServerEntry *CServerBrowser::Find(const NETADDR &Addr)
 {
-	CServerEntry *pEntry = m_aServerlistIp[Addr.ip[0]];
-
-	for(; pEntry; pEntry = pEntry->m_pNextIp)
+	auto Entry = m_ByAddr.find(Addr);
+	if(Entry == m_ByAddr.end())
 	{
-		if(net_addr_comp(&pEntry->m_Addr, &Addr) == 0)
-			return pEntry;
+		return nullptr;
 	}
-	return (CServerEntry *)0;
+	return m_ppServerlist[Entry->second];
 }
 
 void CServerBrowser::QueueRequest(CServerEntry *pEntry)
@@ -488,15 +484,42 @@ void CServerBrowser::QueueRequest(CServerEntry *pEntry)
 	m_NumRequests++;
 }
 
+void ServerBrowserFormatAddresses(char *pBuffer, int BufferSize, NETADDR *pAddrs, int NumAddrs)
+{
+	for(int i = 0; i < NumAddrs; i++)
+	{
+		if(i != 0)
+		{
+			if(BufferSize <= 1)
+			{
+				return;
+			}
+			pBuffer[0] = ',';
+			pBuffer[1] = 0;
+			pBuffer += 1;
+			BufferSize -= 1;
+		}
+		if(BufferSize <= 1)
+		{
+			return;
+		}
+		net_addr_str(&pAddrs[i], pBuffer, BufferSize, true);
+		int Length = str_length(pBuffer);
+		pBuffer += Length;
+		BufferSize -= Length;
+	}
+}
+
 void CServerBrowser::SetInfo(CServerEntry *pEntry, const CServerInfo &Info)
 {
-	bool Fav = pEntry->m_Info.m_Favorite;
-	bool Off = pEntry->m_Info.m_Official;
+	CServerInfo TmpInfo = pEntry->m_Info;
 	pEntry->m_Info = Info;
-	pEntry->m_Info.m_Favorite = Fav;
-	pEntry->m_Info.m_Official = Off;
-	pEntry->m_Info.m_NetAddr = pEntry->m_Addr;
-	net_addr_str(&pEntry->m_Info.m_NetAddr, pEntry->m_Info.m_aAddress, sizeof(pEntry->m_Info.m_aAddress), 1);
+	pEntry->m_Info.m_Favorite = TmpInfo.m_Favorite;
+	pEntry->m_Info.m_FavoriteAllowPing = TmpInfo.m_FavoriteAllowPing;
+	pEntry->m_Info.m_Official = TmpInfo.m_Official;
+	mem_copy(pEntry->m_Info.m_aAddresses, TmpInfo.m_aAddresses, sizeof(pEntry->m_Info.m_aAddresses));
+	pEntry->m_Info.m_NumAddresses = TmpInfo.m_NumAddresses;
+	ServerBrowserFormatAddresses(pEntry->m_Info.m_aAddress, sizeof(pEntry->m_Info.m_aAddress), pEntry->m_Info.m_aAddresses, pEntry->m_Info.m_NumAddresses);
 
 	class CPlayerScoreNameLess
 	{
@@ -530,23 +553,42 @@ void CServerBrowser::SetInfo(CServerEntry *pEntry, const CServerInfo &Info)
 
 void CServerBrowser::SetLatency(NETADDR Addr, int Latency)
 {
-	Addr.port = 0;
-	for(CServerEntry *pEntry = m_aServerlistIp[Addr.ip[0]]; pEntry; pEntry = pEntry->m_pNextIp)
-	{
-		NETADDR Other = pEntry->m_Addr;
-		Other.port = 0;
-		if(net_addr_comp(&Addr, &Other) == 0 && pEntry->m_GotInfo)
-		{
-			pEntry->m_Info.m_Latency = Latency;
-			pEntry->m_Info.m_LatencyIsEstimated = false;
-		}
-	}
 	m_pPingCache->CachePing(Addr, Latency);
+
+	Addr.port = 0;
+	for(int i = 0; i < m_NumServers; i++)
+	{
+		if(!m_ppServerlist[i]->m_GotInfo)
+		{
+			continue;
+		}
+		bool Found = false;
+		for(int j = 0; j < m_ppServerlist[i]->m_Info.m_NumAddresses; j++)
+		{
+			NETADDR Other = m_ppServerlist[i]->m_Info.m_aAddresses[j];
+			Other.port = 0;
+			if(Addr == Other)
+			{
+				Found = true;
+				break;
+			}
+		}
+		if(!Found)
+		{
+			continue;
+		}
+		int Ping = m_pPingCache->GetPing(m_ppServerlist[i]->m_Info.m_aAddresses, m_ppServerlist[i]->m_Info.m_NumAddresses);
+		if(Ping == -1)
+		{
+			continue;
+		}
+		m_ppServerlist[i]->m_Info.m_Latency = Ping;
+		m_ppServerlist[i]->m_Info.m_LatencyIsEstimated = false;
+	}
 }
 
-CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
+CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR *pAddrs, int NumAddrs)
 {
-	int Hash = Addr.ip[0];
 	CServerEntry *pEntry = 0;
 
 	// create new pEntry
@@ -554,37 +596,43 @@ CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
 	mem_zero(pEntry, sizeof(CServerEntry));
 
 	// set the info
-	pEntry->m_Addr = Addr;
-	pEntry->m_Info.m_NetAddr = Addr;
+	mem_copy(pEntry->m_Info.m_aAddresses, pAddrs, NumAddrs * sizeof(pAddrs[0]));
+	pEntry->m_Info.m_NumAddresses = NumAddrs;
 
 	pEntry->m_Info.m_Latency = 999;
 	pEntry->m_Info.m_HasRank = -1;
-	net_addr_str(&Addr, pEntry->m_Info.m_aAddress, sizeof(pEntry->m_Info.m_aAddress), true);
+	ServerBrowserFormatAddresses(pEntry->m_Info.m_aAddress, sizeof(pEntry->m_Info.m_aAddress), pEntry->m_Info.m_aAddresses, pEntry->m_Info.m_NumAddresses);
 	str_copy(pEntry->m_Info.m_aName, pEntry->m_Info.m_aAddress, sizeof(pEntry->m_Info.m_aName));
 
 	// check if it's a favorite
-	pEntry->m_Info.m_Favorite = IsFavorite(Addr);
+	pEntry->m_Info.m_Favorite = m_pFavorites->IsFavorite(pEntry->m_Info.m_aAddresses, pEntry->m_Info.m_NumAddresses);
+	pEntry->m_Info.m_FavoriteAllowPing = m_pFavorites->IsPingAllowed(pEntry->m_Info.m_aAddresses, pEntry->m_Info.m_NumAddresses);
 
 	// check if it's an official server
-	for(auto &Network : m_aNetworks)
+	bool Official = false;
+	for(int i = 0; !Official && i < (int)std::size(m_aNetworks); i++)
 	{
-		for(int i = 0; i < Network.m_NumCountries; i++)
+		for(int j = 0; !Official && j < m_aNetworks[i].m_NumCountries; j++)
 		{
-			CNetworkCountry *pCntr = &Network.m_aCountries[i];
-			for(int j = 0; j < pCntr->m_NumServers; j++)
+			CNetworkCountry *pCntr = &m_aNetworks[i].m_aCountries[j];
+			for(int k = 0; !Official && k < pCntr->m_NumServers; k++)
 			{
-				if(net_addr_comp(&Addr, &pCntr->m_aServers[j]) == 0)
+				for(int l = 0; !Official && l < NumAddrs; l++)
 				{
-					pEntry->m_Info.m_Official = true;
-					break;
+					if(pAddrs[l] == pCntr->m_aServers[k])
+					{
+						Official = true;
+					}
 				}
 			}
 		}
 	}
+	pEntry->m_Info.m_Official = Official;
 
-	// add to the hash list
-	pEntry->m_pNextIp = m_aServerlistIp[Hash];
-	m_aServerlistIp[Hash] = pEntry;
+	for(int i = 0; i < NumAddrs; i++)
+	{
+		m_ByAddr[pAddrs[i]] = m_NumServers;
+	}
 
 	if(m_NumServers == m_NumServerCapacity)
 	{
@@ -605,153 +653,94 @@ CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
 	return pEntry;
 }
 
-void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServerInfo *pInfo)
+void CServerBrowser::OnServerInfoUpdate(const NETADDR &Addr, int Token, const CServerInfo *pInfo)
 {
-	CServerEntry *pEntry = 0;
-	if(Type == IServerBrowser::SET_MASTER_ADD)
+	int BasicToken = Token;
+	int ExtraToken = 0;
+	if(pInfo->m_Type == SERVERINFO_EXTENDED)
 	{
-		if(m_ServerlistType != IServerBrowser::TYPE_INTERNET)
-			return;
-		if(!Find(Addr))
-		{
-			pEntry = Add(Addr);
-			QueueRequest(pEntry);
-		}
+		BasicToken = Token & 0xff;
+		ExtraToken = Token >> 8;
 	}
-	else if(Type == IServerBrowser::SET_FAV_ADD)
-	{
-		if(m_ServerlistType != IServerBrowser::TYPE_FAVORITES)
-			return;
 
-		if(!Find(Addr))
-		{
-			pEntry = Add(Addr);
-			QueueRequest(pEntry);
-		}
-	}
-	else if(Type == IServerBrowser::SET_DDNET_ADD)
-	{
-		if(m_ServerlistType != IServerBrowser::TYPE_DDNET)
-			return;
+	CServerEntry *pEntry = Find(Addr);
 
-		if(!Find(Addr))
-		{
-			pEntry = Add(Addr);
-			QueueRequest(pEntry);
-		}
-	}
-	else if(Type == IServerBrowser::SET_KOG_ADD)
+	if(m_ServerlistType == IServerBrowser::TYPE_LAN)
 	{
-		if(m_ServerlistType != IServerBrowser::TYPE_KOG)
-			return;
-
-		if(!Find(Addr))
+		NETADDR Broadcast;
+		mem_zero(&Broadcast, sizeof(Broadcast));
+		Broadcast.type = m_pNetClient->NetType() | NETTYPE_LINK_BROADCAST;
+		int TokenBC = GenerateToken(Broadcast);
+		bool Drop = false;
+		Drop = Drop || BasicToken != GetBasicToken(TokenBC);
+		Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenBC));
+		if(Drop)
 		{
-			pEntry = Add(Addr);
-			QueueRequest(pEntry);
+			return;
 		}
+
+		if(!pEntry)
+			pEntry = Add(&Addr, 1);
 	}
-	else if(Type == IServerBrowser::SET_HTTPINFO)
+	else
 	{
 		if(!pEntry)
 		{
-			pEntry = Add(Addr);
+			return;
 		}
-		if(pEntry)
+		int TokenAddr = GenerateToken(Addr);
+		bool Drop = false;
+		Drop = Drop || BasicToken != GetBasicToken(TokenAddr);
+		Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenAddr));
+		if(Drop)
 		{
-			SetInfo(pEntry, *pInfo);
-			pEntry->m_RequestIgnoreInfo = true;
+			return;
 		}
 	}
-	else if(Type == IServerBrowser::SET_TOKEN)
+
+	if(m_ServerlistType == IServerBrowser::TYPE_LAN)
 	{
-		int BasicToken = Token;
-		int ExtraToken = 0;
-		if(pInfo->m_Type == SERVERINFO_EXTENDED)
+		SetInfo(pEntry, *pInfo);
+		pEntry->m_Info.m_Latency = minimum(static_cast<int>((time_get() - m_BroadcastTime) * 1000 / time_freq()), 999);
+		if(pInfo->m_Type == SERVERINFO_VANILLA && Is64Player(pInfo))
 		{
-			BasicToken = Token & 0xff;
-			ExtraToken = Token >> 8;
+			pEntry->m_Request64Legacy = true;
+			// Force a quick update.
+			RequestImpl64(Addr, pEntry);
+		}
+	}
+	else if(pEntry->m_RequestTime > 0)
+	{
+		if(!pEntry->m_RequestIgnoreInfo)
+		{
+			SetInfo(pEntry, *pInfo);
 		}
 
-		pEntry = Find(Addr);
-
-		if(m_ServerlistType == IServerBrowser::TYPE_LAN)
+		int Latency = minimum(static_cast<int>((time_get() - pEntry->m_RequestTime) * 1000 / time_freq()), 999);
+		if(!pEntry->m_RequestIgnoreInfo)
 		{
-			NETADDR Broadcast;
-			mem_zero(&Broadcast, sizeof(Broadcast));
-			Broadcast.type = m_pNetClient->NetType() | NETTYPE_LINK_BROADCAST;
-			int TokenBC = GenerateToken(Broadcast);
-			bool Drop = false;
-			Drop = Drop || BasicToken != GetBasicToken(TokenBC);
-			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenBC));
-			if(Drop)
-			{
-				return;
-			}
-
-			if(!pEntry)
-				pEntry = Add(Addr);
+			pEntry->m_Info.m_Latency = Latency;
 		}
 		else
 		{
-			if(!pEntry)
-			{
-				return;
-			}
-			int TokenAddr = GenerateToken(Addr);
-			bool Drop = false;
-			Drop = Drop || BasicToken != GetBasicToken(TokenAddr);
-			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenAddr));
-			if(Drop)
-			{
-				return;
-			}
+			char aAddr[NETADDR_MAXSTRSIZE];
+			net_addr_str(&Addr, aAddr, sizeof(aAddr), true);
+			dbg_msg("serverbrowse/dbg", "received ping response from %s", aAddr);
+			SetLatency(Addr, Latency);
 		}
+		pEntry->m_RequestTime = -1; // Request has been answered
 
-		if(m_ServerlistType == IServerBrowser::TYPE_LAN)
+		if(!pEntry->m_RequestIgnoreInfo)
 		{
-			SetInfo(pEntry, *pInfo);
-			pEntry->m_Info.m_Latency = minimum(static_cast<int>((time_get() - m_BroadcastTime) * 1000 / time_freq()), 999);
 			if(pInfo->m_Type == SERVERINFO_VANILLA && Is64Player(pInfo))
 			{
 				pEntry->m_Request64Legacy = true;
 				// Force a quick update.
-				RequestImpl64(pEntry->m_Addr, pEntry);
+				RequestImpl64(Addr, pEntry);
 			}
 		}
-		else if(pEntry->m_RequestTime > 0)
-		{
-			if(!pEntry->m_RequestIgnoreInfo)
-			{
-				SetInfo(pEntry, *pInfo);
-			}
-
-			int Latency = minimum(static_cast<int>((time_get() - pEntry->m_RequestTime) * 1000 / time_freq()), 999);
-			if(!pEntry->m_RequestIgnoreInfo)
-			{
-				pEntry->m_Info.m_Latency = Latency;
-			}
-			else
-			{
-				char aAddr[NETADDR_MAXSTRSIZE];
-				net_addr_str(&Addr, aAddr, sizeof(aAddr), true);
-				dbg_msg("serverbrowse/dbg", "received ping response from %s", aAddr);
-				SetLatency(Addr, Latency);
-			}
-			pEntry->m_RequestTime = -1; // Request has been answered
-
-			if(!pEntry->m_RequestIgnoreInfo)
-			{
-				if(pInfo->m_Type == SERVERINFO_VANILLA && Is64Player(pInfo))
-				{
-					pEntry->m_Request64Legacy = true;
-					// Force a quick update.
-					RequestImpl64(pEntry->m_Addr, pEntry);
-				}
-			}
-		}
-		RemoveRequest(pEntry);
 	}
+	RemoveRequest(pEntry);
 
 	m_SortOnNextUpdate = true;
 }
@@ -768,7 +757,7 @@ void CServerBrowser::Refresh(int Type)
 
 	if(Type == IServerBrowser::TYPE_LAN)
 	{
-		unsigned char Buffer[sizeof(SERVERBROWSE_GETINFO) + 1];
+		unsigned char aBuffer[sizeof(SERVERBROWSE_GETINFO) + 1];
 		CNetChunk Packet;
 		int i;
 
@@ -777,13 +766,13 @@ void CServerBrowser::Refresh(int Type)
 		mem_zero(&Packet, sizeof(Packet));
 		Packet.m_Address.type = m_pNetClient->NetType() | NETTYPE_LINK_BROADCAST;
 		Packet.m_Flags = NETSENDFLAG_CONNLESS | NETSENDFLAG_EXTENDED;
-		Packet.m_DataSize = sizeof(Buffer);
-		Packet.m_pData = Buffer;
+		Packet.m_DataSize = sizeof(aBuffer);
+		Packet.m_pData = aBuffer;
 		mem_zero(&Packet.m_aExtraData, sizeof(Packet.m_aExtraData));
 
 		int Token = GenerateToken(Packet.m_Address);
-		mem_copy(Buffer, SERVERBROWSE_GETINFO, sizeof(SERVERBROWSE_GETINFO));
-		Buffer[sizeof(SERVERBROWSE_GETINFO)] = GetBasicToken(Token);
+		mem_copy(aBuffer, SERVERBROWSE_GETINFO, sizeof(SERVERBROWSE_GETINFO));
+		aBuffer[sizeof(SERVERBROWSE_GETINFO)] = GetBasicToken(Token);
 
 		Packet.m_aExtraData[0] = GetExtraToken(Token) >> 8;
 		Packet.m_aExtraData[1] = GetExtraToken(Token) & 0xff;
@@ -816,7 +805,7 @@ void CServerBrowser::Refresh(int Type)
 
 void CServerBrowser::RequestImpl(const NETADDR &Addr, CServerEntry *pEntry, int *pBasicToken, int *pToken, bool RandomToken) const
 {
-	unsigned char Buffer[sizeof(SERVERBROWSE_GETINFO) + 1];
+	unsigned char aBuffer[sizeof(SERVERBROWSE_GETINFO) + 1];
 	CNetChunk Packet;
 
 	if(g_Config.m_Debug)
@@ -847,14 +836,14 @@ void CServerBrowser::RequestImpl(const NETADDR &Addr, CServerEntry *pEntry, int
 		*pBasicToken = GetBasicToken(Token);
 	}
 
-	mem_copy(Buffer, SERVERBROWSE_GETINFO, sizeof(SERVERBROWSE_GETINFO));
-	Buffer[sizeof(SERVERBROWSE_GETINFO)] = GetBasicToken(Token);
+	mem_copy(aBuffer, SERVERBROWSE_GETINFO, sizeof(SERVERBROWSE_GETINFO));
+	aBuffer[sizeof(SERVERBROWSE_GETINFO)] = GetBasicToken(Token);
 
 	Packet.m_ClientID = -1;
 	Packet.m_Address = Addr;
 	Packet.m_Flags = NETSENDFLAG_CONNLESS | NETSENDFLAG_EXTENDED;
-	Packet.m_DataSize = sizeof(Buffer);
-	Packet.m_pData = Buffer;
+	Packet.m_DataSize = sizeof(aBuffer);
+	Packet.m_pData = aBuffer;
 	mem_zero(&Packet.m_aExtraData, sizeof(Packet.m_aExtraData));
 	Packet.m_aExtraData[0] = GetExtraToken(Token) >> 8;
 	Packet.m_aExtraData[1] = GetExtraToken(Token) & 0xff;
@@ -867,7 +856,7 @@ void CServerBrowser::RequestImpl(const NETADDR &Addr, CServerEntry *pEntry, int
 
 void CServerBrowser::RequestImpl64(const NETADDR &Addr, CServerEntry *pEntry) const
 {
-	unsigned char Buffer[sizeof(SERVERBROWSE_GETINFO_64_LEGACY) + 1];
+	unsigned char aBuffer[sizeof(SERVERBROWSE_GETINFO_64_LEGACY) + 1];
 	CNetChunk Packet;
 
 	if(g_Config.m_Debug)
@@ -879,14 +868,14 @@ void CServerBrowser::RequestImpl64(const NETADDR &Addr, CServerEntry *pEntry) co
 		m_pConsole->Print(IConsole::OUTPUT_LEVEL_DEBUG, "client_srvbrowse", aBuf);
 	}
 
-	mem_copy(Buffer, SERVERBROWSE_GETINFO_64_LEGACY, sizeof(SERVERBROWSE_GETINFO_64_LEGACY));
-	Buffer[sizeof(SERVERBROWSE_GETINFO_64_LEGACY)] = GetBasicToken(GenerateToken(Addr));
+	mem_copy(aBuffer, SERVERBROWSE_GETINFO_64_LEGACY, sizeof(SERVERBROWSE_GETINFO_64_LEGACY));
+	aBuffer[sizeof(SERVERBROWSE_GETINFO_64_LEGACY)] = GetBasicToken(GenerateToken(Addr));
 
 	Packet.m_ClientID = -1;
 	Packet.m_Address = Addr;
 	Packet.m_Flags = NETSENDFLAG_CONNLESS;
-	Packet.m_DataSize = sizeof(Buffer);
-	Packet.m_pData = Buffer;
+	Packet.m_DataSize = sizeof(aBuffer);
+	Packet.m_pData = aBuffer;
 
 	m_pNetClient->Send(&Packet);
 
@@ -909,28 +898,8 @@ void CServerBrowser::SetCurrentServerPing(const NETADDR &Addr, int Ping)
 	SetLatency(Addr, minimum(Ping, 999));
 }
 
-void ServerBrowserFillEstimatedLatency(int OwnLocation, const IServerBrowserPingCache::CEntry *pEntries, int NumEntries, int *pIndex, NETADDR Addr, CServerInfo *pInfo)
-{
-	Addr.port = 0;
-	while(*pIndex < NumEntries && net_addr_comp(&pEntries[*pIndex].m_Addr, &Addr) < 0)
-	{
-		*pIndex += 1;
-	}
-	if(*pIndex >= NumEntries || net_addr_comp(&pEntries[*pIndex].m_Addr, &Addr) != 0)
-	{
-		pInfo->m_LatencyIsEstimated = true;
-		pInfo->m_Latency = CServerInfo::EstimateLatency(OwnLocation, pInfo->m_Location);
-		return;
-	}
-	pInfo->m_LatencyIsEstimated = false;
-	pInfo->m_Latency = pEntries[*pIndex].m_Ping;
-}
-
 void CServerBrowser::UpdateFromHttp()
 {
-	const IServerBrowserPingCache::CEntry *pPingEntries;
-	int NumPingEntries;
-	m_pPingCache->GetPingCache(&pPingEntries, &NumPingEntries);
 	int OwnLocation;
 	if(str_comp(g_Config.m_BrLocation, "auto") == 0)
 	{
@@ -948,24 +917,13 @@ void CServerBrowser::UpdateFromHttp()
 
 	int NumServers = m_pHttp->NumServers();
 	int NumLegacyServers = m_pHttp->NumLegacyServers();
+	std::unordered_set<NETADDR> WantedAddrs;
+	std::function<bool(const NETADDR *, int)> Want = [](const NETADDR *pAddrs, int NumAddrs) { return true; };
 	if(m_ServerlistType != IServerBrowser::TYPE_INTERNET)
 	{
-		class CWantedAddr
-		{
-		public:
-			NETADDR m_Addr;
-			bool m_FallbackToPing;
-			bool m_Got;
-		};
-		std::vector<CWantedAddr> aWantedAddresses;
-		int LegacySetType;
 		if(m_ServerlistType == IServerBrowser::TYPE_FAVORITES)
 		{
-			for(int i = 0; i < m_NumFavoriteServers; i++)
-			{
-				aWantedAddresses.push_back(CWantedAddr{m_aFavoriteServers[i], m_aFavoriteServersAllowPing[i], false});
-			}
-			LegacySetType = IServerBrowser::SET_FAV_ADD;
+			Want = [&](const NETADDR *pAddrs, int NumAddrs) -> bool { return m_pFavorites->IsFavorite(pAddrs, NumAddrs) != TRISTATE::NONE; };
 		}
 		else
 		{
@@ -976,13 +934,11 @@ void CServerBrowser::UpdateFromHttp()
 			{
 			case IServerBrowser::TYPE_DDNET:
 				Network = NETWORK_DDNET;
-				LegacySetType = IServerBrowser::SET_DDNET_ADD;
 				pExcludeCountries = g_Config.m_BrFilterExcludeCountries;
 				pExcludeTypes = g_Config.m_BrFilterExcludeTypes;
 				break;
 			case IServerBrowser::TYPE_KOG:
 				Network = NETWORK_KOG;
-				LegacySetType = IServerBrowser::SET_KOG_ADD;
 				pExcludeCountries = g_Config.m_BrFilterExcludeCountriesKoG;
 				pExcludeTypes = g_Config.m_BrFilterExcludeTypesKoG;
 				break;
@@ -1016,139 +972,83 @@ void CServerBrowser::UpdateFromHttp()
 
 					if(DDNetFiltered(pExcludeTypes, pCntr->m_aTypes[g]))
 						continue;
-					aWantedAddresses.push_back(CWantedAddr{pCntr->m_aServers[g], false, false});
+					WantedAddrs.insert(pCntr->m_aServers[g]);
 				}
 			}
-		}
-		std::vector<int> aSortedServers;
-		std::vector<int> aSortedLegacyServers;
-		aSortedServers.reserve(NumServers);
-		for(int i = 0; i < NumServers; i++)
-		{
-			aSortedServers.push_back(i);
-		}
-		aSortedLegacyServers.reserve(NumLegacyServers);
-		for(int i = 0; i < NumLegacyServers; i++)
-		{
-			aSortedLegacyServers.push_back(i);
-		}
-
-		class CWantedAddrComparer
-		{
-		public:
-			bool operator()(const CWantedAddr &a, const CWantedAddr &b)
-			{
-				return net_addr_comp(&a.m_Addr, &b.m_Addr) < 0;
-			}
-		};
-		class CAddrComparer
-		{
-		public:
-			IServerBrowserHttp *m_pHttp;
-			bool operator()(int i, int j)
-			{
-				return net_addr_comp(&m_pHttp->ServerAddress(i), &m_pHttp->ServerAddress(j)) < 0;
-			}
-		};
-		class CLegacyAddrComparer
-		{
-		public:
-			IServerBrowserHttp *m_pHttp;
-			bool operator()(int i, int j)
-			{
-				return net_addr_comp(&m_pHttp->LegacyServer(i), &m_pHttp->LegacyServer(j)) < 0;
-			}
-		};
-
-		std::sort(aWantedAddresses.begin(), aWantedAddresses.end(), CWantedAddrComparer());
-		std::sort(aSortedServers.begin(), aSortedServers.end(), CAddrComparer{m_pHttp});
-		std::sort(aSortedLegacyServers.begin(), aSortedLegacyServers.end(), CLegacyAddrComparer{m_pHttp});
-
-		unsigned i = 0;
-		unsigned j = 0;
-		int p = 0;
-		while(i < aWantedAddresses.size() && j < aSortedServers.size())
-		{
-			int Cmp = net_addr_comp(&aWantedAddresses[i].m_Addr, &m_pHttp->ServerAddress(aSortedServers[j]));
-			if(Cmp != 0)
-			{
-				if(Cmp < 0)
+			Want = [&](const NETADDR *pAddrs, int NumAddrs) -> bool {
+				for(int i = 0; i < NumAddrs; i++)
 				{
-					i++;
-				}
-				else
-				{
-					j++;
-				}
-				continue;
-			}
-			aWantedAddresses[i].m_Got = true;
-			NETADDR Addr;
-			CServerInfo Info;
-			m_pHttp->Server(aSortedServers[j], &Addr, &Info);
-			ServerBrowserFillEstimatedLatency(OwnLocation, pPingEntries, NumPingEntries, &p, Addr, &Info);
-			Info.m_HasRank = HasRank(Info.m_aMap);
-			Set(Addr, IServerBrowser::SET_HTTPINFO, -1, &Info);
-			i++;
-			j++;
-		}
-		i = 0;
-		j = 0;
-		while(i < aWantedAddresses.size() && j < aSortedLegacyServers.size())
-		{
-			int Cmp = net_addr_comp(&aWantedAddresses[i].m_Addr, &m_pHttp->LegacyServer(aSortedLegacyServers[j]));
-			if(Cmp != 0)
-			{
-				if(Cmp < 0)
-				{
-					i++;
-				}
-				else
-				{
-					j++;
-				}
-				continue;
-			}
-			aWantedAddresses[i].m_Got = true;
-			Set(m_pHttp->LegacyServer(aSortedLegacyServers[j]), LegacySetType, -1, nullptr);
-			i++;
-			j++;
-		}
-		for(const CWantedAddr &Wanted : aWantedAddresses)
-		{
-			if(!Wanted.m_Got)
-			{
-				if(Wanted.m_FallbackToPing)
-				{
-					Set(Wanted.m_Addr, LegacySetType, -1, nullptr);
-				}
-				else
-				{
-					// Also add favorites we're not allowed to ping.
-					if(LegacySetType == IServerBrowser::SET_FAV_ADD && !Find(Wanted.m_Addr))
+					if(WantedAddrs.count(pAddrs[i]))
 					{
-						Add(Wanted.m_Addr);
+						return true;
 					}
 				}
-			}
+				return false;
+			};
 		}
-		return;
 	}
-	int p = 0;
 	for(int i = 0; i < NumServers; i++)
 	{
-		NETADDR Addr;
-		CServerInfo Info;
-		m_pHttp->Server(i, &Addr, &Info);
-		ServerBrowserFillEstimatedLatency(OwnLocation, pPingEntries, NumPingEntries, &p, Addr, &Info);
+		CServerInfo Info = m_pHttp->Server(i);
+		if(!Want(Info.m_aAddresses, Info.m_NumAddresses))
+		{
+			continue;
+		}
+		int Ping = m_pPingCache->GetPing(Info.m_aAddresses, Info.m_NumAddresses);
+		Info.m_LatencyIsEstimated = Ping == -1;
+		if(Info.m_LatencyIsEstimated)
+		{
+			Info.m_Latency = CServerInfo::EstimateLatency(OwnLocation, Info.m_Location);
+		}
+		else
+		{
+			Info.m_Latency = Ping;
+		}
 		Info.m_HasRank = HasRank(Info.m_aMap);
-		Set(Addr, IServerBrowser::SET_HTTPINFO, -1, &Info);
+		CServerEntry *pEntry = Add(Info.m_aAddresses, Info.m_NumAddresses);
+		SetInfo(pEntry, Info);
+		pEntry->m_RequestIgnoreInfo = true;
 	}
 	for(int i = 0; i < NumLegacyServers; i++)
 	{
 		NETADDR Addr = m_pHttp->LegacyServer(i);
-		Set(Addr, IServerBrowser::SET_MASTER_ADD, -1, nullptr);
+		if(!Want(&Addr, 1))
+		{
+			continue;
+		}
+		QueueRequest(Add(&Addr, 1));
 	}
+
+	if(m_ServerlistType == IServerBrowser::TYPE_FAVORITES)
+	{
+		const IFavorites::CEntry *pFavorites;
+		int NumFavorites;
+		m_pFavorites->AllEntries(&pFavorites, &NumFavorites);
+		for(int i = 0; i < NumFavorites; i++)
+		{
+			bool Found = false;
+			for(int j = 0; j < pFavorites[i].m_NumAddrs; j++)
+			{
+				if(Find(pFavorites[i].m_aAddrs[j]))
+				{
+					Found = true;
+					break;
+				}
+			}
+			if(Found)
+			{
+				continue;
+			}
+			// (Also add favorites we're not allowed to ping.)
+			CServerEntry *pEntry = Add(pFavorites[i].m_aAddrs, pFavorites[i].m_NumAddrs);
+			if(pFavorites->m_AllowPing)
+			{
+				QueueRequest(pEntry);
+			}
+		}
+	}
+
+	m_SortOnNextUpdate = true;
 }
 
 void CServerBrowser::CleanUp()
@@ -1157,7 +1057,7 @@ void CServerBrowser::CleanUp()
 	m_ServerlistHeap.Reset();
 	m_NumServers = 0;
 	m_NumSortedServers = 0;
-	mem_zero(m_aServerlistIp, sizeof(m_aServerlistIp));
+	m_ByAddr.clear();
 	m_pFirstReqServer = 0;
 	m_pLastReqServer = 0;
 	m_NumRequests = 0;
@@ -1172,7 +1072,7 @@ void CServerBrowser::Update(bool ForceResort)
 	const char *pHttpBestUrl;
 	if(!m_pHttp->GetBestUrl(&pHttpBestUrl) && pHttpBestUrl != m_pHttpPrevBestUrl)
 	{
-		str_copy(g_Config.m_BrCachedBestServerinfoUrl, pHttpBestUrl, sizeof(g_Config.m_BrCachedBestServerinfoUrl));
+		str_copy(g_Config.m_BrCachedBestServerinfoUrl, pHttpBestUrl);
 		m_pHttpPrevBestUrl = pHttpBestUrl;
 	}
 
@@ -1206,9 +1106,9 @@ void CServerBrowser::Update(bool ForceResort)
 		if(pEntry->m_RequestTime == 0)
 		{
 			if(pEntry->m_Request64Legacy)
-				RequestImpl64(pEntry->m_Addr, pEntry);
+				RequestImpl64(pEntry->m_Info.m_aAddresses[0], pEntry);
 			else
-				RequestImpl(pEntry->m_Addr, pEntry, nullptr, nullptr, false);
+				RequestImpl(pEntry->m_Info.m_aAddresses[0], pEntry, nullptr, nullptr, false);
 		}
 
 		Count++;
@@ -1248,94 +1148,17 @@ void CServerBrowser::Update(bool ForceResort)
 	// check if we need to resort
 	if(m_Sorthash != SortHash() || ForceResort || m_SortOnNextUpdate)
 	{
+		for(int i = 0; i < m_NumServers; i++)
+		{
+			CServerInfo *pInfo = &m_ppServerlist[i]->m_Info;
+			pInfo->m_Favorite = m_pFavorites->IsFavorite(pInfo->m_aAddresses, pInfo->m_NumAddresses);
+			pInfo->m_FavoriteAllowPing = m_pFavorites->IsPingAllowed(pInfo->m_aAddresses, pInfo->m_NumAddresses);
+		}
 		Sort();
 		m_SortOnNextUpdate = false;
 	}
 }
 
-int CServerBrowser::FindFavorite(const NETADDR &Addr) const
-{
-	// search for the address
-	for(int i = 0; i < m_NumFavoriteServers; i++)
-	{
-		if(net_addr_comp(&Addr, &m_aFavoriteServers[i]) == 0)
-			return i;
-	}
-	return -1;
-}
-
-bool CServerBrowser::GotInfo(const NETADDR &Addr) const
-{
-	CServerEntry *pEntry = ((CServerBrowser *)this)->Find(Addr);
-	return pEntry && pEntry->m_GotInfo;
-}
-
-bool CServerBrowser::IsFavorite(const NETADDR &Addr) const
-{
-	return FindFavorite(Addr) >= 0;
-}
-
-bool CServerBrowser::IsFavoritePingAllowed(const NETADDR &Addr) const
-{
-	int i = FindFavorite(Addr);
-	dbg_assert(i >= 0, "invalid favorite");
-	return i >= 0 && m_aFavoriteServersAllowPing[i];
-}
-
-void CServerBrowser::AddFavorite(const NETADDR &Addr)
-{
-	CServerEntry *pEntry;
-
-	if(m_NumFavoriteServers == MAX_FAVORITES)
-		return;
-
-	// make sure that we don't already have the server in our list
-	if(IsFavorite(Addr))
-	{
-		return;
-	}
-
-	// add the server to the list
-	m_aFavoriteServers[m_NumFavoriteServers] = Addr;
-	m_aFavoriteServersAllowPing[m_NumFavoriteServers] = false;
-	m_NumFavoriteServers++;
-	pEntry = Find(Addr);
-	if(pEntry)
-		pEntry->m_Info.m_Favorite = true;
-
-	if(g_Config.m_Debug)
-	{
-		char aAddrStr[NETADDR_MAXSTRSIZE];
-		net_addr_str(&Addr, aAddrStr, sizeof(aAddrStr), true);
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "added fav, %s", aAddrStr);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_DEBUG, "client_srvbrowse", aBuf);
-	}
-}
-
-void CServerBrowser::FavoriteAllowPing(const NETADDR &Addr, bool AllowPing)
-{
-	int i = FindFavorite(Addr);
-	dbg_assert(i >= 0, "invalid favorite");
-	m_aFavoriteServersAllowPing[i] = AllowPing;
-}
-
-void CServerBrowser::RemoveFavorite(const NETADDR &Addr)
-{
-	int i = FindFavorite(Addr);
-	if(i < 0)
-	{
-		return;
-	}
-	mem_move(&m_aFavoriteServers[i], &m_aFavoriteServers[i + 1], sizeof(NETADDR) * (m_NumFavoriteServers - (i + 1)));
-	mem_move(&m_aFavoriteServersAllowPing[i], &m_aFavoriteServersAllowPing[i + 1], sizeof(bool) * (m_NumFavoriteServers - (i + 1)));
-	m_NumFavoriteServers--;
-
-	CServerEntry *pEntry = Find(Addr);
-	if(pEntry)
-		pEntry->m_Info.m_Favorite = false;
-}
-
 void CServerBrowser::LoadDDNetServers()
 {
 	if(!m_pDDNetInfo)
@@ -1374,7 +1197,7 @@ void CServerBrowser::LoadDDNetServers()
 
 			pCntr->Reset();
 
-			str_copy(pCntr->m_aName, json_string_get(pName), sizeof(pCntr->m_aName));
+			str_copy(pCntr->m_aName, json_string_get(pName));
 			pCntr->m_FlagID = json_int_get(pFlagID);
 
 			// add country
@@ -1400,7 +1223,7 @@ void CServerBrowser::LoadDDNetServers()
 					}
 					if(Pos == pNet->m_NumTypes)
 					{
-						str_copy(pNet->m_aTypes[pNet->m_NumTypes], pType, sizeof(pNet->m_aTypes[pNet->m_NumTypes]));
+						str_copy(pNet->m_aTypes[pNet->m_NumTypes], pType);
 						pNet->m_NumTypes++;
 					}
 				}
@@ -1416,7 +1239,7 @@ void CServerBrowser::LoadDDNetServers()
 					}
 					const char *pStr = json_string_get(pAddr);
 					net_addr_from_str(&pCntr->m_aServers[pCntr->m_NumServers], pStr);
-					str_copy(pCntr->m_aTypes[pCntr->m_NumServers], pType, sizeof(pCntr->m_aTypes[pCntr->m_NumServers]));
+					str_copy(pCntr->m_aTypes[pCntr->m_NumServers], pType);
 				}
 			}
 
@@ -1480,26 +1303,14 @@ int CServerBrowser::HasRank(const char *pMap)
 
 void CServerBrowser::LoadDDNetInfoJson()
 {
-	IOHANDLE File = m_pStorage->OpenFile(DDNET_INFO, IOFLAG_READ | IOFLAG_SKIP_BOM, IStorage::TYPE_SAVE);
-	if(!File)
+	void *pBuf;
+	unsigned Length;
+	if(!m_pStorage->ReadFile(DDNET_INFO, IStorage::TYPE_SAVE, &pBuf, &Length))
 		return;
 
-	const int Length = io_length(File);
-	if(Length <= 0)
-	{
-		io_close(File);
-		return;
-	}
-
-	char *pBuf = (char *)malloc(Length);
-	pBuf[0] = '\0';
-
-	io_read(File, pBuf, Length);
-	io_close(File);
-
 	json_value_free(m_pDDNetInfo);
 
-	m_pDDNetInfo = json_parse(pBuf, Length);
+	m_pDDNetInfo = json_parse((json_char *)pBuf, Length);
 
 	free(pBuf);
 
@@ -1585,30 +1396,6 @@ int CServerBrowser::LoadingProgression() const
 	return 100.0f * Loaded / Servers;
 }
 
-void CServerBrowser::ConfigSaveCallback(IConfigManager *pConfigManager, void *pUserData)
-{
-	CServerBrowser *pSelf = (CServerBrowser *)pUserData;
-
-	char aAddrStr[128];
-	char aBuffer[256];
-	for(int i = 0; i < pSelf->m_NumFavoriteServers; i++)
-	{
-		net_addr_str(&pSelf->m_aFavoriteServers[i], aAddrStr, sizeof(aAddrStr), true);
-		if(!pSelf->m_aFavoriteServersAllowPing[i])
-		{
-			str_format(aBuffer, sizeof(aBuffer), "add_favorite %s", aAddrStr);
-		}
-		else
-		{
-			// Add quotes to the first parameter for backward
-			// compatibility with versions that took a `r` console
-			// parameter.
-			str_format(aBuffer, sizeof(aBuffer), "add_favorite \"%s\" allow_ping", aAddrStr);
-		}
-		pConfigManager->WriteLine(aBuffer);
-	}
-}
-
 void CServerBrowser::DDNetFilterAdd(char *pFilter, const char *pName)
 {
 	if(DDNetFiltered(pFilter, pName))
@@ -1627,11 +1414,11 @@ void CServerBrowser::DDNetFilterRem(char *pFilter, const char *pName)
 	// rewrite exclude/filter list
 	char aBuf[128];
 
-	str_copy(aBuf, pFilter, sizeof(aBuf));
+	str_copy(aBuf, pFilter);
 	pFilter[0] = '\0';
 
 	char aToken[128];
-	for(const char *tok = aBuf; (tok = str_next_token(tok, ",", aToken, sizeof(aToken)));)
+	for(const char *pTok = aBuf; (pTok = str_next_token(pTok, ",", aToken, sizeof(aToken)));)
 	{
 		if(str_comp_nocase(pName, aToken) != 0)
 		{
@@ -1711,7 +1498,7 @@ bool CServerInfo::ParseLocation(int *pResult, const char *pString)
 		return true;
 	}
 	// ISO continent code. Allow antarctica, but treat it as unknown.
-	static const char LOCATIONS[][6] = {
+	static const char s_apLocations[][6] = {
 		"an", // LOC_UNKNOWN
 		"af", // LOC_AFRICA
 		"as", // LOC_ASIA
@@ -1721,9 +1508,9 @@ bool CServerInfo::ParseLocation(int *pResult, const char *pString)
 		"sa", // LOC_SOUTH_AMERICA
 		"as:cn", // LOC_CHINA
 	};
-	for(int i = sizeof(LOCATIONS) / sizeof(LOCATIONS[0]) - 1; i >= 0; i--)
+	for(int i = std::size(s_apLocations) - 1; i >= 0; i--)
 	{
-		if(str_startswith(pString, LOCATIONS[i]))
+		if(str_startswith(pString, s_apLocations[i]))
 		{
 			*pResult = i;
 			return false;
