@@ -1,17 +1,29 @@
 #if defined(CONF_VIDEORECORDER)
 
-#include <engine/console.h>
 #include <engine/shared/config.h>
 #include <engine/storage.h>
 
+#include <base/lock_scope.h>
 #include <engine/client/graphics_threaded.h>
 #include <engine/sound.h>
 
+extern "C" {
+#include <libavutil/avutil.h>
+#include <libavutil/opt.h>
+#include <libswresample/swresample.h>
+#include <libswscale/swscale.h>
+};
+
 #include <memory>
 #include <mutex>
 
 #include "video.h"
 
+#include <chrono>
+#include <thread>
+
+using namespace std::chrono_literals;
+
 // This code is mostly stolen from https://github.com/FFmpeg/FFmpeg/blob/master/doc/examples/muxing.c
 
 #define STREAM_PIX_FMT AV_PIX_FMT_YUV420P /* default pix_fmt */
@@ -19,7 +31,7 @@
 const size_t FORMAT_GL_NCHANNELS = 4;
 LOCK g_WriteLock = 0;
 
-CVideo::CVideo(CGraphics_Threaded *pGraphics, ISound *pSound, IStorage *pStorage, IConsole *pConsole, int Width, int Height, const char *pName) :
+CVideo::CVideo(CGraphics_Threaded *pGraphics, ISound *pSound, IStorage *pStorage, int Width, int Height, const char *pName) :
 	m_pGraphics(pGraphics),
 	m_pStorage(pStorage),
 	m_pSound(pSound)
@@ -28,12 +40,12 @@ CVideo::CVideo(CGraphics_Threaded *pGraphics, ISound *pSound, IStorage *pStorage
 	m_pFormat = 0;
 	m_pOptDict = 0;
 
-	m_VideoCodec = 0;
-	m_AudioCodec = 0;
+	m_pVideoCodec = 0;
+	m_pAudioCodec = 0;
 
 	m_Width = Width;
 	m_Height = Height;
-	str_copy(m_Name, pName, sizeof(m_Name));
+	str_copy(m_aName, pName);
 
 	m_FPS = g_Config.m_ClVideoRecorderFPS;
 
@@ -42,9 +54,6 @@ CVideo::CVideo(CGraphics_Threaded *pGraphics, ISound *pSound, IStorage *pStorage
 	m_ProcessingVideoFrame = 0;
 	m_ProcessingAudioFrame = 0;
 
-	m_NextFrame = false;
-
-	// TODO:
 	m_HasAudio = g_Config.m_ClVideoSndEnable;
 
 	dbg_assert(ms_pCurrentVideo == 0, "ms_pCurrentVideo is NOT set to NULL while creating a new Video.");
@@ -71,8 +80,8 @@ void CVideo::Start()
 	char aDate[20];
 	str_timestamp(aDate, sizeof(aDate));
 	char aBuf[256];
-	if(str_length(m_Name) != 0)
-		str_format(aBuf, sizeof(aBuf), "videos/%s", m_Name);
+	if(str_length(m_aName) != 0)
+		str_format(aBuf, sizeof(aBuf), "videos/%s", m_aName);
 	else
 		str_format(aBuf, sizeof(aBuf), "videos/%s.mp4", aDate);
 
@@ -124,7 +133,7 @@ void CVideo::Start()
 	 * and initialize the codecs. */
 	if(m_pFormat->video_codec != AV_CODEC_ID_NONE)
 	{
-		if(!AddStream(&m_VideoStream, m_pFormatContext, &m_VideoCodec, m_pFormat->video_codec))
+		if(!AddStream(&m_VideoStream, m_pFormatContext, &m_pVideoCodec, m_pFormat->video_codec))
 			return;
 	}
 	else
@@ -134,7 +143,7 @@ void CVideo::Start()
 
 	if(m_HasAudio && m_pFormat->audio_codec != AV_CODEC_ID_NONE)
 	{
-		if(!AddStream(&m_AudioStream, m_pFormatContext, &m_AudioCodec, m_pFormat->audio_codec))
+		if(!AddStream(&m_AudioStream, m_pFormatContext, &m_pAudioCodec, m_pFormat->audio_codec))
 			return;
 	}
 	else
@@ -150,7 +159,7 @@ void CVideo::Start()
 	for(size_t i = 0; i < m_VideoThreads; ++i)
 	{
 		std::unique_lock<std::mutex> Lock(m_vVideoThreads[i]->m_Mutex);
-		m_vVideoThreads[i]->m_Thread = std::thread([this, i]() { RunVideoThread(i == 0 ? (m_VideoThreads - 1) : (i - 1), i); });
+		m_vVideoThreads[i]->m_Thread = std::thread([this, i]() REQUIRES(!g_WriteLock) { RunVideoThread(i == 0 ? (m_VideoThreads - 1) : (i - 1), i); });
 		m_vVideoThreads[i]->m_Cond.wait(Lock, [this, i]() -> bool { return m_vVideoThreads[i]->m_Started; });
 	}
 
@@ -162,7 +171,7 @@ void CVideo::Start()
 	for(size_t i = 0; i < m_AudioThreads; ++i)
 	{
 		std::unique_lock<std::mutex> Lock(m_vAudioThreads[i]->m_Mutex);
-		m_vAudioThreads[i]->m_Thread = std::thread([this, i]() { RunAudioThread(i == 0 ? (m_AudioThreads - 1) : (i - 1), i); });
+		m_vAudioThreads[i]->m_Thread = std::thread([this, i]() REQUIRES(!g_WriteLock) { RunAudioThread(i == 0 ? (m_AudioThreads - 1) : (i - 1), i); });
 		m_vAudioThreads[i]->m_Cond.wait(Lock, [this, i]() -> bool { return m_vAudioThreads[i]->m_Started; });
 	}
 
@@ -258,7 +267,7 @@ void CVideo::Stop()
 	m_vAudioThreads.clear();
 
 	while(m_ProcessingVideoFrame > 0 || m_ProcessingAudioFrame > 0)
-		thread_sleep(10);
+		std::this_thread::sleep_for(10us);
 
 	m_Recording = false;
 
@@ -290,7 +299,7 @@ void CVideo::Stop()
 
 void CVideo::NextVideoFrameThread()
 {
-	if(m_NextFrame && m_Recording)
+	if(m_Recording)
 	{
 		// #ifdef CONF_PLATFORM_MACOS
 		// 	CAutoreleasePool AutoreleasePool;
@@ -343,7 +352,6 @@ void CVideo::NextVideoFrameThread()
 				m_CurVideoThreadIndex = 0;
 		}
 
-		m_NextFrame = false;
 		// sync_barrier();
 		// m_Semaphore.signal();
 	}
@@ -353,21 +361,9 @@ void CVideo::NextVideoFrame()
 {
 	if(m_Recording)
 	{
-		// #ifdef CONF_PLATFORM_MACOS
-		// 	CAutoreleasePool AutoreleasePool;
-		// #endif
-
-		//dbg_msg("video_recorder", "called");
-
 		ms_Time += ms_TickTime;
 		ms_LocalTime = (ms_Time - ms_LocalStartTime) / (float)time_freq();
-		m_NextFrame = true;
 		m_Vframe += 1;
-
-		// m_pGraphics->KickCommandBuffer();
-		//thread_sleep(500);
-
-		// m_Semaphore.wait();
 	}
 }
 
@@ -473,10 +469,11 @@ void CVideo::RunAudioThread(size_t ParentThreadIndex, size_t ThreadIndex)
 			{
 				std::unique_lock<std::mutex> LockAudio(pThreadData->m_AudioFillMutex);
 
-				lock_wait(g_WriteLock);
-				m_AudioStream.m_vpFrames[ThreadIndex]->pts = av_rescale_q(pThreadData->m_SampleCountStart, AVRational{1, m_AudioStream.pEnc->sample_rate}, m_AudioStream.pEnc->time_base);
-				WriteFrame(&m_AudioStream, ThreadIndex);
-				lock_unlock(g_WriteLock);
+				{
+					CLockScope ls(g_WriteLock);
+					m_AudioStream.m_vpFrames[ThreadIndex]->pts = av_rescale_q(pThreadData->m_SampleCountStart, AVRational{1, m_AudioStream.pEnc->sample_rate}, m_AudioStream.pEnc->time_base);
+					WriteFrame(&m_AudioStream, ThreadIndex);
+				}
 
 				pThreadData->m_AudioFrameToFill = 0;
 				pThreadData->m_AudioFillCond.notify_all();
@@ -553,10 +550,11 @@ void CVideo::RunVideoThread(size_t ParentThreadIndex, size_t ThreadIndex)
 			}
 			{
 				std::unique_lock<std::mutex> LockVideo(pThreadData->m_VideoFillMutex);
-				lock_wait(g_WriteLock);
-				m_VideoStream.m_vpFrames[ThreadIndex]->pts = (int64_t)m_VideoStream.pEnc->frame_number;
-				WriteFrame(&m_VideoStream, ThreadIndex);
-				lock_unlock(g_WriteLock);
+				{
+					CLockScope ls(g_WriteLock);
+					m_VideoStream.m_vpFrames[ThreadIndex]->pts = (int64_t)m_VideoStream.pEnc->frame_number;
+					WriteFrame(&m_VideoStream, ThreadIndex);
+				}
 
 				pThreadData->m_VideoFrameToFill = 0;
 				pThreadData->m_VideoFillCond.notify_all();
@@ -571,9 +569,9 @@ void CVideo::RunVideoThread(size_t ParentThreadIndex, size_t ThreadIndex)
 
 void CVideo::FillVideoFrame(size_t ThreadIndex)
 {
-	const int InLinesize[1] = {4 * m_VideoStream.pEnc->width};
+	const int InLineSize = 4 * m_VideoStream.pEnc->width;
 	auto *pRGBAData = m_vPixelHelper[ThreadIndex].data();
-	sws_scale(m_VideoStream.m_vpSwsCtxs[ThreadIndex], (const uint8_t *const *)&pRGBAData, InLinesize, 0,
+	sws_scale(m_VideoStream.m_vpSwsCtxs[ThreadIndex], (const uint8_t *const *)&pRGBAData, &InLineSize, 0,
 		m_VideoStream.pEnc->height, m_VideoStream.m_vpFrames[ThreadIndex]->data, m_VideoStream.m_vpFrames[ThreadIndex]->linesize);
 }
 
@@ -611,23 +609,27 @@ AVFrame *CVideo::AllocPicture(enum AVPixelFormat PixFmt, int Width, int Height)
 
 AVFrame *CVideo::AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64_t ChannelLayout, int SampleRate, int NbSamples)
 {
-	AVFrame *Frame = av_frame_alloc();
+	AVFrame *pFrame = av_frame_alloc();
 	int Ret;
 
-	if(!Frame)
+	if(!pFrame)
 	{
 		dbg_msg("video_recorder", "Error allocating an audio frame");
 		return nullptr;
 	}
 
-	Frame->format = SampleFmt;
-	Frame->channel_layout = ChannelLayout;
-	Frame->sample_rate = SampleRate;
-	Frame->nb_samples = NbSamples;
+	pFrame->format = SampleFmt;
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(59, 24, 100)
+	dbg_assert(av_channel_layout_from_mask(&pFrame->ch_layout, ChannelLayout) == 0, "Failed to set channel layout");
+#else
+	pFrame->channel_layout = ChannelLayout;
+#endif
+	pFrame->sample_rate = SampleRate;
+	pFrame->nb_samples = NbSamples;
 
 	if(NbSamples)
 	{
-		Ret = av_frame_get_buffer(Frame, 0);
+		Ret = av_frame_get_buffer(pFrame, 0);
 		if(Ret < 0)
 		{
 			dbg_msg("video_recorder", "Error allocating an audio buffer");
@@ -635,20 +637,20 @@ AVFrame *CVideo::AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64_t Channel
 		}
 	}
 
-	return Frame;
+	return pFrame;
 }
 
 bool CVideo::OpenVideo()
 {
 	int Ret;
-	AVCodecContext *c = m_VideoStream.pEnc;
-	AVDictionary *opt = 0;
+	AVCodecContext *pContext = m_VideoStream.pEnc;
+	AVDictionary *pOptions = 0;
 
-	av_dict_copy(&opt, m_pOptDict, 0);
+	av_dict_copy(&pOptions, m_pOptDict, 0);
 
 	/* open the codec */
-	Ret = avcodec_open2(c, m_VideoCodec, &opt);
-	av_dict_free(&opt);
+	Ret = avcodec_open2(pContext, m_pVideoCodec, &pOptions);
+	av_dict_free(&pOptions);
 	if(Ret < 0)
 	{
 		char aBuf[AV_ERROR_MAX_STRING_SIZE];
@@ -664,7 +666,7 @@ bool CVideo::OpenVideo()
 	for(size_t i = 0; i < m_VideoThreads; ++i)
 	{
 		m_VideoStream.m_vpFrames.emplace_back(nullptr);
-		m_VideoStream.m_vpFrames[i] = AllocPicture(c->pix_fmt, c->width, c->height);
+		m_VideoStream.m_vpFrames[i] = AllocPicture(pContext->pix_fmt, pContext->width, pContext->height);
 		if(!m_VideoStream.m_vpFrames[i])
 		{
 			dbg_msg("video_recorder", "Could not allocate video frame");
@@ -678,13 +680,13 @@ bool CVideo::OpenVideo()
 	m_VideoStream.m_vpTmpFrames.clear();
 	m_VideoStream.m_vpTmpFrames.reserve(m_VideoThreads);
 
-	if(c->pix_fmt != AV_PIX_FMT_YUV420P)
+	if(pContext->pix_fmt != AV_PIX_FMT_YUV420P)
 	{
 		/* allocate and init a re-usable frame */
 		for(size_t i = 0; i < m_VideoThreads; ++i)
 		{
 			m_VideoStream.m_vpTmpFrames.emplace_back(nullptr);
-			m_VideoStream.m_vpTmpFrames[i] = AllocPicture(AV_PIX_FMT_YUV420P, c->width, c->height);
+			m_VideoStream.m_vpTmpFrames[i] = AllocPicture(AV_PIX_FMT_YUV420P, pContext->width, pContext->height);
 			if(!m_VideoStream.m_vpTmpFrames[i])
 			{
 				dbg_msg("video_recorder", "Could not allocate temporary video frame");
@@ -694,7 +696,7 @@ bool CVideo::OpenVideo()
 	}
 
 	/* copy the stream parameters to the muxer */
-	Ret = avcodec_parameters_from_context(m_VideoStream.pSt->codecpar, c);
+	Ret = avcodec_parameters_from_context(m_VideoStream.pSt->codecpar, pContext);
 	if(Ret < 0)
 	{
 		dbg_msg("video_recorder", "Could not copy the stream parameters");
@@ -706,18 +708,18 @@ bool CVideo::OpenVideo()
 
 bool CVideo::OpenAudio()
 {
-	AVCodecContext *c;
+	AVCodecContext *pContext;
 	int NbSamples;
 	int Ret;
-	AVDictionary *opt = NULL;
+	AVDictionary *pOptions = NULL;
 
-	c = m_AudioStream.pEnc;
+	pContext = m_AudioStream.pEnc;
 
 	/* open it */
 	//m_dbgfile = fopen("/tmp/pcm_dbg", "wb");
-	av_dict_copy(&opt, m_pOptDict, 0);
-	Ret = avcodec_open2(c, m_AudioCodec, &opt);
-	av_dict_free(&opt);
+	av_dict_copy(&pOptions, m_pOptDict, 0);
+	Ret = avcodec_open2(pContext, m_pAudioCodec, &pOptions);
+	av_dict_free(&pOptions);
 	if(Ret < 0)
 	{
 		char aBuf[AV_ERROR_MAX_STRING_SIZE];
@@ -726,10 +728,10 @@ bool CVideo::OpenAudio()
 		return false;
 	}
 
-	if(c->codec->capabilities & AV_CODEC_CAP_VARIABLE_FRAME_SIZE)
+	if(pContext->codec->capabilities & AV_CODEC_CAP_VARIABLE_FRAME_SIZE)
 		NbSamples = 10000;
 	else
-		NbSamples = c->frame_size;
+		NbSamples = pContext->frame_size;
 
 	m_AudioStream.m_vpFrames.clear();
 	m_AudioStream.m_vpFrames.reserve(m_AudioThreads);
@@ -741,7 +743,11 @@ bool CVideo::OpenAudio()
 	for(size_t i = 0; i < m_AudioThreads; ++i)
 	{
 		m_AudioStream.m_vpFrames.emplace_back(nullptr);
-		m_AudioStream.m_vpFrames[i] = AllocAudioFrame(c->sample_fmt, c->channel_layout, c->sample_rate, NbSamples);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(59, 24, 100)
+		m_AudioStream.m_vpFrames[i] = AllocAudioFrame(pContext->sample_fmt, pContext->ch_layout.u.mask, pContext->sample_rate, NbSamples);
+#else
+		m_AudioStream.m_vpFrames[i] = AllocAudioFrame(pContext->sample_fmt, pContext->channel_layout, pContext->sample_rate, NbSamples);
+#endif
 		if(!m_AudioStream.m_vpFrames[i])
 		{
 			dbg_msg("video_recorder", "Could not allocate audio frame");
@@ -758,7 +764,7 @@ bool CVideo::OpenAudio()
 	}
 
 	/* copy the stream parameters to the muxer */
-	Ret = avcodec_parameters_from_context(m_AudioStream.pSt->codecpar, c);
+	Ret = avcodec_parameters_from_context(m_AudioStream.pSt->codecpar, pContext);
 	if(Ret < 0)
 	{
 		dbg_msg("video_recorder", "Could not copy the stream parameters");
@@ -781,9 +787,13 @@ bool CVideo::OpenAudio()
 		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "in_channel_count", 2, 0);
 		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "in_sample_rate", g_Config.m_SndRate, 0);
 		av_opt_set_sample_fmt(m_AudioStream.m_vpSwrCtxs[i], "in_sample_fmt", AV_SAMPLE_FMT_S16, 0);
-		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "out_channel_count", c->channels, 0);
-		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "out_sample_rate", c->sample_rate, 0);
-		av_opt_set_sample_fmt(m_AudioStream.m_vpSwrCtxs[i], "out_sample_fmt", c->sample_fmt, 0);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(59, 24, 100)
+		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "out_channel_count", pContext->ch_layout.nb_channels, 0);
+#else
+		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "out_channel_count", pContext->channels, 0);
+#endif
+		av_opt_set_int(m_AudioStream.m_vpSwrCtxs[i], "out_sample_rate", pContext->sample_rate, 0);
+		av_opt_set_sample_fmt(m_AudioStream.m_vpSwrCtxs[i], "out_sample_fmt", pContext->sample_fmt, 0);
 
 		/* initialize the resampling context */
 		if(swr_init(m_AudioStream.m_vpSwrCtxs[i]) < 0)
@@ -800,7 +810,7 @@ bool CVideo::OpenAudio()
 /* Add an output stream. */
 bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, const AVCodec **ppCodec, enum AVCodecID CodecId)
 {
-	AVCodecContext *c;
+	AVCodecContext *pContext;
 
 	/* find the encoder */
 	*ppCodec = avcodec_find_encoder(CodecId);
@@ -818,80 +828,82 @@ bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, const AVCode
 		return false;
 	}
 	pStream->pSt->id = pOC->nb_streams - 1;
-	c = avcodec_alloc_context3(*ppCodec);
-	if(!c)
+	pContext = avcodec_alloc_context3(*ppCodec);
+	if(!pContext)
 	{
 		dbg_msg("video_recorder", "Could not alloc an encoding context");
 		return false;
 	}
-	pStream->pEnc = c;
+	pStream->pEnc = pContext;
 
 #if defined(CONF_ARCH_IA32) || defined(CONF_ARCH_ARM)
 	// use only 1 ffmpeg thread on 32-bit to save memory
-	c->thread_count = 1;
+	pContext->thread_count = 1;
 #endif
 
 	switch((*ppCodec)->type)
 	{
 	case AVMEDIA_TYPE_AUDIO:
-		c->sample_fmt = (*ppCodec)->sample_fmts ? (*ppCodec)->sample_fmts[0] : AV_SAMPLE_FMT_FLTP;
-		c->bit_rate = g_Config.m_SndRate * 2 * 16;
-		c->sample_rate = g_Config.m_SndRate;
+		pContext->sample_fmt = (*ppCodec)->sample_fmts ? (*ppCodec)->sample_fmts[0] : AV_SAMPLE_FMT_FLTP;
+		pContext->bit_rate = g_Config.m_SndRate * 2 * 16;
+		pContext->sample_rate = g_Config.m_SndRate;
 		if((*ppCodec)->supported_samplerates)
 		{
-			c->sample_rate = (*ppCodec)->supported_samplerates[0];
+			pContext->sample_rate = (*ppCodec)->supported_samplerates[0];
 			for(int i = 0; (*ppCodec)->supported_samplerates[i]; i++)
 			{
 				if((*ppCodec)->supported_samplerates[i] == g_Config.m_SndRate)
 				{
-					c->sample_rate = g_Config.m_SndRate;
+					pContext->sample_rate = g_Config.m_SndRate;
 					break;
 				}
 			}
 		}
-		c->channels = 2;
-		c->channel_layout = AV_CH_LAYOUT_STEREO;
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(59, 24, 100)
+		dbg_assert(av_channel_layout_from_mask(&pContext->ch_layout, AV_CH_LAYOUT_STEREO) == 0, "Failed to set channel layout");
+#else
+		pContext->channels = 2;
+		pContext->channel_layout = AV_CH_LAYOUT_STEREO;
+#endif
 
 		pStream->pSt->time_base.num = 1;
-		pStream->pSt->time_base.den = c->sample_rate;
+		pStream->pSt->time_base.den = pContext->sample_rate;
 		break;
 
 	case AVMEDIA_TYPE_VIDEO:
-		c->codec_id = CodecId;
+		pContext->codec_id = CodecId;
 
-		c->bit_rate = 400000;
+		pContext->bit_rate = 400000;
 		/* Resolution must be a multiple of two. */
-		c->width = m_Width;
-		c->height = m_Height % 2 == 0 ? m_Height : m_Height - 1;
+		pContext->width = m_Width;
+		pContext->height = m_Height % 2 == 0 ? m_Height : m_Height - 1;
 		/* timebase: This is the fundamental unit of time (in seconds) in terms
 			 * of which frame timestamps are represented. For fixed-fps content,
 			 * timebase should be 1/framerate and timestamp increments should be
 			 * identical to 1. */
 		pStream->pSt->time_base.num = 1;
 		pStream->pSt->time_base.den = m_FPS;
-		c->time_base = pStream->pSt->time_base;
+		pContext->time_base = pStream->pSt->time_base;
 
-		c->gop_size = 12; /* emit one intra frame every twelve frames at most */
-		c->pix_fmt = STREAM_PIX_FMT;
-		if(c->codec_id == AV_CODEC_ID_MPEG2VIDEO)
+		pContext->gop_size = 12; /* emit one intra frame every twelve frames at most */
+		pContext->pix_fmt = STREAM_PIX_FMT;
+		if(pContext->codec_id == AV_CODEC_ID_MPEG2VIDEO)
 		{
 			/* just for testing, we also add B-frames */
-			c->max_b_frames = 2;
+			pContext->max_b_frames = 2;
 		}
-		if(c->codec_id == AV_CODEC_ID_MPEG1VIDEO)
+		if(pContext->codec_id == AV_CODEC_ID_MPEG1VIDEO)
 		{
 			/* Needed to avoid using macroblocks in which some coeffs overflow.
 				 * This does not happen with normal video, it just happens here as
 				 * the motion of the chroma plane does not match the luma plane. */
-			c->mb_decision = 2;
+			pContext->mb_decision = 2;
 		}
 		if(CodecId == AV_CODEC_ID_H264)
 		{
-			const char *presets[10] = {"ultrafast", "superfast", "veryfast", "faster", "fast", "medium", "slow", "slower", "veryslow", "placebo"};
-			//av_opt_set(c->priv_data, "preset", "slow", 0);
-			//av_opt_set_int(c->priv_data, "crf", 22, 0);
-			av_opt_set(c->priv_data, "preset", presets[g_Config.m_ClVideoX264Preset], 0);
-			av_opt_set_int(c->priv_data, "crf", g_Config.m_ClVideoX264Crf, 0);
+			static const char *s_apPresets[10] = {"ultrafast", "superfast", "veryfast", "faster", "fast", "medium", "slow", "slower", "veryslow", "placebo"};
+			av_opt_set(pContext->priv_data, "preset", s_apPresets[g_Config.m_ClVideoX264Preset], 0);
+			av_opt_set_int(pContext->priv_data, "crf", g_Config.m_ClVideoX264Crf, 0);
 		}
 		break;
 
@@ -901,7 +913,7 @@ bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, const AVCode
 
 	/* Some formats want stream headers to be separate. */
 	if(pOC->oformat->flags & AVFMT_GLOBALHEADER)
-		c->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
+		pContext->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
 
 	return true;
 }
@@ -971,7 +983,6 @@ void CVideo::FinishFrames(OutputStream *pStream)
 		if(!RetRecv)
 		{
 			/* rescale output packet timestamp values from codec to stream timebase */
-			//if(pStream->pSt->codec->codec_type == AVMEDIA_TYPE_AUDIO)
 			av_packet_rescale_ts(pPacket, pStream->pEnc->time_base, pStream->pSt->time_base);
 			pPacket->stream_index = pStream->pSt->index;
 
