@@ -10,7 +10,8 @@ class IEditor : public IInterface
 public:
 	virtual ~IEditor() {}
 	virtual void Init() = 0;
-	virtual void UpdateAndRender() = 0;
+	virtual void OnUpdate() = 0;
+	virtual void OnRender() = 0;
 	virtual bool HasUnsavedData() const = 0;
 	virtual int Load(const char *pFilename, int StorageType) = 0;
 	virtual int Save(const char *pFilename) = 0;
