@@ -6,10 +6,13 @@
 #include "kernel.h"
 #include <engine/shared/jobs.h>
 
+class CFutureLogger;
+class ILogger;
+
 class CHostLookup : public IJob
 {
 private:
-	virtual void Run();
+	void Run() override;
 
 public:
 	CHostLookup();
@@ -32,12 +35,12 @@ public:
 	virtual ~IEngine() = default;
 
 	virtual void Init() = 0;
-	virtual void InitLogfile() = 0;
 	virtual void AddJob(std::shared_ptr<IJob> pJob) = 0;
+	virtual void SetAdditionalLogger(std::unique_ptr<ILogger> &&pLogger) = 0;
 	static void RunJobBlocking(IJob *pJob);
 };
 
-extern IEngine *CreateEngine(const char *pAppname, bool Silent, int Jobs);
+extern IEngine *CreateEngine(const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs);
 extern IEngine *CreateTestEngine(const char *pAppname, int Jobs);
 
 #endif
