@@ -19,7 +19,7 @@ public:
 	virtual int GetDataSize(int Index) = 0;
 	virtual void *GetDataSwapped(int Index) = 0;
 	virtual void UnloadData(int Index) = 0;
-	virtual void *GetItem(int Index, int *Type, int *pID) = 0;
+	virtual void *GetItem(int Index, int *pType, int *pID) = 0;
 	virtual int GetItemSize(int Index) = 0;
 	virtual void GetType(int Type, int *pStart, int *pNum) = 0;
 	virtual void *FindItem(int Type, int ID) = 0;
