@@ -1,10 +1,15 @@
 #include "antibot.h"
+
 #include <antibot/antibot_interface.h>
 
+#include <base/system.h>
+
 #include <engine/console.h>
 #include <engine/kernel.h>
 #include <engine/server.h>
 
+class IEngineAntibot;
+
 #ifdef CONF_ANTIBOT
 CAntibot::CAntibot() :
 	m_pServer(0), m_pConsole(0), m_pGameServer(0), m_Initialized(false)
