@@ -1,21 +1,15 @@
 #ifndef ENGINE_SERVER_AUTHMANAGER_H
 #define ENGINE_SERVER_AUTHMANAGER_H
 
+#include <vector>
+
 #include <base/hash.h>
-#include <base/tl/array.h>
 
 #define SALT_BYTES 8
 
 class CAuthManager
 {
 private:
-	enum
-	{
-		AUTHED_NO = 0,
-		AUTHED_HELPER,
-		AUTHED_MOD,
-		AUTHED_ADMIN
-	};
 	struct CKey
 	{
 		char m_aIdent[64];
@@ -23,7 +17,7 @@ private:
 		unsigned char m_aSalt[SALT_BYTES];
 		int m_Level;
 	};
-	array<CKey> m_aKeys;
+	std::vector<CKey> m_vKeys;
 
 	int m_aDefault[3];
 	bool m_Generated;
