@@ -13,11 +13,11 @@ class IDbConnection
 public:
 	IDbConnection(const char *pPrefix)
 	{
-		str_copy(m_aPrefix, pPrefix, sizeof(m_aPrefix));
+		str_copy(m_aPrefix, pPrefix);
 	}
 	virtual ~IDbConnection() {}
 	IDbConnection &operator=(const IDbConnection &) = delete;
-	virtual void Print(IConsole *pConsole, const char *Mode) = 0;
+	virtual void Print(IConsole *pConsole, const char *pMode) = 0;
 
 	// copies the credentials, not the active connection
 	virtual IDbConnection *Copy() = 0;
@@ -108,6 +108,7 @@ std::unique_ptr<IDbConnection> CreateMysqlConnection(
 	const char *pUser,
 	const char *pPass,
 	const char *pIp,
+	const char *pBindaddr,
 	int Port,
 	bool Setup);
 
