@@ -4,6 +4,8 @@
 #include <base/system.h>
 #include <engine/shared/protocol.h>
 
+#include <vector>
+
 enum
 {
 	MAX_NAME_SKELETON_LENGTH = MAX_NAME_LENGTH * 4,
@@ -17,9 +19,9 @@ public:
 	CNameBan(const char *pName, int Distance, int IsSubstring, const char *pReason = "") :
 		m_Distance(Distance), m_IsSubstring(IsSubstring)
 	{
-		str_copy(m_aName, pName, sizeof(m_aName));
-		m_SkeletonLength = str_utf8_to_skeleton(m_aName, m_aSkeleton, sizeof(m_aSkeleton) / sizeof(m_aSkeleton[0]));
-		str_copy(m_aReason, pReason, sizeof(m_aReason));
+		str_copy(m_aName, pName);
+		m_SkeletonLength = str_utf8_to_skeleton(m_aName, m_aSkeleton, std::size(m_aSkeleton));
+		str_copy(m_aReason, pReason);
 	}
 	char m_aName[MAX_NAME_LENGTH];
 	char m_aReason[MAX_NAMEBAN_REASON_LENGTH];
@@ -29,6 +31,6 @@ public:
 	int m_IsSubstring;
 };
 
-CNameBan *IsNameBanned(const char *pName, CNameBan *pNameBans, int NumNameBans);
+CNameBan *IsNameBanned(const char *pName, std::vector<CNameBan> &vNameBans);
 
 #endif // ENGINE_SERVER_NAME_BAN_H
