@@ -2,7 +2,6 @@
 
 #include <base/system.h>
 #include <cmath>
-#include <cstring>
 
 void sqlstr::FuzzyString(char *pString, int Size)
 {
@@ -77,7 +76,7 @@ void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString, int Size)
 	for(i = 0; i < 7; i++)
 	{
 		Seconds = aTimes[i];
-		str_copy(aName, aaNames[i], sizeof(aName));
+		str_copy(aName, aaNames[i]);
 
 		Count = std::floor((float)AgoTime / (float)Seconds);
 		if(Count != 0)
@@ -101,7 +100,7 @@ void sqlstr::AgoTimeToString(int AgoTime, char *pAgoString, int Size)
 		// getting second piece now
 		int Seconds2 = aTimes[i + 1];
 		char aName2[6];
-		str_copy(aName2, aaNames[i + 1], sizeof(aName2));
+		str_copy(aName2, aaNames[i + 1]);
 
 		// add second piece if it's greater than 0
 		int Count2 = std::floor((float)(AgoTime - (Seconds * Count)) / (float)Seconds2);
