@@ -8,30 +8,32 @@
 #include <miniupnpc/upnpcommands.h>
 #include <miniupnpc/upnperrors.h>
 
+#include <cstdlib>
+
 void CUPnP::Open(NETADDR Address)
 {
 	if(g_Config.m_SvUseUPnP)
 	{
 		m_Enabled = false;
 		m_Addr = Address;
-		m_UPnPUrls = (struct UPNPUrls *)malloc(sizeof(struct UPNPUrls));
-		m_UPnPData = (struct IGDdatas *)malloc(sizeof(struct IGDdatas));
+		m_pUPnPUrls = (struct UPNPUrls *)malloc(sizeof(struct UPNPUrls));
+		m_pUPnPData = (struct IGDdatas *)malloc(sizeof(struct IGDdatas));
 
 		char aLanAddr[64];
 		char aPort[6];
 		int Error;
 
-		m_UPnPDevice = upnpDiscover(2000, NULL, NULL, 0, 0, 2, &Error);
+		m_pUPnPDevice = upnpDiscover(2000, NULL, NULL, 0, 0, 2, &Error);
 
-		int Status = UPNP_GetValidIGD(m_UPnPDevice, m_UPnPUrls, m_UPnPData, aLanAddr, sizeof(aLanAddr));
+		int Status = UPNP_GetValidIGD(m_pUPnPDevice, m_pUPnPUrls, m_pUPnPData, aLanAddr, sizeof(aLanAddr));
 		dbg_msg("upnp", "status=%d, lan_addr=%s", Status, aLanAddr);
 
 		if(Status == 1)
 		{
 			m_Enabled = true;
-			dbg_msg("upnp", "found valid IGD: %s", m_UPnPUrls->controlURL);
+			dbg_msg("upnp", "found valid IGD: %s", m_pUPnPUrls->controlURL);
 			str_format(aPort, sizeof(aPort), "%d", m_Addr.port);
-			Error = UPNP_AddPortMapping(m_UPnPUrls->controlURL, m_UPnPData->first.servicetype,
+			Error = UPNP_AddPortMapping(m_pUPnPUrls->controlURL, m_pUPnPData->first.servicetype,
 				aPort, aPort, aLanAddr,
 				"DDNet Server " GAME_RELEASE_VERSION,
 				"UDP", NULL, "0");
@@ -54,20 +56,20 @@ void CUPnP::Shutdown()
 		{
 			char aPort[6];
 			str_format(aPort, sizeof(aPort), "%d", m_Addr.port);
-			int Error = UPNP_DeletePortMapping(m_UPnPUrls->controlURL, m_UPnPData->first.servicetype, aPort, "UDP", NULL);
+			int Error = UPNP_DeletePortMapping(m_pUPnPUrls->controlURL, m_pUPnPData->first.servicetype, aPort, "UDP", NULL);
 
 			if(Error != 0)
 			{
 				dbg_msg("upnp", "failed to delete port mapping on shutdown: %s", strupnperror(Error));
 			}
-			FreeUPNPUrls(m_UPnPUrls);
-			freeUPNPDevlist(m_UPnPDevice);
+			FreeUPNPUrls(m_pUPnPUrls);
+			freeUPNPDevlist(m_pUPnPDevice);
 		}
-		free(m_UPnPUrls);
-		free(m_UPnPData);
-		m_UPnPUrls = NULL;
-		m_UPnPData = NULL;
+		free(m_pUPnPUrls);
+		free(m_pUPnPData);
+		m_pUPnPUrls = NULL;
+		m_pUPnPData = NULL;
 	}
 }
 
-#endif
\ No newline at end of file
+#endif
