@@ -5,9 +5,9 @@
 class CUPnP
 {
 	NETADDR m_Addr;
-	struct UPNPUrls *m_UPnPUrls;
-	struct IGDdatas *m_UPnPData;
-	struct UPNPDev *m_UPnPDevice;
+	struct UPNPUrls *m_pUPnPUrls;
+	struct IGDdatas *m_pUPnPData;
+	struct UPNPDev *m_pUPnPDevice;
 	bool m_Enabled;
 
 public:
