@@ -4,12 +4,13 @@
 #include "datafile.h"
 
 #include <base/hash_ctxt.h>
-#include <base/math.h>
 #include <base/system.h>
 #include <engine/storage.h>
 
 #include "uuid_manager.h"
 
+#include <cstdlib>
+
 static const int DEBUG = 0;
 
 enum
@@ -38,11 +39,6 @@ struct CItemEx
 	}
 };
 
-static int GetTypeFromIndex(int Index)
-{
-	return ITEMTYPE_EX - Index - 1;
-}
-
 struct CDatafileItemType
 {
 	int m_Type;
@@ -206,7 +202,7 @@ bool CDataFileReader::Open(class IStorage *pStorage, const char *pFilename, int
 	swap_endian(m_pDataFile->m_pData, sizeof(int), minimum(static_cast<unsigned>(Header.m_Swaplen), Size) / sizeof(int));
 #endif
 
-	//if(DEBUG)
+	if(DEBUG)
 	{
 		dbg_msg("datafile", "allocsize=%d", AllocSize);
 		dbg_msg("datafile", "readsize=%d", ReadSize);
@@ -227,42 +223,6 @@ bool CDataFileReader::Open(class IStorage *pStorage, const char *pFilename, int
 
 	dbg_msg("datafile", "loading done. datafile='%s'", pFilename);
 
-	if(DEBUG)
-	{
-		/*
-		for(int i = 0; i < m_pDataFile->data.num_raw_data; i++)
-		{
-			void *p = datafile_get_data(df, i);
-			dbg_msg("datafile", "%d %d", (int)((char*)p - (char*)(&m_pDataFile->data)), size);
-		}
-
-		for(int i = 0; i < datafile_num_items(df); i++)
-		{
-			int type, id;
-			void *data = datafile_get_item(df, i, &type, &id);
-			dbg_msg("map", "\t%d: type=%x id=%x p=%p offset=%d", i, type, id, data, m_pDataFile->info.item_offsets[i]);
-			int *idata = (int*)data;
-			for(int k = 0; k < 3; k++)
-				dbg_msg("datafile", "\t\t%d=%d (%x)", k, idata[k], idata[k]);
-		}
-
-		for(int i = 0; i < m_pDataFile->data.num_m_aItemTypes; i++)
-		{
-			dbg_msg("map", "\t%d: type=%x start=%d num=%d", i,
-				m_pDataFile->info.m_aItemTypes[i].type,
-				m_pDataFile->info.m_aItemTypes[i].start,
-				m_pDataFile->info.m_aItemTypes[i].num);
-			for(int k = 0; k < m_pDataFile->info.m_aItemTypes[i].num; k++)
-			{
-				int type, id;
-				datafile_get_item(df, m_pDataFile->info.m_aItemTypes[i].start+k, &type, &id);
-				if(type != m_pDataFile->info.m_aItemTypes[i].type)
-					dbg_msg("map", "\tERROR");
-			}
-		}
-		*/
-	}
-
 	return true;
 }
 
@@ -443,17 +403,17 @@ void *CDataFileReader::GetItem(int Index, int *pType, int *pID)
 		return 0;
 	}
 
-	CDatafileItem *i = (CDatafileItem *)(m_pDataFile->m_Info.m_pItemStart + m_pDataFile->m_Info.m_pItemOffsets[Index]);
+	CDatafileItem *pItem = (CDatafileItem *)(m_pDataFile->m_Info.m_pItemStart + m_pDataFile->m_Info.m_pItemOffsets[Index]);
 	if(pType)
 	{
 		// remove sign extension
-		*pType = GetExternalItemType((i->m_TypeAndID >> 16) & 0xffff);
+		*pType = GetExternalItemType((pItem->m_TypeAndID >> 16) & 0xffff);
 	}
 	if(pID)
 	{
-		*pID = i->m_TypeAndID & 0xffff;
+		*pID = pItem->m_TypeAndID & 0xffff;
 	}
-	return (void *)(i + 1);
+	return (void *)(pItem + 1);
 }
 
 void CDataFileReader::GetType(int Type, int *pStart, int *pNum)
@@ -617,6 +577,11 @@ bool CDataFileWriter::Open(class IStorage *pStorage, const char *pFilename, int
 	return OpenFile(pStorage, pFilename, StorageType);
 }
 
+int CDataFileWriter::GetTypeFromIndex(int Index)
+{
+	return ITEMTYPE_EX - Index - 1;
+}
+
 int CDataFileWriter::GetExtendedItemTypeIndex(int Type)
 {
 	for(int i = 0; i < m_NumExtendedItemTypes; i++)
