@@ -12,21 +12,24 @@
 #include <engine/shared/video.h>
 #endif
 
-#include <game/generated/protocol.h>
-
 #include "compression.h"
 #include "demo.h"
 #include "memheap.h"
 #include "network.h"
 #include "snapshot.h"
 
-static const unsigned char s_aHeaderMarker[7] = {'T', 'W', 'D', 'E', 'M', 'O', 0};
-static const unsigned char s_CurVersion = 6;
-static const unsigned char s_OldVersion = 3;
-static const unsigned char s_Sha256Version = 6;
-static const unsigned char s_VersionTickCompression = 5; // demo files with this version or higher will use `CHUNKTICKFLAG_TICK_COMPRESSED`
-static const int s_LengthOffset = 152;
-static const int s_NumMarkersOffset = 176;
+const double g_aSpeeds[g_DemoSpeeds] = {0.1, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 2.0, 3.0, 4.0, 6.0, 8.0, 12.0, 16.0, 20.0, 24.0, 28.0, 32.0, 40.0, 48.0, 56.0, 64.0};
+const CUuid SHA256_EXTENSION =
+	{{0x6b, 0xe6, 0xda, 0x4a, 0xce, 0xbd, 0x38, 0x0c,
+		0x9b, 0x5b, 0x12, 0x89, 0xc8, 0x42, 0xd7, 0x80}};
+
+static const unsigned char gs_aHeaderMarker[7] = {'T', 'W', 'D', 'E', 'M', 'O', 0};
+static const unsigned char gs_CurVersion = 6;
+static const unsigned char gs_OldVersion = 3;
+static const unsigned char gs_Sha256Version = 6;
+static const unsigned char gs_VersionTickCompression = 5; // demo files with this version or higher will use `CHUNKTICKFLAG_TICK_COMPRESSED`
+static const int gs_LengthOffset = 152;
+static const int gs_NumMarkersOffset = 176;
 
 static const ColorRGBA gs_DemoPrintColor{0.75f, 0.7f, 0.7f, 1.0f};
 
@@ -121,18 +124,20 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 		CloseMapFile = true;
 	}
 
-	if(MapFile)
+	if(m_NoMapData)
+		MapSize = 0;
+	else if(MapFile)
 		MapSize = io_length(MapFile);
 
 	// write header
 	mem_zero(&Header, sizeof(Header));
-	mem_copy(Header.m_aMarker, s_aHeaderMarker, sizeof(Header.m_aMarker));
-	Header.m_Version = s_CurVersion;
-	str_copy(Header.m_aNetversion, pNetVersion, sizeof(Header.m_aNetversion));
-	str_copy(Header.m_aMapName, pMap, sizeof(Header.m_aMapName));
+	mem_copy(Header.m_aMarker, gs_aHeaderMarker, sizeof(Header.m_aMarker));
+	Header.m_Version = gs_CurVersion;
+	str_copy(Header.m_aNetversion, pNetVersion);
+	str_copy(Header.m_aMapName, pMap);
 	uint_to_bytes_be(Header.m_aMapSize, MapSize);
 	uint_to_bytes_be(Header.m_aMapCrc, Crc);
-	str_copy(Header.m_aType, pType, sizeof(Header.m_aType));
+	str_copy(Header.m_aType, pType);
 	// Header.m_Length - add this on stop
 	str_timestamp(Header.m_aTimestamp, sizeof(Header.m_aTimestamp));
 	io_write(DemoFile, &Header, sizeof(Header));
@@ -155,6 +160,7 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 		while(true)
 		{
 			unsigned char aChunk[1024 * 64];
+			mem_zero(aChunk, sizeof(aChunk));
 			int Bytes = io_read(MapFile, &aChunk, sizeof(aChunk));
 			if(Bytes <= 0)
 				break;
@@ -178,7 +184,7 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf, gs_DemoPrintColor);
 	}
 	m_File = DemoFile;
-	str_copy(m_aCurrentFilename, pFilename, sizeof(m_aCurrentFilename));
+	str_copy(m_aCurrentFilename, pFilename);
 
 	return 0;
 }
@@ -240,10 +246,6 @@ void CDemoRecorder::WriteTickMarker(int Tick, int Keyframe)
 
 void CDemoRecorder::Write(int Type, const void *pData, int Size)
 {
-	char aBuffer[64 * 1024];
-	char aBuffer2[64 * 1024];
-	unsigned char aChunk[3];
-
 	if(!m_File)
 		return;
 
@@ -252,6 +254,8 @@ void CDemoRecorder::Write(int Type, const void *pData, int Size)
 
 	/* pad the data with 0 so we get an alignment of 4,
 	else the compression won't work and miss some bytes */
+	char aBuffer[64 * 1024];
+	char aBuffer2[64 * 1024];
 	mem_copy(aBuffer2, pData, Size);
 	while(Size & 3)
 		aBuffer2[Size++] = 0;
@@ -263,6 +267,7 @@ void CDemoRecorder::Write(int Type, const void *pData, int Size)
 	if(Size < 0)
 		return;
 
+	unsigned char aChunk[3];
 	aChunk[0] = ((Type & 0x3) << 5);
 	if(Size < 30)
 	{
@@ -339,13 +344,13 @@ int CDemoRecorder::Stop()
 		return -1;
 
 	// add the demo length to the header
-	io_seek(m_File, s_LengthOffset, IOSEEK_START);
+	io_seek(m_File, gs_LengthOffset, IOSEEK_START);
 	unsigned char aLength[4];
 	int_to_bytes_be(aLength, Length());
 	io_write(m_File, aLength, sizeof(aLength));
 
 	// add the timeline markers to the header
-	io_seek(m_File, s_NumMarkersOffset, IOSEEK_START);
+	io_seek(m_File, gs_NumMarkersOffset, IOSEEK_START);
 	unsigned char aNumMarkers[4];
 	int_to_bytes_be(aNumMarkers, m_NumTimelineMarkers);
 	io_write(m_File, aNumMarkers, sizeof(aNumMarkers));
@@ -401,9 +406,6 @@ void CDemoPlayer::Construct(class CSnapshotDelta *pSnapshotDelta)
 	m_pKeyFrames = 0;
 	m_SpeedIndex = 4;
 
-	m_TickTime = 0;
-	m_Time = 0;
-
 	m_pSnapshotDelta = pSnapshotDelta;
 	m_LastSnapshotDataSize = -1;
 }
@@ -415,14 +417,13 @@ void CDemoPlayer::SetListener(IListener *pListener)
 
 int CDemoPlayer::ReadChunkHeader(int *pType, int *pSize, int *pTick)
 {
-	unsigned char Chunk = 0;
-
 	*pSize = 0;
 	*pType = 0;
 
 	if(m_File == NULL)
 		return -1;
 
+	unsigned char Chunk = 0;
 	if(io_read(m_File, &Chunk, sizeof(Chunk)) != sizeof(Chunk))
 		return -1;
 
@@ -432,7 +433,7 @@ int CDemoPlayer::ReadChunkHeader(int *pType, int *pSize, int *pTick)
 		int Tickdelta_legacy = Chunk & (CHUNKMASK_TICK_LEGACY); // compatibility
 		*pType = Chunk & (CHUNKTYPEFLAG_TICKMARKER | CHUNKTICKFLAG_KEYFRAME);
 
-		if(m_Info.m_Header.m_Version < s_VersionTickCompression && Tickdelta_legacy != 0)
+		if(m_Info.m_Header.m_Version < gs_VersionTickCompression && Tickdelta_legacy != 0)
 		{
 			*pTick += Tickdelta_legacy;
 		}
@@ -476,21 +477,19 @@ int CDemoPlayer::ReadChunkHeader(int *pType, int *pSize, int *pTick)
 
 void CDemoPlayer::ScanFile()
 {
-	long StartPos;
 	CHeap Heap;
 	CKeyFrameSearch *pFirstKey = 0;
 	CKeyFrameSearch *pCurrentKey = 0;
-	//DEMOREC_CHUNK chunk;
-	int ChunkSize, ChunkType, ChunkTick = 0;
-	int i;
+	int ChunkTick = 0;
 
-	StartPos = io_tell(m_File);
+	long StartPos = io_tell(m_File);
 	m_Info.m_SeekablePoints = 0;
 
 	while(true)
 	{
 		long CurrentPos = io_tell(m_File);
 
+		int ChunkSize, ChunkType;
 		if(ReadChunkHeader(&ChunkType, &ChunkSize, &ChunkTick))
 			break;
 
@@ -523,6 +522,7 @@ void CDemoPlayer::ScanFile()
 	}
 
 	// copy all the frames to an array instead for fast access
+	int i;
 	m_pKeyFrames = (CKeyFrame *)calloc(maximum(m_Info.m_SeekablePoints, 1), sizeof(CKeyFrame));
 	for(pCurrentKey = pFirstKey, i = 0; pCurrentKey; pCurrentKey = pCurrentKey->m_pNext, i++)
 		m_pKeyFrames[i] = pCurrentKey->m_Frame;
@@ -533,17 +533,10 @@ void CDemoPlayer::ScanFile()
 
 void CDemoPlayer::DoTick()
 {
-	static char s_aCompresseddata[CSnapshot::MAX_SIZE];
-	static char s_aDecompressed[CSnapshot::MAX_SIZE];
-	static char s_aData[CSnapshot::MAX_SIZE];
-	int ChunkType, ChunkTick, ChunkSize;
-	int DataSize = 0;
-	int GotSnapshot = 0;
-
 	// update ticks
 	m_Info.m_PreviousTick = m_Info.m_Info.m_CurrentTick;
 	m_Info.m_Info.m_CurrentTick = m_Info.m_NextTick;
-	ChunkTick = m_Info.m_Info.m_CurrentTick;
+	int ChunkTick = m_Info.m_Info.m_CurrentTick;
 
 	int64_t Freq = time_freq();
 	int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
@@ -554,8 +547,10 @@ void CDemoPlayer::DoTick()
 	if(m_UpdateIntraTimesFunc)
 		m_UpdateIntraTimesFunc();
 
+	bool GotSnapshot = false;
 	while(true)
 	{
+		int ChunkType, ChunkSize;
 		if(ReadChunkHeader(&ChunkType, &ChunkSize, &ChunkTick))
 		{
 			// stop on error or eof
@@ -577,8 +572,11 @@ void CDemoPlayer::DoTick()
 		}
 
 		// read the chunk
+		int DataSize = 0;
+		static char s_aData[CSnapshot::MAX_SIZE];
 		if(ChunkSize)
 		{
+			static char s_aCompresseddata[CSnapshot::MAX_SIZE];
 			if(io_read(m_File, s_aCompresseddata, ChunkSize) != (unsigned)ChunkSize)
 			{
 				// stop on error or eof
@@ -588,6 +586,7 @@ void CDemoPlayer::DoTick()
 				break;
 			}
 
+			static char s_aDecompressed[CSnapshot::MAX_SIZE];
 			DataSize = CNetBase::Decompress(s_aCompresseddata, ChunkSize, s_aDecompressed, sizeof(s_aDecompressed));
 			if(DataSize < 0)
 			{
@@ -613,20 +612,10 @@ void CDemoPlayer::DoTick()
 		{
 			// process delta snapshot
 			static char s_aNewsnap[CSnapshot::MAX_SIZE];
+			CSnapshot *pNewsnap = (CSnapshot *)s_aNewsnap;
+			DataSize = m_pSnapshotDelta->UnpackDelta((CSnapshot *)m_aLastSnapshotData, pNewsnap, s_aData, DataSize);
 
-			GotSnapshot = 1;
-
-			DataSize = m_pSnapshotDelta->UnpackDelta((CSnapshot *)m_aLastSnapshotData, (CSnapshot *)s_aNewsnap, s_aData, DataSize);
-
-			if(DataSize >= 0)
-			{
-				if(m_pListener)
-					m_pListener->OnDemoPlayerSnapshot(s_aNewsnap, DataSize);
-
-				m_LastSnapshotDataSize = DataSize;
-				mem_copy(m_aLastSnapshotData, s_aNewsnap, DataSize);
-			}
-			else
+			if(DataSize < 0)
 			{
 				if(m_pConsole)
 				{
@@ -635,23 +624,54 @@ void CDemoPlayer::DoTick()
 					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
 				}
 			}
+			else if(!pNewsnap->IsValid(DataSize))
+			{
+				if(m_pConsole)
+				{
+					char aBuf[256];
+					str_format(aBuf, sizeof(aBuf), "snapshot delta invalid. DataSize=%d", DataSize);
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+				}
+			}
+			else
+			{
+				if(m_pListener)
+					m_pListener->OnDemoPlayerSnapshot(s_aNewsnap, DataSize);
+
+				m_LastSnapshotDataSize = DataSize;
+				mem_copy(m_aLastSnapshotData, s_aNewsnap, DataSize);
+				GotSnapshot = true;
+			}
 		}
 		else if(ChunkType == CHUNKTYPE_SNAPSHOT)
 		{
 			// process full snapshot
-			GotSnapshot = 1;
+			CSnapshot *pSnap = (CSnapshot *)s_aData;
+			if(!pSnap->IsValid(DataSize))
+			{
+				if(m_pConsole)
+				{
+					char aBuf[256];
+					str_format(aBuf, sizeof(aBuf), "snapshot invalid. DataSize=%d", DataSize);
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+				}
+			}
+			else
+			{
+				GotSnapshot = true;
 
-			m_LastSnapshotDataSize = DataSize;
-			mem_copy(m_aLastSnapshotData, s_aData, DataSize);
-			if(m_pListener)
-				m_pListener->OnDemoPlayerSnapshot(s_aData, DataSize);
+				m_LastSnapshotDataSize = DataSize;
+				mem_copy(m_aLastSnapshotData, s_aData, DataSize);
+				if(m_pListener)
+					m_pListener->OnDemoPlayerSnapshot(s_aData, DataSize);
+			}
 		}
 		else
 		{
 			// if there were no snapshots in this tick, replay the last one
 			if(!GotSnapshot && m_pListener && m_LastSnapshotDataSize != -1)
 			{
-				GotSnapshot = 1;
+				GotSnapshot = true;
 				m_pListener->OnDemoPlayerSnapshot(m_aLastSnapshotData, m_LastSnapshotDataSize);
 			}
 
@@ -681,12 +701,7 @@ void CDemoPlayer::Pause()
 
 void CDemoPlayer::Unpause()
 {
-	if(m_Info.m_Info.m_Paused)
-	{
-		/*m_Info.start_tick = m_Info.current_tick;
-		m_Info.start_time = time_get();*/
-		m_Info.m_Info.m_Paused = false;
-	}
+	m_Info.m_Info.m_Paused = false;
 #if defined(CONF_VIDEORECORDER)
 	if(IVideo::Current() && g_Config.m_ClVideoPauseWithDemo)
 		IVideo::Current()->Pause(false);
@@ -709,7 +724,7 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	}
 
 	// store the filename
-	str_copy(m_aFilename, pFilename, sizeof(m_aFilename));
+	str_copy(m_aFilename, pFilename);
 
 	// clear the playback info
 	mem_zero(&m_Info, sizeof(m_Info));
@@ -725,7 +740,7 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 
 	// read the header
 	io_read(m_File, &m_Info.m_Header, sizeof(m_Info.m_Header));
-	if(mem_comp(m_Info.m_Header.m_aMarker, s_aHeaderMarker, sizeof(s_aHeaderMarker)) != 0)
+	if(mem_comp(m_Info.m_Header.m_aMarker, gs_aHeaderMarker, sizeof(gs_aHeaderMarker)) != 0)
 	{
 		if(m_pConsole)
 		{
@@ -738,7 +753,7 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 		return -1;
 	}
 
-	if(m_Info.m_Header.m_Version < s_OldVersion)
+	if(m_Info.m_Header.m_Version < gs_OldVersion)
 	{
 		if(m_pConsole)
 		{
@@ -750,11 +765,11 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 		m_File = 0;
 		return -1;
 	}
-	else if(m_Info.m_Header.m_Version > s_OldVersion)
+	else if(m_Info.m_Header.m_Version > gs_OldVersion)
 		io_read(m_File, &m_Info.m_TimelineMarkers, sizeof(m_Info.m_TimelineMarkers));
 
 	SHA256_DIGEST Sha256 = SHA256_ZEROED;
-	if(m_Info.m_Header.m_Version >= s_Sha256Version)
+	if(m_Info.m_Header.m_Version >= gs_Sha256Version)
 	{
 		CUuid ExtensionUuid = {};
 		io_read(m_File, &ExtensionUuid.m_aData, sizeof(ExtensionUuid.m_aData));
@@ -794,13 +809,13 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	m_MapInfo.m_Crc = Crc;
 	m_MapInfo.m_Sha256 = Sha256;
 	m_MapInfo.m_Size = MapSize;
-	str_copy(m_MapInfo.m_aName, m_Info.m_Header.m_aMapName, sizeof(m_MapInfo.m_aName));
+	str_copy(m_MapInfo.m_aName, m_Info.m_Header.m_aMapName);
 
-	if(m_Info.m_Header.m_Version > s_OldVersion)
+	if(m_Info.m_Header.m_Version > gs_OldVersion)
 	{
 		// get timeline markers
 		int Num = bytes_be_to_int(m_Info.m_TimelineMarkers.m_aNumTimelineMarkers);
-		m_Info.m_Info.m_NumTimelineMarkers = minimum(Num, (int)MAX_TIMELINE_MARKERS);
+		m_Info.m_Info.m_NumTimelineMarkers = clamp<int>(Num, 0, MAX_TIMELINE_MARKERS);
 		for(int i = 0; i < m_Info.m_Info.m_NumTimelineMarkers; i++)
 		{
 			m_Info.m_Info.m_aTimelineMarkers[i] = bytes_be_to_int(m_Info.m_TimelineMarkers.m_aTimelineMarkers[i]);
@@ -841,7 +856,7 @@ bool CDemoPlayer::ExtractMap(class IStorage *pStorage)
 
 	// handle sha256
 	SHA256_DIGEST Sha256 = SHA256_ZEROED;
-	if(m_Info.m_Header.m_Version >= s_Sha256Version)
+	if(m_Info.m_Header.m_Version >= gs_Sha256Version)
 		Sha256 = m_MapInfo.m_Sha256;
 	else
 	{
@@ -867,13 +882,7 @@ bool CDemoPlayer::ExtractMap(class IStorage *pStorage)
 	return true;
 }
 
-int CDemoPlayer::NextFrame()
-{
-	DoTick();
-	return IsPlaying();
-}
-
-int64_t CDemoPlayer::time()
+int64_t CDemoPlayer::Time()
 {
 #if defined(CONF_VIDEORECORDER)
 	static bool s_Recording = false;
@@ -908,10 +917,8 @@ int CDemoPlayer::Play()
 		DoTick();
 
 	// set start info
-	/*m_Info.start_tick = m_Info.previous_tick;
-	m_Info.start_time = time_get();*/
 	m_Info.m_CurrentTime = m_Info.m_PreviousTick * time_freq() / SERVER_TICK_SPEED;
-	m_Info.m_LastUpdate = time();
+	m_Info.m_LastUpdate = Time();
 	return 0;
 }
 
@@ -927,24 +934,26 @@ int CDemoPlayer::SeekTime(float Seconds)
 	return SetPos(WantedTick);
 }
 
+int CDemoPlayer::SeekTick(ETickOffset TickOffset)
+{
+	return SetPos(m_Info.m_Info.m_CurrentTick + (int)TickOffset);
+}
+
 int CDemoPlayer::SetPos(int WantedTick)
 {
 	if(!m_File)
 		return -1;
 
-	// -5 because we have to have a current tick and previous tick when we do the playback
-	WantedTick = clamp(WantedTick, m_Info.m_Info.m_FirstTick, m_Info.m_Info.m_LastTick) - 5;
+	WantedTick = clamp(WantedTick, m_Info.m_Info.m_FirstTick, m_Info.m_Info.m_LastTick);
+	const int KeyFrameWantedTick = WantedTick - 5; // -5 because we have to have a current tick and previous tick when we do the playback
+	const float Percent = (KeyFrameWantedTick - m_Info.m_Info.m_FirstTick) / (float)(m_Info.m_Info.m_LastTick - m_Info.m_Info.m_FirstTick);
 
 	// get correct key frame
-	int KeyFrame = 0;
-	while(KeyFrame < m_Info.m_SeekablePoints - 1 && m_pKeyFrames[KeyFrame].m_Tick < WantedTick)
-	{
+	int KeyFrame = clamp((int)(m_Info.m_SeekablePoints * Percent), 0, m_Info.m_SeekablePoints - 1);
+	while(KeyFrame < m_Info.m_SeekablePoints - 1 && m_pKeyFrames[KeyFrame].m_Tick < KeyFrameWantedTick)
 		KeyFrame++;
-	}
-	while(KeyFrame > 0 && m_pKeyFrames[KeyFrame].m_Tick > WantedTick)
-	{
+	while(KeyFrame > 0 && m_pKeyFrames[KeyFrame].m_Tick > KeyFrameWantedTick)
 		KeyFrame--;
-	}
 
 	// seek to the correct key frame
 	io_seek(m_File, m_pKeyFrames[KeyFrame].m_Filepos, IOSEEK_START);
@@ -954,7 +963,7 @@ int CDemoPlayer::SetPos(int WantedTick)
 	m_Info.m_PreviousTick = -1;
 
 	// playback everything until we hit our tick
-	while(m_Info.m_PreviousTick < WantedTick && IsPlaying())
+	while(m_Info.m_NextTick < WantedTick)
 		DoTick();
 
 	Play();
@@ -969,28 +978,25 @@ void CDemoPlayer::SetSpeed(float Speed)
 
 void CDemoPlayer::SetSpeedIndex(int Offset)
 {
-	m_SpeedIndex = clamp(m_SpeedIndex + Offset, 0, (int)(sizeof(g_aSpeeds) / sizeof(g_aSpeeds[0]) - 1));
+	m_SpeedIndex = clamp(m_SpeedIndex + Offset, 0, (int)(std::size(g_aSpeeds) - 1));
 	SetSpeed(g_aSpeeds[m_SpeedIndex]);
 }
 
 int CDemoPlayer::Update(bool RealTime)
 {
-	int64_t Now = time();
+	int64_t Now = Time();
 	int64_t Deltatime = Now - m_Info.m_LastUpdate;
 	m_Info.m_LastUpdate = Now;
 
 	if(!IsPlaying())
 		return 0;
 
-	if(m_Info.m_Info.m_Paused)
+	const int64_t Freq = time_freq();
+	if(!m_Info.m_Info.m_Paused)
 	{
-	}
-	else
-	{
-		int64_t Freq = time_freq();
 		m_Info.m_CurrentTime += (int64_t)(Deltatime * (double)m_Info.m_Info.m_Speed);
 
-		while(true)
+		while(!m_Info.m_Info.m_Paused)
 		{
 			int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
 
@@ -1000,35 +1006,31 @@ int CDemoPlayer::Update(bool RealTime)
 
 			// do one more tick
 			DoTick();
-
-			if(m_Info.m_Info.m_Paused)
-				return 0;
 		}
-
-		// update intratick
-		{
-			int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
-			int64_t PrevtickStart = (m_Info.m_PreviousTick) * Freq / SERVER_TICK_SPEED;
-			m_Info.m_IntraTick = (m_Info.m_CurrentTime - PrevtickStart) / (float)(CurtickStart - PrevtickStart);
-			m_Info.m_IntraTickSincePrev = (m_Info.m_CurrentTime - PrevtickStart) / (float)(Freq / SERVER_TICK_SPEED);
-			m_Info.m_TickTime = (m_Info.m_CurrentTime - PrevtickStart) / (float)Freq;
-			if(m_UpdateIntraTimesFunc)
-				m_UpdateIntraTimesFunc();
-		}
-
-		if(m_Info.m_Info.m_CurrentTick == m_Info.m_PreviousTick ||
-			m_Info.m_Info.m_CurrentTick == m_Info.m_NextTick)
-		{
-			if(m_pConsole)
-			{
-				char aBuf[256];
-				str_format(aBuf, sizeof(aBuf), "tick error prev=%d cur=%d next=%d",
-					m_Info.m_PreviousTick, m_Info.m_Info.m_CurrentTick, m_Info.m_NextTick);
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
-			}
-		}
-		m_Time += m_TickTime;
 	}
+
+	// update intratick
+	{
+		int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
+		int64_t PrevtickStart = (m_Info.m_PreviousTick) * Freq / SERVER_TICK_SPEED;
+		m_Info.m_IntraTick = (m_Info.m_CurrentTime - PrevtickStart) / (float)(CurtickStart - PrevtickStart);
+		m_Info.m_IntraTickSincePrev = (m_Info.m_CurrentTime - PrevtickStart) / (float)(Freq / SERVER_TICK_SPEED);
+		m_Info.m_TickTime = (m_Info.m_CurrentTime - PrevtickStart) / (float)Freq;
+		if(m_UpdateIntraTimesFunc)
+			m_UpdateIntraTimesFunc();
+	}
+
+	if(m_Info.m_Info.m_CurrentTick == m_Info.m_PreviousTick || m_Info.m_Info.m_CurrentTick == m_Info.m_NextTick)
+	{
+		if(m_pConsole)
+		{
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "tick error prev=%d cur=%d next=%d",
+				m_Info.m_PreviousTick, m_Info.m_Info.m_CurrentTick, m_Info.m_NextTick);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+		}
+	}
+
 	return 0;
 }
 
@@ -1048,7 +1050,7 @@ int CDemoPlayer::Stop()
 	m_File = 0;
 	free(m_pKeyFrames);
 	m_pKeyFrames = 0;
-	str_copy(m_aFilename, "", sizeof(m_aFilename));
+	str_copy(m_aFilename, "");
 	return 0;
 }
 
@@ -1084,11 +1086,11 @@ bool CDemoPlayer::GetDemoInfo(class IStorage *pStorage, const char *pFilename, i
 	io_read(File, pDemoHeader, sizeof(CDemoHeader));
 	io_read(File, pTimelineMarkers, sizeof(CTimelineMarkers));
 
-	str_copy(pMapInfo->m_aName, pDemoHeader->m_aMapName, sizeof(pMapInfo->m_aName));
+	str_copy(pMapInfo->m_aName, pDemoHeader->m_aMapName);
 	pMapInfo->m_Crc = bytes_be_to_int(pDemoHeader->m_aMapCrc);
 
 	SHA256_DIGEST Sha256 = SHA256_ZEROED;
-	if(pDemoHeader->m_Version >= s_Sha256Version)
+	if(pDemoHeader->m_Version >= gs_Sha256Version)
 	{
 		CUuid ExtensionUuid = {};
 		io_read(File, &ExtensionUuid.m_aData, sizeof(ExtensionUuid.m_aData));
@@ -1109,7 +1111,7 @@ bool CDemoPlayer::GetDemoInfo(class IStorage *pStorage, const char *pFilename, i
 	pMapInfo->m_Size = bytes_be_to_int(pDemoHeader->m_aMapSize);
 
 	io_close(File);
-	return !(mem_comp(pDemoHeader->m_aMarker, s_aHeaderMarker, sizeof(s_aHeaderMarker)) || pDemoHeader->m_Version < s_OldVersion);
+	return !(mem_comp(pDemoHeader->m_aMarker, gs_aHeaderMarker, sizeof(gs_aHeaderMarker)) || pDemoHeader->m_Version < gs_OldVersion);
 }
 
 int CDemoPlayer::GetDemoType() const
@@ -1148,7 +1150,7 @@ void CDemoEditor::Slice(const char *pDemo, const char *pDst, int StartTick, int
 	const CDemoPlayer::CPlaybackInfo *pInfo = m_pDemoPlayer->Info();
 
 	SHA256_DIGEST Sha256 = pMapInfo->m_Sha256;
-	if(pInfo->m_Header.m_Version < s_Sha256Version)
+	if(pInfo->m_Header.m_Version < gs_Sha256Version)
 	{
 		if(m_pDemoPlayer->ExtractMap(m_pStorage))
 			Sha256 = pMapInfo->m_Sha256;
