@@ -1,6 +1,7 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 
+#include <base/logger.h>
 #include <base/system.h>
 
 #include <engine/console.h>
@@ -9,13 +10,11 @@
 #include <engine/shared/network.h>
 #include <engine/storage.h>
 
-#include <engine/shared/assertion_logger.h>
-
 CHostLookup::CHostLookup() = default;
 
 CHostLookup::CHostLookup(const char *pHostname, int Nettype)
 {
-	str_copy(m_aHostname, pHostname, sizeof(m_aHostname));
+	str_copy(m_aHostname, pHostname);
 	m_Nettype = Nettype;
 }
 
@@ -31,7 +30,7 @@ public:
 	IStorage *m_pStorage;
 	bool m_Logging;
 
-	CAssertionLogger m_AssertionLogger;
+	std::shared_ptr<CFutureLogger> m_pFutureLogger;
 
 	char m_aAppName[256];
 
@@ -57,15 +56,12 @@ public:
 		}
 	}
 
-	CEngine(bool Test, const char *pAppname, bool Silent, int Jobs)
+	CEngine(bool Test, const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs) :
+		m_pFutureLogger(std::move(pFutureLogger))
 	{
-		str_copy(m_aAppName, pAppname, std::size(m_aAppName));
+		str_copy(m_aAppName, pAppname);
 		if(!Test)
 		{
-			if(!Silent)
-				dbg_logger_stdout();
-			dbg_logger_debugger();
-
 			//
 			dbg_msg("engine", "running on %s-%s-%s", CONF_FAMILY_STRING, CONF_PLATFORM_STRING, CONF_ARCH_STRING);
 #ifdef CONF_ARCH_ENDIAN_LITTLE
@@ -101,24 +97,20 @@ public:
 
 		char aFullPath[IO_MAX_PATH_LENGTH];
 		m_pStorage->GetCompletePath(IStorage::TYPE_SAVE, "dumps/", aFullPath, sizeof(aFullPath));
-		m_AssertionLogger.Init(aFullPath, m_aAppName);
-
 		m_pConsole->Register("dbg_lognetwork", "", CFGFLAG_SERVER | CFGFLAG_CLIENT, Con_DbgLognetwork, this, "Log the network");
 	}
 
-	void InitLogfile() override
-	{
-		// open logfile if needed
-		if(g_Config.m_Logfile[0])
-			dbg_logger_file(g_Config.m_Logfile);
-	}
-
 	void AddJob(std::shared_ptr<IJob> pJob) override
 	{
 		if(g_Config.m_Debug)
 			dbg_msg("engine", "job added");
 		m_JobPool.Add(std::move(pJob));
 	}
+
+	void SetAdditionalLogger(std::unique_ptr<ILogger> &&pLogger) override
+	{
+		m_pFutureLogger->Set(std::move(pLogger));
+	}
 };
 
 void IEngine::RunJobBlocking(IJob *pJob)
@@ -126,5 +118,5 @@ void IEngine::RunJobBlocking(IJob *pJob)
 	CJobPool::RunBlocking(pJob);
 }
 
-IEngine *CreateEngine(const char *pAppname, bool Silent, int Jobs) { return new CEngine(false, pAppname, Silent, Jobs); }
-IEngine *CreateTestEngine(const char *pAppname, int Jobs) { return new CEngine(true, pAppname, true, Jobs); }
+IEngine *CreateEngine(const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs) { return new CEngine(false, pAppname, std::move(pFutureLogger), Jobs); }
+IEngine *CreateTestEngine(const char *pAppname, int Jobs) { return new CEngine(true, pAppname, nullptr, Jobs); }
