@@ -2,6 +2,8 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include "jobs.h"
 
+#include <base/lock_scope.h>
+
 IJob::IJob() :
 	m_Status(STATE_PENDING)
 {
@@ -54,15 +56,16 @@ void CJobPool::WorkerThread(void *pUser)
 
 		// fetch job from queue
 		sphore_wait(&pPool->m_Semaphore);
-		lock_wait(pPool->m_Lock);
-		if(pPool->m_pFirstJob)
 		{
-			pJob = pPool->m_pFirstJob;
-			pPool->m_pFirstJob = pPool->m_pFirstJob->m_pNext;
-			if(!pPool->m_pFirstJob)
-				pPool->m_pLastJob = 0;
+			CLockScope ls(pPool->m_Lock);
+			if(pPool->m_pFirstJob)
+			{
+				pJob = pPool->m_pFirstJob;
+				pPool->m_pFirstJob = pPool->m_pFirstJob->m_pNext;
+				if(!pPool->m_pFirstJob)
+					pPool->m_pLastJob = 0;
+			}
 		}
-		lock_unlock(pPool->m_Lock);
 
 		// do the job if we have one
 		if(pJob)
@@ -96,16 +99,16 @@ void CJobPool::Destroy()
 
 void CJobPool::Add(std::shared_ptr<IJob> pJob)
 {
-	lock_wait(m_Lock);
+	{
+		CLockScope ls(m_Lock);
+		// add job to queue
+		if(m_pLastJob)
+			m_pLastJob->m_pNext = pJob;
+		m_pLastJob = std::move(pJob);
+		if(!m_pFirstJob)
+			m_pFirstJob = m_pLastJob;
+	}
 
-	// add job to queue
-	if(m_pLastJob)
-		m_pLastJob->m_pNext = pJob;
-	m_pLastJob = std::move(pJob);
-	if(!m_pFirstJob)
-		m_pFirstJob = m_pLastJob;
-
-	lock_unlock(m_Lock);
 	sphore_signal(&m_Semaphore);
 }
 
