@@ -1,42 +1,46 @@
 #include <base/system.h>
 #include <engine/shared/json.h>
 
-const struct _json_value *json_object_get (const json_value * object, const char * index)
+const struct _json_value *json_object_get(const json_value *object, const char *index)
 {
-   unsigned int i;
+	unsigned int i;
 
-   if (object->type != json_object)
-      return &json_value_none;
+	if(object->type != json_object)
+		return &json_value_none;
 
-   for (i = 0; i < object->u.object.length; ++ i)
-      if (!str_comp(object->u.object.values [i].name, index))
-         return object->u.object.values [i].value;
+	for(i = 0; i < object->u.object.length; ++i)
+		if(!str_comp(object->u.object.values[i].name, index))
+			return object->u.object.values[i].value;
 
-   return &json_value_none;
+	return &json_value_none;
 }
 
-const struct _json_value *json_array_get (const json_value * array, int index)
+const struct _json_value *json_array_get(const json_value *array, int index)
 {
-   if (array->type != json_array || index >= (int)array->u.array.length)
-      return &json_value_none;
+	if(array->type != json_array || index >= (int)array->u.array.length)
+		return &json_value_none;
 
-   return array->u.array.values[index];
+	return array->u.array.values[index];
 }
 
-int json_array_length (const json_value * array) {
-   return array->u.array.length;
+int json_array_length(const json_value *array)
+{
+	return array->u.array.length;
 }
 
-const char * json_string_get (const json_value * string) {
-   return string->u.string.ptr;
+const char *json_string_get(const json_value *string)
+{
+	return string->u.string.ptr;
 }
 
-int json_int_get (const json_value * integer) {
-   return integer->u.integer;
+int json_int_get(const json_value *integer)
+{
+	return integer->u.integer;
 }
 
-int json_boolean_get(const json_value * boolean) {
-   return boolean->u.boolean != 0;
+int json_boolean_get(const json_value *boolean)
+{
+	return boolean->u.boolean != 0;
 }
 
 static char EscapeJsonChar(char c)
