@@ -7,7 +7,7 @@
 // buffered stream for reading lines, should perhaps be something smaller
 class CLineReader
 {
-	char m_aBuffer[4 * 8192];
+	char m_aBuffer[4 * 8192 + 1]; // 1 additional byte for null termination
 	unsigned m_BufferPos;
 	unsigned m_BufferSize;
 	unsigned m_BufferMaxSize;
