@@ -46,6 +46,8 @@ public:
 	void Reset(const void *pData, int Size);
 	int GetInt();
 	int GetIntOrDefault(int Default);
+	int GetUncompressedInt();
+	int GetUncompressedIntOrDefault(int Default);
 	const char *GetString(int SanitizeType = SANITIZE);
 	const unsigned char *GetRaw(int Size);
 	bool Error() const { return m_Error; }
