@@ -82,9 +82,11 @@ enum
 	SERVERINFO_LEVEL_MIN = 0,
 	SERVERINFO_LEVEL_MAX = 2,
 
+	MAX_SERVER_ADDRESSES = 16,
 	SERVERINFO_MAX_CLIENTS = 128,
 	MAX_CLIENTS = 64,
 	VANILLA_MAX_CLIENTS = 16,
+	MAX_CHECKPOINTS = 25,
 
 	MAX_INPUT_SIZE = 128,
 	MAX_SNAPSHOT_PACKSIZE = 900,
@@ -108,16 +110,17 @@ enum
 	VERSION_DDNET_OLD = 2,
 	VERSION_DDNET_WHISPER = 217,
 	VERSION_DDNET_GOODHOOK = 221,
-	VERSION_DDNET_EXTRATUNES = 302,
 	VERSION_DDNET_RCONPROTECT = 408,
 	VERSION_DDNET_ANTIPING_PROJECTILE = 604,
-	VERSION_DDNET_HOOKDURATION_TUNE = 607,
-	VERSION_DDNET_FIREDELAY_TUNE = 701,
 	VERSION_DDNET_UPDATER_FIXED = 707,
 	VERSION_DDNET_GAMETICK = 10042,
+	VERSION_DDNET_EARLY_VERSION = 13020,
 	VERSION_DDNET_MSG_LEGACY = 15040,
 	VERSION_DDNET_SWITCH = 15060,
 	VERSION_DDNET_INDEPENDENT_SPECTATORS_TEAM = 16000,
+	VERSION_DDNET_WEAPON_SHIELDS = 16010,
+	VERSION_DDNET_NEW_HUD = 16020,
+	VERSION_DDNET_MULTI_LASER = 16040,
 };
 
 #endif
