@@ -66,7 +66,7 @@ void *CRingBufferBase::Allocate(int Size)
 	if(WantedSize > m_Size)
 		return 0;
 
-	while(1)
+	while(true)
 	{
 		// check for space
 		if(m_pProduce->m_Free)
@@ -153,7 +153,7 @@ void *CRingBufferBase::Prev(void *pCurrent)
 {
 	CItem *pItem = ((CItem *)pCurrent) - 1;
 
-	while(1)
+	while(true)
 	{
 		pItem = PrevBlock(pItem);
 		if(pItem == m_pProduce)
@@ -167,7 +167,7 @@ void *CRingBufferBase::Next(void *pCurrent)
 {
 	CItem *pItem = ((CItem *)pCurrent) - 1;
 
-	while(1)
+	while(true)
 	{
 		pItem = NextBlock(pItem);
 		if(pItem == m_pProduce)
