@@ -12,12 +12,9 @@ class CCubicBezier
 	float b;
 	float c;
 	float d;
-	CCubicBezier(float a, float b, float c, float d)
+	CCubicBezier(float a_, float b_, float c_, float d_) :
+		a(a_), b(b_), c(c_), d(d_)
 	{
-		this->a = a;
-		this->b = b;
-		this->c = c;
-		this->d = d;
 	}
 
 public:
