@@ -13,10 +13,10 @@ class CAnimState
 	CAnimKeyframe m_Attach;
 
 public:
-	CAnimKeyframe *GetBody() { return &m_Body; };
-	CAnimKeyframe *GetBackFoot() { return &m_BackFoot; };
-	CAnimKeyframe *GetFrontFoot() { return &m_FrontFoot; };
-	CAnimKeyframe *GetAttach() { return &m_Attach; };
+	CAnimKeyframe *GetBody() { return &m_Body; }
+	CAnimKeyframe *GetBackFoot() { return &m_BackFoot; }
+	CAnimKeyframe *GetFrontFoot() { return &m_FrontFoot; }
+	CAnimKeyframe *GetAttach() { return &m_Attach; }
 	void Set(CAnimation *pAnim, float Time);
 	void Add(CAnimation *pAnim, float Time, float Amount);
 
