@@ -7,15 +7,8 @@
 #include <engine/shared/video.h>
 #endif
 
-#include <base/color.h>
 #include <engine/input.h>
 
-#include <engine/client.h>
-#include <engine/console.h>
-#include <game/localization.h>
-
-#include <engine/config.h>
-
 class CGameClient;
 
 /**
@@ -36,6 +29,7 @@ protected:
 	 * Get the kernel interface.
 	 */
 	class IKernel *Kernel() const;
+	class IEngine *Engine() const;
 	/**
 	 * Get the graphics interface.
 	 */
@@ -86,6 +80,7 @@ protected:
 	 * @see RECORDER_REPLAYS
 	 */
 	class IDemoRecorder *DemoRecorder(int Recorder) const;
+	class IFavorites *Favorites() const;
 	/**
 	 * Get the server browser interface.
 	 */
@@ -205,8 +200,9 @@ public:
 	 *
 	 * @param x The amount of change in the x coordinate since the last call.
 	 * @param y The amount of change in the y coordinate since the last call.
+	 * @param CursorType The type of cursor that caused the movement.
 	 */
-	virtual bool OnMouseMove(float x, float y) { return false; }
+	virtual bool OnCursorMove(float x, float y, IInput::ECursorType CursorType) { return false; }
 	/**
 	 * Called on a input event.
 	 * @param e The input event.
