@@ -1,7 +1,7 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 
-#include <base/tl/sorted_array.h>
+#include <base/logger.h>
 
 #include <climits>
 #include <cmath>
@@ -14,21 +14,16 @@
 #include <engine/console.h>
 #include <engine/graphics.h>
 #include <engine/keys.h>
-#include <engine/serverbrowser.h>
 #include <engine/shared/config.h>
 #include <engine/shared/ringbuffer.h>
 #include <engine/storage.h>
 #include <engine/textrender.h>
 
-#include <cstdio>
-#include <cstring>
-
 #include <game/client/ui.h>
 
+#include <game/localization.h>
 #include <game/version.h>
 
-#include <game/client/components/controls.h>
-#include <game/client/components/menus.h>
 #include <game/client/lineinput.h>
 #include <game/client/render.h>
 
@@ -38,6 +33,56 @@
 
 #include "console.h"
 
+class CConsoleLogger : public ILogger
+{
+	CGameConsole *m_pConsole;
+	std::mutex m_ConsoleMutex;
+
+public:
+	CConsoleLogger(CGameConsole *pConsole) :
+		m_pConsole(pConsole)
+	{
+		dbg_assert(pConsole != nullptr, "console pointer must not be null");
+	}
+
+	void Log(const CLogMessage *pMessage) override;
+	void OnConsoleDeletion();
+};
+
+void CConsoleLogger::Log(const CLogMessage *pMessage)
+{
+	// TODO: Fix thread-unsafety of accessing `g_Config.m_ConsoleOutputLevel`
+	if(pMessage->m_Level > IConsole::ToLogLevel(g_Config.m_ConsoleOutputLevel))
+	{
+		return;
+	}
+	ColorRGBA Color = gs_ConsoleDefaultColor;
+	if(pMessage->m_HaveColor)
+	{
+		Color.r = pMessage->m_Color.r / 255.0;
+		Color.g = pMessage->m_Color.g / 255.0;
+		Color.b = pMessage->m_Color.b / 255.0;
+	}
+	std::unique_lock<std::mutex> Guard(m_ConsoleMutex);
+	if(m_pConsole)
+	{
+		m_pConsole->m_LocalConsole.PrintLine(pMessage->m_aLine, pMessage->m_LineLength, Color);
+	}
+}
+
+void CConsoleLogger::OnConsoleDeletion()
+{
+	std::unique_lock<std::mutex> Guard(m_ConsoleMutex);
+	m_pConsole = nullptr;
+}
+
+// TODO: support "tune_zone", which has tuning as second argument
+static const char *gs_apTuningCommands[] = {"tune ", "tune_reset ", "toggle_tune "};
+static bool IsTuningCommandPrefix(const char *pStr)
+{
+	return std::any_of(std::begin(gs_apTuningCommands), std::end(gs_apTuningCommands), [pStr](auto *pCmd) { return str_startswith_nocase(pStr, pCmd); });
+}
+
 CGameConsole::CInstance::CInstance(int Type)
 {
 	m_pHistoryEntry = 0x0;
@@ -45,15 +90,21 @@ CGameConsole::CInstance::CInstance(int Type)
 	m_Type = Type;
 
 	if(Type == CGameConsole::CONSOLETYPE_LOCAL)
+	{
+		m_pName = "local_console";
 		m_CompletionFlagmask = CFGFLAG_CLIENT;
+	}
 	else
+	{
+		m_pName = "remote_console";
 		m_CompletionFlagmask = CFGFLAG_SERVER;
+	}
 
 	m_aCompletionBuffer[0] = 0;
-	m_CompletionUsed = false;
 	m_CompletionChosen = -1;
-	m_CompletionRenderOffset = 0.0f;
-	m_ReverseTAB = false;
+	m_aCompletionBufferArgument[0] = 0;
+	m_CompletionChosenArgument = -1;
+	Reset();
 
 	m_aUser[0] = '\0';
 	m_UserGot = false;
@@ -69,18 +120,22 @@ void CGameConsole::CInstance::Init(CGameConsole *pGameConsole)
 
 void CGameConsole::CInstance::ClearBacklog()
 {
+	m_BacklogLock.lock();
 	m_Backlog.Init();
-	m_BacklogActPage = 0;
+	m_BacklogCurPage = 0;
+	m_BacklogLock.unlock();
 }
 
 void CGameConsole::CInstance::ClearBacklogYOffsets()
 {
+	m_BacklogLock.lock();
 	auto *pEntry = m_Backlog.First();
 	while(pEntry)
 	{
 		pEntry->m_YOffset = -1.0f;
 		pEntry = m_Backlog.Next(pEntry);
 	}
+	m_BacklogLock.unlock();
 }
 
 void CGameConsole::CInstance::ClearHistory()
@@ -89,6 +144,12 @@ void CGameConsole::CInstance::ClearHistory()
 	m_pHistoryEntry = 0;
 }
 
+void CGameConsole::CInstance::Reset()
+{
+	m_CompletionRenderOffset = 0.0f;
+	m_CompletionRenderOffsetChange = 0.0f;
+}
+
 void CGameConsole::CInstance::ExecuteLine(const char *pLine)
 {
 	if(m_Type == CGameConsole::CONSOLETYPE_LOCAL)
@@ -102,7 +163,7 @@ void CGameConsole::CInstance::ExecuteLine(const char *pLine)
 			if(!m_UserGot && m_UsernameReq)
 			{
 				m_UserGot = true;
-				str_copy(m_aUser, pLine, sizeof m_aUser);
+				str_copy(m_aUser, pLine);
 			}
 			else
 			{
@@ -113,12 +174,33 @@ void CGameConsole::CInstance::ExecuteLine(const char *pLine)
 	}
 }
 
-void CGameConsole::CInstance::PossibleCommandsCompleteCallback(const char *pStr, void *pUser)
+void CGameConsole::CInstance::PossibleCommandsCompleteCallback(int Index, const char *pStr, void *pUser)
 {
 	CGameConsole::CInstance *pInstance = (CGameConsole::CInstance *)pUser;
-	if(pInstance->m_CompletionChosen == pInstance->m_CompletionEnumerationCount)
+	if(pInstance->m_CompletionChosen == Index)
 		pInstance->m_Input.Set(pStr);
-	pInstance->m_CompletionEnumerationCount++;
+}
+
+static void StrCopyUntilSpace(char *pDest, size_t DestSize, const char *pSrc)
+{
+	const char *pSpace = str_find(pSrc, " ");
+	str_copy(pDest, pSrc, minimum<size_t>(pSpace ? pSpace - pSrc + 1 : 1, DestSize));
+}
+
+void CGameConsole::CInstance::PossibleArgumentsCompleteCallback(int Index, const char *pStr, void *pUser)
+{
+	CGameConsole::CInstance *pInstance = (CGameConsole::CInstance *)pUser;
+	if(pInstance->m_CompletionChosenArgument == Index)
+	{
+		// get command
+		char aBuf[512];
+		StrCopyUntilSpace(aBuf, sizeof(aBuf), pInstance->GetString());
+		str_append(aBuf, " ", sizeof(aBuf));
+
+		// append argument
+		str_append(aBuf, pStr, sizeof(aBuf));
+		pInstance->m_Input.Set(aBuf);
+	}
 }
 
 void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
@@ -188,14 +270,14 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 	}
 	if(m_pGameConsole->Input()->ModifierIsPressed() && m_pGameConsole->Input()->KeyPress(KEY_V))
 	{
-		const char *Text = m_pGameConsole->Input()->GetClipboardText();
-		if(Text)
+		const char *pText = m_pGameConsole->Input()->GetClipboardText();
+		if(pText)
 		{
 			char aLine[256];
 			int i, Begin = 0;
-			for(i = 0; i < str_length(Text); i++)
+			for(i = 0; i < str_length(pText); i++)
 			{
-				if(Text[i] == '\n')
+				if(pText[i] == '\n')
 				{
 					if(i == Begin)
 					{
@@ -203,13 +285,13 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 						continue;
 					}
 					int max = minimum(i - Begin + 1, (int)sizeof(aLine));
-					str_copy(aLine, Text + Begin, max);
+					str_copy(aLine, pText + Begin, max);
 					Begin = i + 1;
 					ExecuteLine(aLine);
 				}
 			}
 			int max = minimum(i - Begin + 1, (int)sizeof(aLine));
-			str_copy(aLine, Text + Begin, max);
+			str_copy(aLine, pText + Begin, max);
 			m_Input.Append(aLine);
 		}
 	}
@@ -243,7 +325,7 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 				if(m_Type == CONSOLETYPE_LOCAL || m_pGameConsole->Client()->RconAuthed())
 				{
 					char *pEntry = m_History.Allocate(m_Input.GetLength() + 1);
-					mem_copy(pEntry, m_Input.GetString(), m_Input.GetLength() + 1);
+					str_copy(pEntry, m_Input.GetString(), m_Input.GetLength() + 1);
 				}
 				ExecuteLine(m_Input.GetString());
 				m_Input.Clear();
@@ -281,60 +363,73 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 		}
 		else if(Event.m_Key == KEY_TAB)
 		{
+			const int Direction = m_pGameConsole->m_pClient->Input()->KeyIsPressed(KEY_LSHIFT) || m_pGameConsole->m_pClient->Input()->KeyIsPressed(KEY_RSHIFT) ? -1 : 1;
+
+			// command completion
 			if(m_Type == CGameConsole::CONSOLETYPE_LOCAL || m_pGameConsole->Client()->RconAuthed())
 			{
-				if(m_ReverseTAB && m_CompletionUsed)
-					m_CompletionChosen--;
-				else if(!m_ReverseTAB)
-					m_CompletionChosen++;
-				m_CompletionEnumerationCount = 0;
-				m_pGameConsole->m_pConsole->PossibleCommands(m_aCompletionBuffer, m_CompletionFlagmask, m_Type != CGameConsole::CONSOLETYPE_LOCAL && m_pGameConsole->Client()->RconAuthed() && m_pGameConsole->Client()->UseTempRconCommands(), PossibleCommandsCompleteCallback, this);
-
-				m_CompletionUsed = true;
-
-				// handle wrapping
-				if(m_CompletionEnumerationCount && (m_CompletionChosen >= m_CompletionEnumerationCount || m_CompletionChosen < 0))
+				const bool UseTempCommands = m_Type == CGameConsole::CONSOLETYPE_REMOTE && m_pGameConsole->Client()->RconAuthed() && m_pGameConsole->Client()->UseTempRconCommands();
+				const int CompletionEnumerationCount = m_pGameConsole->m_pConsole->PossibleCommands(m_aCompletionBuffer, m_CompletionFlagmask, UseTempCommands);
+				if(CompletionEnumerationCount)
 				{
-					m_CompletionChosen = (m_CompletionChosen + m_CompletionEnumerationCount) % m_CompletionEnumerationCount;
-					m_CompletionEnumerationCount = 0;
-					m_pGameConsole->m_pConsole->PossibleCommands(m_aCompletionBuffer, m_CompletionFlagmask, m_Type != CGameConsole::CONSOLETYPE_LOCAL && m_pGameConsole->Client()->RconAuthed() && m_pGameConsole->Client()->UseTempRconCommands(), PossibleCommandsCompleteCallback, this);
+					if(m_CompletionChosen == -1 && Direction < 0)
+						m_CompletionChosen = 0;
+					m_CompletionChosen = (m_CompletionChosen + Direction + CompletionEnumerationCount) % CompletionEnumerationCount;
+					m_pGameConsole->m_pConsole->PossibleCommands(m_aCompletionBuffer, m_CompletionFlagmask, UseTempCommands, PossibleCommandsCompleteCallback, this);
+				}
+				else if(m_CompletionChosen != -1)
+				{
+					m_CompletionChosen = -1;
+					Reset();
+				}
+			}
+
+			// argument completion (tuning, ...)
+			if(m_Type == CGameConsole::CONSOLETYPE_REMOTE && m_pGameConsole->Client()->RconAuthed())
+			{
+				const bool TuningCompletion = IsTuningCommandPrefix(GetString());
+				if(TuningCompletion)
+				{
+					int CompletionEnumerationCount = m_pGameConsole->m_pClient->m_aTuning[g_Config.m_ClDummy].PossibleTunings(m_aCompletionBufferArgument);
+					if(CompletionEnumerationCount)
+					{
+						if(m_CompletionChosenArgument == -1 && Direction < 0)
+							m_CompletionChosenArgument = 0;
+						m_CompletionChosenArgument = (m_CompletionChosenArgument + Direction + CompletionEnumerationCount) % CompletionEnumerationCount;
+						m_pGameConsole->m_pClient->m_aTuning[g_Config.m_ClDummy].PossibleTunings(m_aCompletionBufferArgument, PossibleArgumentsCompleteCallback, this);
+					}
+					else if(m_CompletionChosenArgument != -1)
+					{
+						m_CompletionChosenArgument = -1;
+						Reset();
+					}
 				}
 			}
 		}
 		else if(Event.m_Key == KEY_PAGEUP)
 		{
-			++m_BacklogActPage;
+			++m_BacklogCurPage;
 			m_pGameConsole->m_HasSelection = false;
 		}
 		else if(Event.m_Key == KEY_PAGEDOWN)
 		{
 			m_pGameConsole->m_HasSelection = false;
-			--m_BacklogActPage;
-			if(m_BacklogActPage < 0)
-				m_BacklogActPage = 0;
+			--m_BacklogCurPage;
+			if(m_BacklogCurPage < 0)
+				m_BacklogCurPage = 0;
 		}
 		// in order not to conflict with CLineInput's handling of Home/End only
 		// react to it when the input is empty
 		else if(Event.m_Key == KEY_HOME && m_Input.GetString()[0] == '\0')
 		{
-			m_BacklogActPage = INT_MAX;
+			m_BacklogCurPage = INT_MAX;
 			m_pGameConsole->m_HasSelection = false;
 		}
 		else if(Event.m_Key == KEY_END && m_Input.GetString()[0] == '\0')
 		{
-			m_BacklogActPage = 0;
+			m_BacklogCurPage = 0;
 			m_pGameConsole->m_HasSelection = false;
 		}
-		else if(Event.m_Key == KEY_LSHIFT)
-		{
-			m_ReverseTAB = true;
-			Handled = true;
-		}
-	}
-	if(Event.m_Flags & IInput::FLAG_RELEASE && Event.m_Key == KEY_LSHIFT)
-	{
-		m_ReverseTAB = false;
-		Handled = true;
 	}
 
 	if(!Handled)
@@ -342,31 +437,35 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 
 	if(Event.m_Flags & (IInput::FLAG_PRESS | IInput::FLAG_TEXT))
 	{
-		if((Event.m_Key != KEY_TAB) && (Event.m_Key != KEY_LSHIFT))
+		if(Event.m_Key != KEY_TAB && Event.m_Key != KEY_LSHIFT && Event.m_Key != KEY_RSHIFT)
 		{
-			m_CompletionUsed = false;
 			m_CompletionChosen = -1;
-			str_copy(m_aCompletionBuffer, m_Input.GetString(), sizeof(m_aCompletionBuffer));
-			m_CompletionRenderOffset = 0.0f;
+			str_copy(m_aCompletionBuffer, m_Input.GetString());
+
+			for(const auto *pCmd : gs_apTuningCommands)
+			{
+				if(str_startswith_nocase(m_Input.GetString(), pCmd))
+				{
+					m_CompletionChosenArgument = -1;
+					str_copy(m_aCompletionBufferArgument, &m_Input.GetString()[str_length(pCmd)]);
+				}
+			}
+
+			Reset();
 		}
 
 		// find the current command
 		{
-			char aBuf[64] = {0};
-			const char *pSrc = GetString();
-			int i = 0;
-			for(; i < (int)sizeof(aBuf) - 1 && *pSrc && *pSrc != ' '; i++, pSrc++)
-				aBuf[i] = *pSrc;
-			aBuf[i] = 0;
-
+			char aBuf[sizeof(m_aCommandName)];
+			StrCopyUntilSpace(aBuf, sizeof(aBuf), GetString());
 			const IConsole::CCommandInfo *pCommand = m_pGameConsole->m_pConsole->GetCommandInfo(aBuf, m_CompletionFlagmask,
 				m_Type != CGameConsole::CONSOLETYPE_LOCAL && m_pGameConsole->Client()->RconAuthed() && m_pGameConsole->Client()->UseTempRconCommands());
 			if(pCommand)
 			{
 				m_IsCommand = true;
-				str_copy(m_aCommandName, pCommand->m_pName, IConsole::TEMPCMD_NAME_LENGTH);
-				str_copy(m_aCommandHelp, pCommand->m_pHelp, IConsole::TEMPCMD_HELP_LENGTH);
-				str_copy(m_aCommandParams, pCommand->m_pParams, IConsole::TEMPCMD_PARAMS_LENGTH);
+				str_copy(m_aCommandName, pCommand->m_pName);
+				str_copy(m_aCommandHelp, pCommand->m_pHelp);
+				str_copy(m_aCommandParams, pCommand->m_pParams);
 			}
 			else
 				m_IsCommand = false;
@@ -374,20 +473,19 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 	}
 }
 
-void CGameConsole::CInstance::PrintLine(const char *pLine, ColorRGBA PrintColor)
+void CGameConsole::CInstance::PrintLine(const char *pLine, int Len, ColorRGBA PrintColor)
 {
-	int Len = str_length(pLine);
-
 	if(Len > 255)
 		Len = 255;
 
+	m_BacklogLock.lock();
 	CBacklogEntry *pEntry = m_Backlog.Allocate(sizeof(CBacklogEntry) + Len);
 	pEntry->m_YOffset = -1.0f;
 	pEntry->m_PrintColor = PrintColor;
-	mem_copy(pEntry->m_aText, pLine, Len);
-	pEntry->m_aText[Len] = 0;
+	str_copy(pEntry->m_aText, pLine, Len + 1);
 	if(m_pGameConsole->m_ConsoleType == m_Type)
 		m_pGameConsole->m_NewLineCounter++;
+	m_BacklogLock.unlock();
 }
 
 CGameConsole::CGameConsole() :
@@ -399,9 +497,15 @@ CGameConsole::CGameConsole() :
 	m_StateChangeDuration = 0.1f;
 }
 
+CGameConsole::~CGameConsole()
+{
+	if(m_pConsoleLogger)
+		m_pConsoleLogger->OnConsoleDeletion();
+}
+
 float CGameConsole::TimeNow()
 {
-	static long long s_TimeStart = time_get();
+	static int64_t s_TimeStart = time_get();
 	return float(time_get() - s_TimeStart) / float(time_freq());
 }
 
@@ -414,6 +518,7 @@ CGameConsole::CInstance *CGameConsole::CurrentConsole()
 
 void CGameConsole::OnReset()
 {
+	m_RemoteConsole.Reset();
 }
 
 // only defined for 0<=t<=1
@@ -423,35 +528,35 @@ static float ConsoleScaleFunc(float t)
 	return sinf(acosf(1.0f - t));
 }
 
-struct CRenderInfo
+struct CCompletionOptionRenderInfo
 {
 	CGameConsole *m_pSelf;
 	CTextCursor m_Cursor;
 	const char *m_pCurrentCmd;
 	int m_WantedCompletion;
-	int m_EnumCount;
 	float m_Offset;
+	float *m_pOffsetChange;
 	float m_Width;
+	float m_TotalWidth;
 };
 
-void CGameConsole::PossibleCommandsRenderCallback(const char *pStr, void *pUser)
+void CGameConsole::PossibleCommandsRenderCallback(int Index, const char *pStr, void *pUser)
 {
-	CRenderInfo *pInfo = static_cast<CRenderInfo *>(pUser);
+	CCompletionOptionRenderInfo *pInfo = static_cast<CCompletionOptionRenderInfo *>(pUser);
 
-	if(pInfo->m_EnumCount == pInfo->m_WantedCompletion)
+	if(Index == pInfo->m_WantedCompletion)
 	{
-		float tw = pInfo->m_pSelf->TextRender()->TextWidth(pInfo->m_Cursor.m_pFont, pInfo->m_Cursor.m_FontSize, pStr, -1, -1.0f);
-		pInfo->m_pSelf->Graphics()->TextureClear();
-		pInfo->m_pSelf->Graphics()->QuadsBegin();
-		pInfo->m_pSelf->Graphics()->SetColor(229.0f / 255.0f, 185.0f / 255.0f, 4.0f / 255.0f, 0.85f);
-		pInfo->m_pSelf->RenderTools()->DrawRoundRect(pInfo->m_Cursor.m_X - 2.5f, pInfo->m_Cursor.m_Y - 4.f / 2.f, tw + 5.f, pInfo->m_Cursor.m_FontSize + 4.f, pInfo->m_Cursor.m_FontSize / 3.f);
-		pInfo->m_pSelf->Graphics()->QuadsEnd();
+		float TextWidth = pInfo->m_pSelf->TextRender()->TextWidth(pInfo->m_Cursor.m_pFont, pInfo->m_Cursor.m_FontSize, pStr, -1, -1.0f);
+		const CUIRect Rect = {pInfo->m_Cursor.m_X - 2.5f, pInfo->m_Cursor.m_Y - 4.f / 2.f, TextWidth + 5.f, pInfo->m_Cursor.m_FontSize + 4.f};
+		Rect.Draw(ColorRGBA(229.0f / 255.0f, 185.0f / 255.0f, 4.0f / 255.0f, 0.85f), IGraphics::CORNER_ALL, pInfo->m_Cursor.m_FontSize / 3.f);
 
 		// scroll when out of sight
-		if(pInfo->m_Cursor.m_X < 3.0f)
-			pInfo->m_Offset = 0.0f;
-		else if(pInfo->m_Cursor.m_X + tw > pInfo->m_Width)
-			pInfo->m_Offset -= pInfo->m_Width / 2;
+		const bool MoveLeft = Rect.x - *pInfo->m_pOffsetChange < 0.0f;
+		const bool MoveRight = Rect.x + Rect.w - *pInfo->m_pOffsetChange > pInfo->m_Width;
+		if(MoveLeft && !MoveRight)
+			*pInfo->m_pOffsetChange -= -Rect.x + pInfo->m_Width / 4.0f;
+		else if(!MoveLeft && MoveRight)
+			*pInfo->m_pOffsetChange += Rect.x + Rect.w - pInfo->m_Width + pInfo->m_Width / 4.0f;
 
 		pInfo->m_pSelf->TextRender()->TextColor(0.05f, 0.05f, 0.05f, 1);
 		pInfo->m_pSelf->TextRender()->TextEx(&pInfo->m_Cursor, pStr, -1);
@@ -476,8 +581,8 @@ void CGameConsole::PossibleCommandsRenderCallback(const char *pStr, void *pUser)
 		}
 	}
 
-	pInfo->m_EnumCount++;
 	pInfo->m_Cursor.m_X += 7.0f;
+	pInfo->m_TotalWidth = pInfo->m_Cursor.m_X + pInfo->m_Offset;
 }
 
 void CGameConsole::OnRender()
@@ -575,15 +680,8 @@ void CGameConsole::OnRender()
 		float x = 3;
 		float y = ConsoleHeight - RowHeight - 5.0f;
 
-		CRenderInfo Info;
-		Info.m_pSelf = this;
-		Info.m_WantedCompletion = pConsole->m_CompletionUsed ? pConsole->m_CompletionChosen : -1;
-		Info.m_EnumCount = 0;
-		Info.m_Offset = pConsole->m_CompletionRenderOffset;
-		Info.m_Width = Screen.w;
-		Info.m_pCurrentCmd = pConsole->m_aCompletionBuffer;
-		TextRender()->SetCursor(&Info.m_Cursor, x + Info.m_Offset, y + RowHeight + 2.0f, FontSize, TEXTFLAG_RENDER | TEXTFLAG_STOP_AT_END);
-		Info.m_Cursor.m_LineWidth = std::numeric_limits<float>::max();
+		const float InitialX = x;
+		const float InitialY = y;
 
 		// render prompt
 		CTextCursor Cursor;
@@ -629,7 +727,7 @@ void CGameConsole::OnRender()
 
 		//hide rcon password
 		char aInputString[512];
-		str_copy(aInputString, pConsole->m_Input.GetString(Editing), sizeof(aInputString));
+		str_copy(aInputString, pConsole->m_Input.GetString(Editing));
 		if(m_ConsoleType == CONSOLETYPE_REMOTE && Client()->State() == IClient::STATE_ONLINE && !Client()->RconAuthed() && (pConsole->m_UserGot || !pConsole->m_UsernameReq))
 		{
 			for(int i = 0; i < pConsole->m_Input.GetLength(Editing); ++i)
@@ -663,29 +761,51 @@ void CGameConsole::OnRender()
 		Input()->SetEditingPosition(Marker.m_X, Marker.m_Y + Marker.m_FontSize);
 
 		// render possible commands
-		if(m_ConsoleType == CONSOLETYPE_LOCAL || Client()->RconAuthed())
+		if((m_ConsoleType == CONSOLETYPE_LOCAL || Client()->RconAuthed()) && pConsole->m_Input.GetString()[0])
 		{
-			if(pConsole->m_Input.GetString()[0] != 0)
-			{
-				m_pConsole->PossibleCommands(pConsole->m_aCompletionBuffer, pConsole->m_CompletionFlagmask, m_ConsoleType != CGameConsole::CONSOLETYPE_LOCAL && Client()->RconAuthed() && Client()->UseTempRconCommands(), PossibleCommandsRenderCallback, &Info);
-				pConsole->m_CompletionRenderOffset = Info.m_Offset;
+			CCompletionOptionRenderInfo Info;
+			Info.m_pSelf = this;
+			Info.m_WantedCompletion = pConsole->m_CompletionChosen;
+			Info.m_Offset = pConsole->m_CompletionRenderOffset;
+			Info.m_pOffsetChange = &pConsole->m_CompletionRenderOffsetChange;
+			Info.m_Width = Screen.w;
+			Info.m_TotalWidth = 0.0f;
+			Info.m_pCurrentCmd = pConsole->m_aCompletionBuffer;
+			TextRender()->SetCursor(&Info.m_Cursor, InitialX - Info.m_Offset, InitialY + RowHeight + 2.0f, FontSize, TEXTFLAG_RENDER | TEXTFLAG_STOP_AT_END);
+			Info.m_Cursor.m_LineWidth = std::numeric_limits<float>::max();
+			const int NumCommands = m_pConsole->PossibleCommands(Info.m_pCurrentCmd, pConsole->m_CompletionFlagmask, m_ConsoleType != CGameConsole::CONSOLETYPE_LOCAL && Client()->RconAuthed() && Client()->UseTempRconCommands(), PossibleCommandsRenderCallback, &Info);
+			pConsole->m_CompletionRenderOffset = Info.m_Offset;
 
-				if(Info.m_EnumCount <= 0)
+			if(NumCommands <= 0 && pConsole->m_IsCommand)
+			{
+				const bool TuningCompletion = IsTuningCommandPrefix(Info.m_pCurrentCmd);
+				int NumArguments = 0;
+				if(TuningCompletion)
 				{
-					if(pConsole->m_IsCommand)
-					{
-						char aBuf[512];
-						str_format(aBuf, sizeof(aBuf), "Help: %s ", pConsole->m_aCommandHelp);
-						TextRender()->TextEx(&Info.m_Cursor, aBuf, -1);
-						TextRender()->TextColor(0.75f, 0.75f, 0.75f, 1);
-						str_format(aBuf, sizeof(aBuf), "Usage: %s %s", pConsole->m_aCommandName, pConsole->m_aCommandParams);
-						TextRender()->TextEx(&Info.m_Cursor, aBuf, -1);
-					}
+					Info.m_WantedCompletion = pConsole->m_CompletionChosenArgument;
+					Info.m_TotalWidth = 0.0f;
+					Info.m_pCurrentCmd = pConsole->m_aCompletionBufferArgument;
+					NumArguments = m_pClient->m_aTuning[g_Config.m_ClDummy].PossibleTunings(Info.m_pCurrentCmd, PossibleCommandsRenderCallback, &Info);
+					pConsole->m_CompletionRenderOffset = Info.m_Offset;
+				}
+
+				if(NumArguments <= 0 && pConsole->m_IsCommand)
+				{
+					char aBuf[512];
+					str_format(aBuf, sizeof(aBuf), "Help: %s ", pConsole->m_aCommandHelp);
+					TextRender()->TextEx(&Info.m_Cursor, aBuf, -1);
+					TextRender()->TextColor(0.75f, 0.75f, 0.75f, 1);
+					str_format(aBuf, sizeof(aBuf), "Usage: %s %s", pConsole->m_aCommandName, pConsole->m_aCommandParams);
+					TextRender()->TextEx(&Info.m_Cursor, aBuf, -1);
 				}
 			}
+
+			UI()->DoSmoothScrollLogic(&pConsole->m_CompletionRenderOffset, &pConsole->m_CompletionRenderOffsetChange, Info.m_Width, Info.m_TotalWidth);
 		}
 
-		//	render log (actual page, wrap lines)
+		pConsole->m_BacklogLock.lock();
+
+		// render log (current page, wrap lines)
 		CInstance::CBacklogEntry *pEntry = pConsole->m_Backlog.Last();
 		float OffsetY = 0.0f;
 		float LineOffset = 1.0f;
@@ -714,7 +834,9 @@ void CGameConsole::OnRender()
 			m_MouseIsPress = false;
 		}
 
-		for(int Page = 0; Page <= pConsole->m_BacklogActPage; ++Page, OffsetY = 0.0f)
+		static int s_LastActivePage = pConsole->m_BacklogCurPage;
+		int TotalPages = 1;
+		for(int Page = 0; Page <= maximum(s_LastActivePage, pConsole->m_BacklogCurPage); ++Page, OffsetY = 0.0f)
 		{
 			while(pEntry)
 			{
@@ -742,8 +864,8 @@ void CGameConsole::OnRender()
 				if(y - OffsetY <= RowHeight)
 					break;
 
-				// just render output from actual backlog page (render bottom up)
-				if(Page == pConsole->m_BacklogActPage)
+				// just render output from current backlog page (render bottom up)
+				if(Page == s_LastActivePage)
 				{
 					TextRender()->SetCursor(&Cursor, 0.0f, y - OffsetY, FontSize, TEXTFLAG_RENDER);
 					Cursor.m_LineWidth = Screen.w - 10.0f;
@@ -788,31 +910,23 @@ void CGameConsole::OnRender()
 				Input()->SetClipboardText(SelectionString.c_str());
 			}
 
-			//	actual backlog page number is too high, render last available page (current checked one, render top down)
-			if(!pEntry && Page < pConsole->m_BacklogActPage)
-			{
-				pConsole->m_BacklogActPage = Page;
-				pEntry = pConsole->m_Backlog.First();
-				while(OffsetY > 0.0f && pEntry)
-				{
-					TextRender()->SetCursor(&Cursor, 0.0f, y - OffsetY, FontSize, TEXTFLAG_RENDER);
-					Cursor.m_LineWidth = Screen.w - 10.0f;
-					TextRender()->TextEx(&Cursor, pEntry->m_aText, -1);
-					OffsetY -= pEntry->m_YOffset;
-					pEntry = pConsole->m_Backlog.Next(pEntry);
-				}
+			if(!pEntry)
 				break;
-			}
+			TotalPages++;
 		}
+		pConsole->m_BacklogCurPage = clamp(pConsole->m_BacklogCurPage, 0, TotalPages - 1);
+		s_LastActivePage = pConsole->m_BacklogCurPage;
+
+		pConsole->m_BacklogLock.unlock();
 
 		// render page
 		char aBuf[128];
 		TextRender()->TextColor(1, 1, 1, 1);
-		str_format(aBuf, sizeof(aBuf), Localize("-Page %d-"), pConsole->m_BacklogActPage + 1);
+		str_format(aBuf, sizeof(aBuf), Localize("-Page %d-"), pConsole->m_BacklogCurPage + 1);
 		TextRender()->Text(0, 10.0f, FontSize / 2.f, FontSize, aBuf, -1.0f);
 
 		// render version
-		str_copy(aBuf, "v" GAME_VERSION " on " CONF_PLATFORM_STRING " " CONF_ARCH_STRING, sizeof(aBuf));
+		str_copy(aBuf, "v" GAME_VERSION " on " CONF_PLATFORM_STRING " " CONF_ARCH_STRING);
 		float Width = TextRender()->TextWidth(0, FontSize, aBuf, -1, -1.0f);
 		TextRender()->Text(0, Screen.w - Width - 10.0f, FontSize / 2.f, FontSize, aBuf, -1.0f);
 	}
@@ -860,18 +974,15 @@ void CGameConsole::Toggle(int Type)
 
 		if(m_ConsoleState == CONSOLE_CLOSED || m_ConsoleState == CONSOLE_CLOSING)
 		{
-			/*Input()->MouseModeAbsolute();*/
-			m_pClient->m_Menus.UseMouseButtons(false);
+			UI()->SetEnabled(false);
 			m_ConsoleState = CONSOLE_OPENING;
-			/*// reset controls
-			m_pClient->m_Controls.OnReset();*/
 
 			Input()->SetIMEState(true);
 		}
 		else
 		{
 			Input()->MouseModeRelative();
-			m_pClient->m_Menus.UseMouseButtons(true);
+			UI()->SetEnabled(true);
 			m_pClient->OnRelease();
 			m_ConsoleState = CONSOLE_CLOSING;
 
@@ -886,21 +997,28 @@ void CGameConsole::Toggle(int Type)
 void CGameConsole::Dump(int Type)
 {
 	CInstance *pConsole = Type == CONSOLETYPE_REMOTE ? &m_RemoteConsole : &m_LocalConsole;
+	char aBuf[IO_MAX_PATH_LENGTH + 64];
 	char aFilename[IO_MAX_PATH_LENGTH];
-	char aDate[20];
-
-	str_timestamp(aDate, sizeof(aDate));
-	str_format(aFilename, sizeof(aFilename), "dumps/%s_dump_%s.txt", Type == CONSOLETYPE_REMOTE ? "remote_console" : "local_console", aDate);
-	IOHANDLE io = Storage()->OpenFile(aFilename, IOFLAG_WRITE, IStorage::TYPE_SAVE);
-	if(io)
+	str_timestamp(aBuf, sizeof(aBuf));
+	str_format(aFilename, sizeof(aFilename), "dumps/%s_dump_%s.txt", pConsole->m_pName, aBuf);
+	IOHANDLE File = Storage()->OpenFile(aFilename, IOFLAG_WRITE, IStorage::TYPE_SAVE);
+	if(File)
 	{
+		pConsole->m_BacklogLock.lock();
 		for(CInstance::CBacklogEntry *pEntry = pConsole->m_Backlog.First(); pEntry; pEntry = pConsole->m_Backlog.Next(pEntry))
 		{
-			io_write(io, pEntry->m_aText, str_length(pEntry->m_aText));
-			io_write_newline(io);
+			io_write(File, pEntry->m_aText, str_length(pEntry->m_aText));
+			io_write_newline(File);
 		}
-		io_close(io);
+		pConsole->m_BacklogLock.unlock();
+		io_close(File);
+		str_format(aBuf, sizeof(aBuf), "%s contents were written to '%s'", pConsole->m_pName, aFilename);
 	}
+	else
+	{
+		str_format(aBuf, sizeof(aBuf), "Failed to open '%s'", aFilename);
+	}
+	Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
 }
 
 void CGameConsole::ConToggleLocalConsole(IConsole::IResult *pResult, void *pUserData)
@@ -933,33 +1051,18 @@ void CGameConsole::ConDumpRemoteConsole(IConsole::IResult *pResult, void *pUserD
 	((CGameConsole *)pUserData)->Dump(CONSOLETYPE_REMOTE);
 }
 
-void CGameConsole::ClientConsolePrintCallback(const char *pStr, void *pUserData, ColorRGBA PrintColor)
-{
-	((CGameConsole *)pUserData)->m_LocalConsole.PrintLine(pStr, PrintColor);
-}
-
 void CGameConsole::ConConsolePageUp(IConsole::IResult *pResult, void *pUserData)
 {
 	CInstance *pConsole = ((CGameConsole *)pUserData)->CurrentConsole();
-	pConsole->m_BacklogActPage++;
+	pConsole->m_BacklogCurPage++;
 }
 
 void CGameConsole::ConConsolePageDown(IConsole::IResult *pResult, void *pUserData)
 {
 	CInstance *pConsole = ((CGameConsole *)pUserData)->CurrentConsole();
-	--pConsole->m_BacklogActPage;
-	if(pConsole->m_BacklogActPage < 0)
-		pConsole->m_BacklogActPage = 0;
-}
-
-void CGameConsole::ConchainConsoleOutputLevelUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData)
-{
-	pfnCallback(pResult, pCallbackUserData);
-	if(pResult->NumArguments() == 1)
-	{
-		CGameConsole *pThis = static_cast<CGameConsole *>(pUserData);
-		pThis->Console()->SetPrintOutputLevel(pThis->m_PrintCBIndex, pResult->GetInteger(0));
-	}
+	--pConsole->m_BacklogCurPage;
+	if(pConsole->m_BacklogCurPage < 0)
+		pConsole->m_BacklogCurPage = 0;
 }
 
 void CGameConsole::RequireUsername(bool UsernameReq)
@@ -974,9 +1077,9 @@ void CGameConsole::RequireUsername(bool UsernameReq)
 void CGameConsole::PrintLine(int Type, const char *pLine)
 {
 	if(Type == CONSOLETYPE_LOCAL)
-		m_LocalConsole.PrintLine(pLine);
+		m_LocalConsole.PrintLine(pLine, str_length(pLine), ColorRGBA{1, 1, 1, 1});
 	else if(Type == CONSOLETYPE_REMOTE)
-		m_RemoteConsole.PrintLine(pLine);
+		m_RemoteConsole.PrintLine(pLine, str_length(pLine), ColorRGBA{1, 1, 1, 1});
 }
 
 void CGameConsole::OnConsoleInit()
@@ -987,24 +1090,21 @@ void CGameConsole::OnConsoleInit()
 
 	m_pConsole = Kernel()->RequestInterface<IConsole>();
 
-	//
-	m_PrintCBIndex = Console()->RegisterPrintCallback(g_Config.m_ConsoleOutputLevel, ClientConsolePrintCallback, this);
-
 	Console()->Register("toggle_local_console", "", CFGFLAG_CLIENT, ConToggleLocalConsole, this, "Toggle local console");
 	Console()->Register("toggle_remote_console", "", CFGFLAG_CLIENT, ConToggleRemoteConsole, this, "Toggle remote console");
 	Console()->Register("clear_local_console", "", CFGFLAG_CLIENT, ConClearLocalConsole, this, "Clear local console");
 	Console()->Register("clear_remote_console", "", CFGFLAG_CLIENT, ConClearRemoteConsole, this, "Clear remote console");
-	Console()->Register("dump_local_console", "", CFGFLAG_CLIENT, ConDumpLocalConsole, this, "Dump local console");
-	Console()->Register("dump_remote_console", "", CFGFLAG_CLIENT, ConDumpRemoteConsole, this, "Dump remote console");
+	Console()->Register("dump_local_console", "", CFGFLAG_CLIENT, ConDumpLocalConsole, this, "Write local console contents to a text file");
+	Console()->Register("dump_remote_console", "", CFGFLAG_CLIENT, ConDumpRemoteConsole, this, "Write remote console contents to a text file");
 
 	Console()->Register("console_page_up", "", CFGFLAG_CLIENT, ConConsolePageUp, this, "Previous page in console");
 	Console()->Register("console_page_down", "", CFGFLAG_CLIENT, ConConsolePageDown, this, "Next page in console");
-
-	Console()->Chain("console_output_level", ConchainConsoleOutputLevelUpdate, this);
 }
 
 void CGameConsole::OnInit()
 {
+	m_pConsoleLogger = new CConsoleLogger(this);
+	Engine()->SetAdditionalLogger(std::unique_ptr<ILogger>(m_pConsoleLogger));
 	// add resize event
 	Graphics()->AddWindowResizeListener([this](void *) {
 		m_LocalConsole.ClearBacklogYOffsets();
