@@ -6,6 +6,7 @@
 
 class CEffects : public CComponent
 {
+	bool m_Add5hz;
 	bool m_Add50hz;
 	bool m_Add100hz;
 
@@ -26,6 +27,7 @@ public:
 	void PlayerSpawn(vec2 Pos);
 	void PlayerDeath(vec2 Pos, int ClientID);
 	void PowerupShine(vec2 Pos, vec2 Size);
+	void FreezingFlakes(vec2 Pos, vec2 Size);
 
 	void Update();
 };
