@@ -1,7 +1,6 @@
 /* (c) Rajh, Redix and Sushi. */
 
 #include <engine/ghost.h>
-#include <engine/serverbrowser.h>
 #include <engine/shared/config.h>
 #include <engine/storage.h>
 
@@ -27,7 +26,7 @@ void CGhost::GetGhostSkin(CGhostSkin *pSkin, const char *pSkinName, int UseCusto
 	pSkin->m_ColorFeet = ColorFeet;
 }
 
-void CGhost::GetGhostCharacter(CGhostCharacter *pGhostChar, const CNetObj_Character *pChar)
+void CGhost::GetGhostCharacter(CGhostCharacter *pGhostChar, const CNetObj_Character *pChar, const CNetObj_DDNetCharacter *pDDnetChar)
 {
 	pGhostChar->m_X = pChar->m_X;
 	pGhostChar->m_Y = pChar->m_Y;
@@ -35,7 +34,12 @@ void CGhost::GetGhostCharacter(CGhostCharacter *pGhostChar, const CNetObj_Charac
 	pGhostChar->m_VelY = 0;
 	pGhostChar->m_Angle = pChar->m_Angle;
 	pGhostChar->m_Direction = pChar->m_Direction;
-	pGhostChar->m_Weapon = pChar->m_Weapon;
+	int Weapon = pChar->m_Weapon;
+	if(pDDnetChar != nullptr && pDDnetChar->m_FreezeEnd != 0)
+	{
+		Weapon = WEAPON_NINJA;
+	}
+	pGhostChar->m_Weapon = Weapon;
 	pGhostChar->m_HookState = pChar->m_HookState;
 	pGhostChar->m_HookX = pChar->m_HookX;
 	pGhostChar->m_HookY = pChar->m_HookY;
@@ -62,41 +66,41 @@ void CGhost::GetNetObjCharacter(CNetObj_Character *pChar, const CGhostCharacter
 }
 
 CGhost::CGhostPath::CGhostPath(CGhostPath &&Other) noexcept :
-	m_ChunkSize(Other.m_ChunkSize), m_NumItems(Other.m_NumItems), m_lChunks(std::move(Other.m_lChunks))
+	m_ChunkSize(Other.m_ChunkSize), m_NumItems(Other.m_NumItems), m_vpChunks(std::move(Other.m_vpChunks))
 {
 	Other.m_NumItems = 0;
-	Other.m_lChunks.clear();
+	Other.m_vpChunks.clear();
 }
 
 CGhost::CGhostPath &CGhost::CGhostPath::operator=(CGhostPath &&Other) noexcept
 {
 	Reset(Other.m_ChunkSize);
 	m_NumItems = Other.m_NumItems;
-	m_lChunks = std::move(Other.m_lChunks);
+	m_vpChunks = std::move(Other.m_vpChunks);
 	Other.m_NumItems = 0;
-	Other.m_lChunks.clear();
+	Other.m_vpChunks.clear();
 	return *this;
 }
 
 void CGhost::CGhostPath::Reset(int ChunkSize)
 {
-	for(auto &pChunk : m_lChunks)
+	for(auto &pChunk : m_vpChunks)
 		free(pChunk);
-	m_lChunks.clear();
+	m_vpChunks.clear();
 	m_ChunkSize = ChunkSize;
 	m_NumItems = 0;
 }
 
 void CGhost::CGhostPath::SetSize(int Items)
 {
-	int Chunks = m_lChunks.size();
+	int Chunks = m_vpChunks.size();
 	int NeededChunks = (Items + m_ChunkSize - 1) / m_ChunkSize;
 
 	if(NeededChunks > Chunks)
 	{
-		m_lChunks.resize(NeededChunks);
+		m_vpChunks.resize(NeededChunks);
 		for(int i = Chunks; i < NeededChunks; i++)
-			m_lChunks[i] = (CGhostCharacter *)calloc(m_ChunkSize, sizeof(CGhostCharacter));
+			m_vpChunks[i] = (CGhostCharacter *)calloc(m_ChunkSize, sizeof(CGhostCharacter));
 	}
 
 	m_NumItems = Items;
@@ -115,7 +119,7 @@ CGhostCharacter *CGhost::CGhostPath::Get(int Index)
 
 	int Chunk = Index / m_ChunkSize;
 	int Pos = Index % m_ChunkSize;
-	return &m_lChunks[Chunk][Pos];
+	return &m_vpChunks[Chunk][Pos];
 }
 
 void CGhost::GetPath(char *pBuf, int Size, const char *pPlayerName, int Time) const
@@ -126,7 +130,7 @@ void CGhost::GetPath(char *pBuf, int Size, const char *pPlayerName, int Time) co
 	sha256_str(Sha256, aSha256, sizeof(aSha256));
 
 	char aPlayerName[MAX_NAME_LENGTH];
-	str_copy(aPlayerName, pPlayerName, sizeof(aPlayerName));
+	str_copy(aPlayerName, pPlayerName);
 	str_sanitize_filename(aPlayerName);
 
 	if(Time < 0)
@@ -135,7 +139,7 @@ void CGhost::GetPath(char *pBuf, int Size, const char *pPlayerName, int Time) co
 		str_format(pBuf, Size, "%s/%s_%s_%d.%03d_%s.gho", ms_pGhostDir, pMap, aPlayerName, Time / 1000, Time % 1000, aSha256);
 }
 
-void CGhost::AddInfos(const CNetObj_Character *pChar)
+void CGhost::AddInfos(const CNetObj_Character *pChar, const CNetObj_DDNetCharacter *pDDnetChar)
 {
 	int NumTicks = m_CurGhost.m_Path.Size();
 
@@ -152,7 +156,7 @@ void CGhost::AddInfos(const CNetObj_Character *pChar)
 	}
 
 	CGhostCharacter GhostChar;
-	GetGhostCharacter(&GhostChar, pChar);
+	GetGhostCharacter(&GhostChar, pChar, pDDnetChar);
 	m_CurGhost.m_Path.Add(GhostChar);
 	if(GhostRecorder()->IsRecording())
 		GhostRecorder()->WriteData(GHOSTDATA_TYPE_CHARACTER, &GhostChar, sizeof(CGhostCharacter));
@@ -280,7 +284,7 @@ void CGhost::OnNewSnapshot()
 			CheckStart();
 
 		if(m_Recording)
-			AddInfos(m_pClient->m_Snap.m_pLocalCharacter);
+			AddInfos(m_pClient->m_Snap.m_pLocalCharacter, (m_pClient->m_Snap.m_LocalClientID != -1 && m_pClient->m_Snap.m_aCharacters[m_pClient->m_Snap.m_LocalClientID].m_HasExtendedData) ? &m_pClient->m_Snap.m_aCharacters[m_pClient->m_Snap.m_LocalClientID].m_ExtendedData : nullptr);
 	}
 
 	// Record m_LastRaceTick for g_Config.m_ClConfirmDisconnect/QuitTime anyway
@@ -343,8 +347,37 @@ void CGhost::OnRender()
 
 		Player.m_AttackTick += Client()->GameTick(g_Config.m_ClDummy) - GhostTick;
 
-		m_pClient->m_Players.RenderHook(&Prev, &Player, &Ghost.m_RenderInfo, -2, IntraTick);
-		m_pClient->m_Players.RenderPlayer(&Prev, &Player, &Ghost.m_RenderInfo, -2, IntraTick);
+		CTeeRenderInfo *pRenderInfo = &Ghost.m_RenderInfo;
+		CTeeRenderInfo GhostNinjaRenderInfo;
+		if(Player.m_Weapon == WEAPON_NINJA && g_Config.m_ClShowNinja)
+		{
+			// change the skin for the ghost to the ninja
+			int Skin = m_pClient->m_Skins.Find("x_ninja");
+			if(Skin != -1)
+			{
+				bool IsTeamplay = false;
+				if(m_pClient->m_Snap.m_pGameInfoObj)
+					IsTeamplay = (m_pClient->m_Snap.m_pGameInfoObj->m_GameFlags & GAMEFLAG_TEAMS) != 0;
+
+				GhostNinjaRenderInfo = Ghost.m_RenderInfo;
+				const CSkin *pSkin = m_pClient->m_Skins.Get(Skin);
+				GhostNinjaRenderInfo.m_OriginalRenderSkin = pSkin->m_OriginalSkin;
+				GhostNinjaRenderInfo.m_ColorableRenderSkin = pSkin->m_ColorableSkin;
+				GhostNinjaRenderInfo.m_BloodColor = pSkin->m_BloodColor;
+				GhostNinjaRenderInfo.m_SkinMetrics = pSkin->m_Metrics;
+				GhostNinjaRenderInfo.m_CustomColoredSkin = IsTeamplay;
+				if(!IsTeamplay)
+				{
+					GhostNinjaRenderInfo.m_ColorBody = ColorRGBA(1, 1, 1);
+					GhostNinjaRenderInfo.m_ColorFeet = ColorRGBA(1, 1, 1);
+				}
+				pRenderInfo = &GhostNinjaRenderInfo;
+			}
+		}
+
+		m_pClient->m_Players.RenderHook(&Prev, &Player, pRenderInfo, -2, IntraTick);
+		m_pClient->m_Players.RenderHookCollLine(&Prev, &Player, -2, IntraTick);
+		m_pClient->m_Players.RenderPlayer(&Prev, &Player, pRenderInfo, -2, IntraTick);
 	}
 }
 
@@ -363,8 +396,8 @@ void CGhost::InitRenderInfos(CGhostItem *pGhost)
 	pRenderInfo->m_CustomColoredSkin = pGhost->m_Skin.m_UseCustomColor;
 	if(pGhost->m_Skin.m_UseCustomColor)
 	{
-		pRenderInfo->m_ColorBody = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorBody));
-		pRenderInfo->m_ColorFeet = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorFeet));
+		pRenderInfo->m_ColorBody = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorBody).UnclampLighting());
+		pRenderInfo->m_ColorFeet = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorFeet).UnclampLighting());
 	}
 	else
 	{
@@ -382,7 +415,7 @@ void CGhost::StartRecord(int Tick)
 	m_CurGhost.m_StartTick = Tick;
 
 	const CGameClient::CClientData *pData = &m_pClient->m_aClients[m_pClient->m_Snap.m_LocalClientID];
-	str_copy(m_CurGhost.m_aPlayer, Client()->PlayerName(), sizeof(m_CurGhost.m_aPlayer));
+	str_copy(m_CurGhost.m_aPlayer, Client()->PlayerName());
 	GetGhostSkin(&m_CurGhost.m_Skin, pData->m_aSkinName, pData->m_UseCustomColor, pData->m_ColorBody, pData->m_ColorFeet);
 	InitRenderInfos(&m_CurGhost);
 }
@@ -410,7 +443,7 @@ void CGhost::StopRecord(int Time)
 		CMenus::CGhostItem Item;
 		if(RecordingToFile)
 			GetPath(Item.m_aFilename, sizeof(Item.m_aFilename), m_CurGhost.m_aPlayer, Time);
-		str_copy(Item.m_aPlayer, m_CurGhost.m_aPlayer, sizeof(Item.m_aPlayer));
+		str_copy(Item.m_aPlayer, m_CurGhost.m_aPlayer);
 		Item.m_Time = Time;
 		Item.m_Slot = Slot;
 
@@ -466,7 +499,7 @@ int CGhost::Load(const char *pFilename)
 	pGhost->Reset();
 	pGhost->m_Path.SetSize(pInfo->m_NumTicks);
 
-	str_copy(pGhost->m_aPlayer, pInfo->m_aOwner, sizeof(pGhost->m_aPlayer));
+	str_copy(pGhost->m_aPlayer, pInfo->m_aOwner);
 
 	int Index = 0;
 	bool FoundSkin = false;
