@@ -2,10 +2,9 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include <engine/graphics.h>
 #include <engine/map.h>
-#include <engine/serverbrowser.h>
 #include <engine/storage.h>
 #include <engine/textrender.h>
-#include <game/client/component.h>
+#include <game/generated/client_data.h>
 #include <game/mapitems.h>
 
 #include <game/layers.h>
@@ -14,6 +13,16 @@
 
 #include <game/client/gameclient.h>
 
+const char *const gs_apModEntitiesNames[] = {
+	"ddnet",
+	"ddrace",
+	"race",
+	"blockworlds",
+	"fng",
+	"vanilla",
+	"f-ddrace",
+};
+
 CMapImages::CMapImages() :
 	CMapImages(100)
 {
@@ -23,14 +32,14 @@ CMapImages::CMapImages(int TextureSize)
 {
 	m_Count = 0;
 	m_TextureScale = TextureSize;
-	mem_zero(m_EntitiesIsLoaded, sizeof(m_EntitiesIsLoaded));
+	mem_zero(m_aEntitiesIsLoaded, sizeof(m_aEntitiesIsLoaded));
 	m_SpeedupArrowIsLoaded = false;
 
 	mem_zero(m_aTextureUsedByTileOrQuadLayerFlag, sizeof(m_aTextureUsedByTileOrQuadLayerFlag));
 
-	str_copy(m_aEntitiesPath, "editor/entities_clear", sizeof(m_aEntitiesPath));
+	str_copy(m_aEntitiesPath, "editor/entities_clear");
 
-	static_assert(sizeof(gs_aModEntitiesNames) / sizeof(gs_aModEntitiesNames[0]) == MAP_IMAGE_MOD_TYPE_COUNT, "Mod name string count is not equal to mod type count");
+	static_assert(std::size(gs_apModEntitiesNames) == MAP_IMAGE_MOD_TYPE_COUNT, "Mod name string count is not equal to mod type count");
 }
 
 void CMapImages::OnInit()
@@ -38,7 +47,7 @@ void CMapImages::OnInit()
 	InitOverlayTextures();
 
 	if(str_comp(g_Config.m_ClAssetsEntites, "default") == 0)
-		str_copy(m_aEntitiesPath, "editor/entities_clear", sizeof(m_aEntitiesPath));
+		str_copy(m_aEntitiesPath, "editor/entities_clear");
 	else
 	{
 		str_format(m_aEntitiesPath, sizeof(m_aEntitiesPath), "assets/entities/%s", g_Config.m_ClAssetsEntites);
@@ -74,7 +83,7 @@ void CMapImages::OnMapLoadImpl(class CLayers *pLayers, IMap *pMap)
 			if(pLayer->m_Type == LAYERTYPE_TILES)
 			{
 				CMapItemLayerTilemap *pTLayer = (CMapItemLayerTilemap *)pLayer;
-				if(pTLayer->m_Image != -1 && pTLayer->m_Image < (int)(sizeof(m_aTextures) / sizeof(m_aTextures[0])))
+				if(pTLayer->m_Image != -1 && pTLayer->m_Image < (int)(std::size(m_aTextures)))
 				{
 					m_aTextureUsedByTileOrQuadLayerFlag[pTLayer->m_Image] |= 1;
 				}
@@ -82,7 +91,7 @@ void CMapImages::OnMapLoadImpl(class CLayers *pLayers, IMap *pMap)
 			else if(pLayer->m_Type == LAYERTYPE_QUADS)
 			{
 				CMapItemLayerQuads *pQLayer = (CMapItemLayerQuads *)pLayer;
-				if(pQLayer->m_Image != -1 && pQLayer->m_Image < (int)(sizeof(m_aTextures) / sizeof(m_aTextures[0])))
+				if(pQLayer->m_Image != -1 && pQLayer->m_Image < (int)(std::size(m_aTextures)))
 				{
 					m_aTextureUsedByTileOrQuadLayerFlag[pQLayer->m_Image] |= 2;
 				}
@@ -156,7 +165,7 @@ bool CMapImages::HasTuneLayer(EMapImageModType ModType)
 IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType EntityLayerType)
 {
 	EMapImageModType EntitiesModType = MAP_IMAGE_MOD_TYPE_DDNET;
-	bool EntitesAreMasked = !GameClient()->m_GameInfo.m_DontMaskEntities;
+	bool EntitiesAreMasked = !GameClient()->m_GameInfo.m_DontMaskEntities;
 
 	if(GameClient()->m_GameInfo.m_EntitiesFDDrace)
 		EntitiesModType = MAP_IMAGE_MOD_TYPE_FDDRACE;
@@ -173,21 +182,21 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 	else if(GameClient()->m_GameInfo.m_EntitiesVanilla)
 		EntitiesModType = MAP_IMAGE_MOD_TYPE_VANILLA;
 
-	if(!m_EntitiesIsLoaded[(EntitiesModType * 2) + (int)EntitesAreMasked])
+	if(!m_aEntitiesIsLoaded[(EntitiesModType * 2) + (int)EntitiesAreMasked])
 	{
-		m_EntitiesIsLoaded[(EntitiesModType * 2) + (int)EntitesAreMasked] = true;
+		m_aEntitiesIsLoaded[(EntitiesModType * 2) + (int)EntitiesAreMasked] = true;
 
 		// any mod that does not mask, will get all layers unmasked
-		bool WasUnknwon = !EntitesAreMasked;
+		bool WasUnknown = !EntitiesAreMasked;
 
 		char aPath[64];
-		str_format(aPath, sizeof(aPath), "%s/%s.png", m_aEntitiesPath, gs_aModEntitiesNames[EntitiesModType]);
+		str_format(aPath, sizeof(aPath), "%s/%s.png", m_aEntitiesPath, gs_apModEntitiesNames[EntitiesModType]);
 
-		bool GameTypeHasFrontLayer = HasFrontLayer(EntitiesModType) || WasUnknwon;
-		bool GameTypeHasSpeedupLayer = HasSpeedupLayer(EntitiesModType) || WasUnknwon;
-		bool GameTypeHasSwitchLayer = HasSwitchLayer(EntitiesModType) || WasUnknwon;
-		bool GameTypeHasTeleLayer = HasTeleLayer(EntitiesModType) || WasUnknwon;
-		bool GameTypeHasTuneLayer = HasTuneLayer(EntitiesModType) || WasUnknwon;
+		bool GameTypeHasFrontLayer = HasFrontLayer(EntitiesModType) || WasUnknown;
+		bool GameTypeHasSpeedupLayer = HasSpeedupLayer(EntitiesModType) || WasUnknown;
+		bool GameTypeHasSwitchLayer = HasSwitchLayer(EntitiesModType) || WasUnknown;
+		bool GameTypeHasTeleLayer = HasTeleLayer(EntitiesModType) || WasUnknown;
+		bool GameTypeHasTuneLayer = HasTuneLayer(EntitiesModType) || WasUnknown;
 
 		int TextureLoadFlag = 0;
 		if(Graphics()->IsTileBufferingEnabled())
@@ -214,7 +223,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 			if(!ImagePNGLoaded && TryDefault)
 			{
 				// try default
-				str_format(aPath, sizeof(aPath), "editor/entities_clear/%s.png", gs_aModEntitiesNames[EntitiesModType]);
+				str_format(aPath, sizeof(aPath), "editor/entities_clear/%s.png", gs_apModEntitiesNames[EntitiesModType]);
 				if(Graphics()->LoadPNG(&ImgInfo, aPath, IStorage::TYPE_ALL))
 				{
 					ImagePNGLoaded = true;
@@ -252,7 +261,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 				else if(n == MAP_IMAGE_ENTITY_LAYER_TYPE_TUNE && !GameTypeHasTuneLayer)
 					BuildThisLayer = false;
 
-				dbg_assert(!m_EntitiesTextures[(EntitiesModType * 2) + (int)EntitesAreMasked][n].IsValid(), "entities texture already loaded when it should not be");
+				dbg_assert(!m_aaEntitiesTextures[(EntitiesModType * 2) + (int)EntitiesAreMasked][n].IsValid(), "entities texture already loaded when it should not be");
 
 				if(BuildThisLayer)
 				{
@@ -263,7 +272,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 					{
 						bool ValidTile = i != 0;
 						int TileIndex = i;
-						if(EntitesAreMasked)
+						if(EntitiesAreMasked)
 						{
 							if(EntitiesModType == MAP_IMAGE_MOD_TYPE_DDNET || EntitiesModType == MAP_IMAGE_MOD_TYPE_DDRACE)
 							{
@@ -298,10 +307,6 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 							{
 								ValidTile = false;
 							}
-							/*else if((EntitiesModType == MAP_IMAGE_MOD_TYPE_RACE_BLOCKWORLD) && ...)
-							{
-								ValidTile = false;
-							}*/
 						}
 
 						if(EntitiesModType == MAP_IMAGE_MOD_TYPE_DDNET || EntitiesModType == MAP_IMAGE_MOD_TYPE_DDRACE)
@@ -321,7 +326,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 						}
 					}
 
-					m_EntitiesTextures[(EntitiesModType * 2) + (int)EntitesAreMasked][n] = Graphics()->LoadTextureRaw(ImgInfo.m_Width, ImgInfo.m_Height, ImgInfo.m_Format, pBuildImgData, ImgInfo.m_Format, TextureLoadFlag, aPath);
+					m_aaEntitiesTextures[(EntitiesModType * 2) + (int)EntitiesAreMasked][n] = Graphics()->LoadTextureRaw(ImgInfo.m_Width, ImgInfo.m_Height, ImgInfo.m_Format, pBuildImgData, ImgInfo.m_Format, TextureLoadFlag, aPath);
 				}
 				else
 				{
@@ -332,7 +337,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 
 						m_TransparentTexture = Graphics()->LoadTextureRaw(ImgInfo.m_Width, ImgInfo.m_Height, ImgInfo.m_Format, pBuildImgData, ImgInfo.m_Format, TextureLoadFlag, aPath);
 					}
-					m_EntitiesTextures[(EntitiesModType * 2) + (int)EntitesAreMasked][n] = m_TransparentTexture;
+					m_aaEntitiesTextures[(EntitiesModType * 2) + (int)EntitiesAreMasked][n] = m_TransparentTexture;
 				}
 			}
 
@@ -342,7 +347,7 @@ IGraphics::CTextureHandle CMapImages::GetEntities(EMapImageEntityLayerType Entit
 		}
 	}
 
-	return m_EntitiesTextures[(EntitiesModType * 2) + (int)EntitesAreMasked][EntityLayerType];
+	return m_aaEntitiesTextures[(EntitiesModType * 2) + (int)EntitiesAreMasked][EntityLayerType];
 }
 
 IGraphics::CTextureHandle CMapImages::GetSpeedupArrow()
@@ -376,7 +381,7 @@ IGraphics::CTextureHandle CMapImages::GetOverlayCenter()
 void CMapImages::ChangeEntitiesPath(const char *pPath)
 {
 	if(str_comp(pPath, "default") == 0)
-		str_copy(m_aEntitiesPath, "editor/entities_clear", sizeof(m_aEntitiesPath));
+		str_copy(m_aEntitiesPath, "editor/entities_clear");
 	else
 	{
 		str_format(m_aEntitiesPath, sizeof(m_aEntitiesPath), "assets/entities/%s", pPath);
@@ -384,16 +389,16 @@ void CMapImages::ChangeEntitiesPath(const char *pPath)
 
 	for(int i = 0; i < MAP_IMAGE_MOD_TYPE_COUNT * 2; ++i)
 	{
-		if(m_EntitiesIsLoaded[i])
+		if(m_aEntitiesIsLoaded[i])
 		{
 			for(int n = 0; n < MAP_IMAGE_ENTITY_LAYER_TYPE_COUNT; ++n)
 			{
-				if(m_EntitiesTextures[i][n].IsValid())
-					Graphics()->UnloadTexture(&(m_EntitiesTextures[i][n]));
-				m_EntitiesTextures[i][n] = IGraphics::CTextureHandle();
+				if(m_aaEntitiesTextures[i][n].IsValid())
+					Graphics()->UnloadTexture(&(m_aaEntitiesTextures[i][n]));
+				m_aaEntitiesTextures[i][n] = IGraphics::CTextureHandle();
 			}
 
-			m_EntitiesIsLoaded[i] = false;
+			m_aEntitiesIsLoaded[i] = false;
 		}
 	}
 }
@@ -450,7 +455,7 @@ void CMapImages::UpdateEntityLayerText(void *pTexBuffer, int ImageColorChannelCo
 	if(MaxNumber == -1)
 		MaxNumber = CurrentNumber * 10 - 1;
 
-	str_format(aBuf, 4, "%d", CurrentNumber);
+	str_format(aBuf, sizeof(aBuf), "%d", CurrentNumber);
 
 	int CurrentNumberSuitableFontSize = TextRender()->AdjustFontSize(aBuf, DigitsCount, TextureSize, MaxWidth);
 	int UniversalSuitableFontSize = CurrentNumberSuitableFontSize * 0.92f; // should be smoothed enough to fit any digits combination
@@ -459,7 +464,7 @@ void CMapImages::UpdateEntityLayerText(void *pTexBuffer, int ImageColorChannelCo
 
 	for(; CurrentNumber <= MaxNumber; ++CurrentNumber)
 	{
-		str_format(aBuf, 4, "%d", CurrentNumber);
+		str_format(aBuf, sizeof(aBuf), "%d", CurrentNumber);
 
 		float x = (CurrentNumber % 16) * 64;
 		float y = (CurrentNumber / 16) * 64;
