@@ -1,12 +1,17 @@
-#include "binds.h"
-#include <engine/engine.h>
+#include <base/system.h>
+
 #include <engine/shared/config.h>
 #include <engine/storage.h>
 #include <engine/textrender.h>
 #include <game/client/gameclient.h>
+#include <game/localization.h>
 
 #include "menus.h"
 
+#include <chrono>
+
+using namespace std::chrono_literals;
+
 typedef std::function<void()> TMenuAssetScanLoadedFunc;
 
 struct SMenuAssetScanUser
@@ -15,6 +20,18 @@ struct SMenuAssetScanUser
 	TMenuAssetScanLoadedFunc m_LoadedFunc;
 };
 
+// IDs of the tabs in the Assets menu
+enum
+{
+	ASSETS_TAB_ENTITIES = 0,
+	ASSETS_TAB_GAME = 1,
+	ASSETS_TAB_EMOTICONS = 2,
+	ASSETS_TAB_PARTICLES = 3,
+	ASSETS_TAB_HUD = 4,
+	ASSETS_TAB_EXTRAS = 5,
+	NUMBER_OF_ASSETS_TABS = 6,
+};
+
 void CMenus::LoadEntities(SCustomEntities *pEntitiesItem, void *pUser)
 {
 	auto *pRealUser = (SMenuAssetScanUser *)pUser;
@@ -26,7 +43,7 @@ void CMenus::LoadEntities(SCustomEntities *pEntitiesItem, void *pUser)
 	{
 		for(int i = 0; i < MAP_IMAGE_MOD_TYPE_COUNT; ++i)
 		{
-			str_format(aBuff, sizeof(aBuff), "editor/entities_clear/%s.png", gs_aModEntitiesNames[i]);
+			str_format(aBuff, sizeof(aBuff), "editor/entities_clear/%s.png", gs_apModEntitiesNames[i]);
 			CImageInfo ImgInfo;
 			if(pThis->Graphics()->LoadPNG(&ImgInfo, aBuff, IStorage::TYPE_ALL))
 			{
@@ -42,7 +59,7 @@ void CMenus::LoadEntities(SCustomEntities *pEntitiesItem, void *pUser)
 	{
 		for(int i = 0; i < MAP_IMAGE_MOD_TYPE_COUNT; ++i)
 		{
-			str_format(aBuff, sizeof(aBuff), "assets/entities/%s/%s.png", pEntitiesItem->m_aName, gs_aModEntitiesNames[i]);
+			str_format(aBuff, sizeof(aBuff), "assets/entities/%s/%s.png", pEntitiesItem->m_aName, gs_apModEntitiesNames[i]);
 			CImageInfo ImgInfo;
 			if(pThis->Graphics()->LoadPNG(&ImgInfo, aBuff, IStorage::TYPE_ALL))
 			{
@@ -82,9 +99,9 @@ int CMenus::EntitiesScan(const char *pName, int IsDir, int DirType, void *pUser)
 			return 0;
 
 		SCustomEntities EntitiesItem;
-		str_copy(EntitiesItem.m_aName, pName, sizeof(EntitiesItem.m_aName));
+		str_copy(EntitiesItem.m_aName, pName);
 		CMenus::LoadEntities(&EntitiesItem, pUser);
-		pThis->m_EntitiesList.add(EntitiesItem);
+		pThis->m_vEntitiesList.push_back(EntitiesItem);
 	}
 	else
 	{
@@ -97,9 +114,9 @@ int CMenus::EntitiesScan(const char *pName, int IsDir, int DirType, void *pUser)
 				return 0;
 
 			SCustomEntities EntitiesItem;
-			str_copy(EntitiesItem.m_aName, aName, sizeof(EntitiesItem.m_aName));
+			str_copy(EntitiesItem.m_aName, aName);
 			CMenus::LoadEntities(&EntitiesItem, pUser);
-			pThis->m_EntitiesList.add(EntitiesItem);
+			pThis->m_vEntitiesList.push_back(EntitiesItem);
 		}
 	}
 
@@ -145,7 +162,7 @@ static void LoadAsset(TName *pAssetItem, const char *pAssetName, IGraphics *pGra
 }
 
 template<typename TName>
-static int AssetScan(const char *pName, int IsDir, int DirType, sorted_array<TName> &AssetList, const char *pAssetName, IGraphics *pGraphics, void *pUser)
+static int AssetScan(const char *pName, int IsDir, int DirType, std::vector<TName> &vAssetList, const char *pAssetName, IGraphics *pGraphics, void *pUser)
 {
 	auto *pRealUser = (SMenuAssetScanUser *)pUser;
 	if(IsDir)
@@ -158,9 +175,9 @@ static int AssetScan(const char *pName, int IsDir, int DirType, sorted_array<TNa
 			return 0;
 
 		TName AssetItem;
-		str_copy(AssetItem.m_aName, pName, sizeof(AssetItem.m_aName));
+		str_copy(AssetItem.m_aName, pName);
 		LoadAsset(&AssetItem, pAssetName, pGraphics, pUser);
-		AssetList.add(AssetItem);
+		vAssetList.push_back(AssetItem);
 	}
 	else
 	{
@@ -173,9 +190,9 @@ static int AssetScan(const char *pName, int IsDir, int DirType, sorted_array<TNa
 				return 0;
 
 			TName AssetItem;
-			str_copy(AssetItem.m_aName, aName, sizeof(AssetItem.m_aName));
+			str_copy(AssetItem.m_aName, aName);
 			LoadAsset(&AssetItem, pAssetName, pGraphics, pUser);
-			AssetList.add(AssetItem);
+			vAssetList.push_back(AssetItem);
 		}
 	}
 
@@ -189,7 +206,7 @@ int CMenus::GameScan(const char *pName, int IsDir, int DirType, void *pUser)
 	auto *pRealUser = (SMenuAssetScanUser *)pUser;
 	auto *pThis = (CMenus *)pRealUser->m_pUser;
 	IGraphics *pGraphics = pThis->Graphics();
-	return AssetScan(pName, IsDir, DirType, pThis->m_GameList, "game", pGraphics, pUser);
+	return AssetScan(pName, IsDir, DirType, pThis->m_vGameList, "game", pGraphics, pUser);
 }
 
 int CMenus::EmoticonsScan(const char *pName, int IsDir, int DirType, void *pUser)
@@ -197,7 +214,7 @@ int CMenus::EmoticonsScan(const char *pName, int IsDir, int DirType, void *pUser
 	auto *pRealUser = (SMenuAssetScanUser *)pUser;
 	auto *pThis = (CMenus *)pRealUser->m_pUser;
 	IGraphics *pGraphics = pThis->Graphics();
-	return AssetScan(pName, IsDir, DirType, pThis->m_EmoticonList, "emoticons", pGraphics, pUser);
+	return AssetScan(pName, IsDir, DirType, pThis->m_vEmoticonList, "emoticons", pGraphics, pUser);
 }
 
 int CMenus::ParticlesScan(const char *pName, int IsDir, int DirType, void *pUser)
@@ -205,183 +222,235 @@ int CMenus::ParticlesScan(const char *pName, int IsDir, int DirType, void *pUser
 	auto *pRealUser = (SMenuAssetScanUser *)pUser;
 	auto *pThis = (CMenus *)pRealUser->m_pUser;
 	IGraphics *pGraphics = pThis->Graphics();
-	return AssetScan(pName, IsDir, DirType, pThis->m_ParticlesList, "particles", pGraphics, pUser);
+	return AssetScan(pName, IsDir, DirType, pThis->m_vParticlesList, "particles", pGraphics, pUser);
 }
 
-static sorted_array<const CMenus::SCustomEntities *> s_SearchEntitiesList;
-static sorted_array<const CMenus::SCustomGame *> s_SearchGamesList;
-static sorted_array<const CMenus::SCustomEmoticon *> s_SearchEmoticonsList;
-static sorted_array<const CMenus::SCustomParticle *> s_SearchParticlesList;
+int CMenus::HudScan(const char *pName, int IsDir, int DirType, void *pUser)
+{
+	auto *pRealUser = (SMenuAssetScanUser *)pUser;
+	auto *pThis = (CMenus *)pRealUser->m_pUser;
+	IGraphics *pGraphics = pThis->Graphics();
+	return AssetScan(pName, IsDir, DirType, pThis->m_vHudList, "hud", pGraphics, pUser);
+}
 
-static bool s_InitCustomList[4] = {
+int CMenus::ExtrasScan(const char *pName, int IsDir, int DirType, void *pUser)
+{
+	auto *pRealUser = (SMenuAssetScanUser *)pUser;
+	auto *pThis = (CMenus *)pRealUser->m_pUser;
+	IGraphics *pGraphics = pThis->Graphics();
+	return AssetScan(pName, IsDir, DirType, pThis->m_vExtrasList, "extras", pGraphics, pUser);
+}
+
+static std::vector<const CMenus::SCustomEntities *> gs_vpSearchEntitiesList;
+static std::vector<const CMenus::SCustomGame *> gs_vpSearchGamesList;
+static std::vector<const CMenus::SCustomEmoticon *> gs_vpSearchEmoticonsList;
+static std::vector<const CMenus::SCustomParticle *> gs_vpSearchParticlesList;
+static std::vector<const CMenus::SCustomHud *> gs_vpSearchHudList;
+static std::vector<const CMenus::SCustomExtras *> gs_vpSearchExtrasList;
+
+static bool gs_aInitCustomList[NUMBER_OF_ASSETS_TABS] = {
 	true,
 };
 
-static int s_CustomListSize[4] = {
+static size_t gs_aCustomListSize[NUMBER_OF_ASSETS_TABS] = {
 	0,
 };
 
-static char s_aFilterString[4][50];
+static char s_aFilterString[NUMBER_OF_ASSETS_TABS][50];
 
-static int s_CurCustomTab = 0;
+static int s_CurCustomTab = ASSETS_TAB_ENTITIES;
 
-static const CMenus::SCustomItem *GetCustomItem(int CurTab, int Index)
+static const CMenus::SCustomItem *GetCustomItem(int CurTab, size_t Index)
 {
-	if(CurTab == 0)
-		return s_SearchEntitiesList[Index];
-	else if(CurTab == 1)
-		return s_SearchGamesList[Index];
-	else if(CurTab == 2)
-		return s_SearchEmoticonsList[Index];
-	else if(CurTab == 3)
-		return s_SearchParticlesList[Index];
+	if(CurTab == ASSETS_TAB_ENTITIES)
+		return gs_vpSearchEntitiesList[Index];
+	else if(CurTab == ASSETS_TAB_GAME)
+		return gs_vpSearchGamesList[Index];
+	else if(CurTab == ASSETS_TAB_EMOTICONS)
+		return gs_vpSearchEmoticonsList[Index];
+	else if(CurTab == ASSETS_TAB_PARTICLES)
+		return gs_vpSearchParticlesList[Index];
+	else if(CurTab == ASSETS_TAB_HUD)
+		return gs_vpSearchHudList[Index];
+	else if(CurTab == ASSETS_TAB_EXTRAS)
+		return gs_vpSearchExtrasList[Index];
 
 	return NULL;
 }
 
 template<typename TName>
-void ClearAssetList(sorted_array<TName> &List, IGraphics *pGraphics)
+void ClearAssetList(std::vector<TName> &vList, IGraphics *pGraphics)
 {
-	for(int i = 0; i < List.size(); ++i)
+	for(size_t i = 0; i < vList.size(); ++i)
 	{
-		if(List[i].m_RenderTexture.IsValid())
-			pGraphics->UnloadTexture(&(List[i].m_RenderTexture));
-		List[i].m_RenderTexture = IGraphics::CTextureHandle();
+		if(vList[i].m_RenderTexture.IsValid())
+			pGraphics->UnloadTexture(&(vList[i].m_RenderTexture));
+		vList[i].m_RenderTexture = IGraphics::CTextureHandle();
 	}
-	List.clear();
+	vList.clear();
 }
 
 void CMenus::ClearCustomItems(int CurTab)
 {
-	if(CurTab == 0)
+	if(CurTab == ASSETS_TAB_ENTITIES)
 	{
-		for(int i = 0; i < m_EntitiesList.size(); ++i)
+		for(auto &Entity : m_vEntitiesList)
 		{
-			for(auto &Image : m_EntitiesList[i].m_aImages)
+			for(auto &Image : Entity.m_aImages)
 			{
 				if(Image.m_Texture.IsValid())
 					Graphics()->UnloadTexture(&Image.m_Texture);
 				Image.m_Texture = IGraphics::CTextureHandle();
 			}
 		}
-		m_EntitiesList.clear();
+		m_vEntitiesList.clear();
 
 		// reload current entities
 		m_pClient->m_MapImages.ChangeEntitiesPath(g_Config.m_ClAssetsEntites);
 	}
-	else if(CurTab == 1)
+	else if(CurTab == ASSETS_TAB_GAME)
 	{
-		ClearAssetList(m_GameList, Graphics());
+		ClearAssetList(m_vGameList, Graphics());
 
 		// reload current game skin
 		GameClient()->LoadGameSkin(g_Config.m_ClAssetGame);
 	}
-	else if(CurTab == 2)
+	else if(CurTab == ASSETS_TAB_EMOTICONS)
 	{
-		ClearAssetList(m_EmoticonList, Graphics());
+		ClearAssetList(m_vEmoticonList, Graphics());
 
 		// reload current emoticons skin
 		GameClient()->LoadEmoticonsSkin(g_Config.m_ClAssetEmoticons);
 	}
-	else if(CurTab == 3)
+	else if(CurTab == ASSETS_TAB_PARTICLES)
 	{
-		ClearAssetList(m_ParticlesList, Graphics());
+		ClearAssetList(m_vParticlesList, Graphics());
 
 		// reload current particles skin
 		GameClient()->LoadParticlesSkin(g_Config.m_ClAssetParticles);
 	}
-	s_InitCustomList[CurTab] = true;
+	else if(CurTab == ASSETS_TAB_HUD)
+	{
+		ClearAssetList(m_vHudList, Graphics());
+
+		// reload current hud skin
+		GameClient()->LoadHudSkin(g_Config.m_ClAssetHud);
+	}
+	else if(CurTab == ASSETS_TAB_EXTRAS)
+	{
+		ClearAssetList(m_vExtrasList, Graphics());
+
+		// reload current DDNet particles skin
+		GameClient()->LoadExtrasSkin(g_Config.m_ClAssetExtras);
+	}
+	gs_aInitCustomList[CurTab] = true;
 }
 
 template<typename TName, typename TCaller>
-void InitAssetList(sorted_array<TName> &AssetList, const char *pAssetPath, const char *pAssetName, FS_LISTDIR_CALLBACK pfnCallback, IGraphics *pGraphics, IStorage *pStorage, TCaller Caller)
+void InitAssetList(std::vector<TName> &vAssetList, const char *pAssetPath, const char *pAssetName, FS_LISTDIR_CALLBACK pfnCallback, IGraphics *pGraphics, IStorage *pStorage, TCaller Caller)
 {
-	if(AssetList.size() == 0)
+	if(vAssetList.empty())
 	{
 		TName AssetItem;
-		str_copy(AssetItem.m_aName, "default", sizeof(AssetItem.m_aName));
+		str_copy(AssetItem.m_aName, "default");
 		LoadAsset(&AssetItem, pAssetName, pGraphics, Caller);
-		AssetList.add(AssetItem);
+		vAssetList.push_back(AssetItem);
 
 		// load assets
 		pStorage->ListDirectory(IStorage::TYPE_ALL, pAssetPath, pfnCallback, Caller);
+		std::sort(vAssetList.begin(), vAssetList.end());
 	}
-	if(AssetList.size() != s_CustomListSize[s_CurCustomTab])
-		s_InitCustomList[s_CurCustomTab] = true;
+	if(vAssetList.size() != gs_aCustomListSize[s_CurCustomTab])
+		gs_aInitCustomList[s_CurCustomTab] = true;
 }
 
 template<typename TName>
-int InitSearchList(sorted_array<const TName *> &SearchList, sorted_array<TName> &AssetList)
+int InitSearchList(std::vector<const TName *> &vpSearchList, std::vector<TName> &vAssetList)
 {
-	SearchList.clear();
-	int ListSize = AssetList.size();
+	vpSearchList.clear();
+	int ListSize = vAssetList.size();
 	for(int i = 0; i < ListSize; ++i)
 	{
-		const TName *s = &AssetList[i];
+		const TName *pAsset = &vAssetList[i];
 
 		// filter quick search
-		if(s_aFilterString[s_CurCustomTab][0] != '\0' && !str_utf8_find_nocase(s->m_aName, s_aFilterString[s_CurCustomTab]))
+		if(s_aFilterString[s_CurCustomTab][0] != '\0' && !str_utf8_find_nocase(pAsset->m_aName, s_aFilterString[s_CurCustomTab]))
 			continue;
 
-		SearchList.add_unsorted(s);
+		vpSearchList.push_back(pAsset);
 	}
-	return AssetList.size();
+	return vAssetList.size();
 }
 
 void CMenus::RenderSettingsCustom(CUIRect MainView)
 {
-	CUIRect Label, CustomList, QuickSearch, QuickSearchClearButton, DirectoryButton, Page1Tab, Page2Tab, Page3Tab, Page4Tab, ReloadButton;
+	CUIRect TabBar, CustomList, QuickSearch, QuickSearchClearButton, DirectoryButton, Page1Tab, Page2Tab, Page3Tab, Page4Tab, Page5Tab, Page6Tab, ReloadButton;
 
-	MainView.HSplitTop(20, &Label, &MainView);
-	float TabsW = Label.w;
-	Label.VSplitLeft(TabsW / 4, &Page1Tab, &Page2Tab);
-	Page2Tab.VSplitLeft(TabsW / 4, &Page2Tab, &Page3Tab);
-	Page3Tab.VSplitLeft(TabsW / 4, &Page3Tab, &Page4Tab);
+	MainView.HSplitTop(20, &TabBar, &MainView);
+	float TabsW = TabBar.w;
+	TabBar.VSplitLeft(TabsW / NUMBER_OF_ASSETS_TABS, &Page1Tab, &Page2Tab);
+	Page2Tab.VSplitLeft(TabsW / NUMBER_OF_ASSETS_TABS, &Page2Tab, &Page3Tab);
+	Page3Tab.VSplitLeft(TabsW / NUMBER_OF_ASSETS_TABS, &Page3Tab, &Page4Tab);
+	Page4Tab.VSplitLeft(TabsW / NUMBER_OF_ASSETS_TABS, &Page4Tab, &Page5Tab);
+	Page5Tab.VSplitLeft(TabsW / NUMBER_OF_ASSETS_TABS, &Page5Tab, &Page6Tab);
 
-	static int s_aPageTabs[4] = {};
+	static CButtonContainer s_aPageTabs[NUMBER_OF_ASSETS_TABS] = {};
 
-	if(DoButton_MenuTab((void *)&s_aPageTabs[0], Localize("Entities"), s_CurCustomTab == 0, &Page1Tab, 5, NULL, NULL, NULL, NULL, 4))
-		s_CurCustomTab = 0;
-	if(DoButton_MenuTab((void *)&s_aPageTabs[1], Localize("Game"), s_CurCustomTab == 1, &Page2Tab, 0, NULL, NULL, NULL, NULL, 4))
-		s_CurCustomTab = 1;
-	if(DoButton_MenuTab((void *)&s_aPageTabs[2], Localize("Emoticons"), s_CurCustomTab == 2, &Page3Tab, 0, NULL, NULL, NULL, NULL, 4))
-		s_CurCustomTab = 2;
-	if(DoButton_MenuTab((void *)&s_aPageTabs[3], Localize("Particles"), s_CurCustomTab == 3, &Page4Tab, 10, NULL, NULL, NULL, NULL, 4))
-		s_CurCustomTab = 3;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_ENTITIES], Localize("Entities"), s_CurCustomTab == ASSETS_TAB_ENTITIES, &Page1Tab, IGraphics::CORNER_L, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_ENTITIES;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_GAME], Localize("Game"), s_CurCustomTab == ASSETS_TAB_GAME, &Page2Tab, 0, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_GAME;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_EMOTICONS], Localize("Emoticons"), s_CurCustomTab == ASSETS_TAB_EMOTICONS, &Page3Tab, 0, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_EMOTICONS;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_PARTICLES], Localize("Particles"), s_CurCustomTab == ASSETS_TAB_PARTICLES, &Page4Tab, 0, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_PARTICLES;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_HUD], Localize("HUD"), s_CurCustomTab == ASSETS_TAB_HUD, &Page5Tab, 0, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_HUD;
+	if(DoButton_MenuTab((CButtonContainer *)&s_aPageTabs[ASSETS_TAB_EXTRAS], Localize("Extras"), s_CurCustomTab == ASSETS_TAB_EXTRAS, &Page6Tab, IGraphics::CORNER_R, NULL, NULL, NULL, NULL, 4))
+		s_CurCustomTab = ASSETS_TAB_EXTRAS;
 
-	int64_t LoadStartTime = time_get_microseconds();
+	auto LoadStartTime = time_get_nanoseconds();
 	SMenuAssetScanUser User;
 	User.m_pUser = this;
 	User.m_LoadedFunc = [&]() {
-		if(time_get_microseconds() - LoadStartTime > 500000)
-			RenderLoading(false, false);
+		if(time_get_nanoseconds() - LoadStartTime > 500ms)
+			RenderLoading(Localize("Loading assets"), "", 0, false);
 	};
-	if(s_CurCustomTab == 0)
+	if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
 	{
-		if(m_EntitiesList.size() == 0)
+		if(m_vEntitiesList.empty())
 		{
 			SCustomEntities EntitiesItem;
-			str_copy(EntitiesItem.m_aName, "default", sizeof(EntitiesItem.m_aName));
+			str_copy(EntitiesItem.m_aName, "default");
 			LoadEntities(&EntitiesItem, &User);
-			m_EntitiesList.add(EntitiesItem);
+			m_vEntitiesList.push_back(EntitiesItem);
 
 			// load entities
 			Storage()->ListDirectory(IStorage::TYPE_ALL, "assets/entities", EntitiesScan, &User);
+			std::sort(m_vEntitiesList.begin(), m_vEntitiesList.end());
 		}
-		if(m_EntitiesList.size() != s_CustomListSize[s_CurCustomTab])
-			s_InitCustomList[s_CurCustomTab] = true;
+		if(m_vEntitiesList.size() != gs_aCustomListSize[s_CurCustomTab])
+			gs_aInitCustomList[s_CurCustomTab] = true;
 	}
-	else if(s_CurCustomTab == 1)
+	else if(s_CurCustomTab == ASSETS_TAB_GAME)
 	{
-		InitAssetList(m_GameList, "assets/game", "game", GameScan, Graphics(), Storage(), &User);
+		InitAssetList(m_vGameList, "assets/game", "game", GameScan, Graphics(), Storage(), &User);
 	}
-	else if(s_CurCustomTab == 2)
+	else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
 	{
-		InitAssetList(m_EmoticonList, "assets/emoticons", "emoticons", EmoticonsScan, Graphics(), Storage(), &User);
+		InitAssetList(m_vEmoticonList, "assets/emoticons", "emoticons", EmoticonsScan, Graphics(), Storage(), &User);
 	}
-	else if(s_CurCustomTab == 3)
+	else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
 	{
-		InitAssetList(m_ParticlesList, "assets/particles", "particles", ParticlesScan, Graphics(), Storage(), &User);
+		InitAssetList(m_vParticlesList, "assets/particles", "particles", ParticlesScan, Graphics(), Storage(), &User);
+	}
+	else if(s_CurCustomTab == ASSETS_TAB_HUD)
+	{
+		InitAssetList(m_vHudList, "assets/hud", "hud", HudScan, Graphics(), Storage(), &User);
+	}
+	else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+	{
+		InitAssetList(m_vExtrasList, "assets/extras", "extras", ExtrasScan, Graphics(), Storage(), &User);
 	}
 
 	MainView.HSplitTop(10.0f, 0, &MainView);
@@ -389,38 +458,46 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 	// skin selector
 	MainView.HSplitTop(MainView.h - 10.0f - ms_ButtonHeight, &CustomList, &MainView);
 	static float s_ScrollValue = 0.0f;
-	if(s_InitCustomList[s_CurCustomTab])
+	if(gs_aInitCustomList[s_CurCustomTab])
 	{
 		int ListSize = 0;
-		if(s_CurCustomTab == 0)
+		if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
 		{
-			s_SearchEntitiesList.clear();
-			ListSize = m_EntitiesList.size();
+			gs_vpSearchEntitiesList.clear();
+			ListSize = m_vEntitiesList.size();
 			for(int i = 0; i < ListSize; ++i)
 			{
-				const SCustomEntities *s = &m_EntitiesList[i];
+				const SCustomEntities *pEntity = &m_vEntitiesList[i];
 
 				// filter quick search
-				if(s_aFilterString[s_CurCustomTab][0] != '\0' && !str_utf8_find_nocase(s->m_aName, s_aFilterString[s_CurCustomTab]))
+				if(s_aFilterString[s_CurCustomTab][0] != '\0' && !str_utf8_find_nocase(pEntity->m_aName, s_aFilterString[s_CurCustomTab]))
 					continue;
 
-				s_SearchEntitiesList.add_unsorted(s);
+				gs_vpSearchEntitiesList.push_back(pEntity);
 			}
 		}
-		else if(s_CurCustomTab == 1)
+		else if(s_CurCustomTab == ASSETS_TAB_GAME)
 		{
-			ListSize = InitSearchList(s_SearchGamesList, m_GameList);
+			ListSize = InitSearchList(gs_vpSearchGamesList, m_vGameList);
 		}
-		else if(s_CurCustomTab == 2)
+		else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
 		{
-			ListSize = InitSearchList(s_SearchEmoticonsList, m_EmoticonList);
+			ListSize = InitSearchList(gs_vpSearchEmoticonsList, m_vEmoticonList);
 		}
-		else if(s_CurCustomTab == 3)
+		else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
 		{
-			ListSize = InitSearchList(s_SearchParticlesList, m_ParticlesList);
+			ListSize = InitSearchList(gs_vpSearchParticlesList, m_vParticlesList);
 		}
-		s_InitCustomList[s_CurCustomTab] = false;
-		s_CustomListSize[s_CurCustomTab] = ListSize;
+		else if(s_CurCustomTab == ASSETS_TAB_HUD)
+		{
+			ListSize = InitSearchList(gs_vpSearchHudList, m_vHudList);
+		}
+		else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+		{
+			ListSize = InitSearchList(gs_vpSearchExtrasList, m_vExtrasList);
+		}
+		gs_aInitCustomList[s_CurCustomTab] = false;
+		gs_aCustomListSize[s_CurCustomTab] = ListSize;
 	}
 
 	int OldSelected = -1;
@@ -428,55 +505,73 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 	float TextureWidth = 150;
 	float TextureHeight = 150;
 
-	int SearchListSize = 0;
+	size_t SearchListSize = 0;
 
-	if(s_CurCustomTab == 0)
+	if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
 	{
-		SearchListSize = s_SearchEntitiesList.size();
+		SearchListSize = gs_vpSearchEntitiesList.size();
 	}
-	else if(s_CurCustomTab == 1)
+	else if(s_CurCustomTab == ASSETS_TAB_GAME)
 	{
-		SearchListSize = s_SearchGamesList.size();
+		SearchListSize = gs_vpSearchGamesList.size();
 		TextureHeight = 75;
 	}
-	else if(s_CurCustomTab == 2)
+	else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
 	{
-		SearchListSize = s_SearchEmoticonsList.size();
+		SearchListSize = gs_vpSearchEmoticonsList.size();
 	}
-	else if(s_CurCustomTab == 3)
+	else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
 	{
-		SearchListSize = s_SearchParticlesList.size();
+		SearchListSize = gs_vpSearchParticlesList.size();
+	}
+	else if(s_CurCustomTab == ASSETS_TAB_HUD)
+	{
+		SearchListSize = gs_vpSearchHudList.size();
+	}
+	else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+	{
+		SearchListSize = gs_vpSearchExtrasList.size();
 	}
 
-	UiDoListboxStart(&s_InitCustomList[s_CurCustomTab], &CustomList, TextureHeight + 15.0f + 10.0f + Margin, "", "", SearchListSize, CustomList.w / (Margin + TextureWidth), OldSelected, s_ScrollValue, true);
-	for(int i = 0; i < SearchListSize; ++i)
+	UiDoListboxStart(&gs_aInitCustomList[s_CurCustomTab], &CustomList, TextureHeight + 15.0f + 10.0f + Margin, "", "", SearchListSize, CustomList.w / (Margin + TextureWidth), OldSelected, s_ScrollValue, true);
+	for(size_t i = 0; i < SearchListSize; ++i)
 	{
-		const SCustomItem *s = GetCustomItem(s_CurCustomTab, i);
-		if(s == NULL)
+		const SCustomItem *pItem = GetCustomItem(s_CurCustomTab, i);
+		if(pItem == NULL)
 			continue;
 
-		if(s_CurCustomTab == 0)
+		if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
 		{
-			if(str_comp(s->m_aName, g_Config.m_ClAssetsEntites) == 0)
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetsEntites) == 0)
 				OldSelected = i;
 		}
-		else if(s_CurCustomTab == 1)
+		else if(s_CurCustomTab == ASSETS_TAB_GAME)
 		{
-			if(str_comp(s->m_aName, g_Config.m_ClAssetGame) == 0)
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetGame) == 0)
 				OldSelected = i;
 		}
-		else if(s_CurCustomTab == 2)
+		else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
 		{
-			if(str_comp(s->m_aName, g_Config.m_ClAssetEmoticons) == 0)
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetEmoticons) == 0)
 				OldSelected = i;
 		}
-		else if(s_CurCustomTab == 3)
+		else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
 		{
-			if(str_comp(s->m_aName, g_Config.m_ClAssetParticles) == 0)
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetParticles) == 0)
+				OldSelected = i;
+		}
+		else if(s_CurCustomTab == ASSETS_TAB_HUD)
+		{
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetHud) == 0)
+				OldSelected = i;
+		}
+		else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+		{
+			if(str_comp(pItem->m_aName, g_Config.m_ClAssetExtras) == 0)
 				OldSelected = i;
 		}
 
-		CListboxItem Item = UiDoListboxNextItem(s, OldSelected == i);
+		CListboxItem Item = UiDoListboxNextItem(pItem, OldSelected >= 0 && (size_t)OldSelected == i);
 		CUIRect ItemRect = Item.m_Rect;
 		ItemRect.Margin(Margin / 2, &ItemRect);
 		if(Item.m_Visible)
@@ -484,11 +579,11 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 			CUIRect TextureRect;
 			ItemRect.HSplitTop(15, &ItemRect, &TextureRect);
 			TextureRect.HSplitTop(10, NULL, &TextureRect);
-			UI()->DoLabelScaled(&ItemRect, s->m_aName, ItemRect.h - 2, TEXTALIGN_CENTER);
-			if(s->m_RenderTexture.IsValid())
+			UI()->DoLabel(&ItemRect, pItem->m_aName, ItemRect.h - 2, TEXTALIGN_CENTER);
+			if(pItem->m_RenderTexture.IsValid())
 			{
 				Graphics()->WrapClamp();
-				Graphics()->TextureSet(s->m_RenderTexture);
+				Graphics()->TextureSet(pItem->m_RenderTexture);
 				Graphics()->QuadsBegin();
 				Graphics()->SetColor(1, 1, 1, 1);
 				IGraphics::CQuadItem QuadItem(TextureRect.x + (TextureRect.w - TextureWidth) / 2, TextureRect.y + (TextureRect.h - TextureHeight) / 2, TextureWidth, TextureHeight);
@@ -504,26 +599,36 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 	{
 		if(GetCustomItem(s_CurCustomTab, NewSelected)->m_aName[0] != '\0')
 		{
-			if(s_CurCustomTab == 0)
+			if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
 			{
-				str_copy(g_Config.m_ClAssetsEntites, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName, sizeof(g_Config.m_ClAssetsEntites));
+				str_copy(g_Config.m_ClAssetsEntites, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
 				m_pClient->m_MapImages.ChangeEntitiesPath(GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
 			}
-			else if(s_CurCustomTab == 1)
+			else if(s_CurCustomTab == ASSETS_TAB_GAME)
 			{
-				str_copy(g_Config.m_ClAssetGame, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName, sizeof(g_Config.m_ClAssetGame));
+				str_copy(g_Config.m_ClAssetGame, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
 				GameClient()->LoadGameSkin(g_Config.m_ClAssetGame);
 			}
-			else if(s_CurCustomTab == 2)
+			else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
 			{
-				str_copy(g_Config.m_ClAssetEmoticons, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName, sizeof(g_Config.m_ClAssetEmoticons));
+				str_copy(g_Config.m_ClAssetEmoticons, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
 				GameClient()->LoadEmoticonsSkin(g_Config.m_ClAssetEmoticons);
 			}
-			else if(s_CurCustomTab == 3)
+			else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
 			{
-				str_copy(g_Config.m_ClAssetParticles, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName, sizeof(g_Config.m_ClAssetParticles));
+				str_copy(g_Config.m_ClAssetParticles, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
 				GameClient()->LoadParticlesSkin(g_Config.m_ClAssetParticles);
 			}
+			else if(s_CurCustomTab == ASSETS_TAB_HUD)
+			{
+				str_copy(g_Config.m_ClAssetHud, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
+				GameClient()->LoadHudSkin(g_Config.m_ClAssetHud);
+			}
+			else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+			{
+				str_copy(g_Config.m_ClAssetExtras, GetCustomItem(s_CurCustomTab, NewSelected)->m_aName);
+				GameClient()->LoadExtrasSkin(g_Config.m_ClAssetExtras);
+			}
 		}
 	}
 
@@ -538,7 +643,7 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 
 		SLabelProperties Props;
 		Props.m_AlignVertically = 0;
-		UI()->DoLabelScaled(&QuickSearch, pSearchLabel, 14.0f, TEXTALIGN_LEFT, Props);
+		UI()->DoLabel(&QuickSearch, pSearchLabel, 14.0f, TEXTALIGN_LEFT, Props);
 		float wSearch = TextRender()->TextWidth(0, 14.0f, pSearchLabel, -1, -1.0f);
 		TextRender()->SetRenderFlags(0);
 		TextRender()->SetCurFont(NULL);
@@ -554,40 +659,44 @@ void CMenus::RenderSettingsCustom(CUIRect MainView)
 			EditProps.m_SelectText = true;
 		}
 		EditProps.m_pEmptyText = Localize("Search");
-		if(UIEx()->DoClearableEditBox(&s_aFilterString[s_CurCustomTab], &s_ClearButton, &QuickSearch, s_aFilterString[s_CurCustomTab], sizeof(s_aFilterString[0]), 14.0f, &s_Offset, false, CUI::CORNER_ALL, EditProps))
-			s_InitCustomList[s_CurCustomTab] = true;
+		if(UI()->DoClearableEditBox(&s_aFilterString[s_CurCustomTab], &s_ClearButton, &QuickSearch, s_aFilterString[s_CurCustomTab], sizeof(s_aFilterString[0]), 14.0f, &s_Offset, false, IGraphics::CORNER_ALL, EditProps))
+			gs_aInitCustomList[s_CurCustomTab] = true;
 	}
 
 	DirectoryButton.HSplitTop(5.0f, 0, &DirectoryButton);
 	DirectoryButton.VSplitRight(175.0f, 0, &DirectoryButton);
 	DirectoryButton.VSplitRight(25.0f, &DirectoryButton, &ReloadButton);
 	DirectoryButton.VSplitRight(10.0f, &DirectoryButton, 0);
-	static int s_AssetsDirID = 0;
+	static CButtonContainer s_AssetsDirID;
 	if(DoButton_Menu(&s_AssetsDirID, Localize("Assets directory"), 0, &DirectoryButton))
 	{
 		char aBuf[IO_MAX_PATH_LENGTH];
 		char aBufFull[IO_MAX_PATH_LENGTH + 7];
-		if(s_CurCustomTab == 0)
-			str_copy(aBufFull, "assets/entities", sizeof(aBufFull));
-		else if(s_CurCustomTab == 1)
-			str_copy(aBufFull, "assets/game", sizeof(aBufFull));
-		else if(s_CurCustomTab == 2)
-			str_copy(aBufFull, "assets/emoticons", sizeof(aBufFull));
-		else if(s_CurCustomTab == 3)
-			str_copy(aBufFull, "assets/particles", sizeof(aBufFull));
+		if(s_CurCustomTab == ASSETS_TAB_ENTITIES)
+			str_copy(aBufFull, "assets/entities");
+		else if(s_CurCustomTab == ASSETS_TAB_GAME)
+			str_copy(aBufFull, "assets/game");
+		else if(s_CurCustomTab == ASSETS_TAB_EMOTICONS)
+			str_copy(aBufFull, "assets/emoticons");
+		else if(s_CurCustomTab == ASSETS_TAB_PARTICLES)
+			str_copy(aBufFull, "assets/particles");
+		else if(s_CurCustomTab == ASSETS_TAB_HUD)
+			str_copy(aBufFull, "assets/hud");
+		else if(s_CurCustomTab == ASSETS_TAB_EXTRAS)
+			str_copy(aBufFull, "assets/extras");
 		Storage()->GetCompletePath(IStorage::TYPE_SAVE, aBufFull, aBuf, sizeof(aBuf));
 		Storage()->CreateFolder("assets", IStorage::TYPE_SAVE);
 		Storage()->CreateFolder(aBufFull, IStorage::TYPE_SAVE);
 		if(!open_file(aBuf))
 		{
-			dbg_msg("menus", "couldn't open file");
+			dbg_msg("menus", "couldn't open file '%s'", aBuf);
 		}
 	}
 
 	TextRender()->SetCurFont(TextRender()->GetFont(TEXT_FONT_ICON_FONT));
 	TextRender()->SetRenderFlags(ETextRenderFlags::TEXT_RENDER_FLAG_ONLY_ADVANCE_WIDTH | ETextRenderFlags::TEXT_RENDER_FLAG_NO_X_BEARING | ETextRenderFlags::TEXT_RENDER_FLAG_NO_Y_BEARING | ETextRenderFlags::TEXT_RENDER_FLAG_NO_PIXEL_ALIGMENT | ETextRenderFlags::TEXT_RENDER_FLAG_NO_OVERSIZE);
-	static int s_AssetsReloadBtnID = 0;
-	if(DoButton_Menu(&s_AssetsReloadBtnID, "\xEF\x80\x9E", 0, &ReloadButton, NULL, 15, 5, 0, vec4(1.0f, 1.0f, 1.0f, 0.75f), vec4(1, 1, 1, 0.5f), 0))
+	static CButtonContainer s_AssetsReloadBtnID;
+	if(DoButton_Menu(&s_AssetsReloadBtnID, "\xEF\x80\x9E", 0, &ReloadButton, nullptr, IGraphics::CORNER_ALL, 5, 0, vec4(1.0f, 1.0f, 1.0f, 0.75f), vec4(1, 1, 1, 0.5f), 0))
 	{
 		ClearCustomItems(s_CurCustomTab);
 	}
@@ -654,3 +763,33 @@ void CMenus::ConchainAssetEmoticons(IConsole::IResult *pResult, void *pUserData,
 
 	pfnCallback(pResult, pCallbackUserData);
 }
+
+void CMenus::ConchainAssetHud(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData)
+{
+	CMenus *pThis = (CMenus *)pUserData;
+	if(pResult->NumArguments() == 1)
+	{
+		const char *pArg = pResult->GetString(0);
+		if(str_comp(pArg, g_Config.m_ClAssetHud) != 0)
+		{
+			pThis->GameClient()->LoadHudSkin(pArg);
+		}
+	}
+
+	pfnCallback(pResult, pCallbackUserData);
+}
+
+void CMenus::ConchainAssetExtras(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData)
+{
+	CMenus *pThis = (CMenus *)pUserData;
+	if(pResult->NumArguments() == 1)
+	{
+		const char *pArg = pResult->GetString(0);
+		if(str_comp(pArg, g_Config.m_ClAssetExtras) != 0)
+		{
+			pThis->GameClient()->LoadExtrasSkin(pArg);
+		}
+	}
+
+	pfnCallback(pResult, pCallbackUserData);
+}
