@@ -26,13 +26,22 @@ class CPlayers : public CComponent
 		const CTeeRenderInfo *pRenderInfo,
 		int ClientID,
 		float Intra = 0.f);
+	void RenderHookCollLine(
+		const CNetObj_Character *pPrevChar,
+		const CNetObj_Character *pPlayerChar,
+		int ClientID,
+		float Intra = 0.f);
+	float GetPlayerTargetAngle(
+		const CNetObj_Character *pPrevChar,
+		const CNetObj_Character *pPlayerChar,
+		int ClientID,
+		float Intra = 0.f);
 	bool IsPlayerInfoAvailable(int ClientID) const;
 
 	int m_WeaponEmoteQuadContainerIndex;
-	int m_WeaponSpriteMuzzleQuadContainerIndex[NUM_WEAPONS];
+	int m_aWeaponSpriteMuzzleQuadContainerIndex[NUM_WEAPONS];
 
 public:
-	vec2 m_CurPredictedPos[MAX_CLIENTS];
 	virtual int Sizeof() const override { return sizeof(*this); }
 	virtual void OnInit() override;
 	virtual void OnRender() override;
