@@ -1,6 +1,5 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
-#include <cmath>
 
 #include <base/math.h>
 #include <base/system.h>
@@ -14,6 +13,7 @@
 #include <game/generated/client_data.h>
 
 #include <game/client/gameclient.h>
+#include <game/localization.h>
 
 #include "skins.h"
 
@@ -29,18 +29,22 @@ static bool IsVanillaSkin(const char *pName)
 
 int CSkins::CGetPngFile::OnCompletion(int State)
 {
-	State = CGetFile::OnCompletion(State);
+	State = CHttpRequest::OnCompletion(State);
 
-	if(State != HTTP_ERROR && State != HTTP_ABORTED && !m_pSkins->LoadSkinPNG(m_Info, m_aDest, m_aDest, m_StorageType))
+	if(State != HTTP_ERROR && State != HTTP_ABORTED && !m_pSkins->LoadSkinPNG(m_Info, Dest(), Dest(), IStorage::TYPE_SAVE))
 	{
 		State = HTTP_ERROR;
 	}
 	return State;
 }
 
-CSkins::CGetPngFile::CGetPngFile(CSkins *pSkins, IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType, CTimeout Timeout, HTTPLOG LogProgress) :
-	CGetFile(pStorage, pUrl, pDest, StorageType, Timeout, LogProgress), m_pSkins(pSkins)
+CSkins::CGetPngFile::CGetPngFile(CSkins *pSkins, const char *pUrl, IStorage *pStorage, const char *pDest) :
+	CHttpRequest(pUrl),
+	m_pSkins(pSkins)
 {
+	WriteToFile(pStorage, pDest, IStorage::TYPE_SAVE);
+	Timeout(CTimeout{0, 0, 0, 0});
+	LogProgress(HTTPLOG::NONE);
 }
 
 struct SSkinScanUser
@@ -58,9 +62,12 @@ int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 		return 0;
 
 	char aNameWithoutPng[128];
-	str_copy(aNameWithoutPng, pName, sizeof(aNameWithoutPng));
+	str_copy(aNameWithoutPng, pName);
 	aNameWithoutPng[str_length(aNameWithoutPng) - 4] = 0;
 
+	if(g_Config.m_ClVanillaSkinsOnly && !IsVanillaSkin(aNameWithoutPng))
+		return 0;
+
 	// Don't add duplicate skins (one from user's config directory, other from
 	// client itself)
 	for(int i = 0; i < pSelf->Num(); i++)
@@ -150,7 +157,6 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	}
 
 	CSkin Skin;
-	Skin.m_IsVanilla = IsVanillaSkin(pName);
 	Skin.m_OriginalSkin.m_Body = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_BODY]);
 	Skin.m_OriginalSkin.m_BodyOutline = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_BODY_OUTLINE]);
 	Skin.m_OriginalSkin.m_Feet = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_FOOT]);
@@ -159,7 +165,7 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	Skin.m_OriginalSkin.m_HandsOutline = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_HAND_OUTLINE]);
 
 	for(int i = 0; i < 6; ++i)
-		Skin.m_OriginalSkin.m_Eyes[i] = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_EYE_NORMAL + i]);
+		Skin.m_OriginalSkin.m_aEyes[i] = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_EYE_NORMAL + i]);
 
 	int FeetGridPixelsWidth = (Info.m_Width / g_pData->m_aSprites[SPRITE_TEE_FOOT].m_pSet->m_Gridx);
 	int FeetGridPixelsHeight = (Info.m_Height / g_pData->m_aSprites[SPRITE_TEE_FOOT].m_pSet->m_Gridy);
@@ -189,7 +195,7 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	int BodyHeight = g_pData->m_aSprites[SPRITE_TEE_BODY].m_H * (Info.m_Height / g_pData->m_aSprites[SPRITE_TEE_BODY].m_pSet->m_Gridy); // body height
 	if(BodyWidth > Info.m_Width || BodyHeight > Info.m_Height)
 		return 0;
-	unsigned char *d = (unsigned char *)Info.m_pData;
+	unsigned char *pData = (unsigned char *)Info.m_pData;
 	const int PixelStep = 4;
 	int Pitch = Info.m_Width * PixelStep;
 
@@ -199,12 +205,12 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 		for(int y = 0; y < BodyHeight; y++)
 			for(int x = 0; x < BodyWidth; x++)
 			{
-				uint8_t AlphaValue = d[y * Pitch + x * PixelStep + 3];
+				uint8_t AlphaValue = pData[y * Pitch + x * PixelStep + 3];
 				if(AlphaValue > 128)
 				{
-					aColors[0] += d[y * Pitch + x * PixelStep + 0];
-					aColors[1] += d[y * Pitch + x * PixelStep + 1];
-					aColors[2] += d[y * Pitch + x * PixelStep + 2];
+					aColors[0] += pData[y * Pitch + x * PixelStep + 0];
+					aColors[1] += pData[y * Pitch + x * PixelStep + 1];
+					aColors[2] += pData[y * Pitch + x * PixelStep + 2];
 				}
 			}
 		if(aColors[0] != 0 && aColors[1] != 0 && aColors[2] != 0)
@@ -213,27 +219,27 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 			Skin.m_BloodColor = ColorRGBA(0, 0, 0, 1);
 	}
 
-	CheckMetrics(Skin.m_Metrics.m_Body, d, Pitch, 0, 0, BodyWidth, BodyHeight);
+	CheckMetrics(Skin.m_Metrics.m_Body, pData, Pitch, 0, 0, BodyWidth, BodyHeight);
 
 	// body outline metrics
-	CheckMetrics(Skin.m_Metrics.m_Body, d, Pitch, BodyOutlineOffsetX, BodyOutlineOffsetY, BodyOutlineWidth, BodyOutlineHeight);
+	CheckMetrics(Skin.m_Metrics.m_Body, pData, Pitch, BodyOutlineOffsetX, BodyOutlineOffsetY, BodyOutlineWidth, BodyOutlineHeight);
 
 	// get feet size
-	CheckMetrics(Skin.m_Metrics.m_Feet, d, Pitch, FeetOffsetX, FeetOffsetY, FeetWidth, FeetHeight);
+	CheckMetrics(Skin.m_Metrics.m_Feet, pData, Pitch, FeetOffsetX, FeetOffsetY, FeetWidth, FeetHeight);
 
 	// get feet outline size
-	CheckMetrics(Skin.m_Metrics.m_Feet, d, Pitch, FeetOutlineOffsetX, FeetOutlineOffsetY, FeetOutlineWidth, FeetOutlineHeight);
+	CheckMetrics(Skin.m_Metrics.m_Feet, pData, Pitch, FeetOutlineOffsetX, FeetOutlineOffsetY, FeetOutlineWidth, FeetOutlineHeight);
 
 	// make the texture gray scale
 	for(int i = 0; i < Info.m_Width * Info.m_Height; i++)
 	{
-		int v = (d[i * PixelStep] + d[i * PixelStep + 1] + d[i * PixelStep + 2]) / 3;
-		d[i * PixelStep] = v;
-		d[i * PixelStep + 1] = v;
-		d[i * PixelStep + 2] = v;
+		int v = (pData[i * PixelStep] + pData[i * PixelStep + 1] + pData[i * PixelStep + 2]) / 3;
+		pData[i * PixelStep] = v;
+		pData[i * PixelStep + 1] = v;
+		pData[i * PixelStep + 2] = v;
 	}
 
-	int Freq[256] = {0};
+	int aFreq[256] = {0};
 	int OrgWeight = 0;
 	int NewWeight = 192;
 
@@ -241,13 +247,13 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	for(int y = 0; y < BodyHeight; y++)
 		for(int x = 0; x < BodyWidth; x++)
 		{
-			if(d[y * Pitch + x * PixelStep + 3] > 128)
-				Freq[d[y * Pitch + x * PixelStep]]++;
+			if(pData[y * Pitch + x * PixelStep + 3] > 128)
+				aFreq[pData[y * Pitch + x * PixelStep]]++;
 		}
 
 	for(int i = 1; i < 256; i++)
 	{
-		if(Freq[OrgWeight] < Freq[i])
+		if(aFreq[OrgWeight] < aFreq[i])
 			OrgWeight = i;
 	}
 
@@ -257,7 +263,7 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	for(int y = 0; y < BodyHeight; y++)
 		for(int x = 0; x < BodyWidth; x++)
 		{
-			int v = d[y * Pitch + x * PixelStep];
+			int v = pData[y * Pitch + x * PixelStep];
 			if(v <= OrgWeight && OrgWeight == 0)
 				v = 0;
 			else if(v <= OrgWeight)
@@ -266,9 +272,9 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 				v = NewWeight;
 			else
 				v = (int)(((v - OrgWeight) / (float)InvOrgWeight) * InvNewWeight + NewWeight);
-			d[y * Pitch + x * PixelStep] = v;
-			d[y * Pitch + x * PixelStep + 1] = v;
-			d[y * Pitch + x * PixelStep + 2] = v;
+			pData[y * Pitch + x * PixelStep] = v;
+			pData[y * Pitch + x * PixelStep + 1] = v;
+			pData[y * Pitch + x * PixelStep + 2] = v;
 		}
 
 	Skin.m_ColorableSkin.m_Body = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_BODY]);
@@ -279,88 +285,87 @@ int CSkins::LoadSkin(const char *pName, CImageInfo &Info)
 	Skin.m_ColorableSkin.m_HandsOutline = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_HAND_OUTLINE]);
 
 	for(int i = 0; i < 6; ++i)
-		Skin.m_ColorableSkin.m_Eyes[i] = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_EYE_NORMAL + i]);
+		Skin.m_ColorableSkin.m_aEyes[i] = Graphics()->LoadSpriteTexture(Info, &g_pData->m_aSprites[SPRITE_TEE_EYE_NORMAL + i]);
 
 	Graphics()->FreePNG(&Info);
 
 	// set skin data
-	str_copy(Skin.m_aName, pName, sizeof(Skin.m_aName));
+	str_copy(Skin.m_aName, pName);
 	if(g_Config.m_Debug)
 	{
 		str_format(aBuf, sizeof(aBuf), "load skin %s", Skin.m_aName);
 		Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
 	}
 
-	m_aSkins.add(Skin);
+	m_vSkins.insert(std::lower_bound(m_vSkins.begin(), m_vSkins.end(), Skin), Skin);
 
 	return 0;
 }
 
 void CSkins::OnInit()
 {
-	m_EventSkinPrefix[0] = '\0';
+	m_aEventSkinPrefix[0] = '\0';
 
 	if(g_Config.m_Events)
 	{
-		time_t rawtime;
-		struct tm *timeinfo;
-		std::time(&rawtime);
-		timeinfo = localtime(&rawtime);
-		if(timeinfo->tm_mon == 11 && timeinfo->tm_mday >= 24 && timeinfo->tm_mday <= 26)
+		time_t RawTime;
+		struct tm *pTimeInfo;
+		std::time(&RawTime);
+		pTimeInfo = localtime(&RawTime);
+		if(pTimeInfo->tm_mon == 11 && pTimeInfo->tm_mday >= 24 && pTimeInfo->tm_mday <= 26)
 		{ // Christmas
-			str_copy(m_EventSkinPrefix, "santa", sizeof(m_EventSkinPrefix));
+			str_copy(m_aEventSkinPrefix, "santa");
 		}
 	}
 
 	// load skins;
 	Refresh([this](int SkinID) {
-		GameClient()->m_Menus.RenderLoading(false);
+		GameClient()->m_Menus.RenderLoading(Localize("Loading DDNet Client"), Localize("Loading skin files"), 0);
 	});
 }
 
 void CSkins::Refresh(TSkinLoadedCBFunc &&SkinLoadedFunc)
 {
-	for(int i = 0; i < m_aSkins.size(); ++i)
+	for(auto &Skin : m_vSkins)
 	{
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_Body);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_BodyOutline);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_Feet);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_FeetOutline);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_Hands);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_OriginalSkin.m_HandsOutline);
-		for(auto &Eye : m_aSkins[i].m_OriginalSkin.m_Eyes)
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_Body);
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_BodyOutline);
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_Feet);
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_FeetOutline);
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_Hands);
+		Graphics()->UnloadTexture(&Skin.m_OriginalSkin.m_HandsOutline);
+		for(auto &Eye : Skin.m_OriginalSkin.m_aEyes)
 			Graphics()->UnloadTexture(&Eye);
 
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_Body);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_BodyOutline);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_Feet);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_FeetOutline);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_Hands);
-		Graphics()->UnloadTexture(&m_aSkins[i].m_ColorableSkin.m_HandsOutline);
-		for(auto &Eye : m_aSkins[i].m_ColorableSkin.m_Eyes)
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_Body);
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_BodyOutline);
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_Feet);
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_FeetOutline);
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_Hands);
+		Graphics()->UnloadTexture(&Skin.m_ColorableSkin.m_HandsOutline);
+		for(auto &Eye : Skin.m_ColorableSkin.m_aEyes)
 			Graphics()->UnloadTexture(&Eye);
 	}
 
-	m_aSkins.clear();
-	m_aDownloadSkins.clear();
+	m_vSkins.clear();
+	m_vDownloadSkins.clear();
 	SSkinScanUser SkinScanUser;
 	SkinScanUser.m_pThis = this;
 	SkinScanUser.m_SkinLoadedFunc = SkinLoadedFunc;
 	Storage()->ListDirectory(IStorage::TYPE_ALL, "skins", SkinScan, &SkinScanUser);
-	if(!m_aSkins.size())
+	if(m_vSkins.empty())
 	{
 		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "gameclient", "failed to load skins. folder='skins/'");
 		CSkin DummySkin;
-		DummySkin.m_IsVanilla = true;
-		str_copy(DummySkin.m_aName, "dummy", sizeof(DummySkin.m_aName));
+		str_copy(DummySkin.m_aName, "dummy");
 		DummySkin.m_BloodColor = ColorRGBA(1.0f, 1.0f, 1.0f);
-		m_aSkins.add(DummySkin);
+		m_vSkins.push_back(DummySkin);
 	}
 }
 
 int CSkins::Num()
 {
-	return m_aSkins.size();
+	return m_vSkins.size();
 }
 
 const CSkin *CSkins::Get(int Index)
@@ -372,12 +377,12 @@ const CSkin *CSkins::Get(int Index)
 		if(Index < 0)
 			Index = 0;
 	}
-	return &m_aSkins[Index % m_aSkins.size()];
+	return &m_vSkins[Index % m_vSkins.size()];
 }
 
 int CSkins::Find(const char *pName)
 {
-	const char *pSkinPrefix = m_EventSkinPrefix[0] ? m_EventSkinPrefix : g_Config.m_ClSkinPrefix;
+	const char *pSkinPrefix = m_aEventSkinPrefix[0] ? m_aEventSkinPrefix : g_Config.m_ClSkinPrefix;
 	if(g_Config.m_ClVanillaSkinsOnly && !IsVanillaSkin(pName))
 	{
 		return -1;
@@ -398,9 +403,12 @@ int CSkins::Find(const char *pName)
 
 int CSkins::FindImpl(const char *pName)
 {
-	auto r = ::find_binary(m_aSkins.all(), pName);
-	if(!r.empty())
-		return &r.front() - m_aSkins.base_ptr();
+	CSkin Needle;
+	mem_zero(&Needle, sizeof(Needle));
+	str_copy(Needle.m_aName, pName);
+	auto Range = std::equal_range(m_vSkins.begin(), m_vSkins.end(), Needle);
+	if(std::distance(Range.first, Range.second) == 1)
+		return Range.first - m_vSkins.begin();
 
 	if(str_comp(pName, "default") == 0)
 		return -1;
@@ -411,35 +419,38 @@ int CSkins::FindImpl(const char *pName)
 	if(str_find(pName, "/") != 0)
 		return -1;
 
-	auto d = ::find_binary(m_aDownloadSkins.all(), pName);
-	if(!d.empty())
+	CDownloadSkin DownloadNeedle;
+	mem_zero(&DownloadNeedle, sizeof(DownloadNeedle));
+	str_copy(DownloadNeedle.m_aName, pName);
+	const auto &[RangeBegin, RangeEnd] = std::equal_range(m_vDownloadSkins.begin(), m_vDownloadSkins.end(), DownloadNeedle);
+	if(std::distance(RangeBegin, RangeEnd) == 1)
 	{
-		if(d.front().m_pTask && d.front().m_pTask->State() == HTTP_DONE)
+		if(RangeBegin->m_pTask && RangeBegin->m_pTask->State() == HTTP_DONE)
 		{
 			char aPath[IO_MAX_PATH_LENGTH];
-			str_format(aPath, sizeof(aPath), "downloadedskins/%s.png", d.front().m_aName);
-			Storage()->RenameFile(d.front().m_aPath, aPath, IStorage::TYPE_SAVE);
-			LoadSkin(d.front().m_aName, d.front().m_pTask->m_Info);
-			d.front().m_pTask = nullptr;
+			str_format(aPath, sizeof(aPath), "downloadedskins/%s.png", RangeBegin->m_aName);
+			Storage()->RenameFile(RangeBegin->m_aPath, aPath, IStorage::TYPE_SAVE);
+			LoadSkin(RangeBegin->m_aName, RangeBegin->m_pTask->m_Info);
+			RangeBegin->m_pTask = nullptr;
 		}
-		if(d.front().m_pTask && (d.front().m_pTask->State() == HTTP_ERROR || d.front().m_pTask->State() == HTTP_ABORTED))
+		if(RangeBegin->m_pTask && (RangeBegin->m_pTask->State() == HTTP_ERROR || RangeBegin->m_pTask->State() == HTTP_ABORTED))
 		{
-			d.front().m_pTask = nullptr;
+			RangeBegin->m_pTask = nullptr;
 		}
 		return -1;
 	}
 
 	CDownloadSkin Skin;
-	str_copy(Skin.m_aName, pName, sizeof(Skin.m_aName));
+	str_copy(Skin.m_aName, pName);
 
 	char aUrl[IO_MAX_PATH_LENGTH];
 	char aEscapedName[256];
 	EscapeUrl(aEscapedName, sizeof(aEscapedName), pName);
-	str_format(aUrl, sizeof(aUrl), "%s%s.png", g_Config.m_ClSkinDownloadUrl, aEscapedName);
+	str_format(aUrl, sizeof(aUrl), "%s%s.png", g_Config.m_ClDownloadCommunitySkins != 0 ? g_Config.m_ClSkinCommunityDownloadUrl : g_Config.m_ClSkinDownloadUrl, aEscapedName);
 	char aBuf[IO_MAX_PATH_LENGTH];
 	str_format(Skin.m_aPath, sizeof(Skin.m_aPath), "downloadedskins/%s", IStorage::FormatTmpPath(aBuf, sizeof(aBuf), pName));
-	Skin.m_pTask = std::make_shared<CGetPngFile>(this, Storage(), aUrl, Skin.m_aPath, IStorage::TYPE_SAVE, CTimeout{0, 0, 0}, HTTPLOG::NONE);
+	Skin.m_pTask = std::make_shared<CGetPngFile>(this, aUrl, Storage(), Skin.m_aPath);
 	m_pClient->Engine()->AddJob(Skin.m_pTask);
-	m_aDownloadSkins.add(Skin);
+	m_vDownloadSkins.insert(std::lower_bound(m_vDownloadSkins.begin(), m_vDownloadSkins.end(), Skin), std::move(Skin));
 	return -1;
 }
