@@ -2,17 +2,16 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #ifndef GAME_CLIENT_COMPONENTS_SKINS_H
 #define GAME_CLIENT_COMPONENTS_SKINS_H
-#include <base/color.h>
-#include <base/tl/sorted_array.h>
-#include <base/vmath.h>
-#include <engine/client/http.h>
+
+#include <engine/shared/http.h>
 #include <game/client/component.h>
 #include <game/client/skin.h>
+#include <vector>
 
 class CSkins : public CComponent
 {
 public:
-	class CGetPngFile : public CGetFile
+	class CGetPngFile : public CHttpRequest
 	{
 		CSkins *m_pSkins;
 
@@ -20,7 +19,7 @@ public:
 		virtual int OnCompletion(int State) override;
 
 	public:
-		CGetPngFile(CSkins *pSkins, IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType = -2, CTimeout Timeout = CTimeout{4000, 500, 5}, HTTPLOG LogProgress = HTTPLOG::ALL);
+		CGetPngFile(CSkins *pSkins, const char *pUrl, IStorage *pStorage, const char *pDest);
 		CImageInfo m_Info;
 	};
 
@@ -30,9 +29,19 @@ public:
 		char m_aPath[IO_MAX_PATH_LENGTH];
 		char m_aName[24];
 
+		CDownloadSkin(CDownloadSkin &&Other) = default;
+		CDownloadSkin() = default;
+
+		~CDownloadSkin()
+		{
+			if(m_pTask)
+				m_pTask->Abort();
+		}
 		bool operator<(const CDownloadSkin &Other) const { return str_comp(m_aName, Other.m_aName) < 0; }
 		bool operator<(const char *pOther) const { return str_comp(m_aName, pOther) < 0; }
 		bool operator==(const char *pOther) const { return !str_comp(m_aName, pOther); }
+
+		CDownloadSkin &operator=(CDownloadSkin &&Other) = default;
 	};
 
 	typedef std::function<void(int)> TSkinLoadedCBFunc;
@@ -46,9 +55,9 @@ public:
 	int Find(const char *pName);
 
 private:
-	sorted_array<CSkin> m_aSkins;
-	sorted_array<CDownloadSkin> m_aDownloadSkins;
-	char m_EventSkinPrefix[24];
+	std::vector<CSkin> m_vSkins;
+	std::vector<CDownloadSkin> m_vDownloadSkins;
+	char m_aEventSkinPrefix[24];
 
 	bool LoadSkinPNG(CImageInfo &Info, const char *pName, const char *pPath, int DirType);
 	int LoadSkin(const char *pName, const char *pPath, int DirType);
