@@ -9,15 +9,12 @@
 #include <engine/client.h>
 #include <engine/console.h>
 #include <engine/shared/config.h>
+
+#include <game/collision.h>
 #include <game/gamecore.h>
 #include <game/layers.h>
-#include <game/localization.h>
-
 #include <game/teamscore.h>
 
-#include <game/client/prediction/entities/character.h>
-#include <game/client/prediction/entities/laser.h>
-#include <game/client/prediction/entities/pickup.h>
 #include <game/client/prediction/gameworld.h>
 
 // components
@@ -34,6 +31,7 @@
 #include "components/effects.h"
 #include "components/emoticon.h"
 #include "components/flow.h"
+#include "components/freezebars.h"
 #include "components/ghost.h"
 #include "components/hud.h"
 #include "components/items.h"
@@ -53,6 +51,7 @@
 #include "components/sounds.h"
 #include "components/spectator.h"
 #include "components/statboard.h"
+#include "components/tooltips.h"
 #include "components/voting.h"
 
 class CGameInfo
@@ -91,6 +90,12 @@ public:
 
 	bool m_DontMaskEntities;
 	bool m_AllowXSkins;
+
+	bool m_HudHealthArmor;
+	bool m_HudAmmo;
+	bool m_HudDDRace;
+
+	bool m_NoWeakHookAndBounce;
 };
 
 class CSnapEntities
@@ -131,6 +136,7 @@ public:
 
 	CPlayers m_Players;
 	CNamePlates m_NamePlates;
+	CFreezeBars m_FreezeBars;
 	CItems m_Items;
 	CMapImages m_MapImages;
 
@@ -144,24 +150,11 @@ public:
 	CRaceDemo m_RaceDemo;
 	CGhost m_Ghost;
 
+	CTooltips m_Tooltips;
+
 private:
-	class CStack
-	{
-	public:
-		enum
-		{
-			MAX_COMPONENTS = 64,
-		};
-
-		CStack();
-		void Add(class CComponent *pComponent);
-
-		class CComponent *m_paComponents[MAX_COMPONENTS];
-		int m_Num;
-	};
-
-	CStack m_All;
-	CStack m_Input;
+	std::vector<class CComponent *> m_vpAll;
+	std::vector<class CComponent *> m_vpInput;
 	CNetObjHandler m_NetObjHandler;
 
 	class IEngine *m_pEngine;
@@ -174,6 +167,7 @@ private:
 	class IConsole *m_pConsole;
 	class IStorage *m_pStorage;
 	class IDemoPlayer *m_pDemoPlayer;
+	class IFavorites *m_pFavorites;
 	class IServerBrowser *m_pServerBrowser;
 	class IEditor *m_pEditor;
 	class IFriends *m_pFriends;
@@ -183,21 +177,21 @@ private:
 #endif
 
 	CLayers m_Layers;
-	class CCollision m_Collision;
+	CCollision m_Collision;
 	CUI m_UI;
 
 	void ProcessEvents();
 	void UpdatePositions();
 
 	int m_PredictedTick;
-	int m_LastNewPredictedTick[NUM_DUMMIES];
+	int m_aLastNewPredictedTick[NUM_DUMMIES];
 
 	int m_LastRoundStartTick;
 
 	int m_LastFlagCarrierRed;
 	int m_LastFlagCarrierBlue;
 
-	int m_CheckInfo[NUM_DUMMIES];
+	int m_aCheckInfo[NUM_DUMMIES];
 
 	char m_aDDNetVersionStr[64];
 
@@ -227,10 +221,11 @@ public:
 	class ITextRender *TextRender() const { return m_pTextRender; }
 	class IDemoPlayer *DemoPlayer() const { return m_pDemoPlayer; }
 	class IDemoRecorder *DemoRecorder(int Recorder) const { return Client()->DemoRecorder(Recorder); }
+	class IFavorites *Favorites() const { return m_pFavorites; }
 	class IServerBrowser *ServerBrowser() const { return m_pServerBrowser; }
 	class CRenderTools *RenderTools() { return &m_RenderTools; }
 	class CLayers *Layers() { return &m_Layers; }
-	class CCollision *Collision() { return &m_Collision; }
+	CCollision *Collision() { return &m_Collision; }
 	class IEditor *Editor() { return m_pEditor; }
 	class IFriends *Friends() { return m_pFriends; }
 	class IFriends *Foes() { return m_pFoes; }
@@ -250,10 +245,10 @@ public:
 	bool m_SuppressEvents;
 	bool m_NewTick;
 	bool m_NewPredictedTick;
-	int m_FlagDropTick[2];
+	int m_aFlagDropTick[2];
 
 	// TODO: move this
-	CTuningParams m_Tuning[NUM_DUMMIES];
+	CTuningParams m_aTuning[NUM_DUMMIES];
 
 	enum
 	{
@@ -280,16 +275,16 @@ public:
 		const CNetObj_PlayerInfo *m_pLocalInfo;
 		const CNetObj_SpectatorInfo *m_pSpectatorInfo;
 		const CNetObj_SpectatorInfo *m_pPrevSpectatorInfo;
-		const CNetObj_Flag *m_paFlags[2];
+		const CNetObj_Flag *m_apFlags[2];
 		const CNetObj_GameInfo *m_pGameInfoObj;
 		const CNetObj_GameData *m_pGameDataObj;
 		int m_GameDataSnapID;
 
-		const CNetObj_PlayerInfo *m_paPlayerInfos[MAX_CLIENTS];
-		const CNetObj_PlayerInfo *m_paInfoByScore[MAX_CLIENTS];
-		const CNetObj_PlayerInfo *m_paInfoByName[MAX_CLIENTS];
-		const CNetObj_PlayerInfo *m_paInfoByDDTeamScore[MAX_CLIENTS];
-		const CNetObj_PlayerInfo *m_paInfoByDDTeamName[MAX_CLIENTS];
+		const CNetObj_PlayerInfo *m_apPlayerInfos[MAX_CLIENTS];
+		const CNetObj_PlayerInfo *m_apInfoByScore[MAX_CLIENTS];
+		const CNetObj_PlayerInfo *m_apInfoByName[MAX_CLIENTS];
+		const CNetObj_PlayerInfo *m_apInfoByDDTeamScore[MAX_CLIENTS];
+		const CNetObj_PlayerInfo *m_apInfoByDDTeamName[MAX_CLIENTS];
 
 		int m_LocalClientID;
 		int m_NumPlayers;
@@ -314,7 +309,9 @@ public:
 			CNetObj_Character m_Cur;
 
 			CNetObj_DDNetCharacter m_ExtendedData;
+			const CNetObj_DDNetCharacter *m_PrevExtendedData;
 			bool m_HasExtendedData;
+			bool m_HasExtendedDisplayInfo;
 
 			// interpolated position
 			vec2 m_Position;
@@ -324,10 +321,10 @@ public:
 	};
 
 	CSnapState m_Snap;
-	int m_LocalTuneZone[2];
-	bool m_ReceivedTuning[2];
-	int m_ExpectingTuningForZone[2];
-	int m_ExpectingTuningSince[2];
+	int m_aLocalTuneZone[NUM_DUMMIES];
+	bool m_aReceivedTuning[NUM_DUMMIES];
+	int m_aExpectingTuningForZone[NUM_DUMMIES];
+	int m_aExpectingTuningSince[NUM_DUMMIES];
 
 	// client data
 	struct CClientData
@@ -347,14 +344,14 @@ public:
 		int m_EmoticonStartTick;
 		bool m_Solo;
 		bool m_Jetpack;
-		bool m_NoCollision;
+		bool m_CollisionDisabled;
 		bool m_EndlessHook;
 		bool m_EndlessJump;
-		bool m_NoHammerHit;
-		bool m_NoGrenadeHit;
-		bool m_NoLaserHit;
-		bool m_NoShotgunHit;
-		bool m_NoHookHit;
+		bool m_HammerHitDisabled;
+		bool m_GrenadeHitDisabled;
+		bool m_LaserHitDisabled;
+		bool m_ShotgunHitDisabled;
+		bool m_HookHitDisabled;
 		bool m_Super;
 		bool m_HasTelegunGun;
 		bool m_HasTelegunGrenade;
@@ -382,7 +379,7 @@ public:
 		bool m_Spec;
 
 		// Editor allows 256 switches for now.
-		bool m_SwitchStates[256];
+		bool m_aSwitchStates[256];
 
 		CNetObj_Character m_Snapped;
 		CNetObj_Character m_Evolved;
@@ -396,10 +393,10 @@ public:
 		vec2 m_RenderPos;
 		bool m_IsPredicted;
 		bool m_IsPredictedLocal;
-		int64_t m_SmoothStart[2];
-		int64_t m_SmoothLen[2];
-		vec2 m_PredPos[200];
-		int m_PredTick[200];
+		int64_t m_aSmoothStart[2];
+		int64_t m_aSmoothLen[2];
+		vec2 m_aPredPos[200];
+		int m_aPredTick[200];
 		bool m_SpecCharPresent;
 		vec2 m_SpecChar;
 	};
@@ -449,26 +446,28 @@ public:
 
 	void OnReset();
 
+	size_t ComponentCount() { return m_vpAll.size(); }
+
 	// hooks
-	virtual void OnConnected();
-	virtual void OnRender();
-	virtual void OnUpdate();
-	virtual void OnDummyDisconnect();
+	void OnConnected() override;
+	void OnRender() override;
+	void OnUpdate() override;
+	void OnDummyDisconnect() override;
 	virtual void OnRelease();
-	virtual void OnInit();
-	virtual void OnConsoleInit();
-	virtual void OnStateChange(int NewState, int OldState);
-	virtual void OnMessage(int MsgId, CUnpacker *pUnpacker, int Conn, bool Dummy);
-	virtual void InvalidateSnapshot();
-	virtual void OnNewSnapshot();
-	virtual void OnPredict();
-	virtual void OnActivateEditor();
-	virtual void OnDummySwap();
-	virtual int OnSnapInput(int *pData, bool Dummy, bool Force);
-	virtual void OnShutdown();
-	virtual void OnEnterGame();
-	virtual void OnRconType(bool UsernameReq);
-	virtual void OnRconLine(const char *pLine);
+	void OnInit() override;
+	void OnConsoleInit() override;
+	void OnStateChange(int NewState, int OldState) override;
+	void OnMessage(int MsgId, CUnpacker *pUnpacker, int Conn, bool Dummy) override;
+	void InvalidateSnapshot() override;
+	void OnNewSnapshot() override;
+	void OnPredict() override;
+	void OnActivateEditor() override;
+	void OnDummySwap() override;
+	int OnSnapInput(int *pData, bool Dummy, bool Force) override;
+	void OnShutdown() override;
+	void OnEnterGame() override;
+	void OnRconType(bool UsernameReq) override;
+	void OnRconLine(const char *pLine) override;
 	virtual void OnGameOver();
 	virtual void OnStartGame();
 	virtual void OnFlagGrab(int TeamID);
@@ -478,36 +477,36 @@ public:
 
 	void OnLanguageChange();
 
-	virtual const char *GetItemName(int Type) const;
-	virtual const char *Version() const;
-	virtual const char *NetVersion() const;
-	virtual int DDNetVersion() const;
-	virtual const char *DDNetVersionStr() const;
+	const char *GetItemName(int Type) const override;
+	const char *Version() const override;
+	const char *NetVersion() const override;
+	int DDNetVersion() const override;
+	const char *DDNetVersionStr() const override;
 
 	// actions
 	// TODO: move these
 	void SendSwitchTeam(int Team);
 	void SendInfo(bool Start);
-	virtual void SendDummyInfo(bool Start);
+	void SendDummyInfo(bool Start) override;
 	void SendKill(int ClientID);
 
 	// DDRace
 
-	int m_LocalIDs[NUM_DUMMIES];
+	int m_aLocalIDs[NUM_DUMMIES];
 	CNetObj_PlayerInput m_DummyInput;
 	CNetObj_PlayerInput m_HammerInput;
-	int m_DummyFire;
+	unsigned int m_DummyFire;
 	bool m_ReceivedDDNetPlayer;
 
 	class CTeamsCore m_Teams;
 
 	int IntersectCharacter(vec2 HookPos, vec2 NewPos, vec2 &NewPos2, int ownID);
 
-	virtual int GetLastRaceTick();
+	int GetLastRaceTick() override;
 
 	bool IsTeamPlay() { return m_Snap.m_pGameInfoObj && m_Snap.m_pGameInfoObj->m_GameFlags & GAMEFLAG_TEAMS; }
 
-	bool AntiPingPlayers() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingPlayers && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK && (m_Tuning[g_Config.m_ClDummy].m_PlayerCollision || m_Tuning[g_Config.m_ClDummy].m_PlayerHooking); }
+	bool AntiPingPlayers() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingPlayers && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK && (m_aTuning[g_Config.m_ClDummy].m_PlayerCollision || m_aTuning[g_Config.m_ClDummy].m_PlayerHooking); }
 	bool AntiPingGrenade() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingGrenade && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK; }
 	bool AntiPingWeapons() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingWeapons && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK; }
 	bool AntiPingGunfire() { return AntiPingGrenade() && AntiPingWeapons() && g_Config.m_ClAntiPingGunfire; }
@@ -519,17 +518,23 @@ public:
 	CGameWorld m_PredictedWorld;
 	CGameWorld m_PrevPredictedWorld;
 
-	void DummyResetInput();
-	void Echo(const char *pString);
+	std::vector<SSwitchers> &Switchers() { return m_GameWorld.m_Core.m_vSwitchers; }
+	std::vector<SSwitchers> &PredSwitchers() { return m_PredictedWorld.m_Core.m_vSwitchers; }
+
+	void DummyResetInput() override;
+	void Echo(const char *pString) override;
 	bool IsOtherTeam(int ClientID);
 	int SwitchStateTeam();
 	bool IsLocalCharSuper();
-	bool CanDisplayWarning();
-	bool IsDisplayingWarning();
+	bool CanDisplayWarning() override;
+	bool IsDisplayingWarning() override;
+	CNetObjHandler *GetNetObjHandler() override;
 
 	void LoadGameSkin(const char *pPath, bool AsDir = false);
 	void LoadEmoticonsSkin(const char *pPath, bool AsDir = false);
 	void LoadParticlesSkin(const char *pPath, bool AsDir = false);
+	void LoadHudSkin(const char *pPath, bool AsDir = false);
+	void LoadExtrasSkin(const char *pPath, bool AsDir = false);
 
 	void RefindSkins();
 
@@ -549,7 +554,7 @@ public:
 		IGraphics::CTextureHandle m_SpriteWeaponNinjaCursor;
 		IGraphics::CTextureHandle m_SpriteWeaponLaserCursor;
 
-		IGraphics::CTextureHandle m_SpriteWeaponCursors[6];
+		IGraphics::CTextureHandle m_aSpriteWeaponCursors[6];
 
 		// weapons and hook
 		IGraphics::CTextureHandle m_SpriteHookChain;
@@ -561,13 +566,13 @@ public:
 		IGraphics::CTextureHandle m_SpriteWeaponNinja;
 		IGraphics::CTextureHandle m_SpriteWeaponLaser;
 
-		IGraphics::CTextureHandle m_SpriteWeapons[6];
+		IGraphics::CTextureHandle m_aSpriteWeapons[6];
 
 		// particles
-		IGraphics::CTextureHandle m_SpriteParticles[9];
+		IGraphics::CTextureHandle m_aSpriteParticles[9];
 
 		// stars
-		IGraphics::CTextureHandle m_SpriteStars[3];
+		IGraphics::CTextureHandle m_aSpriteStars[3];
 
 		// projectiles
 		IGraphics::CTextureHandle m_SpriteWeaponGunProjectile;
@@ -577,18 +582,22 @@ public:
 		IGraphics::CTextureHandle m_SpriteWeaponNinjaProjectile;
 		IGraphics::CTextureHandle m_SpriteWeaponLaserProjectile;
 
-		IGraphics::CTextureHandle m_SpriteWeaponProjectiles[6];
+		IGraphics::CTextureHandle m_aSpriteWeaponProjectiles[6];
 
 		// muzzles
-		IGraphics::CTextureHandle m_SpriteWeaponGunMuzzles[3];
-		IGraphics::CTextureHandle m_SpriteWeaponShotgunMuzzles[3];
-		IGraphics::CTextureHandle m_SpriteWeaponNinjaMuzzles[3];
+		IGraphics::CTextureHandle m_aSpriteWeaponGunMuzzles[3];
+		IGraphics::CTextureHandle m_aSpriteWeaponShotgunMuzzles[3];
+		IGraphics::CTextureHandle m_aaSpriteWeaponNinjaMuzzles[3];
 
-		IGraphics::CTextureHandle m_SpriteWeaponsMuzzles[6][3];
+		IGraphics::CTextureHandle m_aaSpriteWeaponsMuzzles[6][3];
 
 		// pickups
 		IGraphics::CTextureHandle m_SpritePickupHealth;
 		IGraphics::CTextureHandle m_SpritePickupArmor;
+		IGraphics::CTextureHandle m_SpritePickupArmorShotgun;
+		IGraphics::CTextureHandle m_SpritePickupArmorGrenade;
+		IGraphics::CTextureHandle m_SpritePickupArmorNinja;
+		IGraphics::CTextureHandle m_SpritePickupArmorLaser;
 		IGraphics::CTextureHandle m_SpritePickupGrenade;
 		IGraphics::CTextureHandle m_SpritePickupShotgun;
 		IGraphics::CTextureHandle m_SpritePickupLaser;
@@ -596,7 +605,8 @@ public:
 		IGraphics::CTextureHandle m_SpritePickupGun;
 		IGraphics::CTextureHandle m_SpritePickupHammer;
 
-		IGraphics::CTextureHandle m_SpritePickupWeapons[6];
+		IGraphics::CTextureHandle m_aSpritePickupWeapons[6];
+		IGraphics::CTextureHandle m_aSpritePickupWeaponArmor[4];
 
 		// flags
 		IGraphics::CTextureHandle m_SpriteFlagBlue;
@@ -621,13 +631,13 @@ public:
 	{
 		IGraphics::CTextureHandle m_SpriteParticleSlice;
 		IGraphics::CTextureHandle m_SpriteParticleBall;
-		IGraphics::CTextureHandle m_SpriteParticleSplat[3];
+		IGraphics::CTextureHandle m_aSpriteParticleSplat[3];
 		IGraphics::CTextureHandle m_SpriteParticleSmoke;
 		IGraphics::CTextureHandle m_SpriteParticleShell;
 		IGraphics::CTextureHandle m_SpriteParticleExpl;
 		IGraphics::CTextureHandle m_SpriteParticleAirJump;
 		IGraphics::CTextureHandle m_SpriteParticleHit;
-		IGraphics::CTextureHandle m_SpriteParticles[10];
+		IGraphics::CTextureHandle m_aSpriteParticles[10];
 	};
 
 	SClientParticlesSkin m_ParticlesSkin;
@@ -635,20 +645,65 @@ public:
 
 	struct SClientEmoticonsSkin
 	{
-		IGraphics::CTextureHandle m_SpriteEmoticons[16];
+		IGraphics::CTextureHandle m_aSpriteEmoticons[16];
 	};
 
 	SClientEmoticonsSkin m_EmoticonsSkin;
 	bool m_EmoticonsSkinLoaded;
 
-	const std::vector<CSnapEntities> &SnapEntities() { return m_aSnapEntities; }
+	struct SClientHudSkin
+	{
+		IGraphics::CTextureHandle m_SpriteHudAirjump;
+		IGraphics::CTextureHandle m_SpriteHudAirjumpEmpty;
+		IGraphics::CTextureHandle m_SpriteHudSolo;
+		IGraphics::CTextureHandle m_SpriteHudCollisionDisabled;
+		IGraphics::CTextureHandle m_SpriteHudEndlessJump;
+		IGraphics::CTextureHandle m_SpriteHudEndlessHook;
+		IGraphics::CTextureHandle m_SpriteHudJetpack;
+		IGraphics::CTextureHandle m_SpriteHudFreezeBarFullLeft;
+		IGraphics::CTextureHandle m_SpriteHudFreezeBarFull;
+		IGraphics::CTextureHandle m_SpriteHudFreezeBarEmpty;
+		IGraphics::CTextureHandle m_SpriteHudFreezeBarEmptyRight;
+		IGraphics::CTextureHandle m_SpriteHudNinjaBarFullLeft;
+		IGraphics::CTextureHandle m_SpriteHudNinjaBarFull;
+		IGraphics::CTextureHandle m_SpriteHudNinjaBarEmpty;
+		IGraphics::CTextureHandle m_SpriteHudNinjaBarEmptyRight;
+		IGraphics::CTextureHandle m_SpriteHudHookHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudHammerHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudShotgunHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudGrenadeHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudLaserHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudGunHitDisabled;
+		IGraphics::CTextureHandle m_SpriteHudDeepFrozen;
+		IGraphics::CTextureHandle m_SpriteHudLiveFrozen;
+		IGraphics::CTextureHandle m_SpriteHudTeleportGrenade;
+		IGraphics::CTextureHandle m_SpriteHudTeleportGun;
+		IGraphics::CTextureHandle m_SpriteHudTeleportLaser;
+		IGraphics::CTextureHandle m_SpriteHudPracticeMode;
+		IGraphics::CTextureHandle m_SpriteHudDummyHammer;
+		IGraphics::CTextureHandle m_SpriteHudDummyCopy;
+	};
+
+	SClientHudSkin m_HudSkin;
+	bool m_HudSkinLoaded;
+
+	struct SClientExtrasSkin
+	{
+		IGraphics::CTextureHandle m_SpriteParticleSnowflake;
+		IGraphics::CTextureHandle m_aSpriteParticles[1];
+	};
+
+	SClientExtrasSkin m_ExtrasSkin;
+	bool m_ExtrasSkinLoaded;
+
+	const std::vector<CSnapEntities> &SnapEntities() { return m_vSnapEntities; }
 
 private:
-	std::vector<CSnapEntities> m_aSnapEntities;
+	std::vector<CSnapEntities> m_vSnapEntities;
 	void SnapCollectEntities();
 
-	bool m_DDRaceMsgSent[NUM_DUMMIES];
-	int m_ShowOthers[NUM_DUMMIES];
+	bool m_aDDRaceMsgSent[NUM_DUMMIES];
+	int m_aShowOthers[NUM_DUMMIES];
 
 	void UpdatePrediction();
 	void UpdateRenderedCharacters();
@@ -658,8 +713,7 @@ private:
 	int m_PredictedDummyID;
 	int m_IsDummySwapping;
 	CCharOrder m_CharOrder;
-	class CCharacter m_aLastWorldCharacters[MAX_CLIENTS];
-	int m_SwitchStateTeam[NUM_DUMMIES];
+	int m_aSwitchStateTeam[NUM_DUMMIES];
 
 	enum
 	{
