@@ -5,6 +5,8 @@
 
 #include <game/client/prediction/entity.h>
 
+class CLaserData;
+
 class CLaser : public CEntity
 {
 	friend class CGameWorld;
@@ -12,14 +14,15 @@ class CLaser : public CEntity
 public:
 	CLaser(CGameWorld *pGameWorld, vec2 Pos, vec2 Direction, float StartEnergy, int Owner, int Type);
 
-	virtual void Tick();
+	void Tick() override;
 
 	const vec2 &GetFrom() { return m_From; }
 	const int &GetOwner() { return m_Owner; }
 	const int &GetEvalTick() { return m_EvalTick; }
-	CLaser(CGameWorld *pGameWorld, int ID, CNetObj_Laser *pLaser);
+	CLaser(CGameWorld *pGameWorld, int ID, CLaserData *pLaser);
 	void FillInfo(CNetObj_Laser *pLaser);
 	bool Match(CLaser *pLaser);
+	CLaserData GetData() const;
 
 protected:
 	bool HitCharacter(vec2 From, vec2 To);
@@ -34,6 +37,7 @@ private:
 	int m_Bounces;
 	int m_EvalTick;
 	int m_Owner;
+	bool m_ZeroEnergyBounceInLastTick;
 
 	// DDRace
 
