@@ -3,32 +3,25 @@
 #ifndef GAME_EDITOR_EDITOR_H
 #define GAME_EDITOR_EDITOR_H
 
-#include <math.h>
 #include <string>
 #include <vector>
 
-#include <base/math.h>
 #include <base/system.h>
 
-#include <base/tl/algorithm.h>
-#include <base/tl/array.h>
-#include <base/tl/sorted_array.h>
-#include <base/tl/string.h>
-
 #include <game/client/render.h>
 #include <game/client/ui.h>
-#include <game/client/ui_ex.h>
 #include <game/mapitems.h>
+#include <game/mapitems_ex.h>
 
 #include <engine/editor.h>
 #include <engine/graphics.h>
-#include <engine/shared/config.h>
-#include <engine/shared/datafile.h>
-#include <engine/sound.h>
-#include <engine/storage.h>
 
 #include "auto_map.h"
 
+#include <chrono>
+
+using namespace std::chrono_literals;
+
 typedef void (*INDEX_MODIFY_FUNC)(int *pIndex);
 
 //CRenderTools m_RenderTools;
@@ -48,7 +41,7 @@ class CEnvelope
 {
 public:
 	int m_Channels;
-	array<CEnvPoint> m_lPoints;
+	std::vector<CEnvPoint> m_vPoints;
 	char m_aName[32];
 	float m_Bottom, m_Top;
 	bool m_Synchronized;
@@ -64,7 +57,7 @@ public:
 
 	void Resort()
 	{
-		sort(m_lPoints.all());
+		std::sort(m_vPoints.begin(), m_vPoints.end());
 		FindTopBottom(0xf);
 	}
 
@@ -72,13 +65,13 @@ public:
 	{
 		m_Top = -1000000000.0f;
 		m_Bottom = 1000000000.0f;
-		for(int i = 0; i < m_lPoints.size(); i++)
+		for(auto &Point : m_vPoints)
 		{
 			for(int c = 0; c < m_Channels; c++)
 			{
 				if(ChannelMask & (1 << c))
 				{
-					float v = fx2f(m_lPoints[i].m_aValues[c]);
+					float v = fx2f(Point.m_aValues[c]);
 					if(v > m_Top)
 						m_Top = v;
 					if(v < m_Bottom)
@@ -88,9 +81,9 @@ public:
 		}
 	}
 
-	int Eval(float Time, float *pResult)
+	int Eval(float Time, ColorRGBA &Color)
 	{
-		CRenderTools::RenderEvalEnvelope(m_lPoints.base_ptr(), m_lPoints.size(), m_Channels, (int64_t)((double)Time * 1000000.0), pResult);
+		CRenderTools::RenderEvalEnvelope(&m_vPoints[0], m_vPoints.size(), m_Channels, std::chrono::nanoseconds((int64_t)((double)Time * (double)std::chrono::nanoseconds(1s).count())), Color);
 		return m_Channels;
 	}
 
@@ -103,21 +96,19 @@ public:
 		p.m_aValues[2] = v2;
 		p.m_aValues[3] = v3;
 		p.m_Curvetype = CURVETYPE_LINEAR;
-		m_lPoints.add(p);
+		m_vPoints.push_back(p);
 		Resort();
 	}
 
 	float EndTime() const
 	{
-		if(m_lPoints.size())
-			return m_lPoints[m_lPoints.size() - 1].m_Time * (1.0f / 1000.0f);
+		if(!m_vPoints.empty())
+			return m_vPoints[m_vPoints.size() - 1].m_Time * (1.0f / 1000.0f);
 		return 0;
 	}
 };
 
-class CLayer;
 class CLayerGroup;
-class CEditorMap;
 
 class CLayer
 {
@@ -133,10 +124,21 @@ public:
 		m_Visible = true;
 		m_Readonly = false;
 		m_Flags = 0;
-		m_pEditor = 0;
+		m_pEditor = nullptr;
 		m_BrushRefCount = 0;
 	}
 
+	CLayer(const CLayer &Other)
+	{
+		str_copy(m_aName, Other.m_aName, sizeof(m_aName));
+		m_Flags = Other.m_Flags;
+		m_pEditor = Other.m_pEditor;
+		m_Type = Other.m_Type;
+		m_BrushRefCount = 0;
+		m_Visible = true;
+		m_Readonly = false;
+	}
+
 	virtual ~CLayer()
 	{
 	}
@@ -157,10 +159,12 @@ public:
 	virtual void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc) {}
 	virtual void ModifySoundIndex(INDEX_MODIFY_FUNC pfnFunc) {}
 
-	virtual void GetSize(float *w, float *h)
+	virtual CLayer *Duplicate() const = 0;
+
+	virtual void GetSize(float *pWidth, float *pHeight)
 	{
-		*w = 0;
-		*h = 0;
+		*pWidth = 0;
+		*pHeight = 0;
 	}
 
 	char m_aName[12];
@@ -169,7 +173,6 @@ public:
 
 	bool m_Readonly;
 	bool m_Visible;
-
 	int m_BrushRefCount;
 };
 
@@ -178,13 +181,15 @@ class CLayerGroup
 public:
 	class CEditorMap *m_pMap;
 
-	array<CLayer *> m_lLayers;
+	std::vector<CLayer *> m_vpLayers;
 
 	int m_OffsetX;
 	int m_OffsetY;
 
 	int m_ParallaxX;
 	int m_ParallaxY;
+	int m_CustomParallaxZoom;
+	int m_ParallaxZoom;
 
 	int m_UseClipping;
 	int m_ClipX;
@@ -205,14 +210,15 @@ public:
 	void MapScreen();
 	void Mapping(float *pPoints);
 
-	void GetSize(float *w, float *h) const;
+	void GetSize(float *pWidth, float *pHeight) const;
 
 	void DeleteLayer(int Index);
+	void DuplicateLayer(int Index);
 	int SwapLayers(int Index0, int Index1);
 
 	bool IsEmpty() const
 	{
-		return m_lLayers.size() == 0; // stupid function, since its bad for Fillselection: TODO add a function for Fillselection that returns whether a specific tile is used in the given layer
+		return m_vpLayers.size() == 0; // stupid function, since its bad for Fillselection: TODO add a function for Fillselection that returns whether a specific tile is used in the given layer
 	}
 
 	/*bool IsUsedInThisLayer(int Layer, int Index) // <--------- this is what i meant but cause i don't know which Indexes belongs to which layers i can't finish yet
@@ -250,29 +256,35 @@ public:
 		}
 	}*/
 
-	void Clear()
+	void OnEdited()
 	{
-		m_lLayers.clear();
+		if(!m_CustomParallaxZoom)
+			m_ParallaxZoom = GetParallaxZoomDefault(m_ParallaxX, m_ParallaxY);
 	}
 
-	void AddLayer(CLayer *l);
+	void Clear()
+	{
+		m_vpLayers.clear();
+	}
+
+	void AddLayer(CLayer *pLayer);
 
 	void ModifyImageIndex(INDEX_MODIFY_FUNC Func)
 	{
-		for(int i = 0; i < m_lLayers.size(); i++)
-			m_lLayers[i]->ModifyImageIndex(Func);
+		for(auto &pLayer : m_vpLayers)
+			pLayer->ModifyImageIndex(Func);
 	}
 
 	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC Func)
 	{
-		for(int i = 0; i < m_lLayers.size(); i++)
-			m_lLayers[i]->ModifyEnvelopeIndex(Func);
+		for(auto &pLayer : m_vpLayers)
+			pLayer->ModifyEnvelopeIndex(Func);
 	}
 
 	void ModifySoundIndex(INDEX_MODIFY_FUNC Func)
 	{
-		for(int i = 0; i < m_lLayers.size(); i++)
-			m_lLayers[i]->ModifySoundIndex(Func);
+		for(auto &pLayer : m_vpLayers)
+			pLayer->ModifySoundIndex(Func);
 	}
 };
 
@@ -289,7 +301,7 @@ public:
 		m_External = 0;
 		m_Width = 0;
 		m_Height = 0;
-		m_pData = 0;
+		m_pData = nullptr;
 		m_Format = 0;
 	}
 
@@ -315,7 +327,7 @@ public:
 		m_aName[0] = 0;
 		m_SoundID = 0;
 
-		m_pData = 0x0;
+		m_pData = nullptr;
 		m_DataSize = 0;
 	}
 
@@ -347,10 +359,10 @@ public:
 		Clean();
 	}
 
-	array<CLayerGroup *> m_lGroups;
-	array<CEditorImage *> m_lImages;
-	array<CEnvelope *> m_lEnvelopes;
-	array<CEditorSound *> m_lSounds;
+	std::vector<CLayerGroup *> m_vpGroups;
+	std::vector<CEditorImage *> m_vpImages;
+	std::vector<CEnvelope *> m_vpEnvelopes;
+	std::vector<CEditorSound *> m_vpSounds;
 
 	class CMapInfo
 	{
@@ -384,7 +396,7 @@ public:
 	{
 		char m_aCommand[256];
 	};
-	array<CSetting> m_lSettings;
+	std::vector<CSetting> m_vSettings;
 
 	class CLayerGame *m_pGameLayer;
 	CLayerGroup *m_pGameGroup;
@@ -392,9 +404,9 @@ public:
 	CEnvelope *NewEnvelope(int Channels)
 	{
 		m_Modified = true;
-		CEnvelope *e = new CEnvelope(Channels);
-		m_lEnvelopes.add(e);
-		return e;
+		CEnvelope *pEnv = new CEnvelope(Channels);
+		m_vpEnvelopes.push_back(pEnv);
+		return pEnv;
 	}
 
 	void DeleteEnvelope(int Index);
@@ -402,53 +414,53 @@ public:
 	CLayerGroup *NewGroup()
 	{
 		m_Modified = true;
-		CLayerGroup *g = new CLayerGroup;
-		g->m_pMap = this;
-		m_lGroups.add(g);
-		return g;
+		CLayerGroup *pGroup = new CLayerGroup;
+		pGroup->m_pMap = this;
+		m_vpGroups.push_back(pGroup);
+		return pGroup;
 	}
 
 	int SwapGroups(int Index0, int Index1)
 	{
-		if(Index0 < 0 || Index0 >= m_lGroups.size())
+		if(Index0 < 0 || Index0 >= (int)m_vpGroups.size())
 			return Index0;
-		if(Index1 < 0 || Index1 >= m_lGroups.size())
+		if(Index1 < 0 || Index1 >= (int)m_vpGroups.size())
 			return Index0;
 		if(Index0 == Index1)
 			return Index0;
 		m_Modified = true;
-		swap(m_lGroups[Index0], m_lGroups[Index1]);
+		std::swap(m_vpGroups[Index0], m_vpGroups[Index1]);
 		return Index1;
 	}
 
 	void DeleteGroup(int Index)
 	{
-		if(Index < 0 || Index >= m_lGroups.size())
+		if(Index < 0 || Index >= (int)m_vpGroups.size())
 			return;
 		m_Modified = true;
-		delete m_lGroups[Index];
-		m_lGroups.remove_index(Index);
+		delete m_vpGroups[Index];
+		m_vpGroups.erase(m_vpGroups.begin() + Index);
 	}
 
 	void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		for(int i = 0; i < m_lGroups.size(); i++)
-			m_lGroups[i]->ModifyImageIndex(pfnFunc);
+		for(auto &pGroup : m_vpGroups)
+			pGroup->ModifyImageIndex(pfnFunc);
 	}
 
 	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		for(int i = 0; i < m_lGroups.size(); i++)
-			m_lGroups[i]->ModifyEnvelopeIndex(pfnFunc);
+		for(auto &pGroup : m_vpGroups)
+			pGroup->ModifyEnvelopeIndex(pfnFunc);
 	}
 
 	void ModifySoundIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		for(int i = 0; i < m_lGroups.size(); i++)
-			m_lGroups[i]->ModifySoundIndex(pfnFunc);
+		for(auto &pGroup : m_vpGroups)
+			pGroup->ModifySoundIndex(pfnFunc);
 	}
 
 	void Clean();
@@ -504,16 +516,67 @@ enum
 	DIRECTION_DOWN = 8,
 };
 
-typedef struct
+struct RECTi
 {
 	int x, y;
 	int w, h;
-} RECTi;
+};
 
 class CLayerTiles : public CLayer
 {
+protected:
+	template<typename T>
+	void ShiftImpl(T *pTiles, int Direction, int ShiftBy)
+	{
+		switch(Direction)
+		{
+		case DIRECTION_LEFT:
+			for(int y = 0; y < m_Height; ++y)
+			{
+				mem_move(&pTiles[y * m_Width], &pTiles[y * m_Width + ShiftBy], (m_Width - ShiftBy) * sizeof(T));
+				mem_zero(&pTiles[y * m_Width + (m_Width - ShiftBy)], ShiftBy * sizeof(T));
+			}
+			break;
+		case DIRECTION_RIGHT:
+			for(int y = 0; y < m_Height; ++y)
+			{
+				mem_move(&pTiles[y * m_Width + ShiftBy], &pTiles[y * m_Width], (m_Width - ShiftBy) * sizeof(T));
+				mem_zero(&pTiles[y * m_Width], ShiftBy * sizeof(T));
+			}
+			break;
+		case DIRECTION_UP:
+			for(int y = 0; y < m_Height - ShiftBy; ++y)
+			{
+				mem_copy(&pTiles[y * m_Width], &pTiles[(y + ShiftBy) * m_Width], m_Width * sizeof(T));
+				mem_zero(&pTiles[(y + ShiftBy) * m_Width], m_Width * sizeof(T));
+			}
+			break;
+		case DIRECTION_DOWN:
+			for(int y = m_Height - 1; y >= ShiftBy; --y)
+			{
+				mem_copy(&pTiles[y * m_Width], &pTiles[(y - ShiftBy) * m_Width], m_Width * sizeof(T));
+				mem_zero(&pTiles[(y - ShiftBy) * m_Width], m_Width * sizeof(T));
+			}
+		}
+	}
+	template<typename T>
+	void BrushFlipXImpl(T *pTiles)
+	{
+		for(int y = 0; y < m_Height; y++)
+			for(int x = 0; x < m_Width / 2; x++)
+				std::swap(pTiles[y * m_Width + x], pTiles[(y + 1) * m_Width - 1 - x]);
+	}
+	template<typename T>
+	void BrushFlipYImpl(T *pTiles)
+	{
+		for(int y = 0; y < m_Height / 2; y++)
+			for(int x = 0; x < m_Width; x++)
+				std::swap(pTiles[y * m_Width + x], pTiles[(m_Height - 1 - y) * m_Width + x]);
+	}
+
 public:
 	CLayerTiles(int w, int h);
+	CLayerTiles(const CLayerTiles &Other);
 	~CLayerTiles();
 
 	virtual CTile GetTile(int x, int y);
@@ -523,7 +586,7 @@ public:
 	virtual void Shift(int Direction);
 
 	void MakePalette();
-	virtual void Render(bool Tileset = false);
+	void Render(bool Tileset = false) override;
 
 	int ConvertX(float x) const;
 	int ConvertY(float y) const;
@@ -532,35 +595,42 @@ public:
 	void Clamp(RECTi *pRect);
 
 	virtual bool IsEmpty(CLayerTiles *pLayer);
-	virtual void BrushSelecting(CUIRect Rect);
-	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
-	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
+	void BrushSelecting(CUIRect Rect) override;
+	int BrushGrab(CLayerGroup *pBrush, CUIRect Rect) override;
+	void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect) override;
+	void BrushDraw(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
+
+	CLayer *Duplicate() const override;
 
 	virtual void ShowInfo();
-	virtual int RenderProperties(CUIRect *pToolbox);
+	int RenderProperties(CUIRect *pToolbox) override;
 
 	struct SCommonPropState
 	{
-		bool Modified = false;
-		int Width = -1;
-		int Height = -1;
-		int Color = 0;
+		enum
+		{
+			MODIFIED_SIZE = 1 << 0,
+			MODIFIED_COLOR = 1 << 1,
+		};
+		int m_Modified = 0;
+		int m_Width = -1;
+		int m_Height = -1;
+		int m_Color = 0;
 	};
-	static int RenderCommonProperties(SCommonPropState &State, CEditor *pEditor, CUIRect *pToolbox, array<CLayerTiles *> &pLayers);
+	static int RenderCommonProperties(SCommonPropState &State, CEditor *pEditor, CUIRect *pToolbox, std::vector<CLayerTiles *> &vpLayers);
 
-	virtual void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc);
-	virtual void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc);
+	void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc) override;
+	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc) override;
 
 	void PrepareForSave();
 
-	void GetSize(float *w, float *h)
+	void GetSize(float *pWidth, float *pHeight) override
 	{
-		*w = m_Width * 32.0f;
-		*h = m_Height * 32.0f;
+		*pWidth = m_Width * 32.0f;
+		*pHeight = m_Height * 32.0f;
 	}
 
 	void FlagModified(int x, int y, int w, int h);
@@ -592,27 +662,30 @@ class CLayerQuads : public CLayer
 {
 public:
 	CLayerQuads();
+	CLayerQuads(const CLayerQuads &Other);
 	~CLayerQuads();
 
-	virtual void Render(bool QuadPicker = false);
+	void Render(bool QuadPicker = false) override;
 	CQuad *NewQuad(int x, int y, int Width, int Height);
+	int SwapQuads(int Index0, int Index1);
 
-	virtual void BrushSelecting(CUIRect Rect);
-	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
-	virtual void BrushPlace(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
+	void BrushSelecting(CUIRect Rect) override;
+	int BrushGrab(CLayerGroup *pBrush, CUIRect Rect) override;
+	void BrushPlace(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
 
-	virtual int RenderProperties(CUIRect *pToolbox);
+	int RenderProperties(CUIRect *pToolbox) override;
 
-	virtual void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc);
-	virtual void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc);
+	void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc) override;
+	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc) override;
 
-	void GetSize(float *w, float *h);
+	void GetSize(float *pWidth, float *pHeight) override;
+	CLayer *Duplicate() const override;
 
 	int m_Image;
-	array<CQuad> m_lQuads;
+	std::vector<CQuad> m_vQuads;
 };
 
 class CLayerGame : public CLayerTiles
@@ -621,10 +694,10 @@ public:
 	CLayerGame(int w, int h);
 	~CLayerGame();
 
-	virtual CTile GetTile(int x, int y);
-	virtual void SetTile(int x, int y, CTile tile);
+	CTile GetTile(int x, int y) override;
+	void SetTile(int x, int y, CTile tile) override;
 
-	virtual int RenderProperties(CUIRect *pToolbox);
+	int RenderProperties(CUIRect *pToolbox) override;
 };
 
 class CEditor : public IEditor
@@ -639,7 +712,6 @@ class CEditor : public IEditor
 	class IStorage *m_pStorage;
 	CRenderTools m_RenderTools;
 	CUI m_UI;
-	CUIEx m_UIEx;
 
 	bool m_EditorWasUsedBefore = false;
 
@@ -663,22 +735,21 @@ public:
 	class ITextRender *TextRender() { return m_pTextRender; }
 	class IStorage *Storage() { return m_pStorage; }
 	CUI *UI() { return &m_UI; }
-	CUIEx *UIEx() { return &m_UIEx; }
 	CRenderTools *RenderTools() { return &m_RenderTools; }
 
 	CEditor() :
 		m_TilesetPicker(16, 16)
 	{
-		m_pInput = 0;
-		m_pClient = 0;
-		m_pGraphics = 0;
-		m_pTextRender = 0;
-		m_pSound = 0;
+		m_pInput = nullptr;
+		m_pClient = nullptr;
+		m_pGraphics = nullptr;
+		m_pTextRender = nullptr;
+		m_pSound = nullptr;
 
 		m_Mode = MODE_LAYERS;
 		m_Dialog = 0;
 		m_EditBoxActive = 0;
-		m_pTooltip = 0;
+		m_pTooltip = nullptr;
 
 		m_GridActive = false;
 		m_GridFactor = 1;
@@ -694,9 +765,9 @@ public:
 		m_MouseInsidePopup = false;
 
 		m_FileDialogStorageType = 0;
-		m_pFileDialogTitle = 0;
-		m_pFileDialogButtonText = 0;
-		m_pFileDialogUser = 0;
+		m_pFileDialogTitle = nullptr;
+		m_pFileDialogButtonText = nullptr;
+		m_pFileDialogUser = nullptr;
 		m_aFileDialogFileName[0] = 0;
 		m_aFileDialogCurrentFolder[0] = 0;
 		m_aFileDialogCurrentLink[0] = 0;
@@ -727,6 +798,7 @@ public:
 
 		m_GuiActive = true;
 		m_ProofBorders = false;
+		m_PreviewZoom = false;
 
 		m_ShowTileInfo = false;
 		m_ShowDetail = true;
@@ -748,7 +820,7 @@ public:
 		m_CommandBox = 0.0f;
 		m_aSettingsCommand[0] = 0;
 
-		ms_pUiGotContext = 0;
+		ms_pUiGotContext = nullptr;
 
 		// DDRace
 
@@ -763,15 +835,18 @@ public:
 		m_PreventUnusedTilesWasWarned = false;
 		m_AllowPlaceUnusedTiles = 0;
 		m_BrushDrawDestructive = true;
+		m_GotoX = 0;
+		m_GotoY = 0;
 
 		m_Mentions = 0;
 	}
 
-	virtual void Init();
-	virtual void UpdateAndRender();
-	virtual bool HasUnsavedData() const { return m_Map.m_Modified; }
-	virtual void UpdateMentions() { m_Mentions++; }
-	virtual void ResetMentions() { m_Mentions = 0; }
+	void Init() override;
+	void OnUpdate() override;
+	void OnRender() override;
+	bool HasUnsavedData() const override { return m_Map.m_Modified; }
+	void UpdateMentions() override { m_Mentions++; }
+	void ResetMentions() override { m_Mentions = 0; }
 
 	CLayerGroup *m_apSavedBrushes[10];
 
@@ -781,13 +856,13 @@ public:
 		void (*pfnFunc)(const char *pFilename, int StorageType, void *pUser), void *pUser);
 
 	void Reset(bool CreateDefault = true);
-	virtual int Save(const char *pFilename);
-	virtual int Load(const char *pFilename, int StorageType);
+	int Save(const char *pFilename) override;
+	int Load(const char *pFilename, int StorageType) override;
 	int Append(const char *pFilename, int StorageType);
 	void LoadCurrentMap();
 	void Render();
 
-	array<CQuad *> GetSelectedQuads();
+	std::vector<CQuad *> GetSelectedQuads();
 	CLayer *GetSelectedLayerType(int Index, int Type) const;
 	CLayer *GetSelectedLayer(int Index) const;
 	CLayerGroup *GetSelectedGroup() const;
@@ -798,6 +873,7 @@ public:
 	void DeleteSelectedQuads();
 	bool IsQuadSelected(int Index) const;
 	int FindSelectedQuadIndex(int Index) const;
+	bool IsSpecialLayer(const CLayer *pLayer) const;
 
 	float ScaleFontSize(char *pText, int TextSize, float FontSize, int Width);
 	int DoProperties(CUIRect *pToolbox, CProperty *pProps, int *pIDs, int *pNewVal, ColorRGBA Color = ColorRGBA(1, 1, 1, 0.5f));
@@ -864,7 +940,7 @@ public:
 	float m_FileDialogScrollValue;
 	int m_FilesSelectedIndex;
 	char m_aFileDialogNewFolderName[64];
-	char m_FileDialogErrString[64];
+	char m_aFileDialogErrString[64];
 	IGraphics::CTextureHandle m_FilePreviewImage;
 	bool m_PreviewImageIsLoaded;
 	CImageInfo m_FilePreviewImageInfo;
@@ -881,12 +957,12 @@ public:
 
 		bool operator<(const CFilelistItem &Other) const { return !str_comp(m_aFilename, "..") ? true : !str_comp(Other.m_aFilename, "..") ? false : m_IsDir && !Other.m_IsDir ? true : !m_IsDir && Other.m_IsDir ? false : str_comp_nocase(m_aFilename, Other.m_aFilename) < 0; }
 	};
-	sorted_array<CFilelistItem> m_FileList;
+	std::vector<CFilelistItem> m_vFileList;
 	int m_FilesStartAt;
 	int m_FilesCur;
 	int m_FilesStopAt;
 
-	std::vector<std::string> m_SelectEntitiesFiles;
+	std::vector<std::string> m_vSelectEntitiesFiles;
 	std::string m_SelectEntitiesImage;
 
 	float m_WorldOffsetX;
@@ -899,6 +975,12 @@ public:
 	bool m_ShowMousePointer;
 	bool m_GuiActive;
 	bool m_ProofBorders;
+	bool m_PreviewZoom;
+	float m_MouseWScale = 1.0f; // Mouse (i.e. UI) scale relative to the World (selected Group)
+	float m_MouseX = 0.0f;
+	float m_MouseY = 0.0f;
+	float m_MouseWorldX = 0.0f;
+	float m_MouseWorldY = 0.0f;
 	float m_MouseDeltaX;
 	float m_MouseDeltaY;
 	float m_MouseDeltaWx;
@@ -916,8 +998,8 @@ public:
 	bool m_ShowServerSettingsEditor;
 	bool m_ShowPicker;
 
-	array<int> m_lSelectedLayers;
-	array<int> m_lSelectedQuads;
+	std::vector<int> m_vSelectedLayers;
+	std::vector<int> m_vSelectedQuads;
 	int m_SelectedQuadPoint;
 	int m_SelectedQuadIndex;
 	int m_SelectedGroup;
@@ -948,7 +1030,7 @@ public:
 	CEditorMap m_Map;
 	int m_ShiftBy;
 
-	static void EnvelopeEval(int TimeOffsetMillis, int Env, float *pChannels, void *pUser);
+	static void EnvelopeEval(int TimeOffsetMillis, int Env, ColorRGBA &Channels, void *pUser);
 
 	float m_CommandBox;
 	char m_aSettingsCommand[256];
@@ -966,29 +1048,29 @@ public:
 	int DoButton_File(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
 
 	int DoButton_Menu(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
-	int DoButton_MenuItem(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags = 0, const char *pToolTip = 0);
+	int DoButton_MenuItem(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags = 0, const char *pToolTip = nullptr);
 
-	int DoButton_ColorPicker(const void *pID, const CUIRect *pRect, ColorRGBA *pColor, const char *pToolTip = 0);
+	int DoButton_ColorPicker(const void *pID, const CUIRect *pRect, ColorRGBA *pColor, const char *pToolTip = nullptr);
 
-	bool DoEditBox(void *pID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *pOffset, bool Hidden = false, int Corners = CUI::CORNER_ALL);
+	bool DoEditBox(void *pID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *pOffset, bool Hidden = false, int Corners = IGraphics::CORNER_ALL);
 	bool DoClearableEditBox(void *pID, void *pClearID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *pOffset, bool Hidden, int Corners);
 
 	void RenderBackground(CUIRect View, IGraphics::CTextureHandle Texture, float Size, float Brightness);
 
 	void RenderGrid(CLayerGroup *pGroup);
 
-	void UiInvokePopupMenu(void *pID, int Flags, float X, float Y, float W, float H, int (*pfnFunc)(CEditor *pEditor, CUIRect Rect, void *pContext), void *pContext = 0);
+	void UiInvokePopupMenu(void *pID, int Flags, float X, float Y, float W, float H, int (*pfnFunc)(CEditor *pEditor, CUIRect Rect, void *pContext), void *pContext = nullptr);
 	void UiDoPopupMenu();
 	bool UiPopupExists(void *pID);
 	bool UiPopupOpen();
 
-	int UiDoValueSelector(void *pID, CUIRect *pRect, const char *pLabel, int Current, int Min, int Max, int Step, float Scale, const char *pToolTip, bool IsDegree = false, bool IsHex = false, int corners = CUI::CORNER_ALL, ColorRGBA *Color = 0);
+	int UiDoValueSelector(void *pID, CUIRect *pRect, const char *pLabel, int Current, int Min, int Max, int Step, float Scale, const char *pToolTip, bool IsDegree = false, bool IsHex = false, int corners = IGraphics::CORNER_ALL, ColorRGBA *pColor = nullptr);
 
 	static int PopupGroup(CEditor *pEditor, CUIRect View, void *pContext);
 
 	struct CLayerPopupContext
 	{
-		array<CLayerTiles *> m_aLayers;
+		std::vector<CLayerTiles *> m_vpLayers;
 		CLayerTiles::SCommonPropState m_CommonPropState;
 	};
 	static int PopupLayer(CEditor *pEditor, CUIRect View, void *pContext);
@@ -1025,7 +1107,7 @@ public:
 	void PopupSelectSoundInvoke(int Current, float x, float y);
 	int PopupSelectSoundResult();
 
-	void DoQuadEnvelopes(const array<CQuad> &m_lQuads, IGraphics::CTextureHandle Texture = IGraphics::CTextureHandle());
+	void DoQuadEnvelopes(const std::vector<CQuad> &vQuads, IGraphics::CTextureHandle Texture = IGraphics::CTextureHandle());
 	void DoQuadEnvPoint(const CQuad *pQuad, int QIndex, int pIndex);
 	void DoQuadPoint(CQuad *pQuad, int QuadIndex, int v);
 
@@ -1047,9 +1129,10 @@ public:
 
 	bool IsEnvelopeUsed(int EnvelopeIndex) const;
 
-	void RenderImages(CUIRect Toolbox, CUIRect View);
-	void RenderLayers(CUIRect Toolbox, CUIRect View);
-	void RenderSounds(CUIRect Toolbox, CUIRect View);
+	void RenderLayers(CUIRect LayersBox);
+	void RenderImagesList(CUIRect Toolbox);
+	void RenderSelectedImage(CUIRect View);
+	void RenderSounds(CUIRect Toolbox);
 	void RenderModebar(CUIRect View);
 	void RenderStatusbar(CUIRect View);
 	void RenderEnvelopeEditor(CUIRect View);
@@ -1061,6 +1144,7 @@ public:
 	void AddFileDialogEntry(int Index, CUIRect *pView);
 	void SelectGameLayer();
 	void SortImages();
+	bool SelectLayerByTile();
 
 	//Tile Numbers For Explanations - TODO: Add/Improve tiles and explanations
 	enum
@@ -1157,6 +1241,8 @@ public:
 	static int PopupSpeedup(CEditor *pEditor, CUIRect View, void *pContext);
 	static int PopupSwitch(CEditor *pEditor, CUIRect View, void *pContext);
 	static int PopupTune(CEditor *pEditor, CUIRect View, void *pContext);
+	static int PopupGoto(CEditor *pEditor, CUIRect View, void *pContext);
+	void Goto(float X, float Y);
 	unsigned char m_TeleNumber;
 
 	unsigned char m_TuningNum;
@@ -1167,6 +1253,9 @@ public:
 
 	unsigned char m_SwitchNum;
 	unsigned char m_SwitchDelay;
+
+	int m_GotoX;
+	int m_GotoY;
 };
 
 // make sure to inline this function
@@ -1184,14 +1273,14 @@ public:
 	CTeleTile *m_pTeleTile;
 	unsigned char m_TeleNum;
 
-	virtual void Resize(int NewW, int NewH);
-	virtual void Shift(int Direction);
-	virtual bool IsEmpty(CLayerTiles *pLayer);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
-	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
+	void Resize(int NewW, int NewH) override;
+	void Shift(int Direction) override;
+	bool IsEmpty(CLayerTiles *pLayer) override;
+	void BrushDraw(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
+	void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect) override;
 	virtual bool ContainsElementWithId(int Id);
 };
 
@@ -1206,14 +1295,14 @@ public:
 	int m_SpeedupMaxSpeed;
 	int m_SpeedupAngle;
 
-	virtual void Resize(int NewW, int NewH);
-	virtual void Shift(int Direction);
-	virtual bool IsEmpty(CLayerTiles *pLayer);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
-	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
+	void Resize(int NewW, int NewH) override;
+	void Shift(int Direction) override;
+	bool IsEmpty(CLayerTiles *pLayer) override;
+	void BrushDraw(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
+	void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect) override;
 };
 
 class CLayerFront : public CLayerTiles
@@ -1221,8 +1310,8 @@ class CLayerFront : public CLayerTiles
 public:
 	CLayerFront(int w, int h);
 
-	virtual void Resize(int NewW, int NewH);
-	virtual void SetTile(int x, int y, CTile tile);
+	void Resize(int NewW, int NewH) override;
+	void SetTile(int x, int y, CTile tile) override;
 };
 
 class CLayerSwitch : public CLayerTiles
@@ -1235,14 +1324,14 @@ public:
 	unsigned char m_SwitchNumber;
 	unsigned char m_SwitchDelay;
 
-	virtual void Resize(int NewW, int NewH);
-	virtual void Shift(int Direction);
-	virtual bool IsEmpty(CLayerTiles *pLayer);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
-	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
+	void Resize(int NewW, int NewH) override;
+	void Shift(int Direction) override;
+	bool IsEmpty(CLayerTiles *pLayer) override;
+	void BrushDraw(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
+	void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect) override;
 	virtual bool ContainsElementWithId(int Id);
 };
 
@@ -1255,36 +1344,39 @@ public:
 	CTuneTile *m_pTuneTile;
 	unsigned char m_TuningNumber;
 
-	virtual void Resize(int NewW, int NewH);
-	virtual void Shift(int Direction);
-	virtual bool IsEmpty(CLayerTiles *pLayer);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
-	virtual void BrushFlipX();
-	virtual void BrushFlipY();
-	virtual void BrushRotate(float Amount);
-	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
+	void Resize(int NewW, int NewH) override;
+	void Shift(int Direction) override;
+	bool IsEmpty(CLayerTiles *pLayer) override;
+	void BrushDraw(CLayer *pBrush, float wx, float wy) override;
+	void BrushFlipX() override;
+	void BrushFlipY() override;
+	void BrushRotate(float Amount) override;
+	void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect) override;
 };
 
 class CLayerSounds : public CLayer
 {
 public:
 	CLayerSounds();
+	CLayerSounds(const CLayerSounds &Other);
 	~CLayerSounds();
 
-	virtual void Render(bool Tileset = false);
+	void Render(bool Tileset = false) override;
 	CSoundSource *NewSource(int x, int y);
 
-	virtual void BrushSelecting(CUIRect Rect);
-	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
-	virtual void BrushPlace(CLayer *pBrush, float wx, float wy);
+	void BrushSelecting(CUIRect Rect) override;
+	int BrushGrab(CLayerGroup *pBrush, CUIRect Rect) override;
+	void BrushPlace(CLayer *pBrush, float wx, float wy) override;
 
-	virtual int RenderProperties(CUIRect *pToolbox);
+	int RenderProperties(CUIRect *pToolbox) override;
 
-	virtual void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc);
-	virtual void ModifySoundIndex(INDEX_MODIFY_FUNC pfnFunc);
+	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc) override;
+	void ModifySoundIndex(INDEX_MODIFY_FUNC pfnFunc) override;
+
+	CLayer *Duplicate() const override;
 
 	int m_Sound;
-	array<CSoundSource> m_lSources;
+	std::vector<CSoundSource> m_vSources;
 };
 
 #endif
