@@ -2,6 +2,10 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include "gamecore.h"
 
+#include "collision.h"
+#include "mapitems.h"
+#include "teamscore.h"
+
 #include <engine/shared/config.h>
 
 const char *CTuningParams::ms_apNames[] =
@@ -30,7 +34,7 @@ bool CTuningParams::Get(int Index, float *pValue) const
 bool CTuningParams::Set(const char *pName, float Value)
 {
 	for(int i = 0; i < Num(); i++)
-		if(str_comp_nocase(pName, ms_apNames[i]) == 0)
+		if(str_comp_nocase(pName, Name(i)) == 0)
 			return Set(i, Value);
 	return false;
 }
@@ -38,15 +42,38 @@ bool CTuningParams::Set(const char *pName, float Value)
 bool CTuningParams::Get(const char *pName, float *pValue) const
 {
 	for(int i = 0; i < Num(); i++)
-		if(str_comp_nocase(pName, ms_apNames[i]) == 0)
+		if(str_comp_nocase(pName, Name(i)) == 0)
 			return Get(i, pValue);
 
 	return false;
 }
 
-float HermiteBasis1(float v)
+int CTuningParams::PossibleTunings(const char *pStr, IConsole::FPossibleCallback pfnCallback, void *pUser)
 {
-	return 2 * v * v * v - 3 * v * v + 1;
+	int Index = 0;
+	for(int i = 0; i < Num(); i++)
+	{
+		if(str_find_nocase(Name(i), pStr))
+		{
+			pfnCallback(Index, Name(i), pUser);
+			Index++;
+		}
+	}
+	return Index;
+}
+
+float CTuningParams::GetWeaponFireDelay(int Weapon) const
+{
+	switch(Weapon)
+	{
+	case WEAPON_HAMMER: return (float)m_HammerHitFireDelay / 1000.0f;
+	case WEAPON_GUN: return (float)m_GunFireDelay / 1000.0f;
+	case WEAPON_SHOTGUN: return (float)m_ShotgunFireDelay / 1000.0f;
+	case WEAPON_GRENADE: return (float)m_GrenadeFireDelay / 1000.0f;
+	case WEAPON_LASER: return (float)m_LaserFireDelay / 1000.0f;
+	case WEAPON_NINJA: return (float)m_NinjaFireDelay / 1000.0f;
+	default: dbg_assert(false, "invalid weapon"); return 0.0f; // this value should not be reached
+	}
 }
 
 float VelocityRamp(float Value, float Start, float Range, float Curvature)
@@ -66,7 +93,7 @@ void CCharacterCore::Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore
 	m_Id = -1;
 
 	// fail safe, if core's tuning didn't get updated at all, just fallback to world tuning.
-	m_Tuning = m_pWorld->m_Tuning[g_Config.m_ClDummy];
+	m_Tuning = m_pWorld->m_aTuning[g_Config.m_ClDummy];
 	Reset();
 }
 
@@ -79,30 +106,31 @@ void CCharacterCore::Reset()
 	m_HookDir = vec2(0, 0);
 	m_HookTick = 0;
 	m_HookState = HOOK_IDLE;
-	m_HookedPlayer = -1;
+	SetHookedPlayer(-1);
+	m_AttachedPlayers.clear();
 	m_Jumped = 0;
 	m_JumpedTotal = 0;
 	m_Jumps = 2;
 	m_TriggeredEvents = 0;
-	m_Hook = true;
-	m_Collision = true;
 
 	// DDNet Character
 	m_Solo = false;
 	m_Jetpack = false;
-	m_NoCollision = false;
+	m_CollisionDisabled = false;
 	m_EndlessHook = false;
 	m_EndlessJump = false;
-	m_NoHammerHit = false;
-	m_NoGrenadeHit = false;
-	m_NoLaserHit = false;
-	m_NoShotgunHit = false;
-	m_NoHookHit = false;
+	m_HammerHitDisabled = false;
+	m_GrenadeHitDisabled = false;
+	m_LaserHitDisabled = false;
+	m_ShotgunHitDisabled = false;
+	m_HookHitDisabled = false;
 	m_Super = false;
 	m_HasTelegunGun = false;
 	m_HasTelegunGrenade = false;
 	m_HasTelegunLaser = false;
+	m_FreezeStart = 0;
 	m_FreezeEnd = 0;
+	m_IsInFreeze = false;
 	m_DeepFrozen = false;
 	m_LiveFrozen = false;
 
@@ -111,19 +139,13 @@ void CCharacterCore::Reset()
 	m_Input.m_TargetY = -1;
 }
 
-void CCharacterCore::Tick(bool UseInput)
+void CCharacterCore::Tick(bool UseInput, bool DoDeferredTick)
 {
-	float PhysSize = 28.0f;
 	m_MoveRestrictions = m_pCollision->GetMoveRestrictions(UseInput ? IsSwitchActiveCb : 0, this, m_Pos);
 	m_TriggeredEvents = 0;
 
 	// get ground state
-	bool Grounded = false;
-	if(m_pCollision->CheckPoint(m_Pos.x + PhysSize / 2, m_Pos.y + PhysSize / 2 + 5))
-		Grounded = true;
-	if(m_pCollision->CheckPoint(m_Pos.x - PhysSize / 2, m_Pos.y + PhysSize / 2 + 5))
-		Grounded = true;
-
+	const bool Grounded = m_pCollision->CheckPoint(m_Pos.x + PhysicalSize() / 2, m_Pos.y + PhysicalSize() / 2 + 5) || m_pCollision->CheckPoint(m_Pos.x - PhysicalSize() / 2, m_Pos.y + PhysicalSize() / 2 + 5);
 	vec2 TargetDirection = normalize(vec2(m_Input.m_TargetX, m_Input.m_TargetY));
 
 	m_Vel.y += m_Tuning.m_Gravity;
@@ -138,28 +160,40 @@ void CCharacterCore::Tick(bool UseInput)
 		m_Direction = m_Input.m_Direction;
 
 		// setup angle
-		float a = 0;
-		if(m_Input.m_TargetX == 0)
-			a = atanf((float)m_Input.m_TargetY);
+		float TmpAngle = atan2f(m_Input.m_TargetY, m_Input.m_TargetX);
+		if(TmpAngle < -(pi / 2.0f))
+		{
+			m_Angle = (int)((TmpAngle + (2.0f * pi)) * 256.0f);
+		}
 		else
-			a = atanf((float)m_Input.m_TargetY / (float)m_Input.m_TargetX);
+		{
+			m_Angle = (int)(TmpAngle * 256.0f);
+		}
 
-		if(m_Input.m_TargetX < 0)
-			a = a + pi;
-
-		m_Angle = (int)(a * 256.0f);
+		// Special jump cases:
+		// m_Jumps == -1: A tee may only make one ground jump. Second jumped bit is always set
+		// m_Jumps == 0: A tee may not make a jump. Second jumped bit is always set
+		// m_Jumps == 1: A tee may do either a ground jump or an air jump. Second jumped bit is set after the first jump
+		// The second jumped bit can be overridden by special tiles so that the tee can nevertheless jump.
 
 		// handle jump
 		if(m_Input.m_Jump)
 		{
 			if(!(m_Jumped & 1))
 			{
-				if(Grounded)
+				if(Grounded && (!(m_Jumped & 2) || m_Jumps != 0))
 				{
 					m_TriggeredEvents |= COREEVENT_GROUND_JUMP;
 					m_Vel.y = -m_Tuning.m_GroundJumpImpulse;
-					m_Jumped |= 1;
-					m_JumpedTotal = 1;
+					if(m_Jumps > 1)
+					{
+						m_Jumped |= 1;
+					}
+					else
+					{
+						m_Jumped |= 3;
+					}
+					m_JumpedTotal = 0;
 				}
 				else if(!(m_Jumped & 2))
 				{
@@ -171,7 +205,9 @@ void CCharacterCore::Tick(bool UseInput)
 			}
 		}
 		else
+		{
 			m_Jumped &= ~1;
+		}
 
 		// handle hook
 		if(m_Input.m_Hook)
@@ -179,21 +215,30 @@ void CCharacterCore::Tick(bool UseInput)
 			if(m_HookState == HOOK_IDLE)
 			{
 				m_HookState = HOOK_FLYING;
-				m_HookPos = m_Pos + TargetDirection * PhysSize * 1.5f;
+				m_HookPos = m_Pos + TargetDirection * PhysicalSize() * 1.5f;
 				m_HookDir = TargetDirection;
-				m_HookedPlayer = -1;
+				SetHookedPlayer(-1);
 				m_HookTick = (float)SERVER_TICK_SPEED * (1.25f - m_Tuning.m_HookDuration);
 				m_TriggeredEvents |= COREEVENT_HOOK_LAUNCH;
 			}
 		}
 		else
 		{
-			m_HookedPlayer = -1;
+			SetHookedPlayer(-1);
 			m_HookState = HOOK_IDLE;
 			m_HookPos = m_Pos;
 		}
 	}
 
+	// handle jumping
+	// 1 bit = to keep track if a jump has been made on this input (player is holding space bar)
+	// 2 bit = to track if all air-jumps have been used up (tee gets dark feet)
+	if(Grounded)
+	{
+		m_Jumped &= ~2;
+		m_JumpedTotal = 0;
+	}
+
 	// add the speed modification according to players wanted direction
 	if(m_Direction < 0)
 		m_Vel.x = SaturatedAdd(-MaxSpeed, MaxSpeed, m_Vel.x, -Accel);
@@ -202,19 +247,10 @@ void CCharacterCore::Tick(bool UseInput)
 	if(m_Direction == 0)
 		m_Vel.x *= Friction;
 
-	// handle jumping
-	// 1 bit = to keep track if a jump has been made on this input (player is holding space bar)
-	// 2 bit = to keep track if a air-jump has been made (tee gets dark feet)
-	if(Grounded)
-	{
-		m_Jumped &= ~2;
-		m_JumpedTotal = 0;
-	}
-
 	// do hook
 	if(m_HookState == HOOK_IDLE)
 	{
-		m_HookedPlayer = -1;
+		SetHookedPlayer(-1);
 		m_HookState = HOOK_IDLE;
 		m_HookPos = m_Pos;
 	}
@@ -224,7 +260,6 @@ void CCharacterCore::Tick(bool UseInput)
 	}
 	else if(m_HookState == HOOK_RETRACT_END)
 	{
-		m_HookState = HOOK_RETRACTED;
 		m_TriggeredEvents |= COREEVENT_HOOK_RETRACT;
 		m_HookState = HOOK_RETRACTED;
 	}
@@ -235,7 +270,7 @@ void CCharacterCore::Tick(bool UseInput)
 		{
 			m_HookState = HOOK_RETRACT_START;
 			NewPos = m_Pos + normalize(NewPos - m_Pos) * m_Tuning.m_HookLength;
-			m_pReset = true;
+			m_Reset = true;
 		}
 
 		// make sure that the hook doesn't go though the ground
@@ -245,7 +280,7 @@ void CCharacterCore::Tick(bool UseInput)
 		int teleNr = 0;
 		int Hit = m_pCollision->IntersectLineTeleHook(m_HookPos, NewPos, &NewPos, 0, &teleNr);
 
-		//m_NewHook = false;
+		// m_NewHook = false;
 
 		if(Hit)
 		{
@@ -255,11 +290,11 @@ void CCharacterCore::Tick(bool UseInput)
 				GoingThroughTele = true;
 			else
 				GoingToHitGround = true;
-			m_pReset = true;
+			m_Reset = true;
 		}
 
 		// Check against other players first
-		if(this->m_Hook && m_pWorld && m_Tuning.m_PlayerHooking)
+		if(!this->m_HookHitDisabled && m_pWorld && m_Tuning.m_PlayerHooking)
 		{
 			float Distance = 0.0f;
 			for(int i = 0; i < MAX_CLIENTS; i++)
@@ -271,13 +306,13 @@ void CCharacterCore::Tick(bool UseInput)
 				vec2 ClosestPoint;
 				if(closest_point_on_line(m_HookPos, NewPos, pCharCore->m_Pos, ClosestPoint))
 				{
-					if(distance(pCharCore->m_Pos, ClosestPoint) < PhysSize + 2.0f)
+					if(distance(pCharCore->m_Pos, ClosestPoint) < PhysicalSize() + 2.0f)
 					{
 						if(m_HookedPlayer == -1 || distance(m_HookPos, pCharCore->m_Pos) < Distance)
 						{
 							m_TriggeredEvents |= COREEVENT_HOOK_ATTACH_PLAYER;
 							m_HookState = HOOK_GRABBED;
-							m_HookedPlayer = i;
+							SetHookedPlayer(i);
 							Distance = distance(m_HookPos, pCharCore->m_Pos);
 						}
 					}
@@ -302,11 +337,11 @@ void CCharacterCore::Tick(bool UseInput)
 			if(GoingThroughTele && m_pWorld && m_pTeleOuts && !m_pTeleOuts->empty() && !(*m_pTeleOuts)[teleNr - 1].empty())
 			{
 				m_TriggeredEvents = 0;
-				m_HookedPlayer = -1;
+				SetHookedPlayer(-1);
 
 				m_NewHook = true;
 				int RandomOut = m_pWorld->RandomOr0((*m_pTeleOuts)[teleNr - 1].size());
-				m_HookPos = (*m_pTeleOuts)[teleNr - 1][RandomOut] + TargetDirection * PhysSize * 1.5f;
+				m_HookPos = (*m_pTeleOuts)[teleNr - 1][RandomOut] + TargetDirection * PhysicalSize() * 1.5f;
 				m_HookDir = TargetDirection;
 				m_HookTeleBase = m_HookPos;
 			}
@@ -327,14 +362,14 @@ void CCharacterCore::Tick(bool UseInput)
 			else
 			{
 				// release hook
-				m_HookedPlayer = -1;
+				SetHookedPlayer(-1);
 				m_HookState = HOOK_RETRACTED;
 				m_HookPos = m_Pos;
 			}
 
 			// keep players hooked for a max of 1.5sec
-			//if(Server()->Tick() > hook_tick+(Server()->TickSpeed()*3)/2)
-			//release_hooked();
+			// if(Server()->Tick() > hook_tick+(Server()->TickSpeed()*3)/2)
+			// release_hooked();
 		}
 
 		// don't do this hook rutine when we are hook to a player
@@ -364,12 +399,18 @@ void CCharacterCore::Tick(bool UseInput)
 		m_HookTick++;
 		if(m_HookedPlayer != -1 && (m_HookTick > SERVER_TICK_SPEED + SERVER_TICK_SPEED / 5 || (m_pWorld && !m_pWorld->m_apCharacters[m_HookedPlayer])))
 		{
-			m_HookedPlayer = -1;
+			SetHookedPlayer(-1);
 			m_HookState = HOOK_RETRACTED;
 			m_HookPos = m_Pos;
 		}
 	}
 
+	if(DoDeferredTick)
+		TickDeferred();
+}
+
+void CCharacterCore::TickDeferred()
+{
 	if(m_pWorld)
 	{
 		for(int i = 0; i < MAX_CLIENTS; i++)
@@ -378,8 +419,8 @@ void CCharacterCore::Tick(bool UseInput)
 			if(!pCharCore)
 				continue;
 
-			//player *p = (player*)ent;
-			//if(pCharCore == this) // || !(p->flags&FLAG_ALIVE)
+			// player *p = (player*)ent;
+			// if(pCharCore == this) // || !(p->flags&FLAG_ALIVE)
 
 			if(pCharCore == this || (m_Id != -1 && !m_pTeams->CanCollide(m_Id, i)))
 				continue; // make sure that we don't nudge our self
@@ -393,26 +434,26 @@ void CCharacterCore::Tick(bool UseInput)
 			{
 				vec2 Dir = normalize(m_Pos - pCharCore->m_Pos);
 
-				bool CanCollide = (m_Super || pCharCore->m_Super) || (pCharCore->m_Collision && m_Collision && !m_NoCollision && !pCharCore->m_NoCollision && m_Tuning.m_PlayerCollision);
+				bool CanCollide = (m_Super || pCharCore->m_Super) || (!m_CollisionDisabled && !pCharCore->m_CollisionDisabled && m_Tuning.m_PlayerCollision);
 
-				if(CanCollide && Distance < PhysSize * 1.25f && Distance > 0.0f)
+				if(CanCollide && Distance < PhysicalSize() * 1.25f && Distance > 0.0f)
 				{
-					float a = (PhysSize * 1.45f - Distance);
+					float a = (PhysicalSize() * 1.45f - Distance);
 					float Velocity = 0.5f;
 
 					// make sure that we don't add excess force by checking the
 					// direction against the current velocity. if not zero.
-					if(length(m_Vel) > 0.0001)
-						Velocity = 1 - (dot(normalize(m_Vel), Dir) + 1) / 2;
+					if(length(m_Vel) > 0.0001f)
+						Velocity = 1 - (dot(normalize(m_Vel), Dir) + 1) / 2; // Wdouble-promotion don't fix this as this might change game physics
 
 					m_Vel += Dir * a * (Velocity * 0.75f);
 					m_Vel *= 0.85f;
 				}
 
 				// handle hook influence
-				if(m_Hook && m_HookedPlayer == i && m_Tuning.m_PlayerHooking)
+				if(!m_HookHitDisabled && m_HookedPlayer == i && m_Tuning.m_PlayerHooking)
 				{
-					if(Distance > PhysSize * 1.50f) // TODO: fix tweakable variable
+					if(Distance > PhysicalSize() * 1.50f) // TODO: fix tweakable variable
 					{
 						float HookAccel = m_Tuning.m_HookDragAccel * (Distance / m_Tuning.m_HookLength);
 						float DragSpeed = m_Tuning.m_HookDragSpeed;
@@ -451,7 +492,7 @@ void CCharacterCore::Move()
 	vec2 NewPos = m_Pos;
 
 	vec2 OldVel = m_Vel;
-	m_pCollision->MoveBox(&NewPos, &m_Vel, vec2(28.0f, 28.0f), 0);
+	m_pCollision->MoveBox(&NewPos, &m_Vel, PhysicalSizeVec2(), 0);
 
 	m_Colliding = 0;
 	if(m_Vel.x < 0.001f && m_Vel.x > -0.001f)
@@ -466,7 +507,7 @@ void CCharacterCore::Move()
 
 	m_Vel.x = m_Vel.x * (1.0f / RampValue);
 
-	if(m_pWorld && (m_Super || (m_Tuning.m_PlayerCollision && m_Collision && !m_NoCollision && !m_Solo)))
+	if(m_pWorld && (m_Super || (m_Tuning.m_PlayerCollision && !m_CollisionDisabled && !m_Solo)))
 	{
 		// check player collision
 		float Distance = distance(m_Pos, NewPos);
@@ -483,10 +524,10 @@ void CCharacterCore::Move()
 					CCharacterCore *pCharCore = m_pWorld->m_apCharacters[p];
 					if(!pCharCore || pCharCore == this)
 						continue;
-					if((!(pCharCore->m_Super || m_Super) && (m_Solo || pCharCore->m_Solo || !pCharCore->m_Collision || pCharCore->m_NoCollision || (m_Id != -1 && !m_pTeams->CanCollide(m_Id, p)))))
+					if((!(pCharCore->m_Super || m_Super) && (m_Solo || pCharCore->m_Solo || pCharCore->m_CollisionDisabled || (m_Id != -1 && !m_pTeams->CanCollide(m_Id, p)))))
 						continue;
 					float D = distance(Pos, pCharCore->m_Pos);
-					if(D < 28.0f && D >= 0.0f)
+					if(D < PhysicalSize() && D >= 0.0f)
 					{
 						if(a > 0.0f)
 							m_Pos = LastPos;
@@ -534,7 +575,7 @@ void CCharacterCore::Read(const CNetObj_CharacterCore *pObjCore)
 	m_HookPos.y = pObjCore->m_HookY;
 	m_HookDir.x = pObjCore->m_HookDx / 256.0f;
 	m_HookDir.y = pObjCore->m_HookDy / 256.0f;
-	m_HookedPlayer = pObjCore->m_HookedPlayer;
+	SetHookedPlayer(pObjCore->m_HookedPlayer);
 	m_Jumped = pObjCore->m_Jumped;
 	m_Direction = pObjCore->m_Direction;
 	m_Angle = pObjCore->m_Angle;
@@ -545,17 +586,14 @@ void CCharacterCore::ReadDDNet(const CNetObj_DDNetCharacter *pObjDDNet)
 	// Collision
 	m_Solo = pObjDDNet->m_Flags & CHARACTERFLAG_SOLO;
 	m_Jetpack = pObjDDNet->m_Flags & CHARACTERFLAG_JETPACK;
-	m_NoCollision = pObjDDNet->m_Flags & CHARACTERFLAG_NO_COLLISION;
-	m_NoHammerHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_HAMMER_HIT;
-	m_NoGrenadeHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_GRENADE_HIT;
-	m_NoLaserHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_LASER_HIT;
-	m_NoShotgunHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_SHOTGUN_HIT;
-	m_NoHookHit = pObjDDNet->m_Flags & CHARACTERFLAG_NO_HOOK;
+	m_CollisionDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_COLLISION_DISABLED;
+	m_HammerHitDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_HAMMER_HIT_DISABLED;
+	m_ShotgunHitDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_SHOTGUN_HIT_DISABLED;
+	m_GrenadeHitDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_GRENADE_HIT_DISABLED;
+	m_LaserHitDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_LASER_HIT_DISABLED;
+	m_HookHitDisabled = pObjDDNet->m_Flags & CHARACTERFLAG_HOOK_HIT_DISABLED;
 	m_Super = pObjDDNet->m_Flags & CHARACTERFLAG_SUPER;
 
-	m_Hook = !m_NoHookHit;
-	m_Collision = !m_NoCollision;
-
 	// Endless
 	m_EndlessHook = pObjDDNet->m_Flags & CHARACTERFLAG_ENDLESS_HOOK;
 	m_EndlessJump = pObjDDNet->m_Flags & CHARACTERFLAG_ENDLESS_JUMP;
@@ -563,14 +601,39 @@ void CCharacterCore::ReadDDNet(const CNetObj_DDNetCharacter *pObjDDNet)
 	// Freeze
 	m_FreezeEnd = pObjDDNet->m_FreezeEnd;
 	m_DeepFrozen = pObjDDNet->m_FreezeEnd == -1;
-	m_LiveFrozen = (pObjDDNet->m_Flags & CHARACTERFLAG_NO_MOVEMENTS) != 0;
+	m_LiveFrozen = (pObjDDNet->m_Flags & CHARACTERFLAG_MOVEMENTS_DISABLED) != 0;
 
 	// Telegun
 	m_HasTelegunGrenade = pObjDDNet->m_Flags & CHARACTERFLAG_TELEGUN_GRENADE;
 	m_HasTelegunGun = pObjDDNet->m_Flags & CHARACTERFLAG_TELEGUN_GUN;
 	m_HasTelegunLaser = pObjDDNet->m_Flags & CHARACTERFLAG_TELEGUN_LASER;
 
+	// Weapons
+	m_aWeapons[WEAPON_HAMMER].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_HAMMER) != 0;
+	m_aWeapons[WEAPON_GUN].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_GUN) != 0;
+	m_aWeapons[WEAPON_SHOTGUN].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_SHOTGUN) != 0;
+	m_aWeapons[WEAPON_GRENADE].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_GRENADE) != 0;
+	m_aWeapons[WEAPON_LASER].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_LASER) != 0;
+	m_aWeapons[WEAPON_NINJA].m_Got = (pObjDDNet->m_Flags & CHARACTERFLAG_WEAPON_NINJA) != 0;
+
+	// Available jumps
 	m_Jumps = pObjDDNet->m_Jumps;
+
+	// Display Information
+	// We only accept the display information when it is received, which means it is not -1 in each case.
+	if(pObjDDNet->m_JumpedTotal != -1)
+	{
+		m_JumpedTotal = pObjDDNet->m_JumpedTotal;
+	}
+	if(pObjDDNet->m_NinjaActivationTick != -1)
+	{
+		m_Ninja.m_ActivationTick = pObjDDNet->m_NinjaActivationTick;
+	}
+	if(pObjDDNet->m_FreezeStart != -1)
+	{
+		m_FreezeStart = pObjDDNet->m_FreezeStart;
+		m_IsInFreeze = pObjDDNet->m_Flags & CHARACTERFLAG_IN_FREEZE;
+	}
 }
 
 void CCharacterCore::Quantize()
@@ -580,6 +643,30 @@ void CCharacterCore::Quantize()
 	Read(&Core);
 }
 
+void CCharacterCore::SetHookedPlayer(int HookedPlayer)
+{
+	if(HookedPlayer != m_HookedPlayer)
+	{
+		if(m_HookedPlayer != -1 && m_Id != -1 && m_pWorld)
+		{
+			CCharacterCore *pCharCore = m_pWorld->m_apCharacters[m_HookedPlayer];
+			if(pCharCore)
+			{
+				pCharCore->m_AttachedPlayers.erase(m_Id);
+			}
+		}
+		if(HookedPlayer != -1 && m_Id != -1 && m_pWorld)
+		{
+			CCharacterCore *pCharCore = m_pWorld->m_apCharacters[HookedPlayer];
+			if(pCharCore)
+			{
+				pCharCore->m_AttachedPlayers.insert(m_Id);
+			}
+		}
+		m_HookedPlayer = HookedPlayer;
+	}
+}
+
 // DDRace
 
 void CCharacterCore::SetTeamsCore(CTeamsCore *pTeams)
@@ -595,8 +682,28 @@ void CCharacterCore::SetTeleOuts(std::map<int, std::vector<vec2>> *pTeleOuts)
 bool CCharacterCore::IsSwitchActiveCb(int Number, void *pUser)
 {
 	CCharacterCore *pThis = (CCharacterCore *)pUser;
-	if(pThis->Collision()->m_pSwitchers)
+	if(pThis->m_pWorld && !pThis->m_pWorld->m_vSwitchers.empty())
 		if(pThis->m_Id != -1 && pThis->m_pTeams->Team(pThis->m_Id) != (pThis->m_pTeams->m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER))
-			return pThis->Collision()->m_pSwitchers[Number].m_Status[pThis->m_pTeams->Team(pThis->m_Id)];
+			return pThis->m_pWorld->m_vSwitchers[Number].m_aStatus[pThis->m_pTeams->Team(pThis->m_Id)];
 	return false;
 }
+
+void CWorldCore::InitSwitchers(int HighestSwitchNumber)
+{
+	if(HighestSwitchNumber > 0)
+		m_vSwitchers.resize(HighestSwitchNumber + 1);
+	else
+		m_vSwitchers.clear();
+
+	for(auto &Switcher : m_vSwitchers)
+	{
+		Switcher.m_Initial = true;
+		for(int j = 0; j < MAX_CLIENTS; j++)
+		{
+			Switcher.m_aStatus[j] = true;
+			Switcher.m_aEndTick[j] = 0;
+			Switcher.m_aType[j] = 0;
+			Switcher.m_aLastUpdateTick[j] = 0;
+		}
+	}
+}
