@@ -3,20 +3,21 @@
 #ifndef GAME_GAMECORE_H
 #define GAME_GAMECORE_H
 
-#include <base/math.h>
 #include <base/system.h>
+#include <base/vmath.h>
 
 #include <map>
+#include <set>
 #include <vector>
 
-#include "collision.h"
+#include <engine/console.h>
 #include <engine/shared/protocol.h>
 #include <game/generated/protocol.h>
-#include <math.h>
 
-#include "mapitems.h"
 #include "prng.h"
-#include "teamscore.h"
+
+class CCollision;
+class CTeamsCore;
 
 class CTuneParam
 {
@@ -40,17 +41,17 @@ public:
 
 class CTuningParams
 {
+	static const char *ms_apNames[];
+
 public:
 	CTuningParams()
 	{
 		const float TicksPerSecond = 50.0f;
-#define MACRO_TUNING_PARAM(Name, ScriptName, Value, Description) m_##Name.Set((int)(Value * 100.0f));
+#define MACRO_TUNING_PARAM(Name, ScriptName, Value, Description) m_##Name.Set((int)((Value)*100.0f));
 #include "tuning.h"
 #undef MACRO_TUNING_PARAM
 	}
 
-	static const char *ms_apNames[];
-
 #define MACRO_TUNING_PARAM(Name, ScriptName, Value, Description) CTuneParam m_##Name;
 #include "tuning.h"
 #undef MACRO_TUNING_PARAM
@@ -63,6 +64,9 @@ public:
 	bool Set(const char *pName, float Value);
 	bool Get(int Index, float *pValue) const;
 	bool Get(const char *pName, float *pValue) const;
+	static const char *Name(int Index) { return ms_apNames[Index]; }
+	int PossibleTunings(const char *pStr, IConsole::FPossibleCallback pfnCallback = IConsole::EmptyPossibleCommandCallback, void *pUser = nullptr);
+	float GetWeaponFireDelay(int Weapon) const;
 };
 
 inline void StrToInts(int *pInts, int Num, const char *pStr)
@@ -164,7 +168,7 @@ enum
 	COREEVENT_HOOK_ATTACH_GROUND = 0x10,
 	COREEVENT_HOOK_HIT_NOHOOK = 0x20,
 	COREEVENT_HOOK_RETRACT = 0x40,
-	//COREEVENT_HOOK_TELE=0x80,
+	// COREEVENT_HOOK_TELE=0x80,
 };
 
 // show others values - do not change them
@@ -176,13 +180,22 @@ enum
 	SHOW_OTHERS_ONLY_TEAM = 2 // show players that are in solo and are in the same team
 };
 
+struct SSwitchers
+{
+	bool m_aStatus[MAX_CLIENTS];
+	bool m_Initial;
+	int m_aEndTick[MAX_CLIENTS];
+	int m_aType[MAX_CLIENTS];
+	int m_aLastUpdateTick[MAX_CLIENTS];
+};
+
 class CWorldCore
 {
 public:
 	CWorldCore()
 	{
 		mem_zero(m_apCharacters, sizeof(m_apCharacters));
-		m_pPrng = 0;
+		m_pPrng = nullptr;
 	}
 
 	int RandomOr0(int BelowThis)
@@ -197,23 +210,26 @@ public:
 		return m_pPrng->RandomBits() % BelowThis;
 	}
 
-	CTuningParams m_Tuning[2];
+	CTuningParams m_aTuning[2];
 	class CCharacterCore *m_apCharacters[MAX_CLIENTS];
 	CPrng *m_pPrng;
+
+	void InitSwitchers(int HighestSwitchNumber);
+	std::vector<SSwitchers> m_vSwitchers;
 };
 
 class CCharacterCore
 {
 	friend class CCharacter;
-	CWorldCore *m_pWorld;
+	CWorldCore *m_pWorld = nullptr;
 	CCollision *m_pCollision;
 	std::map<int, std::vector<vec2>> *m_pTeleOuts;
 
 public:
+	static constexpr float PhysicalSize() { return 28.0f; };
+	static constexpr vec2 PhysicalSizeVec2() { return vec2(28.0f, 28.0f); };
 	vec2 m_Pos;
 	vec2 m_Vel;
-	bool m_Hook;
-	bool m_Collision;
 
 	vec2 m_HookPos;
 	vec2 m_HookDir;
@@ -221,11 +237,31 @@ public:
 	int m_HookTick;
 	int m_HookState;
 	int m_HookedPlayer;
+	std::set<int> m_AttachedPlayers;
+	void SetHookedPlayer(int HookedPlayer);
+
 	int m_ActiveWeapon;
+	struct WeaponStat
+	{
+		int m_AmmoRegenStart;
+		int m_Ammo;
+		int m_Ammocost;
+		bool m_Got;
+	} m_aWeapons[NUM_WEAPONS];
+
+	// ninja
+	struct
+	{
+		vec2 m_ActivationDir;
+		int m_ActivationTick;
+		int m_CurrentMoveTime;
+		int m_OldVelAmount;
+	} m_Ninja;
 
 	bool m_NewHook;
 
 	int m_Jumped;
+	// m_JumpedTotal counts the jumps performed in the air
 	int m_JumpedTotal;
 	int m_Jumps;
 
@@ -237,7 +273,8 @@ public:
 
 	void Init(CWorldCore *pWorld, CCollision *pCollision, CTeamsCore *pTeams = nullptr, std::map<int, std::vector<vec2>> *pTeleOuts = nullptr);
 	void Reset();
-	void Tick(bool UseInput);
+	void TickDeferred();
+	void Tick(bool UseInput, bool DoDeferredTick = true);
 	void Move();
 
 	void Read(const CNetObj_CharacterCore *pObjCore);
@@ -245,10 +282,9 @@ public:
 	void Quantize();
 
 	// DDRace
-
 	int m_Id;
-	bool m_pReset;
-	class CCollision *Collision() { return m_pCollision; }
+	bool m_Reset;
+	CCollision *Collision() { return m_pCollision; }
 
 	vec2 m_LastVel;
 	int m_Colliding;
@@ -260,19 +296,21 @@ public:
 	void ReadDDNet(const CNetObj_DDNetCharacter *pObjDDNet);
 	bool m_Solo;
 	bool m_Jetpack;
-	bool m_NoCollision;
+	bool m_CollisionDisabled;
 	bool m_EndlessHook;
 	bool m_EndlessJump;
-	bool m_NoHammerHit;
-	bool m_NoGrenadeHit;
-	bool m_NoLaserHit;
-	bool m_NoShotgunHit;
-	bool m_NoHookHit;
+	bool m_HammerHitDisabled;
+	bool m_GrenadeHitDisabled;
+	bool m_LaserHitDisabled;
+	bool m_ShotgunHitDisabled;
+	bool m_HookHitDisabled;
 	bool m_Super;
 	bool m_HasTelegunGun;
 	bool m_HasTelegunGrenade;
 	bool m_HasTelegunLaser;
+	int m_FreezeStart;
 	int m_FreezeEnd;
+	bool m_IsInFreeze;
 	bool m_DeepFrozen;
 	bool m_LiveFrozen;
 	CTuningParams m_Tuning;
@@ -283,7 +321,7 @@ private:
 	static bool IsSwitchActiveCb(int Number, void *pUser);
 };
 
-//input count
+// input count
 struct CInputCount
 {
 	int m_Presses;
