@@ -78,7 +78,7 @@ int CMapBugs::Update(const char *pBug)
 	int Bug = -1;
 	if(false) {} // NOLINT(readability-simplify-boolean-expr)
 #define MAPBUG(constname, string) \
-	else if(str_comp(pBug, string) == 0) { Bug = constname; }
+	else if(str_comp(pBug, string) == 0) { Bug = (constname); }
 #include "mapbugs_list.h"
 #undef MAPBUG
 	if(Bug == -1)
