@@ -3,12 +3,12 @@
 #ifndef GAME_SERVER_ENTITIES_CHARACTER_H
 #define GAME_SERVER_ENTITIES_CHARACTER_H
 
-#include <engine/antibot.h>
 #include <game/server/entity.h>
 #include <game/server/save.h>
 
-class CAntibot;
 class CGameTeams;
+class CGameWorld;
+class IAntibot;
 struct CAntibotCharacterData;
 
 enum
@@ -29,24 +29,27 @@ class CCharacter : public CEntity
 	friend class CSaveTee; // need to use core
 
 public:
-	//character's size
-	static const int ms_PhysSize = 28;
+	CCharacter(CGameWorld *pWorld, CNetObj_PlayerInput LastInput);
 
-	CCharacter(CGameWorld *pWorld);
-
-	virtual void Reset();
-	virtual void Destroy();
-	virtual void Tick();
-	virtual void TickDefered();
-	virtual void TickPaused();
-	virtual void Snap(int SnappingClient);
+	void Reset() override;
+	void Destroy() override;
+	void PreTick() override;
+	void Tick() override;
+	void TickDeferred() override;
+	void TickPaused() override;
+	void Snap(int SnappingClient) override;
+	void SwapClients(int Client1, int Client2) override;
 
 	bool CanSnapCharacter(int SnappingClient);
 
 	bool IsGrounded();
 
 	void SetWeapon(int W);
+	void SetJetpack(bool Active);
 	void SetSolo(bool Solo);
+	void SetSuper(bool Super);
+	void SetLiveFrozen(bool Active);
+	void SetDeepFrozen(bool Active);
 	void HandleWeaponSwitch();
 	void DoWeaponSwitch();
 
@@ -96,15 +99,6 @@ private:
 	CEntity *m_apHitObjects[10];
 	int m_NumObjectsHit;
 
-	struct WeaponStat
-	{
-		int m_AmmoRegenStart;
-		int m_Ammo;
-		int m_Ammocost;
-		bool m_Got;
-
-	} m_aWeapons[NUM_WEAPONS];
-
 	int m_LastWeapon;
 	int m_QueuedWeapon;
 
@@ -130,22 +124,12 @@ private:
 	CNetObj_PlayerInput m_Input;
 	CNetObj_PlayerInput m_SavedInput;
 	int m_NumInputs;
-	int m_Jumped;
 
 	int m_DamageTakenTick;
 
 	int m_Health;
 	int m_Armor;
 
-	// ninja
-	struct
-	{
-		vec2 m_ActivationDir;
-		int m_ActivationTick;
-		int m_CurrentMoveTime;
-		int m_OldVelAmount;
-	} m_Ninja;
-
 	// the player core for the physics
 	CCharacterCore m_Core;
 	CGameTeams *m_pTeams = nullptr;
@@ -162,6 +146,7 @@ private:
 
 	void SnapCharacter(int SnappingClient, int ID);
 	static bool IsSwitchActiveCb(int Number, void *pUser);
+	void SetTimeCheckpoint(int TimeCheckpoint);
 	void HandleTiles(int Index);
 	float m_Time;
 	int m_LastBroadcast;
@@ -177,7 +162,6 @@ private:
 
 	bool m_SetSavePos;
 	CSaveTee m_RescueTee;
-	bool m_Solo;
 
 public:
 	CGameTeams *Teams() { return m_pTeams; }
@@ -195,27 +179,11 @@ public:
 	int Team();
 	bool CanCollide(int ClientID);
 	bool SameTeam(int ClientID);
-	bool m_Super;
-	bool m_SuperJump;
-	bool m_Jetpack;
 	bool m_NinjaJetpack;
 	int m_TeamBeforeSuper;
 	int m_FreezeTime;
-	int m_FreezeTick;
 	bool m_FrozenLastTick;
-	bool m_DeepFreeze;
-	bool m_LiveFreeze;
-	bool m_EndlessHook;
 	bool m_FreezeHammer;
-	enum
-	{
-		HIT_ALL = 0,
-		DISABLE_HIT_HAMMER = 1,
-		DISABLE_HIT_SHOTGUN = 2,
-		DISABLE_HIT_GRENADE = 4,
-		DISABLE_HIT_LASER = 8
-	};
-	int m_Hit;
 	int m_TuneZone;
 	int m_TuneZoneOld;
 	int m_PainSoundTimer;
@@ -223,10 +191,12 @@ public:
 	int m_StartTime;
 	vec2 m_PrevPos;
 	int m_TeleCheckpoint;
-	int m_CpTick;
-	int m_CpActive;
-	int m_CpLastBroadcast;
-	float m_CpCurrent[25];
+
+	int m_TimeCpBroadcastEndTick;
+	int m_LastTimeCp;
+	int m_LastTimeCpBroadcasted;
+	float m_aCurrentTimeCp[MAX_CHECKPOINTS];
+
 	int m_TileIndex;
 	int m_TileFIndex;
 
@@ -257,14 +227,13 @@ public:
 	CCharacterCore GetCore() { return m_Core; }
 	void SetCore(CCharacterCore Core) { m_Core = Core; }
 	CCharacterCore *Core() { return &m_Core; }
-	bool GetWeaponGot(int Type) { return m_aWeapons[Type].m_Got; }
-	void SetWeaponGot(int Type, bool Value) { m_aWeapons[Type].m_Got = Value; }
-	int GetWeaponAmmo(int Type) { return m_aWeapons[Type].m_Ammo; }
-	void SetWeaponAmmo(int Type, int Value) { m_aWeapons[Type].m_Ammo = Value; }
-	bool IsAlive() { return m_Alive; }
-	void SetNinjaActivationDir(vec2 ActivationDir) { m_Ninja.m_ActivationDir = ActivationDir; }
-	void SetNinjaActivationTick(int ActivationTick) { m_Ninja.m_ActivationTick = ActivationTick; }
-	void SetNinjaCurrentMoveTime(int CurrentMoveTime) { m_Ninja.m_CurrentMoveTime = CurrentMoveTime; }
+	bool GetWeaponGot(int Type) { return m_Core.m_aWeapons[Type].m_Got; }
+	void SetWeaponGot(int Type, bool Value) { m_Core.m_aWeapons[Type].m_Got = Value; }
+	int GetWeaponAmmo(int Type) { return m_Core.m_aWeapons[Type].m_Ammo; }
+	void SetWeaponAmmo(int Type, int Value) { m_Core.m_aWeapons[Type].m_Ammo = Value; }
+	void SetNinjaActivationDir(vec2 ActivationDir) { m_Core.m_Ninja.m_ActivationDir = ActivationDir; }
+	void SetNinjaActivationTick(int ActivationTick) { m_Core.m_Ninja.m_ActivationTick = ActivationTick; }
+	void SetNinjaCurrentMoveTime(int CurrentMoveTime) { m_Core.m_Ninja.m_CurrentMoveTime = CurrentMoveTime; }
 
 	int GetLastAction() const { return m_LastAction; }
 
@@ -272,6 +241,13 @@ public:
 	bool HasTelegunGrenade() { return m_Core.m_HasTelegunGrenade; }
 	bool HasTelegunLaser() { return m_Core.m_HasTelegunLaser; }
 
+	bool HammerHitDisabled() { return m_Core.m_HammerHitDisabled; }
+	bool ShotgunHitDisabled() { return m_Core.m_ShotgunHitDisabled; }
+	bool LaserHitDisabled() { return m_Core.m_LaserHitDisabled; }
+	bool GrenadeHitDisabled() { return m_Core.m_GrenadeHitDisabled; }
+
+	bool IsSuper() { return m_Core.m_Super; }
+
 	CSaveTee &GetRescueTeeRef() { return m_RescueTee; }
 };
 
