@@ -1,17 +1,19 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include "pickup.h"
+#include "character.h"
+
 #include <game/generated/protocol.h>
+#include <game/mapitems.h>
+#include <game/teamscore.h>
+
 #include <game/server/gamecontext.h>
 #include <game/server/player.h>
 
-#include <game/server/teams.h>
-#include <game/version.h>
-
-#include "character.h"
+static constexpr int gs_PickupPhysSize = 14;
 
 CPickup::CPickup(CGameWorld *pGameWorld, int Type, int SubType, int Layer, int Number) :
-	CEntity(pGameWorld, CGameWorld::ENTTYPE_PICKUP, vec2(0, 0), PickupPhysSize)
+	CEntity(pGameWorld, CGameWorld::ENTTYPE_PICKUP, vec2(0, 0), gs_PickupPhysSize)
 {
 	m_Type = Type;
 	m_Subtype = SubType;
@@ -26,29 +28,12 @@ CPickup::CPickup(CGameWorld *pGameWorld, int Type, int SubType, int Layer, int N
 
 void CPickup::Reset()
 {
-	/*if (g_pData->m_aPickups[m_Type].m_Spawndelay > 0)
-		m_SpawnTick = Server()->Tick() + Server()->TickSpeed() * g_pData->m_aPickups[m_Type].m_Spawndelay;
-	else
-		m_SpawnTick = -1;*/
 }
 
 void CPickup::Tick()
 {
 	Move();
-	/*// wait for respawn
-	if(m_SpawnTick > 0)
-	{
-		if(Server()->Tick() > m_SpawnTick)
-		{
-			// respawn
-			m_SpawnTick = -1;
 
-			if(m_Type == POWERUP_WEAPON)
-				GameServer()->CreateSound(m_Pos, SOUND_WEAPON_SPAWN);
-		}
-		else
-			return;
-	}*/
 	// Check if a player intersected us
 	CCharacter *apEnts[MAX_CLIENTS];
 	int Num = GameWorld()->FindEntities(m_Pos, 20.0f, (CEntity **)apEnts, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
@@ -58,7 +43,7 @@ void CPickup::Tick()
 
 		if(pChr && pChr->IsAlive())
 		{
-			if(m_Layer == LAYER_SWITCH && m_Number > 0 && !GameServer()->Collision()->m_pSwitchers[m_Number].m_Status[pChr->Team()])
+			if(m_Layer == LAYER_SWITCH && m_Number > 0 && !Switchers()[m_Number].m_aStatus[pChr->Team()])
 				continue;
 			bool Sound = false;
 			// player picked us up, is someone was hooking us, let them go
@@ -93,14 +78,62 @@ void CPickup::Tick()
 					pChr->SetActiveWeapon(WEAPON_HAMMER);
 				break;
 
+			case POWERUP_ARMOR_SHOTGUN:
+				if(pChr->Team() == TEAM_SUPER)
+					continue;
+				if(pChr->GetWeaponGot(WEAPON_SHOTGUN))
+				{
+					pChr->SetWeaponGot(WEAPON_SHOTGUN, false);
+					pChr->SetWeaponAmmo(WEAPON_SHOTGUN, 0);
+					pChr->SetLastWeapon(WEAPON_GUN);
+					GameServer()->CreateSound(m_Pos, SOUND_PICKUP_ARMOR, pChr->TeamMask());
+				}
+				if(pChr->GetActiveWeapon() == WEAPON_SHOTGUN)
+					pChr->SetActiveWeapon(WEAPON_HAMMER);
+				break;
+
+			case POWERUP_ARMOR_GRENADE:
+				if(pChr->Team() == TEAM_SUPER)
+					continue;
+				if(pChr->GetWeaponGot(WEAPON_GRENADE))
+				{
+					pChr->SetWeaponGot(WEAPON_GRENADE, false);
+					pChr->SetWeaponAmmo(WEAPON_GRENADE, 0);
+					pChr->SetLastWeapon(WEAPON_GUN);
+					GameServer()->CreateSound(m_Pos, SOUND_PICKUP_ARMOR, pChr->TeamMask());
+				}
+				if(pChr->GetActiveWeapon() == WEAPON_GRENADE)
+					pChr->SetActiveWeapon(WEAPON_HAMMER);
+				break;
+
+			case POWERUP_ARMOR_NINJA:
+				if(pChr->Team() == TEAM_SUPER)
+					continue;
+				pChr->SetNinjaActivationDir(vec2(0, 0));
+				pChr->SetNinjaActivationTick(-500);
+				pChr->SetNinjaCurrentMoveTime(0);
+				break;
+
+			case POWERUP_ARMOR_LASER:
+				if(pChr->Team() == TEAM_SUPER)
+					continue;
+				if(pChr->GetWeaponGot(WEAPON_LASER))
+				{
+					pChr->SetWeaponGot(WEAPON_LASER, false);
+					pChr->SetWeaponAmmo(WEAPON_LASER, 0);
+					pChr->SetLastWeapon(WEAPON_GUN);
+					GameServer()->CreateSound(m_Pos, SOUND_PICKUP_ARMOR, pChr->TeamMask());
+				}
+				if(pChr->GetActiveWeapon() == WEAPON_LASER)
+					pChr->SetActiveWeapon(WEAPON_HAMMER);
+				break;
+
 			case POWERUP_WEAPON:
 
 				if(m_Subtype >= 0 && m_Subtype < NUM_WEAPONS && (!pChr->GetWeaponGot(m_Subtype) || pChr->GetWeaponAmmo(m_Subtype) != -1))
 				{
 					pChr->GiveWeapon(m_Subtype);
 
-					//RespawnTime = g_pData->m_aPickups[m_Type].m_Respawntime;
-
 					if(m_Subtype == WEAPON_GRENADE)
 						GameServer()->CreateSound(m_Pos, SOUND_PICKUP_GRENADE, pChr->TeamMask());
 					else if(m_Subtype == WEAPON_SHOTGUN)
@@ -117,37 +150,17 @@ void CPickup::Tick()
 			{
 				// activate ninja on target player
 				pChr->GiveNinja();
-				//RespawnTime = g_pData->m_aPickups[m_Type].m_Respawntime;
-
-				/*// loop through all players, setting their emotes
-					CCharacter *pC = static_cast<CCharacter *>(GameServer()->m_World.FindFirst(CGameWorld::ENTTYPE_CHARACTER));
-					for(; pC; pC = (CCharacter *)pC->TypeNext())
-					{
-						if (pC != pChr)
-							pC->SetEmote(EMOTE_SURPRISE, Server()->Tick() + Server()->TickSpeed());
-					}*/
 				break;
 			}
 			default:
 				break;
 			};
-
-			/*if(RespawnTime >= 0)
-			{
-				char aBuf[256];
-				str_format(aBuf, sizeof(aBuf), "pickup player='%d:%s' item=%d/%d",
-					pChr->GetPlayer()->GetCID(), Server()->ClientName(pChr->GetPlayer()->GetCID()), m_Type, m_Subtype);
-				GameServer()->Console()->Print(IConsole::OUTPUT_LEVEL_DEBUG, "game", aBuf);
-				m_SpawnTick = Server()->Tick() + Server()->TickSpeed() * RespawnTime;
-			}*/
 		}
 	}
 }
 
 void CPickup::TickPaused()
 {
-	/*if(m_SpawnTick != -1)
-		++m_SpawnTick;*/
 }
 
 void CPickup::Snap(int SnappingClient)
@@ -155,12 +168,12 @@ void CPickup::Snap(int SnappingClient)
 	if(NetworkClipped(SnappingClient))
 		return;
 
-	CCharacter *Char = GameServer()->GetPlayerChar(SnappingClient);
+	CCharacter *pChar = GameServer()->GetPlayerChar(SnappingClient);
 
 	if(SnappingClient != SERVER_DEMO_CLIENT && (GameServer()->m_apPlayers[SnappingClient]->GetTeam() == TEAM_SPECTATORS || GameServer()->m_apPlayers[SnappingClient]->IsPaused()) && GameServer()->m_apPlayers[SnappingClient]->m_SpectatorID != SPEC_FREEVIEW)
-		Char = GameServer()->GetPlayerChar(GameServer()->m_apPlayers[SnappingClient]->m_SpectatorID);
+		pChar = GameServer()->GetPlayerChar(GameServer()->m_apPlayers[SnappingClient]->m_SpectatorID);
 
-	int SnappingClientVersion = SnappingClient != SERVER_DEMO_CLIENT ? GameServer()->GetClientVersion(SnappingClient) : CLIENT_VERSIONNR;
+	int SnappingClientVersion = GameServer()->GetClientVersion(SnappingClient);
 
 	CNetObj_EntityEx *pEntData = 0;
 	if(SnappingClientVersion >= VERSION_DDNET_SWITCH && (m_Layer == LAYER_SWITCH || length(m_Core) > 0))
@@ -175,27 +188,34 @@ void CPickup::Snap(int SnappingClient)
 	else
 	{
 		int Tick = (Server()->Tick() % Server()->TickSpeed()) % 11;
-		if(Char && Char->IsAlive() && m_Layer == LAYER_SWITCH && m_Number > 0 && !GameServer()->Collision()->m_pSwitchers[m_Number].m_Status[Char->Team()] && !Tick)
+		if(pChar && pChar->IsAlive() && m_Layer == LAYER_SWITCH && m_Number > 0 && !Switchers()[m_Number].m_aStatus[pChar->Team()] && !Tick)
 			return;
 	}
 
 	int Size = Server()->IsSixup(SnappingClient) ? 3 * 4 : sizeof(CNetObj_Pickup);
-	CNetObj_Pickup *pP = static_cast<CNetObj_Pickup *>(Server()->SnapNewItem(NETOBJTYPE_PICKUP, GetID(), Size));
-	if(!pP)
+	CNetObj_Pickup *pPickup = static_cast<CNetObj_Pickup *>(Server()->SnapNewItem(NETOBJTYPE_PICKUP, GetID(), Size));
+	if(!pPickup)
 		return;
 
-	pP->m_X = (int)m_Pos.x;
-	pP->m_Y = (int)m_Pos.y;
-	pP->m_Type = m_Type;
+	pPickup->m_X = (int)m_Pos.x;
+	pPickup->m_Y = (int)m_Pos.y;
+	pPickup->m_Type = m_Type;
+	if(SnappingClientVersion < VERSION_DDNET_WEAPON_SHIELDS)
+	{
+		if(m_Type >= POWERUP_ARMOR_SHOTGUN && m_Type <= POWERUP_ARMOR_LASER)
+		{
+			pPickup->m_Type = POWERUP_ARMOR;
+		}
+	}
 	if(Server()->IsSixup(SnappingClient))
 	{
 		if(m_Type == POWERUP_WEAPON)
-			pP->m_Type = m_Subtype == WEAPON_SHOTGUN ? 3 : m_Subtype == WEAPON_GRENADE ? 2 : 4;
+			pPickup->m_Type = m_Subtype == WEAPON_SHOTGUN ? 3 : m_Subtype == WEAPON_GRENADE ? 2 : 4;
 		else if(m_Type == POWERUP_NINJA)
-			pP->m_Type = 5;
+			pPickup->m_Type = 5;
 	}
 	else
-		pP->m_Subtype = m_Subtype;
+		pPickup->m_Subtype = m_Subtype;
 }
 
 void CPickup::Move()
