@@ -4,7 +4,9 @@
 
 #include "entity.h"
 #include "gamecontext.h"
-#include "player.h"
+
+#include <base/system.h>
+#include <base/vmath.h>
 
 //////////////////////////////////////////////////
 // Event handler
@@ -49,32 +51,32 @@ void CEventHandler::Snap(int SnappingClient)
 	{
 		if(SnappingClient == SERVER_DEMO_CLIENT || CmaskIsSet(m_aClientMasks[i], SnappingClient))
 		{
-			CNetEvent_Common *ev = (CNetEvent_Common *)&m_aData[m_aOffsets[i]];
-			if(!NetworkClipped(GameServer(), SnappingClient, vec2(ev->m_X, ev->m_Y)))
+			CNetEvent_Common *pEvent = (CNetEvent_Common *)&m_aData[m_aOffsets[i]];
+			if(!NetworkClipped(GameServer(), SnappingClient, vec2(pEvent->m_X, pEvent->m_Y)))
 			{
 				int Type = m_aTypes[i];
 				int Size = m_aSizes[i];
-				const char *Data = &m_aData[m_aOffsets[i]];
+				const char *pData = &m_aData[m_aOffsets[i]];
 				if(GameServer()->Server()->IsSixup(SnappingClient))
-					EventToSixup(&Type, &Size, &Data);
+					EventToSixup(&Type, &Size, &pData);
 
-				void *d = GameServer()->Server()->SnapNewItem(Type, i, Size);
-				if(d)
-					mem_copy(d, Data, Size);
+				void *pItem = GameServer()->Server()->SnapNewItem(Type, i, Size);
+				if(pItem)
+					mem_copy(pItem, pData, Size);
 			}
 		}
 	}
 }
 
-void CEventHandler::EventToSixup(int *Type, int *Size, const char **pData)
+void CEventHandler::EventToSixup(int *pType, int *pSize, const char **ppData)
 {
 	static char s_aEventStore[128];
-	if(*Type == NETEVENTTYPE_DAMAGEIND)
+	if(*pType == NETEVENTTYPE_DAMAGEIND)
 	{
-		const CNetEvent_DamageInd *pEvent = (const CNetEvent_DamageInd *)(*pData);
+		const CNetEvent_DamageInd *pEvent = (const CNetEvent_DamageInd *)(*ppData);
 		protocol7::CNetEvent_Damage *pEvent7 = (protocol7::CNetEvent_Damage *)s_aEventStore;
-		*Type = -protocol7::NETEVENTTYPE_DAMAGE;
-		*Size = sizeof(*pEvent7);
+		*pType = -protocol7::NETEVENTTYPE_DAMAGE;
+		*pSize = sizeof(*pEvent7);
 
 		pEvent7->m_X = pEvent->m_X;
 		pEvent7->m_Y = pEvent->m_Y;
@@ -84,20 +86,20 @@ void CEventHandler::EventToSixup(int *Type, int *Size, const char **pData)
 		// or a separate array of "damage ind" events that's added in while snapping
 		pEvent7->m_HealthAmount = 1;
 
-		*pData = s_aEventStore;
+		*ppData = s_aEventStore;
 	}
-	else if(*Type == NETEVENTTYPE_SOUNDGLOBAL) // No more global sounds for the server
+	else if(*pType == NETEVENTTYPE_SOUNDGLOBAL) // No more global sounds for the server
 	{
-		const CNetEvent_SoundGlobal *pEvent = (const CNetEvent_SoundGlobal *)(*pData);
+		const CNetEvent_SoundGlobal *pEvent = (const CNetEvent_SoundGlobal *)(*ppData);
 		protocol7::CNetEvent_SoundWorld *pEvent7 = (protocol7::CNetEvent_SoundWorld *)s_aEventStore;
 
-		*Type = -protocol7::NETEVENTTYPE_SOUNDWORLD;
-		*Size = sizeof(*pEvent7);
+		*pType = -protocol7::NETEVENTTYPE_SOUNDWORLD;
+		*pSize = sizeof(*pEvent7);
 
 		pEvent7->m_SoundID = pEvent->m_SoundID;
 		pEvent7->m_X = pEvent->m_X;
 		pEvent7->m_Y = pEvent->m_Y;
 
-		*pData = s_aEventStore;
+		*ppData = s_aEventStore;
 	}
 }
