@@ -1,25 +1,20 @@
 #include "score.h"
-#include "entities/character.h"
 #include "gamemodes/DDRace.h"
 #include "player.h"
 #include "save.h"
 #include "scoreworker.h"
 
 #include <base/system.h>
-#include <engine/server/databases/connection.h>
 #include <engine/server/databases/connection_pool.h>
-#include <engine/server/sql_string_helpers.h>
 #include <engine/shared/config.h>
 #include <engine/shared/console.h>
 #include <engine/shared/linereader.h>
 #include <engine/storage.h>
 #include <game/generated/wordlist.h>
 
-#include <algorithm>
-#include <cstring>
-#include <fstream>
 #include <memory>
-#include <random>
+
+class IDbConnection;
 
 std::shared_ptr<CScorePlayerResult> CScore::NewSqlPlayerResult(int ClientID)
 {
@@ -68,8 +63,8 @@ void CScore::GeneratePassphrase(char *pBuf, int BufSize)
 		if(i != 0)
 			str_append(pBuf, " ", BufSize);
 		// TODO: decide if the slight bias towards lower numbers is ok
-		int Rand = m_Prng.RandomBits() % m_aWordlist.size();
-		str_append(pBuf, m_aWordlist[Rand].c_str(), BufSize);
+		int Rand = m_Prng.RandomBits() % m_vWordlist.size();
+		str_append(pBuf, m_vWordlist[Rand].c_str(), BufSize);
 	}
 }
 
@@ -98,17 +93,17 @@ CScore::CScore(CGameContext *pGameServer, CDbConnectionPool *pPool) :
 			char aWord[32] = {0};
 			sscanf(pLine, "%*s %31s", aWord);
 			aWord[31] = 0;
-			m_aWordlist.emplace_back(aWord);
+			m_vWordlist.emplace_back(aWord);
 		}
 		io_close(File);
 	}
 	else
 	{
 		dbg_msg("sql", "failed to open wordlist, using fallback");
-		m_aWordlist.assign(std::begin(g_aFallbackWordlist), std::end(g_aFallbackWordlist));
+		m_vWordlist.assign(std::begin(g_aFallbackWordlist), std::end(g_aFallbackWordlist));
 	}
 
-	if(m_aWordlist.size() < 1000)
+	if(m_vWordlist.size() < 1000)
 	{
 		dbg_msg("sql", "too few words in wordlist");
 		Server()->SetErrorShutdown("sql too few words in wordlist");
@@ -118,26 +113,26 @@ CScore::CScore(CGameContext *pGameServer, CDbConnectionPool *pPool) :
 	m_pPool->Execute(CScoreWorker::Init, std::move(Tmp), "load best time");
 }
 
-void CScore::LoadPlayerData(int ClientID)
+void CScore::LoadPlayerData(int ClientID, const char *pName)
 {
-	ExecPlayerThread(CScoreWorker::LoadPlayerData, "load player data", ClientID, "", 0);
+	ExecPlayerThread(CScoreWorker::LoadPlayerData, "load player data", ClientID, pName, 0);
 }
 
-void CScore::MapVote(int ClientID, const char *MapName)
+void CScore::MapVote(int ClientID, const char *pMapName)
 {
 	if(RateLimitPlayer(ClientID))
 		return;
-	ExecPlayerThread(CScoreWorker::MapVote, "map vote", ClientID, MapName, 0);
+	ExecPlayerThread(CScoreWorker::MapVote, "map vote", ClientID, pMapName, 0);
 }
 
-void CScore::MapInfo(int ClientID, const char *MapName)
+void CScore::MapInfo(int ClientID, const char *pMapName)
 {
 	if(RateLimitPlayer(ClientID))
 		return;
-	ExecPlayerThread(CScoreWorker::MapInfo, "map info", ClientID, MapName, 0);
+	ExecPlayerThread(CScoreWorker::MapInfo, "map info", ClientID, pMapName, 0);
 }
 
-void CScore::SaveScore(int ClientID, float Time, const char *pTimestamp, float CpTime[NUM_CHECKPOINTS], bool NotEligible)
+void CScore::SaveScore(int ClientID, float Time, const char *pTimestamp, float aTimeCp[NUM_CHECKPOINTS], bool NotEligible)
 {
 	CConsole *pCon = (CConsole *)GameServer()->Console();
 	if(pCon->m_Cheated || NotEligible)
@@ -155,7 +150,7 @@ void CScore::SaveScore(int ClientID, float Time, const char *pTimestamp, float C
 	Tmp->m_Time = Time;
 	str_copy(Tmp->m_aTimestamp, pTimestamp, sizeof(Tmp->m_aTimestamp));
 	for(int i = 0; i < NUM_CHECKPOINTS; i++)
-		Tmp->m_aCpCurrent[i] = CpTime[i];
+		Tmp->m_aCurrentTimeCp[i] = aTimeCp[i];
 
 	m_pPool->ExecuteWrite(CScoreWorker::SaveScore, std::move(Tmp), "save score");
 }
@@ -210,7 +205,7 @@ void CScore::ShowTeamTop5(int ClientID, int Offset)
 	ExecPlayerThread(CScoreWorker::ShowTeamTop5, "show team top5", ClientID, "", Offset);
 }
 
-void CScore::ShowTeamTop5(int ClientID, const char *pName, int Offset)
+void CScore::ShowPlayerTeamTop5(int ClientID, const char *pName, int Offset)
 {
 	if(RateLimitPlayer(ClientID))
 		return;
@@ -273,7 +268,7 @@ void CScore::RandomUnfinishedMap(int ClientID, int Stars)
 	m_pPool->Execute(CScoreWorker::RandomUnfinishedMap, std::move(Tmp), "random unfinished map");
 }
 
-void CScore::SaveTeam(int ClientID, const char *Code, const char *Server)
+void CScore::SaveTeam(int ClientID, const char *pCode, const char *pServer)
 {
 	if(RateLimitPlayer(ClientID))
 		return;
@@ -290,9 +285,9 @@ void CScore::SaveTeam(int ClientID, const char *Code, const char *Server)
 	pController->m_Teams.SetSaving(Team, SaveResult);
 
 	auto Tmp = std::make_unique<CSqlTeamSave>(SaveResult);
-	str_copy(Tmp->m_aCode, Code, sizeof(Tmp->m_aCode));
+	str_copy(Tmp->m_aCode, pCode, sizeof(Tmp->m_aCode));
 	str_copy(Tmp->m_aMap, g_Config.m_SvMap, sizeof(Tmp->m_aMap));
-	str_copy(Tmp->m_aServer, Server, sizeof(Tmp->m_aServer));
+	str_copy(Tmp->m_aServer, pServer, sizeof(Tmp->m_aServer));
 	str_copy(Tmp->m_aClientName, this->Server()->ClientName(ClientID), sizeof(Tmp->m_aClientName));
 	Tmp->m_aGeneratedCode[0] = '\0';
 	GeneratePassphrase(Tmp->m_aGeneratedCode, sizeof(Tmp->m_aGeneratedCode));
@@ -301,7 +296,7 @@ void CScore::SaveTeam(int ClientID, const char *Code, const char *Server)
 	m_pPool->ExecuteWrite(CScoreWorker::SaveTeam, std::move(Tmp), "save team");
 }
 
-void CScore::LoadTeam(const char *Code, int ClientID)
+void CScore::LoadTeam(const char *pCode, int ClientID)
 {
 	if(RateLimitPlayer(ClientID))
 		return;
@@ -323,7 +318,7 @@ void CScore::LoadTeam(const char *Code, int ClientID)
 	SaveResult->m_Status = CScoreSaveResult::LOAD_FAILED;
 	pController->m_Teams.SetSaving(Team, SaveResult);
 	auto Tmp = std::make_unique<CSqlTeamLoad>(SaveResult);
-	str_copy(Tmp->m_aCode, Code, sizeof(Tmp->m_aCode));
+	str_copy(Tmp->m_aCode, pCode, sizeof(Tmp->m_aCode));
 	str_copy(Tmp->m_aMap, g_Config.m_SvMap, sizeof(Tmp->m_aMap));
 	Tmp->m_ClientID = ClientID;
 	str_copy(Tmp->m_aRequestingPlayer, Server()->ClientName(ClientID), sizeof(Tmp->m_aRequestingPlayer));
