@@ -1,15 +1,15 @@
 #ifndef GAME_SERVER_SCORE_H
 #define GAME_SERVER_SCORE_H
 
-#include <engine/map.h>
-#include <engine/server/databases/connection_pool.h>
 #include <game/prng.h>
-#include <game/voting.h>
 
-#include "save.h"
 #include "scoreworker.h"
 
+class CDbConnectionPool;
 class CGameContext;
+class IDbConnection;
+class IServer;
+struct ISqlData;
 
 class CScore
 {
@@ -21,7 +21,7 @@ class CScore
 	CGameContext *m_pGameServer;
 	IServer *m_pServer;
 
-	std::vector<std::string> m_aWordlist;
+	std::vector<std::string> m_vWordlist;
 	CPrng m_Prng;
 	void GeneratePassphrase(char *pBuf, int BufSize);
 
@@ -46,8 +46,8 @@ public:
 
 	void MapInfo(int ClientID, const char *pMapName);
 	void MapVote(int ClientID, const char *pMapName);
-	void LoadPlayerData(int ClientID);
-	void SaveScore(int ClientID, float Time, const char *pTimestamp, float aCpTime[NUM_CHECKPOINTS], bool NotEligible);
+	void LoadPlayerData(int ClientID, const char *pName = "");
+	void SaveScore(int ClientID, float Time, const char *pTimestamp, float aTimeCp[NUM_CHECKPOINTS], bool NotEligible);
 
 	void SaveTeamScore(int *pClientIDs, unsigned int Size, float Time, const char *pTimestamp);
 
@@ -55,7 +55,7 @@ public:
 	void ShowRank(int ClientID, const char *pName);
 
 	void ShowTeamTop5(int ClientID, int Offset = 1);
-	void ShowTeamTop5(int ClientID, const char *pName, int Offset = 1);
+	void ShowPlayerTeamTop5(int ClientID, const char *pName, int Offset = 1);
 	void ShowTeamRank(int ClientID, const char *pName);
 
 	void ShowTopPoints(int ClientID, int Offset = 1);
