@@ -1,6 +1,5 @@
 /* (c) Shereef Marzouk. See "licence DDRace.txt" and the readme.txt in the root of the distribution for more information. */
 #include "teamscore.h"
-#include <base/math.h>
 #include <engine/shared/config.h>
 
 CTeamsCore::CTeamsCore()
@@ -10,34 +9,34 @@ CTeamsCore::CTeamsCore()
 
 bool CTeamsCore::SameTeam(int ClientID1, int ClientID2) const
 {
-	return m_Team[ClientID1] == TEAM_SUPER || m_Team[ClientID2] == TEAM_SUPER || m_Team[ClientID1] == m_Team[ClientID2];
+	return m_aTeam[ClientID1] == TEAM_SUPER || m_aTeam[ClientID2] == TEAM_SUPER || m_aTeam[ClientID1] == m_aTeam[ClientID2];
 }
 
 int CTeamsCore::Team(int ClientID) const
 {
-	return m_Team[ClientID];
+	return m_aTeam[ClientID];
 }
 
 void CTeamsCore::Team(int ClientID, int Team)
 {
 	dbg_assert(Team >= TEAM_FLOCK && Team <= TEAM_SUPER, "invalid team");
-	m_Team[ClientID] = Team;
+	m_aTeam[ClientID] = Team;
 }
 
 bool CTeamsCore::CanKeepHook(int ClientID1, int ClientID2) const
 {
-	if(m_Team[ClientID1] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || m_Team[ClientID2] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || ClientID1 == ClientID2)
+	if(m_aTeam[ClientID1] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || m_aTeam[ClientID2] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || ClientID1 == ClientID2)
 		return true;
-	return m_Team[ClientID1] == m_Team[ClientID2];
+	return m_aTeam[ClientID1] == m_aTeam[ClientID2];
 }
 
 bool CTeamsCore::CanCollide(int ClientID1, int ClientID2) const
 {
-	if(m_Team[ClientID1] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || m_Team[ClientID2] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || ClientID1 == ClientID2)
+	if(m_aTeam[ClientID1] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || m_aTeam[ClientID2] == (m_IsDDRace16 ? VANILLA_TEAM_SUPER : TEAM_SUPER) || ClientID1 == ClientID2)
 		return true;
-	if(m_IsSolo[ClientID1] || m_IsSolo[ClientID2])
+	if(m_aIsSolo[ClientID1] || m_aIsSolo[ClientID2])
 		return false;
-	return m_Team[ClientID1] == m_Team[ClientID2];
+	return m_aTeam[ClientID1] == m_aTeam[ClientID2];
 }
 
 void CTeamsCore::Reset()
@@ -47,9 +46,9 @@ void CTeamsCore::Reset()
 	for(int i = 0; i < MAX_CLIENTS; ++i)
 	{
 		if(g_Config.m_SvTeam == SV_TEAM_FORCED_SOLO)
-			m_Team[i] = i;
+			m_aTeam[i] = i;
 		else
-			m_Team[i] = TEAM_FLOCK;
-		m_IsSolo[i] = false;
+			m_aTeam[i] = TEAM_FLOCK;
+		m_aIsSolo[i] = false;
 	}
 }
