@@ -22,9 +22,8 @@ TEST(SecureRandom, Below1)
 TEST(SecureRandom, Below)
 {
 	int BOUNDS[] = {2, 3, 4, 5, 10, 100, 127, 128, 129};
-	for(unsigned i = 0; i < sizeof(BOUNDS) / sizeof(BOUNDS[0]); i++)
+	for(auto Below : BOUNDS)
 	{
-		int Below = BOUNDS[i];
 		for(int j = 0; j < 10; j++)
 		{
 			int Random = secure_rand_below(Below);
