@@ -115,7 +115,7 @@ TEST(Str, Utf8FixTruncation)
 		"привет Наташа",
 		"до свидания\xffОлег",
 	};
-	for(unsigned i = 0; i < sizeof(aaBuf) / sizeof(aaBuf[0]); i++)
+	for(unsigned i = 0; i < std::size(aaBuf); i++)
 	{
 		EXPECT_EQ(str_utf8_fix_truncation(aaBuf[i]), str_length(apExpected[i]));
 		EXPECT_STREQ(aaBuf[i], apExpected[i]);
@@ -221,15 +221,93 @@ TEST(Str, HexDecode)
 	EXPECT_STREQ(aOut, "ABCD");
 }
 
+void StrBase64Str(char *pBuffer, int BufferSize, const char *pString)
+{
+	str_base64(pBuffer, BufferSize, pString, str_length(pString));
+}
+
+TEST(Str, Base64)
+{
+	char aBuf[128];
+	str_base64(aBuf, sizeof(aBuf), "\0", 1);
+	EXPECT_STREQ(aBuf, "AA==");
+	str_base64(aBuf, sizeof(aBuf), "\0\0", 2);
+	EXPECT_STREQ(aBuf, "AAA=");
+	str_base64(aBuf, sizeof(aBuf), "\0\0\0", 3);
+	EXPECT_STREQ(aBuf, "AAAA");
+
+	StrBase64Str(aBuf, sizeof(aBuf), "");
+	EXPECT_STREQ(aBuf, "");
+
+	// https://en.wikipedia.org/w/index.php?title=Base64&oldid=1033503483#Output_padding
+	StrBase64Str(aBuf, sizeof(aBuf), "pleasure.");
+	EXPECT_STREQ(aBuf, "cGxlYXN1cmUu");
+	StrBase64Str(aBuf, sizeof(aBuf), "leasure.");
+	EXPECT_STREQ(aBuf, "bGVhc3VyZS4=");
+	StrBase64Str(aBuf, sizeof(aBuf), "easure.");
+	EXPECT_STREQ(aBuf, "ZWFzdXJlLg==");
+	StrBase64Str(aBuf, sizeof(aBuf), "asure.");
+	EXPECT_STREQ(aBuf, "YXN1cmUu");
+	StrBase64Str(aBuf, sizeof(aBuf), "sure.");
+	EXPECT_STREQ(aBuf, "c3VyZS4=");
+}
+
+TEST(Str, Base64Decode)
+{
+	char aOut[17];
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), ""), 0);
+	EXPECT_STREQ(aOut, "XXXXXXXXXXXXXXXX");
+
+	// https://en.wikipedia.org/w/index.php?title=Base64&oldid=1033503483#Output_padding
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "cGxlYXN1cmUu"), 9);
+	EXPECT_STREQ(aOut, "pleasure.XXXXXXX");
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "bGVhc3VyZS4="), 8);
+	EXPECT_STREQ(aOut, "leasure.XXXXXXXX");
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "ZWFzdXJlLg=="), 7);
+	EXPECT_STREQ(aOut, "easure.XXXXXXXXX");
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "YXN1cmUu"), 6);
+	EXPECT_STREQ(aOut, "asure.XXXXXXXXXX");
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "c3VyZS4="), 5);
+	EXPECT_STREQ(aOut, "sure.XXXXXXXXXXX");
+	str_copy(aOut, "XXXXXXXXXXXXXXXX", sizeof(aOut));
+	EXPECT_EQ(str_base64_decode(aOut, sizeof(aOut), "////"), 3);
+	EXPECT_STREQ(aOut, "\xff\xff\xffXXXXXXXXXXXXX");
+}
+
+TEST(Str, Base64DecodeError)
+{
+	char aBuf[128];
+	// Wrong padding.
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "A"), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "AA"), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "AAA"), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "A==="), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "=AAA"), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "===="), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "AAA=AAAA"), 0);
+	// Invalid characters.
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "----"), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "AAAA "), 0);
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "AAA "), 0);
+	// Invalid padding values.
+	EXPECT_LT(str_base64_decode(aBuf, sizeof(aBuf), "//=="), 0);
+}
+
 TEST(Str, Tokenize)
 {
 	char aTest[] = "GER,RUS,ZAF,BRA,CAN";
-	const char *aOut[] = {"GER", "RUS", "ZAF", "BRA", "CAN"};
+	const char *apOut[] = {"GER", "RUS", "ZAF", "BRA", "CAN"};
 	char aBuf[4];
 
 	int n = 0;
-	for(const char *tok = aTest; (tok = str_next_token(tok, ",", aBuf, sizeof(aBuf)));)
-		EXPECT_STREQ(aOut[n++], aBuf);
+	for(const char *pTok = aTest; (pTok = str_next_token(pTok, ",", aBuf, sizeof(aBuf)));)
+		EXPECT_STREQ(apOut[n++], aBuf);
 
 	char aTest2[] = "";
 	EXPECT_EQ(str_next_token(aTest2, ",", aBuf, sizeof(aBuf)), nullptr);
@@ -259,7 +337,7 @@ TEST(Str, InList)
 	EXPECT_TRUE(str_in_list("abc,,def", ",", "def"));
 }
 
-TEST(Str, StrFormat)
+TEST(Str, Format)
 {
 	char aBuf[4];
 	EXPECT_EQ(str_format(aBuf, 4, "%d:", 9), 2);
@@ -270,7 +348,7 @@ TEST(Str, StrFormat)
 	EXPECT_STREQ(aBuf, "99:");
 }
 
-TEST(Str, StrFormatTruncate)
+TEST(Str, FormatTruncate)
 {
 	const char *pStr = "DDNet最好了";
 	char aBuf[64];
@@ -296,32 +374,32 @@ TEST(Str, StrFormatTruncate)
 	EXPECT_STREQ(aBuf, "DDNet最好了");
 }
 
-TEST(Str, StrCopyNum)
+TEST(Str, CopyNum)
 {
-	const char *foo = "Foobaré";
+	const char *pFoo = "Foobaré";
 	char aBuf[64];
-	str_utf8_truncate(aBuf, 3, foo, 1);
+	str_utf8_truncate(aBuf, 3, pFoo, 1);
 	EXPECT_STREQ(aBuf, "F");
-	str_utf8_truncate(aBuf, 3, foo, 2);
+	str_utf8_truncate(aBuf, 3, pFoo, 2);
 	EXPECT_STREQ(aBuf, "Fo");
-	str_utf8_truncate(aBuf, 3, foo, 3);
+	str_utf8_truncate(aBuf, 3, pFoo, 3);
 	EXPECT_STREQ(aBuf, "Fo");
-	str_utf8_truncate(aBuf, sizeof(aBuf), foo, 6);
+	str_utf8_truncate(aBuf, sizeof(aBuf), pFoo, 6);
 	EXPECT_STREQ(aBuf, "Foobar");
-	str_utf8_truncate(aBuf, sizeof(aBuf), foo, 7);
+	str_utf8_truncate(aBuf, sizeof(aBuf), pFoo, 7);
 	EXPECT_STREQ(aBuf, "Foobaré");
-	str_utf8_truncate(aBuf, sizeof(aBuf), foo, 0);
+	str_utf8_truncate(aBuf, sizeof(aBuf), pFoo, 0);
 	EXPECT_STREQ(aBuf, "");
 
 	char aBuf2[8];
-	str_utf8_truncate(aBuf2, sizeof(aBuf2), foo, 7);
+	str_utf8_truncate(aBuf2, sizeof(aBuf2), pFoo, 7);
 	EXPECT_STREQ(aBuf2, "Foobar");
 	char aBuf3[9];
-	str_utf8_truncate(aBuf3, sizeof(aBuf3), foo, 7);
+	str_utf8_truncate(aBuf3, sizeof(aBuf3), pFoo, 7);
 	EXPECT_STREQ(aBuf3, "Foobaré");
 }
 
-TEST(Str, StrCopy)
+TEST(Str, Copy)
 {
 	const char *pStr = "DDNet最好了";
 	char aBuf[64];
@@ -384,7 +462,7 @@ TEST(Str, Utf8Stats)
 	EXPECT_EQ(Count, 3);
 }
 
-TEST(Str, StrTime)
+TEST(Str, Time)
 {
 	char aBuf[32] = "foobar";
 
@@ -436,7 +514,7 @@ TEST(Str, StrTime)
 	EXPECT_STREQ(aBuf, "2057:36.78");
 }
 
-TEST(Str, StrTimeFloat)
+TEST(Str, TimeFloat)
 {
 	char aBuf[64];
 	EXPECT_EQ(str_time_float(123456.78, TIME_DAYS, aBuf, sizeof(aBuf)), 11);
@@ -445,3 +523,19 @@ TEST(Str, StrTimeFloat)
 	EXPECT_EQ(str_time_float(12.16, TIME_HOURS_CENTISECS, aBuf, sizeof(aBuf)), 8);
 	EXPECT_STREQ(aBuf, "00:12.16");
 }
+
+TEST(Str, HasCc)
+{
+	EXPECT_FALSE(str_has_cc(""));
+	EXPECT_FALSE(str_has_cc("a"));
+	EXPECT_FALSE(str_has_cc("Merhaba dünya!"));
+
+	EXPECT_TRUE(str_has_cc("\n"));
+	EXPECT_TRUE(str_has_cc("\n"));
+	EXPECT_TRUE(str_has_cc("\r"));
+	EXPECT_TRUE(str_has_cc("\t"));
+	EXPECT_TRUE(str_has_cc("a\n"));
+	EXPECT_TRUE(str_has_cc("a\rb"));
+	EXPECT_TRUE(str_has_cc("\tb"));
+	EXPECT_TRUE(str_has_cc("\n\n"));
+}
